/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a common header to provide some general functions which are used by examples.
 */

#ifndef _FSDK_COMMON_H_
#define _FSDK_COMMON_H_

/* Common header files. */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <string>
#include <iostream>
#include <vector>
#include <locale.h>
#include<assert.h>

#define FSThreadHandle	void*
#if defined(_MSC_VER)
	/* Include MS VC header files, for Windows platforms. */

	#if _MSC_VER >= 1200
		#define _WIN32_WINNT 0x0501 //_WIN32_WINNT_WINXP
	#endif
	#include <windows.h>
	#include <direct.h>
	#if _MSC_VER > 1200
		#define MBSTOWCS_S(pReturnValue, wcstr, sizeInWords, mbstr, count)	mbstowcs_s(pReturnValue, wcstr, sizeInWords, mbstr, count)
	#else
		#define MBSTOWCS_S(pReturnValue, wcstr, sizeInWords, mbstr, count)	*pReturnValue = mbstowcs(wcstr, mbstr, count)
	#endif

#elif defined(__GNUC__)
	/* Include GCC header files, for Linux/Unix platforms. */
	#include <sys/stat.h>
	#include <unistd.h>
	#include <pthread.h>
	#include "stdarg.h"
	#include <dirent.h>
	#include <dlfcn.h>
	#ifndef FALSE
		#define FALSE 0
	#endif
	#ifndef TRUE
		#define TRUE 1
	#endif

	#define MBSTOWCS_S(pReturnValue, wcstr, sizeInWords, mbstr, count)	*pReturnValue = mbstowcs(wcstr, mbstr, count)
#else
	/* Wrong platforms. */

	#error "Wrong compiler!"
#endif

#if defined(__APPLE__)
    #include "TargetConditionals.h"
    #if (defined(TARGET_OS_IPHONE) && TARGET_OS_IPHONE == 1) || (defined(TARGET_IPHONE_SIMULATOR) && TARGET_IPHONE_SIMULATOR == 1)
        //It means IOS demo
        #define FSDK_IOS_DEMO
    #endif
#endif

/* Include Foxit GSDK header files. */
#include "../../../include/fsdk.h"
/* Ignore namespace check for STL. */
using namespace std;
#define NO_LICENSE "Invalid license!!! Please checking whether the license has related module!!!\r\n";

//SDKFunctionList
//CRT
typedef FS_RESULT (*FSCRTLibraryCreateDefaultMgr)();
typedef FS_RESULT (*FSCRTLibraryCreateMgr)(FS_LPVOID memory, FS_DWORD size, FSCRT_MEMMGRHANDLER* memMgr);
typedef FS_RESULT (*FSCRTLibraryDestroyMgr)();
typedef FS_RESULT (*FSCRTBStrInit)(FSCRT_BSTR* bstr);
typedef FS_RESULT (*FSCRTBStrClear)(FSCRT_BSTR* bstr);
typedef FS_RESULT (*FSCRTLicenseUnlockLibrary)(const FSCRT_BSTR* licenseKey, const FSCRT_BSTR* unlockCode);
typedef FS_RESULT (*FSCRTLicenseGetType)(FS_INT32* type);
typedef FS_RESULT (*FSCRTUTF8FromUTF16LE)(FSCRT_BSTR* utf8Str, const FS_WORD* buffer, FS_DWORD length);
typedef FS_RESULT (*FSCRTUTF8FromUTF32LE)(FSCRT_BSTR* utf8Str, const FS_DWORD* buffer, FS_DWORD length);
typedef FS_RESULT (*FSCRTFileCreateFromFileName)(const FSCRT_BSTR* fileName, FS_DWORD fileModes, FSCRT_FILE* file);
typedef FS_RESULT (*FSCRTFileCreate)(FSCRT_FILEHANDLER* fileStream, FSCRT_FILE* file);
typedef FS_RESULT (*FSCRTFileRelease)(FSCRT_FILE file);
typedef FS_RESULT (*FSCRTProgressContinue)(FSCRT_PROGRESS progress, FSCRT_PAUSEHANDLER* pause);
typedef FS_RESULT (*FSCRTProgressRelease)(FSCRT_PROGRESS progress);

//PDF
typedef FS_RESULT (*FSCRTPDFModuleInitialize)();
typedef FS_RESULT (*FSCRTPDFModuleFinalize)();
typedef FS_RESULT (*FSPDFDocStartLoad)(FSCRT_FILE file, const FSCRT_BSTR* password, FSCRT_DOCUMENT* document, FSCRT_PROGRESS* progress);
typedef FS_RESULT (*FSPDFDocCountPages)(FSCRT_DOCUMENT document, FS_INT32* count);
typedef FS_RESULT (*FSPDFDocGetPage)(FSCRT_DOCUMENT document, FS_INT32 index, FSCRT_PAGE* page);
typedef FS_RESULT (*FSPDFDocStartSaveToFile)(FSCRT_DOCUMENT document, FSCRT_FILE file, FS_INT32 flag, FSCRT_PROGRESS* saveProgress);
typedef FS_RESULT (*FSPDFDocClose)(FSCRT_DOCUMENT document);
typedef FS_RESULT (*FSPDFPageStartParse)(FSCRT_PAGE page, FS_DWORD flag, FSCRT_PROGRESS* parseProgress);
typedef FS_RESULT (*FSPDFPageClear)(FSCRT_PAGE page);
typedef FS_RESULT (*FSPDFTextPageLoad)(FSCRT_PAGE pdfPage, FSPDF_TEXTPAGE* textPage);
typedef FS_RESULT (*FSPDFTextPageExportToFile)(FSPDF_TEXTPAGE textPage, FSCRT_FILE file);
typedef FS_RESULT (*FSPDFTextPageRelease)(FSPDF_TEXTPAGE textPage);


class SDKFunctionList {
public:
	SDKFunctionList(void* hModule);
	~SDKFunctionList();
	//CRT functions
	FSCRTLibraryCreateDefaultMgr	pFSCRTLibraryCreateDefaultMgr;
	FSCRTLibraryCreateMgr			pFSCRTLibraryCreateMgr;
	FSCRTLibraryDestroyMgr			pFSCRTLibraryDestroyMgr;
	FSCRTBStrInit					pFSCRTBStrInit;
	FSCRTBStrClear					pFSCRTBStrClear;
	FSCRTLicenseUnlockLibrary		pFSCRTLicenseUnlockLibrary;
	FSCRTLicenseGetType				pFSCRTLicenseGetType;
	FSCRTUTF8FromUTF16LE			pFSCRTUTF8FromUTF16LE;
	FSCRTUTF8FromUTF32LE			pFSCRTUTF8FromUTF32LE;
	FSCRTFileCreateFromFileName		pFSCRTFileCreateFromFileName;
	FSCRTFileCreate					pFSCRTFileCreate;
	FSCRTFileRelease				pFSCRTFileRelease;
	FSCRTProgressContinue			pFSCRTProgressContinue;
	FSCRTProgressRelease			pFSCRTProgressRelease;

	//PDF functions
	FSCRTPDFModuleInitialize		pFSCRTPDFModuleInitialize;
	FSCRTPDFModuleFinalize			pFSCRTPDFModuleFinalize;
	FSPDFDocStartLoad				pFSPDFDocStartLoad;
	FSPDFDocCountPages				pFSPDFDocCountPages;
	FSPDFDocGetPage					pFSPDFDocGetPage;
	FSPDFDocStartSaveToFile			pFSPDFDocStartSaveToFile;
	FSPDFDocClose					pFSPDFDocClose;

	FSPDFPageStartParse				pFSPDFPageStartParse;
	FSPDFPageClear					pFSPDFPageClear;
	FSPDFTextPageLoad				pFSPDFTextPageLoad;
	FSPDFTextPageExportToFile		pFSPDFTextPageExportToFile;
	FSPDFTextPageRelease			pFSPDFTextPageRelease;

private:
	void*							m_hModule;
};
//SDKManager
class SDKManager {
public:
	SDKManager ();
	~SDKManager();

	FS_RESULT			InitManager(const char* pDLLPath);
	void				DestroyManager();
	SDKFunctionList*	GetSDKFunctionList() { assert(NULL != m_pFunList); return m_pFunList; }

private:
	FS_RESULT			InitializeLibray(bool useDefMgr);
	void				FinalizeLibrary();

private:
	char*				m_pDLLPath;
	void*				m_hModule;
	SDKFunctionList*	m_pFunList;
	//variables for extension manager
	FSCRT_MEMMGRHANDLER	m_MemMgrHandler; 
	FS_LPVOID			m_pGlobalBuffer;
	bool				m_bInit;
};

//DLLManager: contain the dll path and the flag indicated the dll is used or not 
typedef struct _DLLManager {
	string	strDllName;
	bool	bDLLUsed;
}DLLManager;

//FoxitParams: the struct contains some global params 
typedef struct _FoxitParams {
	DLLManager* pDllManager;
	void* pFileDirHandle;
	unsigned int nThreadCount;
	bool bLastFile;
	unsigned int nSuccessCount;
	unsigned int nFailCount;
	string strInputDir;
	string strOutputDir;
} FoxitParams;

/*******************************************************************************/
/* Directories                                                                 */
/*******************************************************************************/
/**
 * @brief	Set the current executing file path.
 *
 * @param[in] exeFile		The current executing file path.
 *
 * @note	In order to track accessing path for each example in Foxit GSDK, this function set the current executing file path and store internally.
 *			In main function of each example, this is the first function call.
 */
void			FSDK_SetExecuteFilePath(const char* exeFile);
/**
 * @brief	Get the current executing folder.
 *
 * @return	The current executing folder.
 *
 * @note	This function returns executing folder according to the file path which is set by calling of FSDK_SetExecuteFilePath.
 */
string			FSDK_GetExecuteFolder();

/*******************************************************************************/
/* Time                                                                        */
/*******************************************************************************/
/**
 * @brief	Get the current processor time.
 *
 * @return	The current processor time in milliseconds.
 *
 * @note	This function provides a simple way to get the current processor time used by the calling process.
 */
unsigned int	FSDK_GetClock();

/*******************************************************************************/
/* File system                                                                 */
/*******************************************************************************/
/**
 * @brief	Open a folder to enumerate all files in it.
 *
 * @param[in] pathName	Folder name to be enumerated.
 *
 * @return	A pointer object to the given folder.
 *
 * @note	This function is used to open a folder when user need enumerate its contents.
 *			Call FSDK_GetNextFile to get a file object in the folder, or call FSDK_CloseFolder to close the folder.
 */
void*			FSDK_OpenFolder(const char* pathName);
 
/**
 * @brief	Close a folder.
 *
 * @param[in] folder	Folder pointer which is returned by previous call to FSDK_OpenFolder.
 *
 * @note	Call this function to close folder after enumerating files.
 */
void			FSDK_CloseFolder(void* folder);
 
/**
 * @brief	Get a file in folder.
 *
 * @param[in] folder	Folder pointer which is returned by previous call to FSDK_OpenFolder.
 * @param[out] isDir	Specifies the file is a directory object if its value is TRUE or a common file object if its value is FALSE.
 *
 * @return	File name enumerated in folder, without path name.
 *
 * @note	If the returned file name is empty, isDir makes no sense.
 */
string			FSDK_GetNextFile(void* folder, FS_BOOL* isDir);
 
/**
 * @brief	Create a folder, if the folder don't exist.
 *
 * @param[in] pathName	Folder name to be created.
 *
 * @return	true if the folder exist or create folder success.
 *
 */
bool			FSDK_CreatFolder(const char* pathName);
 
/**
 * @brief	Check the folder exist or not.
 *
 * @param[in] pathName	Folder name.
 *
 * @return	true if the folder exist or false.
 *
 */
bool			FSDK_FolderExist(const char* pathName);
 
/**
 * @brief	Copy the specific number DLL to the specific path.
 *
 * @param[in] pathName	Folder name.
 *
 * @return	true if the folder exist or false.
 *
 */
bool		FSDK_CopyDLL(void* pParam, const char* pDLLPath, unsigned int nThreadCount);
 
/*******************************************************************************/
/* Multiple threads                                                            */
/*******************************************************************************/
/**
 * @brief	Type definition for callback function of thread.
 *
 * @param[in] param		User-defined parameter passed to thread procedure.
 *
 * @note	Pass address of thread procedure when call FSDK_CreateThread.
 */
typedef void	(*FSDK_CALLBACK_ThreadProc)(void* param);

/**
 * @brief	Create thread.
 *
 * @param[in] threadProc	Address of thread procedure.
 * @param[in] param			User-defined parameter passed to thread procedure.
 *
 * @return	Thread handle, NULL if failed.
 *
 * @note	This function create a new thread. New thread runs immediately when it is created successfully.
 */
void*			FSDK_CreateThread(FSDK_CALLBACK_ThreadProc threadProc, void* param);

/**
 * @brief	Wait exit for single thread.
 *
 * @param[in] thread	Thread handle which is returned by previous call to FSDK_CreateThread.
 *
 * @return	0 if successful, other value if failed.
 *
 * @note	Call this function to wait for the exit status of single thread.
 */
int				FSDK_WaitForSingleThread(void* thread);

/**
 * @brief	Wait full exit for multiple threads.
 *
 * @param[in] threads	Array to multiple threads. Each element is a thread handle which is returned by previous call to FSDK_CreateThread.
 * @param[in] count		Count of multiple threads which are stored in threads array.
 *
 * @return	0 if successful, other value if failed.
 *
 * @note	Call this function to wait for the exit status of multiple threads.
 */
int				FSDK_WaitForMultipleThreads(void** threads, int count);

/**
 * @brief	 Get current thread id.
 *
 * @return	The thread id handle.
*/
FSCRT_THREAD	FSDK_GetCurrentThread();
/*******************************************************************************/
/* Mutex                                                                       */
/*******************************************************************************/
#if defined(_MSC_VER)
	/* Lock definition for MS VC compiler, use critical section. */
	typedef CRITICAL_SECTION	FSDK_LOCK;
#elif defined(__GNUC__)
	/* Lock definition for GCC compiler, use posix mutex. */
	typedef pthread_mutex_t		FSDK_LOCK;
#else
	/* Lock definition for other compilers, use void pointer. */
	typedef void*				FSDK_LOCK;
#endif

/**
 * @brief	Initialize a lock.
 *
 * @param[in] lock	Pointer to a lock object.
 *
 * @return	TRUE if successful, FALSE if failed.
 *
 * @note	Call this function to initialize a lock object before use it.
 */
bool			FSDK_InitializeLock(FSDK_LOCK* lock);
/**
 * @brief	Finalize a lock.
 *
 * @param[in] lock	Pointer to a lock object.
 *
 * @note	Call this function to finalize a lock object after use it.
 */
void			FSDK_FinalizeLock(FSDK_LOCK* lock);
/**
 * @brief	Lock a lock object.
 *
 * @param[in] lock	Pointer to a lock object.
 *
 * @note	Call this function to lock.
 */
void			FSDK_Lock(FSDK_LOCK* lock);
/**
 * @brief	Unlock a lock object.
 *
 * @param[in] lock	Pointer to a lock object.
 *
 * @note	Call this function to unlock.
 */
void			FSDK_Unlock(FSDK_LOCK* lock);

/*******************************************************************************/
/* File access                                                                 */
/*******************************************************************************/
/**
 * @brief	Compare extension name.
 *
 * @param[in] fileName	Source file name.
 * @param[in] extName	Extension name.
 *
 * @return	TRUE if extension name are same, or FALSE if different.
 *
 * @note	Call this function to distinguish file types, just a simple way.
 */
bool			FSDK_CompareExtName(const char* fileName, const char* extName);

/**
 * @brief	Open a file with given access mode.
 *
 * @param[in] fileName	File name to be opened.
 * @param[in] fileMode	File mode string to specify how to access, it has the same value as fopen function.
 * @param[in] useDef	Specifies how to open file.
 *						If it's TRUE, this function calls FSCRT_File_CreateFromFileName to open directly.
 *						If it's FALSE, this function calls FSCRT_File_Create to open.
 *
 * @return	File handle. NULL if failed.
 *
 * @note	Call this function to distinguish file types, just a simple way.
 */
FSCRT_FILE		FSDK_OpenFile(SDKFunctionList* pFunList, const char* fileName, const char* fileMode, bool useDef = false);

/**
 * @brief	Get custom file name from first command line parameter.
 *
 * @param[in] filePath	        Custom input file path.
 * @return	Custom file name.
 */
string			FSDK_GetPDFName(const char* filePath);

/*******************************************************************************/
/* Log                                                                         */
/*******************************************************************************/
typedef struct _FSDK_LOG {
	FILE*		pLogFile;
	FSDK_LOCK	logLock;
}FSDK_LOG;

/**
 * @brief	Open log file.
 *
 * @param[in] logFileName	Full path name of log file.
 * @param[in] log			The SDK log struct.
 *
 * @return	TRUE if successful, or FALSE for failure.
 *
 * @note	Call this function to open a log file. To get log file path name, call FSDK_GetLogFile for an example module.
 */
bool			FSDK_OpenLog(const char* logFileName, FSDK_LOG* log);
/**
 * @brief	Output log data.
 *
  * @param[in] log		The SDK log struct.
 * @param[in] format	Format string of log data.
 *
 * @return	TRUE if successful, or FALSE for failure.
 *
 * @note	This function supports variable argument list.
 */
bool			FSDK_OutputLog(FSDK_LOG* log, const char* format, ...);
/**
 * @brief	Output log string.
 *
 * @param[in] log		The SDK log struct.
 * @param[in] str		String buffer.
 * @param[in] length	Length of string in bytes.
 *
 * @return	TRUE if successful, or FALSE for failure.
 *
 * @note	This function outputs a string into log file.
 */
bool			FSDK_OutputStringLog(FSDK_LOG* log, const char* str, int length);
/**
 * @brief	Close log file.
 *
 * @param[in] log			The SDK log struct.
 *
 * @note	User should call this function to close log file when application exits.
 */
void			FSDK_CloseLog(FSDK_LOG* log);

#endif /* _FSDK_COMMON_H_ */
