// mt_pdf2txt.cpp : Defines the entry point for the console application.
//
#include "../comm_src/fgsdk_common.h"

FSDK_LOCK g_Lock;

FoxitParams g_FoxitParams;

void PrintUsage()
{
	printf( "\nUsage:	mt_pdf2txt /f <folder name> [/t integer] [/d dest folder name]\n" );
}

bool pdf2txt(FSDK_LOG* log, SDKFunctionList* pFunList, string& strSrcDir, string& strFileName, string& strDstDir)
{
	assert(0 != strSrcDir.length());
	assert(0 != strFileName.length());

	string strPdfFile = strSrcDir;
	if (string::npos == strPdfFile.rfind('\\'))
		strPdfFile += "/";
	else
		strPdfFile += "\\";
	strPdfFile += strFileName;
	string strTxtFile;
	if (0 == strDstDir.length())
		strTxtFile = string(strPdfFile.c_str(), strPdfFile.length() - 4) + ".txt";
	else {
		strTxtFile = strDstDir;
		if (string::npos == strTxtFile.rfind('\\'))
			strTxtFile += "/";
		else
			strTxtFile += "\\";
		strTxtFile += string(strFileName.c_str(), strFileName.length() - 4) + ".txt";
	}

	string strLog("\r\n======");
	strLog += "\r\n\topen input file: ";
	strLog += strPdfFile;
	//Open an input PDF file
	FSCRT_FILE file = FSDK_OpenFile(pFunList, strPdfFile.c_str(), "rb");
	if (!file)
	{
		strLog += "failure\r\n";
		FSDK_OutputStringLog(log, strLog.c_str(), (int)strLog.length());
		return false;
	}
	strLog += "\r\n\tSucceed to open input file\r\n";

	strLog += "\r\n\topen output file: ";

	//Open an output txt file
	FSCRT_FILE txtFile = FSDK_OpenFile(pFunList, strTxtFile.c_str(), "wb+");
	if (!txtFile)
	{
		strLog += "failure: Can't open output file!\r\n";
		FSDK_OutputStringLog(log, strLog.c_str(), (int)strLog.length());
		pFunList->pFSCRTFileRelease(file);
		return false;
	}
	strLog += "success to open output file\r\n";

	//Mark the starting time
	unsigned int nStartTime = FSDK_GetClock();

	strLog += "\tload PDF document: ";
	FSCRT_DOCUMENT pdfDoc = NULL;
	//Load a PDF document object from a file
	FS_RESULT ret = pFunList->pFSPDFDocStartLoad(file, NULL, &pdfDoc, NULL);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		strLog += "failure: Can't load documents from file\r\n";
		FSDK_OutputStringLog(log, strLog.c_str(), (int)strLog.length());
		pFunList->pFSCRTFileRelease(file);
		pFunList->pFSCRTFileRelease(txtFile);
		return false;
	}

	//Get page count
	FS_INT32 pageCount = 0;
	ret = pFunList->pFSPDFDocCountPages(pdfDoc, &pageCount);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		strLog += "Failed to get page count\r\n";
		FSDK_OutputStringLog(log, strLog.c_str(), (int)strLog.length());
		pFunList->pFSCRTFileRelease(file);
		pFunList->pFSCRTFileRelease(txtFile);
		return false;
	}

	//Parse and convert PDF page one by one into one text file  
	for (int i=0; i < pageCount; i++)
	{
		FSCRT_PAGE pdfPage = NULL;
		//Get a page based on the given page index
		ret = pFunList->pFSPDFDocGetPage(pdfDoc, i, &pdfPage);
		if (pdfPage == NULL) {
			strLog += "failure: Can't get page %d !\r\n";
			continue;
		}

		//Before loading a text page from a PDF page, the PDF page should be pared successfully first.
		FSCRT_PROGRESS progress;
		ret = pFunList->pFSPDFPageStartParse(pdfPage, FSPDF_PAGEPARSEFLAG_NORMAL, &progress);
		if (FSCRT_ERRCODE_SUCCESS != ret) {
			pFunList->pFSPDFPageClear(pdfPage);
			pdfPage = NULL;
			strLog += "failure: Can't parse the page\r\n";
			continue;
		}
		//Continue to parse it
		ret = pFunList->pFSCRTProgressContinue(progress, NULL);
		if (FSCRT_ERRCODE_FINISHED != ret) {
			pFunList->pFSCRTProgressRelease(progress);
			pFunList->pFSPDFPageClear(pdfPage);
			pdfPage = NULL;
			strLog += "\r\n\tfailure: Can't parse the page\r\n";
			continue;
		}
		strLog += "The current page is parsed successfully\r\n";

		//Release the resource of progress
		pFunList->pFSCRTProgressRelease(progress);

		FSPDF_TEXTPAGE textPage=NULL;
		//Load a text page based on a PDF page
		ret = pFunList->pFSPDFTextPageLoad(pdfPage,&textPage);
		if (NULL == textPage) {
			strLog += "failure: Can't load pages!\r\n";
			if (FSCRT_ERRCODE_INVALIDLICENSE == ret)
				strLog += NO_LICENSE;
			pFunList->pFSPDFPageClear(pdfPage);
			continue;
		}

		//Export a text page to the given text file
		ret = pFunList->pFSPDFTextPageExportToFile(textPage,txtFile);
		if(ret != FSCRT_ERRCODE_SUCCESS) {
			strLog += "failure: Can't export to file!\r\n";
			FSDK_OutputStringLog(log, strLog.c_str(), (int)strLog.length());
			pFunList->pFSPDFTextPageRelease(textPage);
			pFunList->pFSPDFPageClear(pdfPage);
			continue;
		}

		//FSDK_OutputStringLog(log, strLog.c_str(), (int)strLog.length());

		//Release pdfPage and textPage
		pFunList->pFSPDFTextPageRelease(textPage);
		pFunList->pFSPDFPageClear(pdfPage);
	}

	pFunList->pFSPDFDocClose(pdfDoc);
	//Mark the end time
	unsigned int nEndTime = FSDK_GetClock();
	FSDK_OutputStringLog(log, strLog.c_str(), (int)strLog.length());
	//Record the duration
	FSDK_OutputLog(log, "\tpage count: %d\r\n\telapsed time: %dms\r\n", pageCount, nEndTime - nStartTime);
	pFunList->pFSCRTFileRelease(file);
	pFunList->pFSCRTFileRelease(txtFile);
	
	return true;
}

void Multiple_Threads_Proc(void* param)
{
	FSCRT_THREAD threadID = FSDK_GetCurrentThread();
	FSDK_LOG threadLog;
	char threadid[128] = {0};
#if _MSC_VER > 1200
	sprintf_s((char*)threadid, sizeof(threadid), "/mt_pdf2txt_thread_%d.txt", threadID);
#else
	sprintf((char*)threadid, "/mt_pdf2txt_thread_%d.txt", threadID);
#endif
	string threadLogPath = g_FoxitParams.strOutputDir;
	threadLogPath += (char*)threadid;
	FSDK_OpenLog(threadLogPath.c_str(), &threadLog);

	FSDK_Lock(&g_Lock);
	int i = 0;
	for (i = 0; i != g_FoxitParams.nThreadCount; ++i) {
		if (g_FoxitParams.pDllManager[i].bDLLUsed)
			continue;
		else{
			g_FoxitParams.pDllManager[i].bDLLUsed = true;
			break;
		}
	}
	FSDK_Unlock(&g_Lock);

	SDKManager sdkManager;
	FS_RESULT ret = sdkManager.InitManager(g_FoxitParams.pDllManager[i].strDllName.c_str());
	if (ret != FSCRT_ERRCODE_SUCCESS) {
		sdkManager.DestroyManager();
		FSDK_OutputLog(&threadLog, "\r\nThe SDKManager initialize failed!\r\n");
		if (ret == FSCRT_ERRCODE_INVALIDLICENSE)
		{
			FSDK_OutputLog(&threadLog,"\r\nLicense is invalid or expired!!!\r\n");
		}
		FSDK_CloseLog(&threadLog);
		return;
	}
	SDKFunctionList* pFunList = sdkManager.GetSDKFunctionList();
	while(!g_FoxitParams.bLastFile)
	{
		//get the file 
		string strFile;
		FSDK_Lock(&g_Lock);
		do {
			FS_BOOL isDir = FALSE;
			strFile = FSDK_GetNextFile(g_FoxitParams.pFileDirHandle, &isDir);
			if(!isDir) {
				// Check whether a file is PDF or not
				if(FSDK_CompareExtName(strFile.c_str(), ".pdf")) 
					break;
			}
			if (strFile.size() < 1) {
				g_FoxitParams.bLastFile = true;
				break;
			}
		} while (TRUE);
		FSDK_Unlock(&g_Lock);

		if (strFile.length() > 0) {
			if (pdf2txt(&threadLog, pFunList, g_FoxitParams.strInputDir, strFile, g_FoxitParams.strOutputDir)) {
				string logInfo = "\r\n\t" + strFile + " convert success!";
				FSDK_OutputLog(&threadLog, logInfo.c_str());
				FSDK_Lock(&g_Lock);
				g_FoxitParams.nSuccessCount++;
				FSDK_Unlock(&g_Lock);
			}
			else{
				string logInfo = "\r\n\t" + strFile + " convert failed!";
				FSDK_OutputLog(&threadLog, logInfo.c_str());
				FSDK_Unlock(&g_Lock);
				g_FoxitParams.nFailCount++;
				FSDK_Unlock(&g_Lock);
				continue;
			}
		}
	}
	
	sdkManager.DestroyManager();
	FSDK_CloseLog(&threadLog);
	return;
}

int main(int argc, char **argv )
{
	g_FoxitParams.pFileDirHandle = NULL;
	g_FoxitParams.nThreadCount = 2;
	g_FoxitParams.bLastFile = false;
	g_FoxitParams.pDllManager = NULL;
	g_FoxitParams.nSuccessCount = 0;
	g_FoxitParams.nFailCount = 0;

	//command param analysis
	int i = 0;
	bool bNeedParm = false;
	for ( i = 1; i != argc; ++i )
	{
		if( (argv[i][0]) == '/' )
		{
			switch ( argv[i][1] )
			{
			case 'f':
				if( i < argc-1 && 0 == argv[i][2]) {
					g_FoxitParams.strInputDir = argv[i+1];
					if (0 != g_FoxitParams.strInputDir.length()) {
						if (!FSDK_FolderExist(g_FoxitParams.strInputDir.c_str()))
							printf( "\nError: the input file directory does not exist!\n" );
						else
							bNeedParm = true;
					}
					else
						printf( "\nError: the input file directory does not exist!\n" );
				}
				else{
					printf( "\nError, incorrect param!\n" );
					PrintUsage();
					return -1;
				}
				break;
			case 't':
				if( i < argc-1  && 0 == argv[i][2])
					g_FoxitParams.nThreadCount = atoi(argv[i+1]);
				else {
					printf( "\nError, incorrect param!\n" );
					PrintUsage();
					return -1;
				}
				break;
			case 'd':
				if( i < argc-1  && 0 == argv[i][2]) {
					g_FoxitParams.strOutputDir = argv[i+1];
					if (0 != g_FoxitParams.strOutputDir.length()) {
						if (!FSDK_FolderExist(g_FoxitParams.strOutputDir.c_str()))
							printf( "\nError: the output file directory does not exist!\n" );
					}
				}
				else{
					printf( "\nError, incorrect param!\n" );
					PrintUsage();
					return -1;
				}
				break;
			default:
				PrintUsage();
				break;
			}
		}
	}
	if (!bNeedParm) {
		PrintUsage();
		return -1;
	}
	if (0 == g_FoxitParams.strOutputDir.length()) {
		g_FoxitParams.strOutputDir = g_FoxitParams.strInputDir;
		if (string::npos == g_FoxitParams.strOutputDir.rfind('\\'))
			g_FoxitParams.strOutputDir += "/Output";
		else
			g_FoxitParams.strOutputDir += "\\Output";
		if (!FSDK_FolderExist(g_FoxitParams.strOutputDir.c_str()) && !FSDK_CreatFolder(g_FoxitParams.strOutputDir.c_str())) {
			printf("\r\nfailed to create the output folder!");
			return -1;
		}
	}

	FSDK_SetExecuteFilePath(argv[0]);
	//open the main thread log
	FSDK_LOG mainLog;
	string logPath = g_FoxitParams.strOutputDir + "/mt_pdf2txt_mainlog.txt";
	FSDK_OpenLog(logPath.c_str(), &mainLog);
	FSDK_OutputLog(&mainLog, "Foxit PDF SDK example: mt_pdf2txt\r\n");
	string logInfo = "\tInput from: " + g_FoxitParams.strInputDir + "\r\n";
	FSDK_OutputLog(&mainLog, logInfo.c_str());
	logInfo = "\tOutput to: " + g_FoxitParams.strOutputDir + "\r\n";
	FSDK_OutputLog(&mainLog, logInfo.c_str());
	g_FoxitParams.pDllManager = new DLLManager[g_FoxitParams.nThreadCount];
	if (NULL == g_FoxitParams.pDllManager) {
		FSDK_OutputLog(&mainLog, "\r\nOut of memory, DLLManager create failed!\r\n");
		FSDK_CloseLog(&mainLog);
		return-1;
	}

	string strDllPath = FSDK_GetExecuteFolder();
#if defined(_WIN64)  //windows 64bit platforms.
	strDllPath += "\\fsdk_win64.dll";
#elif defined(_WIN32)
	strDllPath += "\\fsdk_win32.dll";
#endif
	if (!FSDK_CopyDLL(g_FoxitParams.pDllManager, strDllPath.c_str(), g_FoxitParams.nThreadCount)) {
		FSDK_OutputLog(&mainLog, "\r\nDLL copy failed!\r\n");
		FSDK_CloseLog(&mainLog);
		if (g_FoxitParams.pDllManager) {
			delete[] g_FoxitParams.pDllManager;
			g_FoxitParams.pDllManager = NULL;
		}
		return -1;
	}
	FSDK_OutputLog(&mainLog, "\r\nDLL copy completed!\r\n");

	g_FoxitParams.pFileDirHandle = FSDK_OpenFolder(g_FoxitParams.strInputDir.c_str());
	if (NULL == g_FoxitParams.pFileDirHandle) {
		FSDK_OutputLog(&mainLog, "\r\nCan not open the input directory!");
		FSDK_CloseLog(&mainLog);
		if (g_FoxitParams.pDllManager) {
			delete[] g_FoxitParams.pDllManager;
			g_FoxitParams.pDllManager = NULL;
		}
		return -1;
	}

	FSThreadHandle* pHandleArray = new FSThreadHandle[g_FoxitParams.nThreadCount];
	if (NULL == pHandleArray) {
		FSDK_OutputLog(&mainLog, "\r\nOut of memory, handle array create failed!\r\n");
		FSDK_CloseLog(&mainLog);
		if (g_FoxitParams.pDllManager) {
			delete[] g_FoxitParams.pDllManager;
			g_FoxitParams.pDllManager = NULL;
		}
		return -1;
	}
	FSDK_InitializeLock(&g_Lock);

	for (i = 0; i != g_FoxitParams.nThreadCount; ++i)
	{
		pHandleArray[i] = FSDK_CreateThread(Multiple_Threads_Proc, NULL) ;
	}

	FSDK_OutputLog(&mainLog, "\r\nCreate %d threads success!\r\n", g_FoxitParams.nThreadCount);

	//calculate the execution time
	unsigned int nStartTime = FSDK_GetClock();
	FSDK_WaitForMultipleThreads((void**)pHandleArray, g_FoxitParams.nThreadCount);
	unsigned int nEndTime = FSDK_GetClock();
	
	FSDK_OutputLog(&mainLog, "\r\n======");
	FSDK_OutputLog(&mainLog, "\r\n\r\nTotal file count is : %d.", g_FoxitParams.nSuccessCount + g_FoxitParams.nFailCount);
	FSDK_OutputLog(&mainLog, "\r\n%d files convert to text success!", g_FoxitParams.nSuccessCount);
	FSDK_OutputLog(&mainLog, "\r\n%d files convert to text failed!", g_FoxitParams.nFailCount);
	FSDK_OutputLog(&mainLog, "\r\nthe total time is %d milliseconds!\r\n", nEndTime - nStartTime);

	FSDK_FinalizeLock(&g_Lock);
	if (g_FoxitParams.pDllManager) {
		delete[] g_FoxitParams.pDllManager;
		g_FoxitParams.pDllManager = NULL;
	}
	//close thread handle
	for (i = 0; i != g_FoxitParams.nThreadCount; ++i){
		CloseHandle(pHandleArray[i]);
	}

	delete[] pHandleArray;
	pHandleArray = NULL;
	//close input folder
	FSDK_CloseFolder(g_FoxitParams.pFileDirHandle);
	g_FoxitParams.pFileDirHandle = NULL;
	FSDK_OutputLog(&mainLog, "\r\nmt_pdf2txt executed success!\r\n\r\n");
	//close main thread log
	FSDK_CloseLog(&mainLog);
	return 1;
}

