/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a source to implement functions of adding form fields to a pdf file.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main addformfields
#endif

FS_RESULT setCharOptions(char** choices,FSPDF_CHOICEOPTION* options,FS_INT32 selectedIndex,int length)
{
	for (int i = 0;i<length;i++)
	{
		FSCRT_BStr_Init(&options[i].optionValue);
		FSCRT_BStr_Init(&options[i].optionLabel);
		options[i].optionLabel.str = choices[i];
		options[i].optionLabel.len = (FS_DWORD)strlen(choices[i]);
		if(i == selectedIndex)
		{
			options[i].selected = TRUE;
			options[i].defaultSelected = TRUE;
		}
		else 
		{
			options[i].selected = FALSE;
			options[i].defaultSelected = FALSE;
		}
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FSCRT_RECTF setRect(FS_FLOAT left, FS_FLOAT top, FS_FLOAT right, FS_FLOAT bottom)
{
	FSCRT_RECTF rect;
	rect.left = left;
	rect.top = top;
	rect.right = right;
	rect.bottom = bottom;
	return rect;
}
//add Text form field
FS_RESULT addTextField(FSPDF_FORM form, FSCRT_PAGE page, FSCRT_RECTF rect, FS_LPCSTR fieldName,FS_LPCSTR value, FSPDF_FORMCONTROL* formControl)
{
	FSCRT_BSTR bstrFieldName;
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;

	FSCRT_BStr_Init(&bstrFieldName);
	ret = FSCRT_BStr_Set(&bstrFieldName,fieldName,strlen(fieldName));
	FSDK_CHECK_RET(ret);
	ret = FSPDF_Form_AddField(form,page,&bstrFieldName,FSPDF_FORMFIELDTYPE_TEXTFIELD,&rect,formControl);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Add field: %s.\r\n",bstrFieldName.str);
	else
	{
		FSDK_OutputLog("Failure: Fail to add field: %s, with error code %d.\r\n", bstrFieldName.str, ret);
		return ret;
	}
	if (value != NULL)
	{
		FSCRT_BSTR bstrValue;
		FSCRT_BStr_Init(&bstrValue);
		ret = FSCRT_BStr_Set(&bstrValue,value,strlen(value));
		FSDK_CHECK_RET(ret);
		ret = FSPDF_FormField_SetValue(form,&bstrFieldName,&bstrValue);
		FSDK_CHECK_RET(ret);
		FSCRT_BStr_Clear(&bstrValue);
	}
	FSCRT_BStr_Clear(&bstrFieldName);
	return ret;
}
//add Radio Button form field
FS_RESULT addRadioButton(FSPDF_FORM form, FSCRT_PAGE page, FSCRT_RECTF rect, FS_LPSTR fieldName, FS_BOOL checked, FS_LPSTR exportValue, FSPDF_FORMCONTROL* formControl)
{
	FSCRT_BSTR bstrFieldName;
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;

	FSCRT_BStr_Init(&bstrFieldName);
	ret = FSCRT_BStr_Set(&bstrFieldName,fieldName,strlen(fieldName));
	FSDK_CHECK_RET(ret);
	ret = FSPDF_Form_AddField(form,page,&bstrFieldName,FSPDF_FORMFIELDTYPE_RADIOBUTTON,&rect,formControl);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Add field: %s.\r\n",bstrFieldName.str);
	else
	{
		FSDK_OutputLog("Failure: Fail to add field: %s, with error code %d.\r\n", bstrFieldName.str, ret);
		return ret;
	}
	ret = FSPDF_FormControl_SetChecked(*formControl,checked);
	FSDK_CHECK_RET(ret)
	if (exportValue != NULL)
	{
		FSCRT_BSTR bstrExportValue = {exportValue, (FS_DWORD)strlen(exportValue)};
		ret = FSPDF_FormControl_SetExportValue(*formControl, &bstrExportValue);
		FSDK_CHECK_RET(ret)
	}
	FSCRT_BStr_Clear(&bstrFieldName);
	return ret;
}
//add Combo Box form field
FS_RESULT addComboBox(FSPDF_FORM form, FSCRT_PAGE page, FS_LPSTR fieldName, FSCRT_RECTF rect, FSPDF_CHOICEOPTION* options, FSPDF_FORMCONTROL* formControl)
{
	FSCRT_BSTR bstrFieldName;
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;

	FSCRT_BStr_Init(&bstrFieldName);
	ret = FSCRT_BStr_Set(&bstrFieldName,fieldName,strlen(fieldName));
	FSDK_CHECK_RET(ret);
	ret = FSPDF_Form_AddField(form,page,&bstrFieldName,FSPDF_FORMFIELDTYPE_COMBOBOX,&rect,formControl);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Add field: %s.\r\n",bstrFieldName.str);
	else
	{
		FSDK_OutputLog("Failure: Fail to add field: %s, with error code %d.\r\n", bstrFieldName.str, ret);
		return ret;
	}
	if (options != NULL)
	{
		ret = FSPDF_FormField_SetOptions(form,&bstrFieldName,options,4);
		FSDK_CHECK_RET(ret);
	}
	FSCRT_BStr_Clear(&bstrFieldName);
	return ret;
}
//add Check Box form field
FS_RESULT addCheckBox(FSPDF_FORM form, FSCRT_PAGE page, FS_LPSTR fieldName, FSCRT_RECTF rect, FS_BOOL checked, FSPDF_FORMCONTROL* formControl)
{
	FSCRT_BSTR bstrFieldName;
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;

	FSCRT_BStr_Init(&bstrFieldName);
	ret = FSCRT_BStr_Set(&bstrFieldName,fieldName,strlen(fieldName));
	FSDK_CHECK_RET(ret);
	ret = FSPDF_Form_AddField(form,page,&bstrFieldName,FSPDF_FORMFIELDTYPE_CHECKBOX,&rect,formControl);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Add field: %s.\r\n",bstrFieldName.str);
	else
	{
		FSDK_OutputLog("Failure: Fail to add field: %s, with error code %d.\r\n", bstrFieldName.str, ret);
		return ret;
	}
	ret = FSPDF_FormControl_SetChecked(*formControl,checked);
	FSDK_CHECK_RET(ret);
	FSCRT_BStr_Clear(&bstrFieldName);
	return ret;
}
//add List Box form field
FS_RESULT addListBox(FSPDF_FORM form, FSCRT_PAGE page, FS_LPSTR fieldName, FSCRT_RECTF rect, FSPDF_CHOICEOPTION* options, FSPDF_FORMCONTROL* formControl)
{
	FSCRT_BSTR bstrFieldName;
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;

	FSCRT_BStr_Init(&bstrFieldName);
	ret = FSCRT_BStr_Set(&bstrFieldName,fieldName,strlen(fieldName));
	FSDK_CHECK_RET(ret);
	ret = FSPDF_Form_AddField(form,page,&bstrFieldName,FSPDF_FORMFIELDTYPE_LISTBOX,&rect,formControl);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Add field: %s.\r\n",bstrFieldName.str);
	else
	{
		FSDK_OutputLog("Failure: Fail to add field: %s, with error code %d.\r\n", bstrFieldName.str, ret);
		return ret;
	}
	if (options != NULL)
	{
		ret = FSPDF_FormField_SetOptions(form,&bstrFieldName,options,4);
		FSDK_CHECK_RET(ret);
	}
	FSCRT_BStr_Clear(&bstrFieldName);
	return ret;
}
//add Push Button form field
FS_RESULT addPushButton(FSPDF_FORM form, FSCRT_PAGE page, FS_LPSTR fieldName, FSCRT_RECTF rect, FSPDF_FORMCONTROL* formControl)
{
	FSCRT_BSTR bstrFieldName;
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;

	FSCRT_BStr_Init(&bstrFieldName);
	ret = FSCRT_BStr_Set(&bstrFieldName,fieldName,strlen(fieldName));
	FSDK_CHECK_RET(ret);
	ret = FSPDF_Form_AddField(form,page,&bstrFieldName,FSPDF_FORMFIELDTYPE_PUSHBUTTON,&rect, formControl);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Add field: %s.\r\n",bstrFieldName.str);
	else
		FSDK_OutputLog("Failure: Fail to add field: %s, with error code %d.\r\n", bstrFieldName.str, ret);
	FSCRT_BStr_Clear(&bstrFieldName);
	return ret;	
}
//set the properties Push Button form field 
FS_RESULT setPushButtomProperties(FSCRT_ANNOT annot, FS_INT32 hlMode, FS_INT32 mkColorEntry, FS_ARGB color, FS_INT32 mkCaptionEntry, FS_LPSTR caption)
{
	FSCRT_BSTR bstrCaption;
	bstrCaption.str = caption;
	bstrCaption.len = (FS_DWORD)strlen(caption);
	FS_RESULT ret = FSPDF_Annot_SetHighlightingMode(annot,FSPDF_ANNOT_HIGHLIGHTINGMODE_PUSH);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_Annot_SetMKColor(annot, FSPDF_ANNOT_MKENTRY_BORDERCOLOR, color);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_Annot_SetMKCaption(annot,FSPDF_ANNOT_MKENTRY_NORMALCAPTION,&bstrCaption);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_Annot_ResetAppearance(annot);
	FSDK_CHECK_RET(ret);
	return ret;
}

struct TextFieldInfo
{
	FSCRT_RECTF	rect;
	FS_LPSTR	sFieldName;
	FS_LPSTR	sValue;
};

struct RadioButtonInfo
{
	FSCRT_RECTF	rect;
	FS_LPSTR	sFieldName;
	FS_LPSTR	sValue;
	FS_BOOL		bChecked;
};

struct CheckBoxInfo
{
	FSCRT_RECTF	rect;
	FS_LPSTR	sFieldName;
	FS_BOOL		bChecked;
};

//add all types of form fields
FS_RESULT addFormFields(string pdfFile,string inputFilesPath,string outputFilesPath)
{
	string strLog("\r\n======Add Form Fields======\r\n");
	strLog += (inputFilesPath + "/" + pdfFile + "\r\n");
	FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());

	CAutoDocument pdfDoc((inputFilesPath + "/" + pdfFile).c_str(), "rb");
	if (!pdfDoc.GetFileHander())
		return FSCRT_ERRCODE_ERROR;
	if (FSCRT_ERRCODE_SUCCESS != pdfDoc.LoadPDFDocument(NULL))
		return FSCRT_ERRCODE_ERROR;
	FSCRT_DOCUMENT document = NULL;
	FSCRT_PAGE page = NULL;
	FSPDF_FORM form = NULL;
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;

	document = pdfDoc.GetDocument();
	ret = FSPDF_Doc_GetPage(document, 0, &page);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_Doc_CreateForm(document,&form);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_Page_LoadAnnots(page);
	FSDK_CHECK_RET(ret);
	
	FSPDF_FORMCONTROL formControl = NULL;
	FSCRT_RECTF rect;
	FSCRT_ANNOT annot = NULL;
	/////////////Without Action///////////////////
	// Text field
	TextFieldInfo tfInfoArray[] = {
									{setRect(191.5, 711.5, 281.5, 693.5), "Text Field0", "Foxit"}
									,{setRect(381.0, 685.5, 499.5, 665.25), "Text Field1", "sale@foxitsoftware.com"}
									,{setRect(192.0, 592.5, 284.25, 573.0), "Text Field0", "Foxit"}
								  };
	for (FS_INT32 i=0; i<(sizeof(tfInfoArray)/sizeof(tfInfoArray[0])); i++)
	{
		FSDK_OutputLog("Add text field. Loop-index=%d.\r\n", i);
		ret = addTextField(form, page, tfInfoArray[i].rect, tfInfoArray[i].sFieldName, tfInfoArray[i].sValue, &formControl);
		FSDK_CHECK_RET(ret);
	}

	//Radio Button
	RadioButtonInfo rbInfoArray[] = {
		{setRect(377.0, 711.0, 404.75, 694.5), "Radio Button0", "male", TRUE}
		,{setRect(440.75, 711.0, 470.0, 693.75), "Radio Button0", "female", FALSE}
	};
	for (FS_INT32 i=0; i<(sizeof(rbInfoArray)/sizeof(rbInfoArray[0])); i++)
	{
		FSDK_OutputLog("Add radio button. Loop-index=%d.\r\n", i);
		ret = addRadioButton(form, page, rbInfoArray[i].rect, rbInfoArray[i].sFieldName, rbInfoArray[i].bChecked, rbInfoArray[i].sValue, &formControl);
		FSDK_CHECK_RET(ret);
	}

	// Check Box
	CheckBoxInfo checkbInfoArray[] = {
									{setRect(208.0, 657.0, 226.0, 640.5), "Check Box0", TRUE}
									,{setRect(290.25, 657.0, 308.25, 640.5), "Check Box1",  FALSE}
									,{setRect(374.5, 657.75, 392.5, 641.25), "Check Box2",  FALSE}
									,{setRect(144.75, 396.75, 163.5, 381.0), "Check Box6",  FALSE}
									,{setRect(348.75, 393.75, 367.5, 379.5), "Check Box6",  FALSE}
									,{setRect(287.25, 394.5, 306.0, 380.25), "Check Box6",  FALSE}
									,{setRect(145.5, 372.0, 164.25, 357.0), "Check Box7",  FALSE}
									,{setRect(287.25, 371.25, 306.0, 357.0), "Check Box7",  FALSE}
									,{setRect(348.75, 369.75, 367.5, 355.5), "Check Box7",  FALSE}
								};
	for (FS_INT32 i=0; i<(sizeof(checkbInfoArray)/sizeof(checkbInfoArray[0])); i++)
	{
		FSDK_OutputLog("Add check box. Loop-index=%d.\r\n", i);
		ret = addCheckBox(form, page, checkbInfoArray[i].sFieldName, checkbInfoArray[i].rect, checkbInfoArray[i].bChecked, &formControl);
		FSDK_CHECK_RET(ret);
	}

	FSPDF_CHOICEOPTION* options = new FSPDF_CHOICEOPTION[4];
	// Combo Box
	rect = setRect(191.5, 683.25, 283.75, 666.0);
	char* choices1[] = {(FS_LPSTR)"Han", (FS_LPSTR)"Gaoshan", (FS_LPSTR)"Manchu", (FS_LPSTR)"other"};
	ret = setCharOptions(choices1,options,0,4);
	FSDK_CHECK_RET(ret);
	ret = addComboBox(form, page, "Combo Box0", rect, options, &formControl);
	FSDK_CHECK_RET(ret);

	rect = setRect(208.0, 323.5, 317.5, 304.75);
	char* choices2[] = {(FS_LPSTR)"sdk", (FS_LPSTR)"core", (FS_LPSTR)"webpdf", (FS_LPSTR)"toolkit"};
	ret = setCharOptions(choices2,options,0,4);
	FSDK_CHECK_RET(ret);
	ret = addComboBox(form, page, "Combo Box1", rect, options, &formControl);
	FSDK_CHECK_RET(ret);

	rect = setRect(208.0, 292.75, 318.25, 275.5);
	ret = setCharOptions(choices2,options,0,4);
	FSDK_CHECK_RET(ret);
	ret = addComboBox(form, page, "Combo Box1", rect, options, &formControl);
	FSDK_CHECK_RET(ret);

	// List Box
	rect = setRect(148.5, 528.75, 228.75, 417.75);
	char* choices3[] = {(FS_LPSTR)"first", (FS_LPSTR)"second", (FS_LPSTR)"third", (FS_LPSTR)"fourth"};
	ret = setCharOptions(choices3,options,3,4);
	FSDK_CHECK_RET(ret);
	ret = addListBox(form, page, "List Box1", rect, options, &formControl);
	FSDK_CHECK_RET(ret);

	rect = setRect(330.5, 528.5, 414.5, 419.0);
	char* choices4[] = {(FS_LPSTR)"first", (FS_LPSTR)"second", (FS_LPSTR)"third", (FS_LPSTR)"fourth"};
	ret = setCharOptions(choices4,options,2,4);
	FSDK_CHECK_RET(ret);
	ret = addListBox(form, page, "List Box0", rect, options, &formControl);
	FSDK_CHECK_RET(ret);

	rect =setRect(434.75, 527.0, 514.25, 419.75);
	char* choices5[] = {(FS_LPSTR)"first", (FS_LPSTR)"second", (FS_LPSTR)"third", (FS_LPSTR)"fourth"};
	ret = setCharOptions(choices5,options,2,4);
	FSDK_CHECK_RET(ret);
	ret = addListBox(form, page, "List Box0", rect, options, &formControl);
	FSDK_CHECK_RET(ret);

	delete[] options;
	
	///////////////With Action////////////////
	//Add Push Button0 and set SUBMITFORM action.
	rect = setRect(238.0, 195.75, 288.25, 171.0);
	ret = addPushButton(form, page, "Push Button0", rect, &formControl);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_FormControl_GetWidgetAnnot(formControl, &annot);
	FSDK_CHECK_RET(ret)
	ret = setPushButtomProperties(annot, FSPDF_ANNOT_HIGHLIGHTINGMODE_PUSH, FSPDF_ANNOT_MKENTRY_BORDERCOLOR, 0xffff0000,
									FSPDF_ANNOT_MKENTRY_NORMALCAPTION, "submit");
	FSDK_CHECK_RET(ret);
	FSCRT_BSTR fileName;
	FSCRT_BStr_Init(&fileName);
	ret = FSCRT_BStr_Set(&fileName,"D:\\field.fdf",strlen("D:\\field.fdf"));
	FSDK_CHECK_RET(ret);
	FSPDF_ATTACHMENT attachment;
	ret = FSPDF_Attachment_Create(document,&attachment);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_Attachment_SetFileName(attachment,&fileName,FALSE);
	FSDK_CHECK_RET(ret);
	FSCRT_BStr_Clear(&fileName); 
	FSPDF_ACTIONDATA actionData;
	FSPDF_ACTIONDATA_SUBMITFORM* submitData = new FSPDF_ACTIONDATA_SUBMITFORM;
    FSCRT_BSTR fields[] ={(FS_LPSTR)"Check Box0", (FS_DWORD)strlen("Check Box0")
        , (FS_LPSTR)"Check Box1", (FS_DWORD)strlen("Check Box1")
        , (FS_LPSTR)"Check Box2", (FS_DWORD)strlen("Check Box2")
        , (FS_LPSTR)"Check Box6", (FS_DWORD)strlen("Check Box6")
        , (FS_LPSTR)"Check Box7", (FS_DWORD)strlen("Check Box7")
        , (FS_LPSTR)"Combo Box0", (FS_DWORD)strlen("Combo Box0")
        , (FS_LPSTR)"Combo Box1", (FS_DWORD)strlen("Combo Box1")
        , (FS_LPSTR)"List Box0", (FS_DWORD)strlen("List Box0")
        , (FS_LPSTR)"List Box1", (FS_DWORD)strlen("List Box1")
        , (FS_LPSTR)"Radio Button0", (FS_DWORD)strlen("Radio Button0")
        , (FS_LPSTR)"Radio Button1", (FS_DWORD)strlen("Radio Button1")
        , (FS_LPSTR)"Text Field0", (FS_DWORD)strlen("Text Field0")
        , (FS_LPSTR)"Text Field1", (FS_DWORD)strlen("Text Field1")};
	submitData->attachment = attachment;
	submitData->count = 13;
	submitData->fields = fields;
	submitData->flags = FSPDF_SUBMITFORMFLAGS_EXPORTFORMAT & FSPDF_SUBMITFORMFLAGS_XFDF & FSPDF_SUBMITFORMFLAGS_SUBMITPDF;
	actionData.actionData = (FS_LPVOID)submitData;
	actionData.actionType = FSPDF_ACTION_SUBMITFORM;
	ret = FSPDF_Annot_InsertAction(annot,FSPDF_TRIGGER_ANNOT_MU,0,&actionData);
	FSDK_CHECK_RET(ret);
	delete submitData;
	ret = FSPDF_Attachment_Release(document, attachment);

	//Add "Push Button1" and set RESETFORM action
	rect =setRect(329.5, 195.75, 379.75, 171.0);
	ret = addPushButton(form, page, "Push Button1", rect, &formControl);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_FormControl_GetWidgetAnnot(formControl,&annot);
	FSDK_CHECK_RET(ret);
	ret = setPushButtomProperties(annot, FSPDF_ANNOT_HIGHLIGHTINGMODE_PUSH, FSPDF_ANNOT_MKENTRY_BORDERCOLOR, 0xffff0000, 
									FSPDF_ANNOT_MKENTRY_NORMALCAPTION, "reset");
	FSDK_CHECK_RET(ret);
	FSPDF_ACTIONDATA_RESETFORM* resetData = new FSPDF_ACTIONDATA_RESETFORM;
	resetData->count = 13;
	resetData->fields = fields;
	resetData->flags = 0;
	actionData.actionData = (FS_LPVOID)resetData;
	actionData.actionType = FSPDF_ACTION_RESETFORM;
	ret = FSPDF_Annot_InsertAction(annot,FSPDF_TRIGGER_ANNOT_MU,1,&actionData);
	FSDK_CHECK_RET(ret);
	delete resetData;

	//Add "Push Button2"
	rect = setRect(425.0, 195.25, 476.25, 168.75);
	ret = addPushButton(form, page, "Push Button2", rect, &formControl);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_FormControl_GetWidgetAnnot(formControl,&annot);
	FSDK_CHECK_RET(ret);
	ret = setPushButtomProperties(annot, FSPDF_ANNOT_HIGHLIGHTINGMODE_PUSH, FSPDF_ANNOT_MKENTRY_BORDERCOLOR, 0xaa00ffff, 
									FSPDF_ANNOT_MKENTRY_NORMALCAPTION, "cancel");
	FSDK_CHECK_RET(ret);

	rect = setRect(491.25, 194.25, 543.0, 168.75);
	ret = addPushButton(form, page, "Push Button2", rect, &formControl);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_FormControl_GetWidgetAnnot(formControl,&annot);
	FSDK_CHECK_RET(ret);
	ret = setPushButtomProperties(annot, FSPDF_ANNOT_HIGHLIGHTINGMODE_PUSH, FSPDF_ANNOT_HIGHLIGHTINGMODE_PUSH,0xff00ffff,
									FSPDF_ANNOT_MKENTRY_NORMALCAPTION, "cancel");
	FSDK_CHECK_RET(ret);

	// Add "Radio Button1" and set HIDE action.
	rect = setRect(425.25, 151.5, 445.5, 136.5);
	ret = addRadioButton(form, page, rect, "Radio Button1", FALSE, "hide", &formControl);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_FormControl_GetWidgetAnnot(formControl,&annot);
	FSDK_CHECK_RET(ret);
	FSCRT_BSTR fields2 = {(FS_LPSTR)"Push Button2", (FS_DWORD)strlen("Push Button2")};
	FSPDF_ACTIONDATA_HIDE* hideData = new FSPDF_ACTIONDATA_HIDE;
	hideData->count = 1;
	hideData->fields = &fields2;
	hideData->hide = TRUE;
	actionData.actionData = hideData;
	actionData.actionType = FSPDF_ACTION_HIDE;
	ret = FSPDF_Annot_InsertAction(annot,FSPDF_TRIGGER_ANNOT_MU,2,&actionData);
	FSDK_CHECK_RET(ret);

	rect = setRect(425.75, 123.0, 445.25, 108.75);
	ret = addRadioButton(form, page, rect, "Radio Button1", TRUE, "display", &formControl);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_FormControl_GetWidgetAnnot(formControl,&annot);
	FSDK_CHECK_RET(ret);
	hideData->count = 1;
	hideData->fields = &fields2;
	hideData->hide = FALSE;
	actionData.actionData = hideData;
	actionData.actionType = FSPDF_ACTION_HIDE;
	ret = FSPDF_Annot_InsertAction(annot,FSPDF_TRIGGER_ANNOT_MU,3,&actionData);
	FSDK_CHECK_RET(ret);
	delete hideData;

	ret = FSPDF_Page_UnloadAnnots(page);
	// Save PDF as result.
	string outputFileName = outputFilesPath + "addFormFields.pdf";
	FS_BOOL result = FSDK_SavePDFFile(document,outputFileName.c_str());
	if (TRUE == result)
		FSDK_OutputLog("Success: Save PDF file: %s.\r\n",outputFileName.c_str());
	else
		FSDK_OutputLog("Failure: Fail to save PDF file: %s, with error code %d.\r\n", outputFileName.c_str(),ret);
	return ret;
}

void  addFormFieldsToPDFs(void* folder, string inputFilesPath, string outputFilesPath)
{
	do
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		string outputFile = outputFilesPath + "/";
		if (strFile.size() > 0 && !isDir)
		{
			if (FSDK_CompareExtName(strFile.c_str(), ".pdf") && strcmp(strFile.c_str(), "test_addFormFields.pdf") == 0)
			{
				//add form fields.
				addFormFields(strFile, inputFilesPath, outputFile);
			}	
		}
		if (strFile.size() < 1) break;
	} while (1);
}

int main(int argc, char* argv[]){

	FSDK_SetExecuteFilePath(argc, argv);
	string logPath = FSDK_GetLogFile("addformfields");
	//Open log file.
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: addformfields\r\n");
	FSDK_OutputLog("\tInput from: input_files folder\r\n");
	FSDK_OutputLog("\tOutput to: output_files/addformfields\r\n\r\n");

	//Initialize SDK library, or no method of SDK can be used.
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module, or no method of PDF module can be used.
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}
	//Get input files folder.
	string inputFilesPath = FSDK_GetInputFilesFolder("addformfields");
	
	//Get input files folder path.
	string outputFilesPath = FSDK_GetOutputFilesFolder("addformfields");

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFilesPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	void* folder = FSDK_OpenFolder(inputFilesPath.c_str());
	if(folder)
	{
		FSDK_OutputLog("\r\nEnumerate files in folder input_files: ");
		addFormFieldsToPDFs(folder,inputFilesPath,outputFilesPath);
		//Close access to specific folder.
		FSDK_CloseFolder(folder);
	}
	else
	{
		string inputFilesPath = FSDK_GetFixFolder();
		string pdffile = "test_addFormFields.pdf";
		string outputFile = outputFilesPath + "/";
		FS_RESULT ret = addFormFields(pdffile,inputFilesPath,outputFile);
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}