/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public,
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a demo to summarize all markup annots and save to a new pdf from a source pdf.
 **/

//Include common header file.
#include "../comm_src/fgsdk_common.h"
#include <map>
#include <algorithm>

#if defined(FSDK_IOS_DEMO)
#define main annots_summary
#endif

#define FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData) if (FSCRT_ERRCODE_SUCCESS != ret) {\
	FSDK_OutputLog("Failed in the line %d with error code %d.\r\n", __LINE__-1, ret);\
	FSCRT_PathData_Clear(pathData);\
	FSCRT_PathData_Release(pathData);\
	return ret;\
							}

#define FSDK_DST_WIDTH	612
#define FSDK_DST_HEIGHT	792

static FS_BOOL g_bCommentsOnly = 0;
static FS_FLOAT g_scale = 1.0f;
static FS_FLOAT g_translation = 0.0f;


FS_RESULT CreateCommentIcon(FSCRT_PAGE& page, FSPDF_PAGEOBJECTS& pageObjs, FS_LPSTR annotType, int index)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	string resPath = FSDK_GetFixFolder();
	resPath += "/";
#ifndef FSDK_IOS_DEMO
	resPath += "res/";
#endif
	resPath += annotType;
	resPath += ".png";
	FSCRT_FILE imageFile = FSDK_OpenFile(resPath.c_str(), "rb");
	FSCRT_IMAGE image = NULL;
	ret = FSCRT_Image_LoadFromFile(imageFile, &image);
	FSDK_CHECK_RET(ret);
	FS_INT32 count = 0;
	ret = FSCRT_Image_CountFrames(image, &count);
	FSDK_CHECK_RET(ret);
	if (0 < count)
	{
		ret = FSCRT_Image_LoadFrame(image, 0);
		FSDK_CHECK_RET(ret);
		FSPDF_PAGEOBJECT imageObj;
		ret = FSPDF_ImageObject_Create(page, &imageObj);
		FSDK_CHECK_RET(ret);
		ret = FSPDF_ImageObject_SetImage(page, imageObj, image);
		FSDK_CHECK_RET(ret);
		FSCRT_MATRIX matrix = {12 , 0, 0, 12, 47, 680 - 30.0f * index};
		ret = FSPDF_PageObject_SetMatrix(page, imageObj, &matrix);
		FSDK_CHECK_RET(ret);
		ret = FSPDF_PageObjects_InsertObject(page, pageObjs, 0, 1000, imageObj);
		FSDK_CHECK_RET(ret);
	}
	FSCRT_Image_Release(image);
	FSCRT_File_Release(imageFile);
	return ret;
}

//get guide rect point.
FS_RESULT GetHotPoint(FSCRT_ANNOT& annot, FS_LPSTR annotType, FS_FLOAT& x, FS_FLOAT& y)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	FSCRT_RECTF annotRect = {0, 0, 0, 0};
	ret = FSPDF_Annot_GetRect(annot, &annotRect);
	FSDK_CHECK_RET(ret);
	x = annotRect.right;
	y = annotRect.top;

	if (!strcmp(FSPDF_ANNOTTYPE_LINE, annotType) || !strcmp(FSPDF_ANNOTTYPE_CIRCLE, annotType) || !strcmp(FSPDF_ANNOTTYPE_SQUARE, annotType))
	{
		x = (annotRect.right + annotRect.left) / 2.0f;
		y = (annotRect.top + annotRect.bottom) / 2.0f;
	}
	else if (!strcmp(FSPDF_ANNOTTYPE_POLYGON, annotType) || !strcmp(FSPDF_ANNOTTYPE_POLYLINE, annotType))
	{
		FS_INT32 count = 0;
		ret = FSPDF_Annot_GetVertices(annot, NULL, &count);
		FSDK_CHECK_RET(ret);
		if (1 < count)
		{
			FS_FLOAT* vertices = new FS_FLOAT[count];
			ret = FSPDF_Annot_GetVertices(annot, vertices, &count);
			FSDK_CHECK_RET(ret);
			x = vertices[0];
			y = vertices[1];
			delete vertices;
		}
	}
	else if (!strcmp(FSPDF_ANNOTTYPE_INK, annotType))
	{
		FSCRT_PATHDATA pathData;
		ret = FSPDF_Annot_GetInkList(annot, &pathData);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		FS_INT32 count = 0;
		ret = FSCRT_PathData_CountPoints(pathData, &count);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		if (0 < count)
		{
			FS_INT32 pointType = 0;
			ret = FSCRT_PathData_GetPoint(pathData, 0, &x, &y, &pointType);
			FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		}
		FSCRT_PathData_Clear(pathData);
		FSCRT_PathData_Release(pathData);
		pathData = NULL;
	}
	else if (!strcmp(FSPDF_ANNOTTYPE_FREETEXT, annotType))
	{
		FSCRT_BSTR intent;
		FSCRT_BStr_Init(&intent);
		ret = FSPDF_Annot_GetIntent(annot, &intent);
		if (FSCRT_ERRCODE_SUCCESS == ret)
		{
			if (intent.str && !strcmp(FSPDF_ANNOT_INTENTNAME_FREETEXT_CALLOUT, intent.str))
			{
				FS_FLOAT x2 = 0, y2 = 0, x3 = 0, y3 = 0;
				ret = FSPDF_Annot_GetCalloutLinePoints(annot, &x, &y, &x2, &y2, &x3, &y3);
				FSDK_CHECK_RET(ret);
			}
		}
		else if (FSCRT_ERRCODE_NOTFOUND == ret)
			ret = FSCRT_ERRCODE_SUCCESS;
		else
			return ret;
		FSCRT_BStr_Clear(&intent);
	}

	x = x  * g_scale;
	y = y * g_scale + g_translation - 9.f;

	return ret;
}

FS_RESULT SummaryComments(FSCRT_FONT& font, FSCRT_PAGE& srcPage, FSCRT_PAGE& dstPage, FSCRT_PAGE& dstPage2)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	FS_FLOAT srcWidth = 0, srcHeight = 0;
	ret = FSPDF_Page_GetSize(srcPage, &srcWidth, &srcHeight);
	FSDK_CHECK_RET(ret);

	g_scale = FSDK_DST_WIDTH / srcWidth;
	g_translation = (FSDK_DST_HEIGHT - (srcHeight * g_scale)) / 2;

	FSCRT_PROGRESS parseProgress = NULL;
	ret = FSPDF_Page_StartParse(srcPage, FSPDF_PAGEPARSEFLAG_NORMAL, &parseProgress);
	FSDK_CHECK_RET(ret);
	ret = FSCRT_Progress_Continue(parseProgress, NULL);
	if (FSCRT_ERRCODE_FINISHED != ret) return FSCRT_ERRCODE_ERROR;
	FSCRT_Progress_Release(parseProgress);
	parseProgress = NULL;
	ret = FSPDF_Page_LoadAnnots(srcPage);
	FSDK_CHECK_RET(ret);

	FSPDF_PAGEOBJECTS pageObjs = NULL;
	if (!g_bCommentsOnly)
	{
		ret = FSPDF_Page_SetSize(dstPage, FSDK_DST_WIDTH, FSDK_DST_HEIGHT);
		FSDK_CHECK_RET(ret);
		ret = FSPDF_Page_GetPageObjects(dstPage, &pageObjs);
		FSDK_CHECK_RET(ret);
		FSPDF_PAGEOBJECT formXObj;
		ret = FSPDF_FormXObject_Create(dstPage, &formXObj);
		FSDK_CHECK_RET(ret);
		ret = FSPDF_FormXObject_ExtractAPFromPage(dstPage, formXObj, srcPage, TRUE);
		FSDK_CHECK_RET(ret);
		FSCRT_MATRIX matrix = {g_scale, 0, 0, g_scale, 0, g_translation};
		ret = FSPDF_PageObject_SetMatrix(dstPage, formXObj, &matrix);
		FSDK_CHECK_RET(ret);
		ret = FSPDF_PageObjects_InsertObject(dstPage, pageObjs, FSPDF_PAGEOBJECT_ALL, 0, formXObj);
		FSDK_CHECK_RET(ret);

		FSPDF_PAGEOBJECT pathObjContent = NULL;
		ret = FSPDF_PathObject_Create(dstPage, &pathObjContent);
		FSDK_CHECK_RET(ret);
		FSCRT_PATHDATA pathData;
		ret = FSCRT_PathData_Create(&pathData);
		FSDK_CHECK_RET(ret);
		ret = FSCRT_PathData_MoveTo(pathData, 0, g_translation);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		ret = FSCRT_PathData_LineTo(pathData, FSDK_DST_WIDTH, g_translation);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		ret = FSCRT_PathData_LineTo(pathData, FSDK_DST_WIDTH, FSDK_DST_HEIGHT - g_translation);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		ret = FSCRT_PathData_LineTo(pathData, 0, FSDK_DST_HEIGHT - g_translation);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		ret = FSCRT_PathData_LineTo(pathData, 0, g_translation);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		ret = FSPDF_PathObject_SetPathData(dstPage, pathObjContent, pathData);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		ret = FSPDF_PageObjects_InsertObject(dstPage, pageObjs, FSPDF_PAGEOBJECT_ALL, 1000, pathObjContent);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		FSCRT_PathData_Clear(pathData);
		FSCRT_PathData_Release(pathData);
		pathData = NULL;
	}
	
	ret = FSPDF_Page_SetSize(dstPage2, FSDK_DST_WIDTH, FSDK_DST_HEIGHT);
	FSDK_CHECK_RET(ret);
	FSPDF_PAGEOBJECTS pageObjs2;
	ret = FSPDF_Page_GetPageObjects(dstPage2, &pageObjs2);
	FSDK_CHECK_RET(ret);

	FSPDF_TEXTSTATE textState;
	textState.charSpace = 0;
	textState.font = font;
	textState.textMatrix[0] = 1;
	textState.textMatrix[1] = 0;
	textState.textMatrix[2] = 0;
	textState.textMatrix[3] = 1;
	textState.textMode = 0;
	textState.wordSpace = 0;

	FSCRT_BSTR bstrTitle;
	FSCRT_BStr_Init(&bstrTitle);
	FSCRT_BStr_Set(&bstrTitle, "Summary of comments: Annot_all.pdf", strlen("Summary of comments: Annot_all.pdf"));
	textState.fontSize = 16.0f;
	textState.x = 50;
	textState.y = 720;
	FSPDF_PAGEOBJECT tilteTextObj = NULL;
	ret = FSPDF_TextObject_Create(dstPage2, &tilteTextObj);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_TextObject_SetTextState(dstPage2, tilteTextObj ,&textState, FALSE, 400);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_TextObject_SetUnicodeString(dstPage2, tilteTextObj, &bstrTitle);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_PageObjects_InsertObject(dstPage2, pageObjs2, FSPDF_PAGEOBJECT_ALL, 1000, tilteTextObj);
	FSDK_CHECK_RET(ret);
	FSCRT_BStr_Clear(&bstrTitle);

	//Page:index
	FS_INT32 srcPageIndex = -1;
	ret = FSPDF_Page_GetIndex(srcPage, &srcPageIndex);
	FSDK_CHECK_RET(ret);
	srcPageIndex += 1;
	char indexStr[128];
	memset(indexStr, 0, 128);
	sprintf(indexStr, "Page:%d", srcPageIndex);
	FSCRT_BSTR pageStr;
	FSCRT_BStr_Init(&pageStr);
	FSCRT_BStr_Set(&pageStr, indexStr, (FS_DWORD)strlen(indexStr));
	textState.fontSize = 13.0f;
	textState.x = 50;
	textState.y = 705;
	FSPDF_PAGEOBJECT textObjPage = NULL;
	ret = FSPDF_TextObject_Create(dstPage2, &textObjPage);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_TextObject_SetTextState(dstPage2, textObjPage ,&textState, FALSE, 400);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_TextObject_SetUnicodeString(dstPage2, textObjPage, &pageStr);
	FSDK_CHECK_RET(ret);
	ret = FSPDF_PageObjects_InsertObject(dstPage2, pageObjs2, FSPDF_PAGEOBJECT_ALL, 1000, textObjPage);
	FSDK_CHECK_RET(ret);
	FSCRT_BStr_Clear(&pageStr);

	FS_LPCSTR filter = "Text, FreeText, Line, Square, Circle, Polygon, PolyLine, Highlight, Underline, Squiggly, StrikeOut, Stamp, Caret, Ink, FileAttachment";
	FSCRT_BSTR filterBstr;
	FSCRT_BStr_Init(&filterBstr);
	FSCRT_BStr_Set(&filterBstr, filter, (FS_DWORD)strlen(filter));
	FS_INT32 annotCount = 0;
	ret = FSPDF_Annot_GetCount(srcPage, &filterBstr, &annotCount);
	FSDK_CHECK_RET(ret);
	if (0 == annotCount)
	{
		FSCRT_BSTR noCommentsBstr;
		FSCRT_BStr_Init(&noCommentsBstr);
		FSCRT_BStr_Set(&noCommentsBstr, "No Comments.", strlen("No Comments."));
		textState.fontSize = 16.0f;
		textState.x = 50;
		textState.y = 685;
		FSPDF_PAGEOBJECT objNoComments = NULL;
		ret = FSPDF_TextObject_Create(dstPage2, &objNoComments);
		FSDK_CHECK_RET(ret);
		ret = FSPDF_TextObject_SetTextState(dstPage2, objNoComments ,&textState, FALSE, 400);
		FSDK_CHECK_RET(ret);
		ret = FSPDF_TextObject_SetUnicodeString(dstPage2, objNoComments, &noCommentsBstr);
		FSDK_CHECK_RET(ret);
		ret = FSPDF_PageObjects_InsertObject(dstPage2, pageObjs2, FSPDF_PAGEOBJECT_ALL, 1000, objNoComments);
		FSDK_CHECK_RET(ret);
		FSCRT_BStr_Clear(&noCommentsBstr);
	}
	else
	{
		struct AnnotRectTop
		{
			FSCRT_ANNOT annot;
			FS_FLOAT recttop;
		};

		int index = 0;
		AnnotRectTop* annotrecttop = new AnnotRectTop[annotCount];
		for (index = 0; index < annotCount; index++)
		{
			FSCRT_ANNOT annot = NULL;
			ret = FSPDF_Annot_Get(srcPage, &filterBstr, index, &annot);
			FSDK_CHECK_RET(ret);
			FSCRT_RECTF annotRect = {0, 0, 0, 0};
			ret = FSPDF_Annot_GetRect(annot, &annotRect);
			FSDK_CHECK_RET(ret);
			annotrecttop[index].annot = annot;
			annotrecttop[index].recttop = annotRect.top;
		}

		//sort by annot rect top in descending order
		for (index = 0; index < annotCount; index++)
		{
			for (int j = 0; j < annotCount-1-index; j++)
			{
				if (annotrecttop[j].recttop < annotrecttop[j+1].recttop)
				{
					AnnotRectTop temp = annotrecttop[j];
					annotrecttop[j] = annotrecttop[j+1];
					annotrecttop[j+1] = temp;
				}
			}
		}

		for (int i = 0; i != annotCount; i++)
		{
			FSCRT_ANNOT& annot = annotrecttop[i].annot;
			FSCRT_BSTR bstr;
			FSCRT_BStr_Init(&bstr);
			ret= FSPDF_Annot_GetType(annot, &bstr);
			FSDK_CHECK_RET(ret);
			char annotType[128];
			memset(annotType, 0, 128);
			strcpy(annotType, bstr.str);
			ret = CreateCommentIcon(dstPage2, pageObjs2, annotType, i);
			FSDK_CHECK_RET(ret);

			if (!g_bCommentsOnly)
			{
				FS_FLOAT x = 0;
				FS_FLOAT y = 0;
				ret = GetHotPoint(annot, annotType, x, y);
				FSDK_CHECK_RET(ret);
				FS_INT32 j = i + 1;
				FS_INT32 digit = 0;
				while (j)
				{
					j /= 10;
					digit++;
				}
				if (x + 2.0f + 5.56f * digit > FSDK_DST_WIDTH)
					x = FSDK_DST_WIDTH - (2.0f + 5.56f * digit);
				//path object
				FSPDF_PAGEOBJECT pathObj = NULL;
				ret = FSPDF_PathObject_Create(dstPage, &pathObj);
				FSDK_CHECK_RET(ret);
				FSCRT_PATHDATA pathData;
				ret = FSCRT_PathData_Create(&pathData);
				FSDK_CHECK_RET(ret);
				ret = FSCRT_PathData_MoveTo(pathData, x, y);
				FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
				ret = FSCRT_PathData_LineTo(pathData, x + 2.0f + 5.56f * digit, y);
				FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
				ret = FSCRT_PathData_LineTo(pathData, x + 2.0f + 5.56f * digit, y + 13.16f);
				FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
				ret = FSCRT_PathData_LineTo(pathData, x, y + 13.16f);
				FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
				ret = FSCRT_PathData_LineTo(pathData, x, y);
				FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
				ret = FSPDF_PathObject_SetPathData(dstPage, pathObj, pathData);
				FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
				ret = FSPDF_PageObjects_InsertObject(dstPage, pageObjs, FSPDF_PAGEOBJECT_ALL, 1000, pathObj);
				FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);

				FSCRT_PathData_Clear(pathData);
				FSCRT_PathData_Release(pathData);
				//text object
				FSPDF_PAGEOBJECT textObj = NULL;
				ret = FSPDF_TextObject_Create(dstPage, &textObj);
				FSDK_CHECK_RET(ret);
				textState.fontSize = 10.0;
				textState.x = x + 1;
				textState.y = y;
				char temp[10];
				memset(temp, 0, 10);
				sprintf(temp, "%d", i + 1);
				FSCRT_BSTR unicodeStr;
				unicodeStr.str = temp;
				unicodeStr.len = sizeof(temp);
				ret = FSPDF_TextObject_SetTextState(dstPage, textObj ,&textState, FALSE, 400);
				FSDK_CHECK_RET(ret);
				ret = FSPDF_TextObject_SetUnicodeString(dstPage, textObj, &unicodeStr);
				FSDK_CHECK_RET(ret);
				ret = FSPDF_PageObjects_InsertObject(dstPage, pageObjs, FSPDF_PAGEOBJECT_ALL, 1000, textObj);
				FSDK_CHECK_RET(ret);
			}

			char tempStr[256];
			memset(tempStr, 0, 256);
			sprintf(tempStr, "Number: %d ", i + 1);
			string annotStr = tempStr;
			annotStr += "Author: ";
			ret= FSPDF_Annot_GetTitle(annot, &bstr);
			if (FSCRT_ERRCODE_SUCCESS != ret && FSCRT_ERRCODE_NOTFOUND != ret) return ret;
			annotStr += bstr.str;
			annotStr += " Subject: ";
			ret = FSPDF_Annot_GetSubject(annot, &bstr);
			if (FSCRT_ERRCODE_SUCCESS != ret && FSCRT_ERRCODE_NOTFOUND != ret) return ret;
			annotStr += bstr.str;
			FSCRT_DATETIMEZONE dt;
			memset(&dt, 0, sizeof(dt));
			ret = FSPDF_Annot_GetModifiedDateTime(annot, &dt);
			if (FSCRT_ERRCODE_SUCCESS != ret && FSCRT_ERRCODE_NOTFOUND != ret) return ret;
			memset(tempStr, 0, 256);
			sprintf(tempStr, " Date: %d-%d-%d %d:%d:%d", dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second);
			annotStr += tempStr;

			FSPDF_PAGEOBJECT textObjAnnot = NULL;
			ret = FSPDF_TextObject_Create(dstPage2, &textObjAnnot);
			FSDK_CHECK_RET(ret);
			textState.fontSize = 10.0;
			textState.x = 60;
			textState.y = 688 - 30.0f * i;
            FSCRT_BStr_Set(&bstr, const_cast<char*>(annotStr.c_str()), (FS_DWORD)annotStr.length());
			ret = FSPDF_TextObject_SetTextState(dstPage2, textObjAnnot ,&textState, FALSE, 400);
			FSDK_CHECK_RET(ret);
			ret = FSPDF_TextObject_SetUnicodeString(dstPage2, textObjAnnot, &bstr);
			FSDK_CHECK_RET(ret);
			ret = FSPDF_PageObjects_InsertObject(dstPage2, pageObjs2, FSPDF_PAGEOBJECT_ALL, 1000, textObjAnnot);
			FSDK_CHECK_RET(ret);

			ret = FSPDF_Annot_GetContents(annot, &bstr);
			if (FSCRT_ERRCODE_SUCCESS == ret)
			{
				FSPDF_PAGEOBJECT textObjAnnot2 = NULL;
				ret = FSPDF_TextObject_Create(dstPage2, &textObjAnnot2);
				FSDK_CHECK_RET(ret);
				textState.fontSize = 6.0;
				textState.x = 60;
				textState.y = 675 - 30.0f * i;
				ret = FSPDF_TextObject_SetTextState(dstPage2, textObjAnnot2 ,&textState, FALSE, 400);
				FSDK_CHECK_RET(ret);
				ret = FSPDF_TextObject_SetUnicodeString(dstPage2, textObjAnnot2, &bstr);
				FSDK_CHECK_RET(ret);
				if (!strcmp(FSPDF_ANNOTTYPE_FREETEXT, annotType))
				{
					FSPDF_DEFAULTAPPEARANCE defAppearance;
					memset(&defAppearance, 0, sizeof(FSPDF_DEFAULTAPPEARANCE));
					ret = FSPDF_Annot_GetDefaultAppearance(annot, &defAppearance);
					FSDK_CHECK_RET(ret);
					ret = FSPDF_PageObject_SetColor(dstPage2, textObjAnnot2, FALSE, defAppearance.textColor);
					FSDK_CHECK_RET(ret);
				}
				ret = FSPDF_PageObjects_InsertObject(dstPage2, pageObjs2, FSPDF_PAGEOBJECT_ALL, 1000, textObjAnnot2);
				FSDK_CHECK_RET(ret);
			}
			else if (FSCRT_ERRCODE_NOTFOUND == ret)
				ret = FSCRT_ERRCODE_SUCCESS;
			else
				return ret;

			FSCRT_PATHDATA pathData;
			ret = FSCRT_PathData_Create(&pathData);
			FSDK_CHECK_RET(ret);
			ret = FSCRT_PathData_MoveTo(pathData, 60, 685 - 30.0f * i);
			FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
			ret = FSCRT_PathData_LineTo(pathData, 562, 685 - 30.0f * i);
			FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
			FSPDF_PAGEOBJECT pathObjAnnot = NULL;
			ret = FSPDF_PathObject_Create(dstPage2, &pathObjAnnot);
			FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
			ret = FSPDF_PathObject_SetPathData(dstPage2, pathObjAnnot, pathData);
			FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
			ret = FSPDF_PageObjects_InsertObject(dstPage2, pageObjs2, FSPDF_PAGEOBJECT_ALL, 1000, pathObjAnnot);
			FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
			FSCRT_BStr_Clear(&bstr);
			FSCRT_PathData_Clear(pathData);
			FSCRT_PathData_Release(pathData);
		}

		delete annotrecttop;

		//two lines below the title
		FSPDF_PAGEOBJECT pathObjTitle = NULL;
		ret = FSPDF_PathObject_Create(dstPage2, &pathObjTitle);
		FSDK_CHECK_RET(ret);
		FSCRT_PATHDATA pathData;
		ret = FSCRT_PathData_Create(&pathData);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		ret = FSCRT_PathData_MoveTo(pathData, 50, 715);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		ret = FSCRT_PathData_LineTo(pathData, 562, 715);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		ret = FSCRT_PathData_MoveTo(pathData, 50, 697);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		ret = FSCRT_PathData_LineTo(pathData, 562, 697);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		ret = FSPDF_PathObject_SetPathData(dstPage2, pathObjTitle, pathData);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		ret = FSPDF_PageObjects_InsertObject(dstPage2, pageObjs2, FSPDF_PAGEOBJECT_ALL, 1000, pathObjTitle);
		FSDK_CHECK_RET_RELEASE_PATHDATA(ret, pathData);
		FSCRT_PathData_Clear(pathData);
		FSCRT_PathData_Release(pathData);
		pathData = NULL;
	}
	FSCRT_BStr_Clear(&filterBstr);

	if (!g_bCommentsOnly)
	{
		ret = FSPDF_PageObjects_GenerateContents(dstPage, pageObjs);
		FSDK_CHECK_RET(ret);
	}
	ret = FSPDF_PageObjects_GenerateContents(dstPage2, pageObjs2);

	return ret;
}

FS_RESULT CreateCommentsSummary(string inputPath, string outputPath)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	inputPath += "/Annot_all.pdf";
	FSCRT_FILE file = FSDK_OpenFile(inputPath.c_str(), "rb");
	FSCRT_DOCUMENT srcDoc = NULL;
	ret = FSPDF_Doc_StartLoad(file, NULL, &srcDoc, NULL);
	FSDK_CHECK_RET(ret);
	FS_INT32 count = 0;
	ret= FSPDF_Doc_CountPages(srcDoc, &count);
	FSDK_CHECK_RET(ret);

	FSCRT_FONT font = NULL;
	FSCRT_BSTR NewfontName;
	NewfontName.str = (FS_LPSTR)"Helvetica";
	NewfontName.len = (FS_DWORD)strlen(NewfontName.str);
	ret = FSCRT_Font_Create(&NewfontName, FSCRT_FONTSTYLE_SERIF, 5, FSCRT_CHARSET_ANSI, &font);
	FSDK_CHECK_RET(ret);

	for (int j = 0; j < 2; j++)
	{
		g_bCommentsOnly = j;
		FSCRT_DOCUMENT dstDoc;
		ret = FSPDF_Doc_Create(&dstDoc);
		FSDK_CHECK_RET(ret);

		for (int i = 0; i < count; i++)
		{
			FSCRT_PAGE srcPage = NULL;
			ret = FSPDF_Doc_GetPage(srcDoc, i, &srcPage);
			FSDK_CHECK_RET(ret);

			FSCRT_PAGE dstPage = NULL;
			FSCRT_PAGE dstPage2 = NULL;
			if (g_bCommentsOnly)
			{
				ret = FSPDF_Page_Create(dstDoc, i, &dstPage2);
				FSDK_CHECK_RET(ret);
			} 
			else
			{
				ret = FSPDF_Page_Create(dstDoc, i*2, &dstPage);
				FSDK_CHECK_RET(ret);
				ret = FSPDF_Page_Create(dstDoc, i*2+1, &dstPage2);
				FSDK_CHECK_RET(ret);
			}

			ret = SummaryComments(font, srcPage, dstPage, dstPage2);
			FSDK_CHECK_RET(ret);
			FSPDF_Page_Clear(srcPage);
		}

		string outputFilePath = outputPath;
		if (g_bCommentsOnly)
			outputFilePath += "/annots_summary_commentsonly.pdf";
		else
			outputFilePath += "/annots_summary.pdf";
		FS_BOOL bRet = FSDK_SavePDFFile(dstDoc, outputFilePath.c_str());
		if (!bRet) return FSCRT_ERRCODE_ERROR;
	}

	FSCRT_Font_Release(font);
	FSPDF_Doc_Close(srcDoc);
	FSCRT_File_Release(file);
	return ret;
}

//Entrance of this example
int main(int argc, char* argv[])
{

	//Set executing file path
	FSDK_SetExecuteFilePath(argc, argv);

	//Open log file and output some basic info
	string logPath = FSDK_GetLogFile("annots_summary");
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: annots_summary\r\n\r\n");

	//Initialize Foxit SDK library
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		//Failed to initialize, output status and return
		FSDK_CloseLog();//Close folder
		return -1;
	}

	//Initialize PDF module
    FS_RESULT ret = FSCRT_PDFModule_Initialize();
	if(FSCRT_ERRCODE_SUCCESS != ret){
        FSDK_OutputLog("Failed to Initialize PDF module with error code %d.\r\n", ret);
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	string inputFolder = FSDK_GetFixFolder();
	string outputFolder = FSDK_GetOutputFilesFolder("annots_summary");

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	ret = CreateCommentsSummary(inputFolder, outputFolder);
	if (FSCRT_ERRCODE_SUCCESS != ret)
        FSDK_OutputLog("Failed to create comments summary with error code %d!\r\n", ret);

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSCRT_PDFModule_Finalize();

	//Finalize Foxit SDK library
	FSDK_FinalizeLibrary();

	//Close log and exit
	FSDK_CloseLog();
	return 0;
}