/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is bitmap transform source to implement bitmap transform examples.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main bitmap_transform
#endif

//Stretch bitmap.
void Stretch_Bitmap(FSCRT_BITMAP dib, FSCRT_BITMAP& dstDib)
{
	FSDK_OutputLog("\r\n====== Stretch bitmap ======\r\n");
	FS_RESULT ret;
	FSCRT_RECT rect = {100, 100, 300, 300};
	FS_INT32 nWidth = 0, nHeight = 0;
	FS_INT32 format;
	//Create destination bitmap.
	FSCRT_Bitmap_GetSize(dib, &nWidth, &nHeight);
	FSCRT_Bitmap_GetFormat(dib, &format);
	ret = FSCRT_Bitmap_Create(nWidth, nHeight, format, NULL, 0, &dstDib);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: Stretch bitmap failed with error code %d.\r\n", ret);
	}
	else {
		//Fill destination bitmap with white.
		FSCRT_Bitmap_FillRect(dstDib, FSCRT_ARGB_Encode(0xff, 0xff, 0xff, 0xff), NULL);
		//Stretch bitmap.
		FSCRT_Bitmap_StretchTo(dib, dstDib, 100, 100, nWidth - 100, nHeight - 100, &rect, FSCRT_INTERPOLATION_QUADRATIC);
		FSDK_OutputLog("Success: Stretch bitmap successfully.\r\n");
	}
}

//Transform bitmap with matrix.
void Transform_Bitmap(const FSCRT_BITMAP dib, FSCRT_BITMAP& dstDib)
{
	FSDK_OutputLog("\r\n======Transform bitmap with matrix ======\r\n");
	FS_RESULT ret;
	FSCRT_RECT rect = {50, 50, 200, 200};
	FS_INT32 nWidth = 0, nHeight = 0;
	FS_INT32 format;
	//Create destination bitmap.
	FSCRT_Bitmap_GetSize(dib, &nWidth, &nHeight);
	FSCRT_Bitmap_GetFormat(dib, &format);
	ret = FSCRT_Bitmap_Create(nWidth, nHeight, format, NULL, 0, &dstDib);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: Transform  bitmap failed with error code %d.\r\n", ret);
	}
	else {
		//Fill destination bitmap with white.
		FSCRT_Bitmap_FillRect(dstDib, FSCRT_ARGB_Encode(0xff, 0xff, 0xff, 0xff), NULL);
		//Transform bitmap with matrix.
		FSCRT_MATRIX matrix;
		matrix.a = (FS_FLOAT)nWidth;
		matrix.b = 0;
		matrix.c = 0;
		matrix.d = (FS_FLOAT) -nHeight;
		matrix.e = 0;
		matrix.f = (FS_FLOAT) nHeight;
		FSCRT_Bitmap_TransformTo(dib, dstDib, &matrix, &rect, FSCRT_INTERPOLATION_QUADRATIC);
		FSDK_OutputLog("Success: Transform bitmap with matrix succeed.\r\n");
	}
}

//Calculate bitmap edge.
void CalcEdge_Bitmap(const FSCRT_BITMAP dib, FSCRT_BITMAP& dstDib)
{
	FSDK_OutputLog("\r\n====== Calculate bitmap edge ======\r\n");
	FS_RESULT ret;
	FS_INT32 nWidth = 0, nHeight = 0;
	FS_INT32 format;
	//Create destination bitmap.
	FSCRT_Bitmap_GetSize(dib, &nWidth, &nHeight);
	FSCRT_Bitmap_GetFormat(dib, &format);
	ret = FSCRT_Bitmap_Create(nWidth, nHeight, format, NULL, 0, &dstDib);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: Create dest bitmap failed with error code %d.\r\n", ret);
	}else {
		//Fill destination bitmap with white.
		FSCRT_Bitmap_FillRect(dstDib, FSCRT_ARGB_Encode(0xff, 0xff, 0xff, 0xff), NULL);
		FSCRT_MATRIX matrix;
		matrix.a = (FS_FLOAT)nWidth;
		matrix.b = 0;
		matrix.c = 0;
		matrix.d = (FS_FLOAT)-nHeight;
		matrix.e = 0;
		matrix.f = (FS_FLOAT)nHeight;
		FSCRT_RECT rectEdge;
		//Calculate destination bitmap edge.
		FSCRT_Bitmap_CalcBBox(dib, FSCRT_BITMAPBBOX_DETECTION, 0xffffff, 0, 0, &rectEdge);
		//Transform bitmap.
		FSCRT_Bitmap_TransformTo(dib, dstDib, &matrix, &rectEdge, FSCRT_INTERPOLATION_DOWNSAMPLE);
		FSDK_OutputLog("Success: Calculate bitmap edge ends.\r\n");
	}
}

//Convert bitmap format.
void ConvertFm_Bitmap(const FSCRT_BITMAP& dib)
{
	FSDK_OutputLog("\r\n====== Convert bitmap format ======\r\n");
	FS_INT32 format, newFormat;
	//get old format.
	FSCRT_Bitmap_GetFormat(dib, &format);
	FSCRT_Bitmap_ConvertFormat(dib, FSCRT_BITMAPFORMAT_24BPP_RGB);
	//get format after convert.
	FS_RESULT ret = FSCRT_Bitmap_GetFormat(dib, &newFormat);
	if (FSCRT_BITMAPFORMAT_24BPP_RGB != newFormat)
		FSDK_OutputLog("Failure: Fail to convert bitmap format from %d to %d, with error code %d.\r\n", format, FSCRT_BITMAPFORMAT_24BPP_RGB, ret);
	else
		FSDK_OutputLog("Success: Convert bitmap format succeed.\r\n");
}

void convertFile(string bitmapName, string inputFilesPath, string outputFilesPath )
{
	//FSCRT_IMAGE image = NULL;
	if(!FSDK_IsSupportedImageFile(bitmapName.c_str())) 
		return ;
	// load image from file
	CAutoImage autoImage((inputFilesPath + "/" + bitmapName).c_str(), "rb");

	int pos = (int)bitmapName.find_first_of('.');
	string strFileName = bitmapName;
	strFileName = strFileName.erase(pos, 4);

	string outFilename = outputFilesPath + "/" +strFileName;

	// Get the image object using the file handler
	FS_RESULT ret = autoImage.LoadImage();
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		return;
	}
	FS_INT32 iframecount = 0;
	
	// count frames
	ret = FSCRT_Image_CountFrames(autoImage.GetImage(), &iframecount);
	if ( FSCRT_ERRCODE_SUCCESS != ret)
		return;
	
	if( iframecount >0)
	{
		FSCRT_BITMAP dstDib = NULL;
		ret = FSCRT_Image_LoadFrame(autoImage.GetImage(), 0);
		if(ret != FSCRT_ERRCODE_SUCCESS){
			FSDK_OutputLog("Failure: In function FSCRT_Image_LoadFrame with error code %d.\r\n", ret);
			return;
		}
			

		//get current Fram Bitmap 
		ret = autoImage.GetCurrentFramBitmap();
		if(ret != FSCRT_ERRCODE_SUCCESS){
			return;
		}

		// test flipp function
		FSDK_OutputLog("\r\n====== Flip bitmap begin ======\r\n");
		FSCRT_Bitmap_GetFlipped(autoImage.GetBitmap(), TRUE, FALSE, &dstDib);  
		if (dstDib)
		{
			FSDK_OutputLog("Success: Flip bitmap succesfully.\r\n");
			FSDK_SaveImageFile(dstDib, FSCRT_IMAGETYPE_BMP, (char*)(outFilename + "_Flipp" + ".bmp").c_str());
			FSCRT_Bitmap_Release(dstDib);
			dstDib = NULL;
		}
		else 
			FSDK_OutputLog("Failure: In function FSCRT_Bitmap_GetFlipped with error code %d.\r\n", ret);

		//test stretch function
		Stretch_Bitmap(autoImage.GetBitmap(), dstDib);
		if (dstDib)
		{
			FSDK_SaveImageFile(dstDib, FSCRT_IMAGETYPE_BMP, (char*)(outFilename + "_Stretch" + ".bmp").c_str());
			FSCRT_Bitmap_Release(dstDib);
			dstDib = NULL;
		}
		
		//test Calculate bitmap edge.
		CalcEdge_Bitmap(autoImage.GetBitmap(), dstDib);
		if (dstDib)
		{
			FSDK_SaveImageFile(dstDib, FSCRT_IMAGETYPE_BMP, (char*)(outFilename + "_Calc" + ".bmp").c_str());
			FSCRT_Bitmap_Release(dstDib);
			dstDib = NULL;
		}

		//Convert bitmap format.
		ConvertFm_Bitmap(autoImage.GetBitmap());
		if(autoImage.GetBitmap())
		{
			FSDK_SaveImageFile(autoImage.GetBitmap(), FSCRT_IMAGETYPE_BMP, (char*)(outFilename + "_Conv" + ".bmp").c_str());
		}

		//Convert transform.
		Transform_Bitmap(autoImage.GetBitmap(), dstDib);
		if (dstDib)
		{
			FSDK_SaveImageFile(dstDib, FSCRT_IMAGETYPE_BMP, (char*)(outFilename + "_Tran" + ".bmp").c_str());
			FSCRT_Bitmap_Release(dstDib);
			dstDib = NULL;
		}
	}
}


void  ConvertFolder(void* folder, string inputFilesPath, string outputFilesPath)
{
	do
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if(!isDir)
		{
			convertFile(strFile, inputFilesPath, outputFilesPath);
		}
		if (strFile.size() < 1) break;
		
	} while (1);
}

int main(int argc, char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);

	//Get log File.
	string logPath = FSDK_GetLogFile("bitmap_transform");
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: bitmap_transform\r\n\r\n");

	//Initialize library.
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Init PDF module.
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	string inputFilesPath = FSDK_GetInputFilesFolder("bitmap_transform");
	//Get input files folder.
	void* folder = FSDK_OpenFolder(inputFilesPath.c_str(), FALSE);
	//Get input files folder path.
	string outputFilesPath = FSDK_GetOutputFilesFolder("bitmap_transform");
	if (folder)
	{
		ConvertFolder(folder, inputFilesPath, outputFilesPath);
		//Close access to specific folder.
		FSDK_CloseFolder(folder);
	}
	else
	{
		string inputFilesPath = FSDK_GetFixFolder();
		// Use the file to test.
		string bitmapName = "Foxit.bmp";
		convertFile(bitmapName, inputFilesPath, outputFilesPath);
	}

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}