/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a source to implement functions of exporting annotations from FDF and exporting annotations to FDF.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main fdf
#endif

bool IsMarkupType(const FSCRT_BSTR* annotType)
{
	if( strcmp(annotType->str, FSPDF_ANNOTTYPE_TEXT) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_FREETEXT) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_LINE) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_SQUARE) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_CIRCLE) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_POLYGON) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_POLYLINE) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_HIGHLIGHT) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_UNDERLINE) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_SQUIGGLY) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_STRIKEOUT) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_STAMP) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_CARET) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_INK) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_FILEATTACHMENT) == 0 ||
		strcmp(annotType->str, FSPDF_ANNOTTYPE_SOUND) == 0 )
		return TRUE;
	return FALSE;
}

//Export annotations to PDF from FDF
void ExportFDFAnnotToPDF(string fdfName, string inputFilesPath, string outputFilesPath)
{
	FSDK_OutputLog("\r\n====== Export annotation from FDF to PDF ======\r\n");
	//Open FDF file
	CAutoDocument autoFDFDoc((inputFilesPath + "/" + fdfName).c_str(), "rb");
	 
	//Load FDF document
	FS_BOOL bRet = autoFDFDoc.LoadFDFDocument();

	if (!bRet)
		return;
	//Get all annotations in the FDF document
	FS_INT32 nAnnotCount = 0;
	FS_RESULT ret = FSFDF_Annot_GetCount(autoFDFDoc.GetDocument(), NULL, &nAnnotCount);
	if (FSCRT_ERRCODE_SUCCESS != ret){
		FSDK_OutputLog("Failure: In function FSFDF_Annot_GetCount with error code %d.\r\n", ret);
		return;
	}
	if(nAnnotCount == 0){
		FSDK_OutputLog("Failure: The FDF document's annotation does not exist.\r\n", ret);
		return;
	}
	FSCRT_ANNOT* pAnnotArray = new FSCRT_ANNOT[nAnnotCount];
	for (int i = 0; i < nAnnotCount; i++)
	{
		FSFDF_Annot_Get(autoFDFDoc.GetDocument(), NULL, i, &pAnnotArray[i]);
	}

	FSCRT_BSTR pdfpath;
	FSCRT_BStr_Init(&pdfpath);
	//Get PDF path from FDF file
	FSFDF_Doc_GetPDFPath(autoFDFDoc.GetDocument(), &pdfpath);
	string strpdf(pdfpath.str, pdfpath.len);
	strpdf = inputFilesPath + "/" + strpdf;
	FSCRT_BStr_Clear(&pdfpath);

	//Open PDF file
	CAutoDocument autoPdfDoc(strpdf.c_str(), "rb");
	//Load PDF document
	ret = autoPdfDoc.LoadPDFDocument(NULL);

	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		//Export annot to pdf
		for (int i = 0; i < nAnnotCount; i++)
		{
			FSCRT_BSTR type = {NULL, 0};
			FSPDF_Annot_GetType(pAnnotArray[i],&type);
			if(!IsMarkupType(&type)){
				FSCRT_BStr_Clear(&type);
				continue;
			}
			FSCRT_BStr_Clear(&type);
			FS_INT32 pageIndex = 0;
			ret = FSFDF_Annot_GetPageIndex(pAnnotArray[i], &pageIndex);
            
			if (FSCRT_ERRCODE_SUCCESS == ret)
			{
				CAutoPDFPage pdfPage;
				ret = pdfPage.LoadPDFPage(autoPdfDoc.GetDocument(),pageIndex);
				if (FSCRT_ERRCODE_SUCCESS != ret)
					return;

				FSCRT_ANNOT annot = NULL;
				ret = FSFDF_Annot_ExportToPDFPage(pAnnotArray[i], pdfPage.GetPDFPage(), &annot);
				if (FSCRT_ERRCODE_SUCCESS == ret)
					FSDK_OutputLog("Success: Export Annot with index %d to PDF document.\r\n", i);
				else
				{	
					FSDK_OutputLog("Failure: Fail to export Annot with index %d to PDF document, and the error code %d.\r\n", i, ret);
				}
			}
		}
	}
	string pdfName = string(fdfName.c_str(), fdfName.length() - 4 );
	pdfName += ".pdf";
	//Save PDF to output files folder
	FSDK_SavePDFFile(autoPdfDoc.GetDocument(), (outputFilesPath + "/" + pdfName).c_str());
	delete []pAnnotArray;
}

//Export annotations from PDF to FDF
void ExportPDFAnnotToFDF(string pdfName, string inputFilesPath, string outputFilesPath)
{
	FSDK_OutputLog("\r\n====== Export annotation from PDF to FDF ======\r\n");
	//open PDF file
	CAutoDocument autoPdfDoc((inputFilesPath + "/" + pdfName).c_str(), "rb");
	//Load PDF document
	if (FSCRT_ERRCODE_SUCCESS != autoPdfDoc.LoadPDFDocument(NULL))
		return;

	FS_INT32 pageCount = 0;
	//Count pages
	FS_RESULT ret = FSPDF_Doc_CountPages(autoPdfDoc.GetDocument(), &pageCount);
	if (FSCRT_ERRCODE_SUCCESS != ret){
		FSDK_OutputLog("Failure: In function FSPDF_Doc_CountPages with error code %d.\r\n", ret);
		return ;
	}

	//Create FDF Document
	FS_INT32 pageIndex = 0;
	CAutoDocument autoFDFDoc;
	if(autoFDFDoc.CreateFDFDocument(FSCRT_DOCUMENTTYPE_FDF) != FSCRT_ERRCODE_SUCCESS)
		return;

	//Get annotations in all pages
	while (pageIndex < pageCount)
	{
		//Load page
		CAutoPDFPage autoPDFPage;
		ret = autoPDFPage.LoadPDFPage(autoPdfDoc.GetDocument(), pageIndex);
		if (FSCRT_ERRCODE_SUCCESS != ret)
			continue;
		autoPDFPage.LoadAnnots();			//Load annots		
		FS_INT32 nAnnotCount = 0;

		//Count annotations
		ret = FSPDF_Annot_GetCount(autoPDFPage.GetPDFPage(), NULL, &nAnnotCount);
		if (FSCRT_ERRCODE_SUCCESS == ret)
		{
			//Export annot to FDF
			for (int i = 0; i < nAnnotCount; i++)
			{
				//Get annotations and export to FDF
				FSCRT_ANNOT annot = NULL;
				FSPDF_Annot_Get(autoPDFPage.GetPDFPage(), NULL, i, &annot);

				FSCRT_BSTR type = {NULL, 0};
				FSPDF_Annot_GetType(annot,&type);
				if(!IsMarkupType(&type)) {
					FSCRT_BStr_Clear(&type);
					continue;
				}
				FSCRT_BStr_Clear(&type);
				ret = FSPDF_Annot_ExportToFDFDoc(annot, autoFDFDoc.GetDocument());
				if (FSCRT_ERRCODE_SUCCESS == ret)
					FSDK_OutputLog("Success: Export annot with index %d to FDF document.\r\n", i);
				else
					FSDK_OutputLog("Failure: Fail to export annot with index %d to FDF document, and error code %d.\r\n", i, ret);
			}
		}else
			FSDK_OutputLog("Failure: In function FSPDF_Annot_GetCount with error code %d.\r\n", ret);
		pageIndex ++;
	} 

	//Save FDF document
	string fdfName = string(pdfName.c_str(), pdfName.length() - 4 );
	fdfName += ".fdf";
	FSCRT_FILE saveFile = FSDK_OpenFile((outputFilesPath + "/" + fdfName).c_str(), "wb");
	if (saveFile)
	{
		ret = FSFDF_Doc_Save(autoFDFDoc.GetDocument(), saveFile);
		if (FSCRT_ERRCODE_SUCCESS == ret)
		{
			FSDK_OutputLog("Success: The FDF document save to %s.\r\n", FSDK_GetFileName(fdfName.c_str()).c_str());
		}else
			FSDK_OutputLog("Failure: In function FSFDF_Doc_Save with error code %d.\r\n", ret);
		FSCRT_File_Release(saveFile);
	}
}

void ExportFolderFDF(void* folder, string inputFilesPath, string outputFilesPath)
{
	//Enumerate files in input file folder and choose FDF or PDF file to load
	//1.Export annotations from FDF to PDF
	//2.Export annotations from PDF to FDF
	do
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if (strFile.size() > 0 && !isDir)
		{
			if (FSDK_CompareExtName(strFile.c_str(), ".fdf"))
			{
				//Export annotations from FDF to PDF
				ExportFDFAnnotToPDF(strFile, inputFilesPath, outputFilesPath);
			}

			if (FSDK_CompareExtName(strFile.c_str(), ".pdf"))
			{
				//Export annotations from PDF to FDF
				ExportPDFAnnotToFDF(strFile, inputFilesPath, outputFilesPath);
			}
		}

		if (strFile.size() < 1) break;
		if (isDir) continue;
	} while (1);
}

int main(int argc, char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);
	string logPath = FSDK_GetLogFile("fdf");
	//Open log file
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: fdf\r\n\r\n");

	//Initialize Foxit SDK library
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_OutputLog("failure\r\n");
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module, or no method of PDF module can be used
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get output files folder path.
	string outputFilesPath = FSDK_GetOutputFilesFolder("fdf");

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFilesPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get path of input file folder for this demo
	string inputFilesPath = FSDK_GetInputFilesFolder("fdf");
	void* folder = FSDK_OpenFolder(inputFilesPath.c_str(), FALSE);

	if (folder)
	{
		//Enumerate files in input file folder and choose FDF or PDF file to load
		FSDK_OutputLog("Input from folder: %s\r\n", inputFilesPath.c_str());
		FSDK_OutputLog("Output to folder: %s \r\n\r\n", outputFilesPath.c_str());
		ExportFolderFDF(folder, inputFilesPath, outputFilesPath);
		//Close access to specific folder
		FSDK_CloseFolder(folder);
	}
	else
	{
		string inputFilesPath = FSDK_GetFixFolder();
		//Export annotations from FDF to PDF
		string fdfName = "AnnotationData.fdf";
		FSDK_OutputLog("Input from folder: %s\r\n", inputFilesPath.c_str());
		FSDK_OutputLog("Output to folder: %s \r\n\r\n", outputFilesPath.c_str());
		ExportFDFAnnotToPDF(fdfName, inputFilesPath, outputFilesPath);
		//Export annotations from PDF to FDF
		string pdfName = "AnnotationDataExport.pdf";
		ExportPDFAnnotToFDF(pdfName, inputFilesPath, outputFilesPath);
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}
