/**
* Copyright (C) 2001-2019, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
* You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
* unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
*
* This file is a source to show how to use functions about layer module.
*/

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main layeredit
#endif

FS_RESULT CreateLayers_And_AddPageObjects(string inputPath, string fileName, string outputPath)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;

	string filePath = inputPath + "/" +fileName;
	CAutoDocument doc(filePath.c_str(), "rb");
	doc.LoadPDFDocument(NULL);
	FSCRT_DOCUMENT sdkDoc = doc.GetDocument();

	FSCRT_BSTR layerNames[] = {{"Text", (FS_DWORD)strlen("Text")},{"Path", (FS_DWORD)strlen("Path")},{"Image", (FS_DWORD)strlen("Image")}, {"Other", (FS_DWORD)strlen("Other")}};
	FS_INT32 objTypes[] = {FSPDF_PAGEOBJECT_TEXT, FSPDF_PAGEOBJECT_PATH, FSPDF_PAGEOBJECT_IMAGE,FSPDF_PAGEOBJECT_ALL};
	FSPDF_LAYER layerAdded[4] = {NULL, NULL,NULL, NULL};

	for (int layerIndex = 0; layerIndex < 4; layerIndex ++)
	{
		ret = FSPDF_Doc_AddLayer(sdkDoc, &layerNames[layerIndex], &layerAdded[layerIndex]);
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			FSDK_OutputLog("Failure: In function FSPDF_Doc_AddLayer with error code %d.\r\n", ret);
		} else 
			FSDK_OutputLog("Success: A layer named %s is added.\r\n", layerNames[layerIndex].str);

		FS_INT32 pageCount = doc.GetPagesCount();
		for(int pageIndex = 0; pageIndex < pageCount; pageIndex++)
		{
			CAutoPDFPage autoPage;
			autoPage.LoadPDFPage(sdkDoc, pageIndex);
			autoPage.ParsePage();
			FSCRT_PAGE page=autoPage.GetPDFPage();
			
			FSPDF_PAGEOBJECTS pageObjs;
			ret = FSPDF_Page_GetPageObjects(page, &pageObjs);
			if (FSCRT_ERRCODE_SUCCESS != ret)
			{
				FSDK_OutputLog("Failure: In function FSPDF_Page_GetPageObjects with error code %d.\r\n", ret);
				FSDK_CHECK_LICENSE(ret);
				continue;
			}

			ret = FSCRT_ERRCODE_TOBECONTINUED;
			FSCRT_POSITION position = NULL;
			FSPDF_PAGEOBJECT pageObj = NULL;
			while (FSCRT_ERRCODE_TOBECONTINUED == ret)
			{
				ret = FSPDF_PageObjects_EnumerateObject(page, pageObjs, FSPDF_PAGEOBJECT_ALL, &position, &pageObj);
				if (FSCRT_ERRCODE_FINISHED != ret && FSCRT_ERRCODE_TOBECONTINUED != ret)
				{
					FSDK_OutputLog("Failure: In function FSPDF_PageObjects_EnumerateObject with error code %d.\r\n", ret);
					continue;
				}

				if(pageObj)
				{
					FS_INT32 objType = -1;
					FS_RESULT ret2 = FSPDF_PageObject_GetType(page, pageObj, &objType);
					if (FSCRT_ERRCODE_SUCCESS != ret2)
					{
						FSDK_OutputLog("Failure: In function FSPDF_PageObject_GetType with error code %d.\r\n", ret);
						continue;
					}

					if (layerIndex != 3 && objType == objTypes[layerIndex])
					{
						ret2 = FSPDF_Layer_AddPageObject(layerAdded[layerIndex], page, pageObj);
						if (FSCRT_ERRCODE_SUCCESS != ret2)
						{
							FSDK_OutputLog("Failure: In function FSPDF_Layer_AddPageObject with error code %d.\r\n", ret);
							continue;
						}
					}
					else if (layerIndex == 3 && objType != objTypes[0] && objType != objTypes[1] && objType != objTypes[2])// the rest objects.
					{
						ret2 = FSPDF_Layer_AddPageObject(layerAdded[layerIndex], page, pageObj);
						if (FSCRT_ERRCODE_SUCCESS != ret2)
						{
							FSDK_OutputLog("Failure: In function FSPDF_Layer_AddPageObject with error code %d.\r\n", ret);
							continue;
						} 
					}
				}
			}// end while (ret == FSCRT_ERRCODE_TOBECONTINUED)

			ret = FSPDF_PageObjects_GenerateContents(page, pageObjs);
			if (FSCRT_ERRCODE_SUCCESS != ret)
			{
				FSDK_OutputLog("Failure: In function FSPDF_PageObjects_GenerateContents with error code %d.\r\n", ret);
			} else {
				FSDK_OutputLog("Success: Generate the page content.\r\n");
			}
		}//end for(int pageIndex = 0; pageIndex < pageCount; pageIndex++)
	}//end 	for (int layerIndex = 0; layerIndex < 3; layerIndex ++)

	ret = FSPDF_Doc_SetLayersBaseState(sdkDoc, FSPDF_LAYER_BASESTATE_OFF);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Doc_SetLayersBaseState with error code %d.\r\n", ret);
	} else {
		FSDK_OutputLog("Success: Set the base state of layers in document .\r\n");
	}

	//generally, layerAdded[2] equal with layers.childrn[2].layer.
	//layerAdded[2] is Image layer. Let it be visible.
	ret = FSPDF_Layer_SetDefaultState(layerAdded[2], TRUE);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Layer_SetDefaultState with error code %d.\r\n", ret);
		return ret;
	} else {
		FSDK_OutputLog("Success: Set layer's default state.\r\n");
	}

	string outputFilePath = outputPath + fileName;
	FSDK_SavePDFFile(sdkDoc,outputFilePath.c_str());
	doc.CloseDocument();
	return ret;
}

FS_RESULT RetriveLayersAndHandleUsage(FSPDF_LAYERNODE* layers, FS_INT32 usageType, FS_BOOL isSetting = TRUE)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	if (layers->layer)
	{
		if (usageType == FSPDF_LAYERUSAGETYPE_DESIGN)
			usageType = FSPDF_LAYERUSAGETYPE_ZOOM;

		FS_LPVOID pData;
		FS_BOOL visible = TRUE;
		FS_BOOL invisible = TRUE;

		FSPDF_LAYERUSAGE_PRINTDATA printData;
		printData.printState = TRUE;

		ret = FSCRT_BStr_Init(&printData.subtype);
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			FSDK_OutputLog("Failure: In function FSCRT_BStr_Init with error code %d.\r\n", ret);
			return ret;
		}
		ret =FSCRT_BStr_Set(&printData.subtype, "watermark", strlen("watermark"));
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			FSDK_OutputLog("Failure: In function FSCRT_BStr_Set with error code %d.\r\n", ret);
			return ret;
		}

		FSPDF_LAYERUSAGE_ZOOMDATA zoomData;
		zoomData.minFactor = 1.0;
		zoomData.maxFactor = 2.0;

		if (isSetting)
		{
			switch(usageType)
			{
			case FSPDF_LAYERUSAGETYPE_VIEW:
				pData = &visible;
				break;
			case FSPDF_LAYERUSAGETYPE_PRINT:
				pData = &printData;
				break;
			case FSPDF_LAYERUSAGETYPE_EXPORT:
				pData = &invisible;
				break;
			case FSPDF_LAYERUSAGETYPE_ZOOM:
				pData = &zoomData;
				break;
			default:
				return FSCRT_ERRCODE_INVALIDTYPE;
			}

			ret = FSPDF_Layer_SetApplicationUsage(layers->layer, usageType,pData);
			if (FSCRT_ERRCODE_SUCCESS != ret)
			{
				FSDK_OutputLog("Failure: In function FSPDF_Layer_SetApplicationUsage with error code %d.\r\n", ret);
			} else {
				FSDK_OutputLog("Success: Set application to a layer.\r\n");
			}

		} else {
			switch(usageType)
			{
			case FSPDF_LAYERUSAGETYPE_VIEW:
				pData = &visible;
				break;
			case FSPDF_LAYERUSAGETYPE_PRINT:
				pData = &printData;
				break;
			case FSPDF_LAYERUSAGETYPE_EXPORT:
				pData = &invisible;
				break;
			case FSPDF_LAYERUSAGETYPE_ZOOM:
				pData = &zoomData;
				break;
			default:
				return FSCRT_ERRCODE_INVALIDTYPE;
			}
			ret = FSPDF_Layer_GetApplicationUsage(layers->layer, usageType, pData);
			if (FSCRT_ERRCODE_SUCCESS != ret)
				FSDK_OutputLog("Failure: In function FSPDF_Layer_GetApplicationUsage(layers->layer, usageType, pData) with error code %d.\r\n", ret);

			//you can check the usage content here.

			ret = FSPDF_Layer_RemoveApplicationUsage(layers->layer,usageType);
			if (FSCRT_ERRCODE_SUCCESS != ret)
			{
				FSDK_OutputLog("Failure: In function FSPDF_Doc_AddLayer() with error code %d.\r\n", ret);
				return ret;
			} else {
				FSDK_OutputLog("Success: Remove the layer's application usage.\r\n");
			}
		}

		ret = FSCRT_BStr_Clear(&printData.subtype);
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			FSDK_OutputLog("Failure: In function FSCRT_BStr_Clear(&printData.subtype) with error code %d.\r\n", ret);
			return ret;
		}
	}

	for(int childIndex = 0; childIndex<layers->count; childIndex++)
	{
		ret = RetriveLayersAndHandleUsage(&(layers->children[childIndex]), (usageType ++)%5, isSetting);
		if (FSCRT_ERRCODE_SUCCESS != ret)
			return ret;
	}

	return ret;
}

FS_RESULT HandleApplicationUsage(string inputPath, string fileName, string outputPath)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;

	string filePath = inputPath + "/" +fileName;
	CAutoDocument doc(filePath.c_str(), "rb");
	doc.LoadPDFDocument(NULL);
	FSCRT_DOCUMENT sdkDoc = doc.GetDocument();

	FSPDF_LAYERNODE layers;
	ret = FSPDF_LayerNode_Init(&layers);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSPDF_LayerNode_Init with error code %d.\r\n", ret);
		return ret;
	}

	ret = FSPDF_Doc_EnumLayers(sdkDoc, &layers);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Doc_EnumLayers with error code %d.\r\n", ret);
		return ret;
	}

	ret = RetriveLayersAndHandleUsage(&layers, FSPDF_LAYERUSAGETYPE_VIEW);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		return ret;

	string outputFilePath = outputPath + "AddLayerUsage_" + fileName;
	FS_BOOL bRet = FSDK_SavePDFFile(sdkDoc,outputFilePath.c_str());

	ret = RetriveLayersAndHandleUsage(&layers, FSPDF_LAYERUSAGETYPE_VIEW, FALSE);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		return ret;

	outputFilePath = outputPath + "RemoveLayerUsage_" + fileName;
	bRet = FSDK_SavePDFFile(sdkDoc,outputFilePath.c_str());

	ret = FSPDF_LayerNode_Clear(&layers);
	doc.CloseDocument();
	return ret;
}

FS_RESULT RetieveLayersToCountObject(FSPDF_LAYERNODE* layers, FSCRT_DOCUMENT doc)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;

	if (layers->layer)
	{
		FS_INT32 pageCount = 0;
		ret = FSPDF_Doc_CountPages(doc, &pageCount);
		if(ret != FSCRT_ERRCODE_SUCCESS)
			FSDK_OutputLog("Failure: In function FSPDF_Doc_CountPages with error code %d.\r\n", ret);

		for(int pageIndex = 0; pageIndex < pageCount; pageIndex++)
		{
			CAutoPDFPage autoPage;
			autoPage.LoadPDFPage(doc, pageIndex);
			autoPage.ParsePage();

			FSCRT_PAGE page = autoPage.GetPDFPage();
			ret = FSPDF_Doc_GetPage(doc, pageIndex, &page);
			if(ret != FSCRT_ERRCODE_SUCCESS)
			{
				FSDK_OutputLog("Failure: In function FSPDF_Doc_GetPage with error code %d.\r\n", ret);
				continue;
			}

			FSCRT_POSITION position = NULL;
			FSPDF_PAGEOBJECT pageObj;
			ret = FSPDF_Layer_EnumeratePageObject(layers->layer, page, &position, &pageObj);
			if(!(ret == FSCRT_ERRCODE_FINISHED && pageObj == NULL))
			{
				FSDK_OutputLog("Failure: In function FSPDF_Layer_EnumeratePageObject with error code %d.\r\n", ret);
				return FSCRT_ERRCODE_ERROR;
			}
		}
	}

	for(int childIndex = 0; childIndex<layers->count; childIndex++)
	{
		ret = RetieveLayersToCountObject(&(layers->children[childIndex]), doc);
		if(ret != FSCRT_ERRCODE_SUCCESS)
			return ret;
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT RetrieveObjectToHandleLayers(CAutoDocument& doc)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	FS_RESULT ret2 = FSCRT_ERRCODE_SUCCESS;

	FSCRT_DOCUMENT sdkDoc = doc.GetDocument();
	FS_INT32 pageCount = doc.GetPagesCount();
	for(int pageIndex = 0; pageIndex < pageCount; pageIndex++)
	{
		CAutoPDFPage autoPage;
		autoPage.LoadPDFPage(sdkDoc, pageIndex);
		autoPage.ParsePage();
		FSCRT_PAGE page = autoPage.GetPDFPage();

		FSPDF_PAGEOBJECTS pageObjs;
		ret = FSPDF_Page_GetPageObjects(page, &pageObjs);
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			FSDK_OutputLog("Failure: In function FSPDF_Page_GetPageObjects with error code %d.\r\n", ret);
			continue;
		} 

		FSCRT_ARRAY layerArray;
		ret = FSCRT_Array_Init(&layerArray, FSCRT_VT_OBJECT_LAYER);
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			FSDK_OutputLog("Failure: In function FSCRT_Array_Init with error code %d.\r\n", ret);
			continue;
		} 

		ret = FSCRT_ERRCODE_TOBECONTINUED;
		FSCRT_POSITION position = NULL;
		FSPDF_PAGEOBJECT pageObj = NULL;
		while (ret == FSCRT_ERRCODE_TOBECONTINUED)
		{
			ret = FSCRT_Array_Clear(&layerArray);
			if (FSCRT_ERRCODE_SUCCESS != ret)
			{
				FSDK_OutputLog("Failure: In function FSCRT_Array_Clear with error code %d.\r\n", ret);
				continue;
			} 

			ret = FSPDF_PageObjects_EnumerateObject(page, pageObjs, FSPDF_PAGEOBJECT_ALL, &position, &pageObj);
			if (FSCRT_ERRCODE_SUCCESS != ret && FSCRT_ERRCODE_FINISHED != ret && FSCRT_ERRCODE_TOBECONTINUED != ret)
			{
				FSDK_OutputLog("Failure: In function FSPDF_PageObjects_EnumerateObject with error code %d.\r\n", ret);
				continue;
			} 

			if(pageObj)
			{
				ret2 = FSPDF_PageObject_GetLayers(page, pageObj, &layerArray);
				if (FSCRT_ERRCODE_SUCCESS == ret2 )
				{
					for (int layerIndex = 0; layerIndex < layerArray.count; layerIndex++)
					{
						FSPDF_LAYER layer = ((FSPDF_LAYER*)layerArray.data)[layerIndex];
						ret2 = FSPDF_Layer_RemovePageObject(layer, pageObj);
						if (FSCRT_ERRCODE_SUCCESS != ret2)
						{
							FSDK_OutputLog("Failure: In function FSPDF_Layer_RemovePageObject with error code %d.\r\n", ret2);
							continue;
						} 
					}
				}
			}
		}// end while (ret == FSCRT_ERRCODE_TOBECONTINUED)
		ret = FSPDF_PageObjects_GenerateContents(page, pageObjs);
		if (FSCRT_ERRCODE_SUCCESS != ret)
			FSDK_OutputLog("Failure: In function FSPDF_PageObjects_GenerateContents with error code %d.\r\n", ret);
	}//End for(int pageIndex = 0; pageIndex < pageCount; pageIndex++)

	FSPDF_LAYERNODE layers;
	ret = FSPDF_LayerNode_Init(&layers);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSPDF_LayerNode_Init with error code %d.\r\n", ret);
		return ret;
	}

	ret = FSPDF_Doc_EnumLayers(sdkDoc, &layers);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Doc_EnumLayers with error code %d.\r\n", ret);
		return ret;
	} else {
		FSDK_OutputLog("Success: Enumerate the document's  layers.\r\n");
	}

	//after remove all page objects, any one of layers contains a page object.
	ret = RetieveLayersToCountObject(&layers, sdkDoc);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: Can't count the page objects in layers. Error code: %d.\r\n", ret);
		return ret;
	}
	else
		FSDK_OutputLog("Success: Count the number of page objects in layers.\r\n");

	ret = FSPDF_LayerNode_Clear(&layers);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure: In function FSPDF_LayerNode_Clear with error code %d.\r\n", ret);
	return ret;
}


FS_RESULT GetLayersThroughObject(string inputPath, string fileName, string outputPath)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;

	string filePath = inputPath + "/" +fileName;
	CAutoDocument doc(filePath.c_str(), "rb");
	doc.LoadPDFDocument(NULL);

	ret = RetrieveObjectToHandleLayers(doc);
	if(ret != FSCRT_ERRCODE_SUCCESS)
		return ret;
			
	string outputFilePath = outputPath + "RemoveObjects_" + fileName;
	FS_BOOL bRet = FSDK_SavePDFFile(doc.GetDocument(),outputFilePath.c_str());

	doc.CloseDocument();
	return ret;
}

FS_RESULT RetieveLayersToRemove(FSPDF_LAYERNODE* layers)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	if (layers->layer)
	{
		ret = FSPDF_Layer_Remove(layers->layer,0);
		if (FSCRT_ERRCODE_SUCCESS != ret)
			FSDK_OutputLog("Failure: In function FSPDF_Layer_Remove with error code %d.\r\n", ret);
	}

	for(int childIndex = 0; childIndex<layers->count; childIndex++)
	{
		ret = RetieveLayersToRemove(&(layers->children[childIndex]));
		if(ret != FSCRT_ERRCODE_SUCCESS)
			return ret;
	}
	return ret;
}

FS_RESULT RemoveLayers(string inputPath, string fileName, string outputPath)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;

	string filePath = inputPath + "/" +fileName;
	CAutoDocument doc(filePath.c_str(), "rb");
	doc.LoadPDFDocument(NULL);

	FSPDF_LAYERNODE layers;
	ret = FSPDF_LayerNode_Init(&layers);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSPDF_LayerNode_Init with error code %d.\r\n", ret);
		return ret;
	}

	ret = FSPDF_Doc_EnumLayers(doc.GetDocument(), &layers);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Doc_EnumLayers with error code %d.\r\n", ret);
		return ret;
	} else {
		FSDK_OutputLog("Success: Enumerate the document's  layers.\r\n");
	}

	ret = RetieveLayersToRemove(&layers);

	string outputFilePath = outputPath + "RemoveLayers_" + fileName;
	FS_BOOL bRet = FSDK_SavePDFFile(doc.GetDocument(),outputFilePath.c_str());

	ret = FSPDF_LayerNode_Clear(&layers);
	if(FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure: In function FSPDF_LayerNode_Clear with error code %d.\r\n", ret);
	doc.CloseDocument();
	return ret;
}

FS_RESULT LayerOperation(string fileName, string filePath, string outputFolder)
{
	FS_RESULT ret = CreateLayers_And_AddPageObjects(filePath, fileName, outputFolder);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: Can't create layers and add page objects to them.\r\n");
		FSDK_CHECK_RET(ret);
	} else
		FSDK_OutputLog("Success: Layers are created and page objects are added to them.\r\n");

	ret = HandleApplicationUsage(outputFolder, fileName, outputFolder);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: Can't set the application usages of layers.\r\n");
		FSDK_CHECK_RET(ret);
	} else
		FSDK_OutputLog("Success: Layers' application usages are set.\r\n");

	ret = GetLayersThroughObject(outputFolder, fileName, outputFolder);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: Can't get layers through page objects.\r\n");
		FSDK_CHECK_RET(ret);
	} else
		FSDK_OutputLog("Success: Page objects are removed from layers.\r\n");

	ret = RemoveLayers(outputFolder, fileName, outputFolder);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: Can't remove all the layers.\r\n");
		FSDK_CHECK_RET(ret);
	} else
		FSDK_OutputLog("Success: All the layers is removed.\r\n");

	return ret;
}

void  BrowseFolderForLayerEdit(void* folder, string inputFilesPath, string outputFilesPath)
{
	FSDK_OutputLog("Enter the input folder\r\n");

	do
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if(!isDir)
		{
			LayerOperation(strFile, inputFilesPath, outputFilesPath);
		}
		if (strFile.size() < 1) break;

	} while (TRUE);
}

/**
* This demo shows how to access layer properties.
* Intent: move all of image objects to image layer, path object to path layer, text object to text layer. And the rest objects will be moved to a layer named "Others".
* Major header file: fpdf_layer_r.h, fpdf_layer_w.h, fspdf_pageobjects_r.h
**/
int main(int argc, char* argv[]){

	FSDK_SetExecuteFilePath(argc, argv);
	string logPath = FSDK_GetLogFile("layeredit");
	//Open log file.
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: layeredit\r\n");
	FSDK_OutputLog("\tInput from: input_files folder\r\n");
	FSDK_OutputLog("\tOutput to: output_files/layeredit\r\n\r\n");

	//Initialize SDK library, or no method of SDK can be used.
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module, or no method of PDF module can be used.
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}
	//Get input files folder.
	string inputFilesPath = FSDK_GetInputFilesFolder("layeredit");

	//Get input files folder path.
	string outputFilesPath = FSDK_GetOutputFilesFolder("layeredit");

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFilesPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	void* folder = FSDK_OpenFolder(inputFilesPath.c_str());
	if(folder)
	{
		FSDK_OutputLog("\r\nEnumerate files in folder input_files: ");
		BrowseFolderForLayerEdit(folder,inputFilesPath,outputFilesPath + "/");
		//Close access to specific folder.
		FSDK_CloseFolder(folder);
	}
	else
	{
		string inputFilesPath = FSDK_GetFixFolder();
		string pdffile = "FoxitBigPreview.pdf";
		string outputFile = outputFilesPath + "/";
		FS_RESULT ret = LayerOperation(pdffile,inputFilesPath ,outputFile);	
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}