/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a demo to demonstrate how to use matrix to translate, scale... objects.
 **/

#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main matrix
#endif

int main(int argc, char* argv[])
{
	int i = 0, j = 0;
	FSDK_SetExecuteFilePath(argc, argv);
	//Initialize SDK library, or no method of SDK can be used
	bool bRet = FSDK_InitializeLibray(false);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}
	//Initialize PDF module, or no method of PDF module can be used
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}
	//Open the sample PDF File
	string strInputFolder = FSDK_GetFixFolder() + "/";
	string strInputFileName = "SamplePDF.pdf";
	string strOutputFilesFolder = FSDK_GetOutputFilesFolder("matrix");

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(strOutputFilesFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	FSCRT_FILE file = FSDK_OpenFile((strInputFolder + strInputFileName).c_str(), "rb");
	FSCRT_DOCUMENT doc = NULL;
	FSCRT_PAGE page = NULL;
	FSCRT_MATRIX matrix = {0};
	FSPDF_PAGEOBJECT textObj = NULL;
	FSPDF_PAGEOBJECT imageObj = NULL;
	FS_INT32 count = 0;
	FSPDF_PAGEOBJECTS pageObjs = NULL;
	FSCRT_PROGRESS progress = NULL;
	//Start to load PDF document
	FS_RESULT ret = FSPDF_Doc_StartLoad(file, NULL, &doc, NULL);
	if(FSCRT_ERRCODE_SUCCESS != ret) goto EXIT;
	//Get the first page.
	ret = FSPDF_Doc_GetPage(doc, 0, &page);
	if(FSCRT_ERRCODE_SUCCESS != ret) goto EXIT;
	//Need to parse the page before get the page objects.
	ret = FSPDF_Page_StartParse(page, FSPDF_PAGEPARSEFLAG_NORMAL, &progress);
	ret = FSCRT_Progress_Continue(progress, NULL);
	if(FSCRT_ERRCODE_FINISHED != ret) goto EXIT;

	ret = FSPDF_Page_GetPageObjects(page, &pageObjs);
	if(FSCRT_ERRCODE_SUCCESS != ret) 
	{
		FSDK_CHECK_LICENSE(ret);
		goto EXIT;
	}
	//Search for the sample text object of "Foxit Software Incorporated" and image object of "Foxit Logo"
	
	ret = FSPDF_PageObjects_CountObjects(page, pageObjs, FSPDF_PAGEOBJECT_ALL, &count);
	for(i=0; i<count; i++)
	{
		FSPDF_PAGEOBJECT pageObj = NULL;
		FSPDF_PageObjects_GetObject(page, pageObjs, FSPDF_PAGEOBJECT_ALL, i, &pageObj);
		if(!pageObj) goto EXIT;

		FS_INT32 type = 0;
		FSPDF_PageObject_GetType(page, pageObj, &type);

		if(FSPDF_PAGEOBJECT_TEXT == type)
		{
			FSCRT_BSTR text;
			FSCRT_BStr_Init(&text);
			FSPDF_TextObject_GetUnicodeString(page, pageObj, &text);
			if(strcmp(text.str, "Foxit Software Incorporated") != 0)
			{
				FSCRT_BStr_Clear(&text);
				continue;
			}
			FSCRT_BStr_Clear(&text);
			textObj = pageObj;
			continue;;
		}
		else if(FSPDF_PAGEOBJECT_IMAGE == type)
		{
			imageObj = pageObj;
			continue;
		}
		else
			continue;
	}
	if(!textObj || !imageObj) goto EXIT;

	//Now we have found the two page objects, transform them.
	for(j=0; j<2; j++)
	{
		FSPDF_PAGEOBJECT pageObj = j==0?textObj:imageObj;
		FS_FLOAT fStartX = j==0?400.0f:150.0f;

		FSPDF_PAGEOBJECT pageObj1 = NULL;
		FSPDF_PageObject_Clone(page, pageObj, &pageObj1);
		FSPDF_PageObject_GetMatrix(page, pageObj1, &matrix);
		//Translate the matrix in vertical direction.
		FSCRT_Matrix_Translate(&matrix, 0, 150);
		FSPDF_PageObject_SetMatrix(page, pageObj1, &matrix);
		FSPDF_PageObjects_InsertObject(page, pageObjs, FSPDF_PAGEOBJECT_ALL, -1, pageObj1);

		FSPDF_PAGEOBJECT pageObj2 = NULL;
		FSPDF_PageObject_Clone(page, pageObj, &pageObj2);
		FSPDF_PageObject_GetMatrix(page, pageObj2, &matrix);
		//FS_FLOAT a = matrix.a;
		FS_FLOAT d = matrix.d;
		FS_FLOAT e = matrix.e;
		FS_FLOAT f = matrix.f;
		//Translate it to the original point(0,0) first
		FSCRT_Matrix_Translate(&matrix, -e, -f);
		//Rotate the matrix 90 degree in anticlockwise direction.
		FSCRT_Matrix_Rotate(&matrix, 1.57f);
		//Page rotation should be considered, assume that the current page rotation is 0.
		FS_FLOAT distance = d;
		if(j==0)
		{
			FSCRT_RECTF rect = {0};
			FSPDF_PageObject_GetRect(page, textObj, &rect);
			distance = rect.top - rect.bottom;
		}
		//Translate it to the specific coordinate(fStartX,400)
		FSCRT_Matrix_Translate(&matrix, distance+fStartX , 400);
		FSPDF_PageObject_SetMatrix(page, pageObj2, &matrix);
		FSPDF_PageObjects_InsertObject(page, pageObjs, FSPDF_PAGEOBJECT_ALL, -1, pageObj2);

		FSPDF_PAGEOBJECT pageObj3 = NULL;
		FSPDF_PageObject_Clone(page, pageObj, &pageObj3);
		FSPDF_PageObject_GetMatrix(page, pageObj3, &matrix);
		//Translate it to the original point(0,0) first
		FSCRT_Matrix_Translate(&matrix, -e, -f);
		//Magnify the matrix 1.5 times in both horizontal and vertical directions.
		FSCRT_Matrix_Scale(&matrix, 1.5, 1.5);
		//Translate it to the specific coordinate(fStartX,600)
		FSCRT_Matrix_Translate(&matrix, fStartX, (FS_FLOAT)600);
		FSPDF_PageObject_SetMatrix(page, pageObj3, &matrix);
		FSPDF_PageObjects_InsertObject(page, pageObjs, FSPDF_PAGEOBJECT_ALL, -1, pageObj3);

		FSPDF_PAGEOBJECT pageObj4 = NULL;
		FSPDF_PageObject_Clone(page, pageObj, &pageObj4);
		FSPDF_PageObject_GetMatrix(page, pageObj4, &matrix);
		//Translate it to the original point(0,0) first
		FSCRT_Matrix_Translate(&matrix, -e, -f);
		//Skews the x axis by an angle 0.5 and the y axis by an angle 0.5.
		FSCRT_Matrix_Shear(&matrix, 0.5, 0.5);
		//Translate it to the specific coordinate(fStartX,800)
		FSCRT_Matrix_Translate(&matrix, fStartX, (FS_FLOAT)800);
		FSPDF_PageObject_SetMatrix(page, pageObj4, &matrix);
		FSPDF_PageObjects_InsertObject(page, pageObjs, FSPDF_PAGEOBJECT_ALL, -1, pageObj4);
	}
	//Generate the page content
	FSPDF_PageObjects_GenerateContents(page, pageObjs);

	//Save the pdf document
	
	FSDK_CreatFolder(strOutputFilesFolder.c_str());
	FSDK_SavePDFFile(doc, (strOutputFilesFolder + "/MatrixTransformResult.pdf").c_str(), FSPDF_SAVEFLAG_INCREMENTAL);

EXIT:
	FSPDF_Page_Clear(page);
	FSPDF_Doc_Close(doc);
	FSCRT_File_Release(file);

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module
	FSDK_PDFModule_Finalize();
	//Finalize SDK library
	FSDK_FinalizeLibrary();
	//Close log file
	FSDK_CloseLog();
	return 0;
}
