/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is an example to add files as PDF document attachements.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfattachments
#endif

class CPDFAttachments
{
public:
	CPDFAttachments();
	~CPDFAttachments();
	void AttachmentsFolder(void* folder);
	void AttachmentsToFile();
	void AttachmentsFile(string strFile);
	FS_RESULT AddFileToAttachments(string fileName, string attchmentFile);
public:
	string m_strInputFilesPath;
	string m_strOutputFilesPath;
	string m_strFile;
	string m_strInsertFile;
};

CPDFAttachments::CPDFAttachments()
	:m_strInputFilesPath("")
	,m_strOutputFilesPath("")
	,m_strFile("")
	,m_strInsertFile("")
{

}

CPDFAttachments::~CPDFAttachments()
{

}

FS_RESULT CPDFAttachments::AddFileToAttachments(string fileName, string attchmentFile/*, string inputFilesPath, string outputFilesPath*/)
{
	//open the document
	string strFile = m_strInputFilesPath + "/" + fileName;
	CAutoDocument autoDoc(strFile.c_str(), "rb");
	if (!autoDoc.GetFileHander()) return FSCRT_ERRCODE_ERROR;

	//Load a PDF document from a PDF file
	if (FSCRT_ERRCODE_SUCCESS != autoDoc.LoadPDFDocument(NULL))
		return FSCRT_ERRCODE_ERROR;

	//Get attachments from document
	autoDoc.LoadAttachments();
	//get attachments count
	FS_INT32 count = 0;
	FS_RESULT ret = FSPDF_Attachments_CountAttachment(autoDoc.GetAttachments(), &count);
	if (ret != FSCRT_ERRCODE_SUCCESS) 
	{
		FSDK_OutputLog("Failure: In function FSPDF_Attachments_CountAttachment with error code %d.\r\n", ret);
		return FSCRT_ERRCODE_ERROR;
	}
	FSDK_OutputLog("Success: The PDF document has %d attachment(s).\r\n", count);

	//create a attachment 
	ret = autoDoc.CreateAttachment();
	if (FSCRT_ERRCODE_SUCCESS != ret) return FSCRT_ERRCODE_ERROR;

	//insert a attachment to attachments
	ret = FSPDF_Attachments_InsertAttachment(autoDoc.GetAttachments() ,count, autoDoc.GetAttachment());
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Attachments_InsertAttachment with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
	}
	FSDK_OutputLog("Success: Insert an attachment to PDF document.\r\n");

	strFile = m_strInputFilesPath + "/" + attchmentFile;
	//Open the input file
	CAutoDocument auotAttachFile(strFile.c_str(), "rb");
	if (!auotAttachFile.GetFileHander()) return FSCRT_ERRCODE_ERROR;

	//set a file to  a attachment
	ret = FSPDF_Attachment_SetFile(autoDoc.GetAttachment(), auotAttachFile.GetFileHander());
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Attachment_SetFile with error code %d.\r\n", ret);
		return FSCRT_ERRCODE_ERROR;
	}
	FSDK_OutputLog("Success: Insert a file of attachment to PDF document.\r\n");

	//set file name
	FSCRT_BSTR bstr;
	FSCRT_BStr_Init(&bstr);
	FSCRT_BStr_Set(&bstr, attchmentFile.c_str(), attchmentFile.length());
	ret = FSPDF_Attachment_SetFileName(autoDoc.GetAttachment(), &bstr, FALSE);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_BStr_Clear(&bstr);
		FSDK_OutputLog("Failure: In function FSPDF_Attachment_SetFileName with error code %d.\r\n", ret);
		return FSCRT_ERRCODE_ERROR;
	}
	FSDK_OutputLog("Success: Insert a file name of attachment to PDF document.\r\n");
	FSCRT_BStr_Clear(&bstr);
	//save PDF file
	string savePath = m_strOutputFilesPath + "/" + fileName;
	if (!FSDK_SavePDFFile(autoDoc.GetDocument(), savePath.c_str())) return FSCRT_ERRCODE_ERROR;
	FSDK_OutputLog("Success: Add file to attachments.\r\n");
	return FSCRT_ERRCODE_SUCCESS;
}

void CPDFAttachments::AttachmentsToFile()
{
	////Compose the absolute input file path
	string strFile = m_strInputFilesPath + "/" + m_strFile;	
	CAutoDocument autoDocument(strFile.c_str(), "rb");
	//Load a PDF document from a PDF file
	if (FSCRT_ERRCODE_SUCCESS != autoDocument.LoadPDFDocument(NULL))
		return;

	//Get attachments from document
	FS_RESULT ret = autoDocument.LoadAttachments();
	if (FSCRT_ERRCODE_SUCCESS != ret) return;

	//get attachments count
	FS_INT32 count = 0;
	ret = FSPDF_Attachments_CountAttachment(autoDocument.GetAttachments(), &count);

	//get all attachment by index
	autoDocument.GetAllAttachment();
	//get attachments name
	autoDocument.GetAllAttachmentFileName();
	//save attachments by filename
	autoDocument.SaveAttachmentByFileByName(m_strOutputFilesPath);
}

void CPDFAttachments::AttachmentsFile(string strFile)
{
	if (FSDK_CompareExtName(strFile.c_str(), ".pdf"))
	{
		string attachmentFile = "FoxitLogo.jpg";
		AttachmentsToFile();
		AddFileToAttachments(strFile, attachmentFile);
	}
}

void CPDFAttachments::AttachmentsFolder(void* folder)
{
	FSDK_OutputLog("success\r\n");

	do
	{
		//Get a file from folder
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if (strFile.size() > 0 && !isDir)
			AttachmentsFile(strFile);

		//If it's an invalid file name
		if (strFile.size() < 1) break;

	}while (1);
}

int main(int argc, char* argv[])
{

	//Set executing file path
	FSDK_SetExecuteFilePath(argc, argv);

	//Open log file and output some basic info
	string logPath = FSDK_GetLogFile("pdfattachments");
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfattachments\r\n\r\n");
	FSDK_OutputLog("\tInput from: input_files folder\r\n");
	FSDK_OutputLog("\tOutput to: output_files/pdfattachments\r\n\r\n");

	//Initialize Foxit SDK library
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		//Failed to initialize, output status and return
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	CPDFAttachments pdfAttachments;

	//Get folder of input files for this example
	pdfAttachments.m_strInputFilesPath = FSDK_GetInputFilesFolder("pdfattachments");

	//Get output files folder path.
	pdfAttachments.m_strOutputFilesPath = FSDK_GetOutputFilesFolder("pdfattachments");

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(pdfAttachments.m_strOutputFilesPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Open folder to enumerate files
	void* folder = FSDK_OpenFolder(pdfAttachments.m_strInputFilesPath.c_str(), FALSE);
	if (folder)
	{
		FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfattachments: ");
		pdfAttachments.AttachmentsFolder(folder);
		//Close access to specific folder.
		FSDK_CloseFolder(folder);
	}
	else
	{
		pdfAttachments.m_strInputFilesPath = FSDK_GetFixFolder();
		pdfAttachments.m_strFile = "AttachMent.pdf";
		pdfAttachments.m_strInsertFile = "FoxitLogo.jpg";
		pdfAttachments.AttachmentsToFile();
		pdfAttachments.AddFileToAttachments(pdfAttachments.m_strFile, pdfAttachments.m_strInsertFile);
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0 ;
}

