/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a source to implement functions of getting and setting document information.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfdocinfo
#endif

//Set properties to viewer preferences
void SetViewerPref(FSCRT_DOCUMENT pdfDoc)
{
	FSCRT_BSTR strItem[6];
	FSCRT_BStr_InitConstString(strItem[0], "HideToolbar");
	FSCRT_BStr_InitConstString(strItem[1], "HideMenubar");
	FSCRT_BStr_InitConstString(strItem[2], "HideWindowUI");
	FSCRT_BStr_InitConstString(strItem[3], "FitWindow");
	FSCRT_BStr_InitConstString(strItem[4], "CenterWindow");
	FSCRT_BStr_InitConstString(strItem[5], "DisplayDocTitle");
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;

	int i = 0;
	for ( i = 0; i < 6; i++)
	{
		//Set UI visibility status to viewer preferences
		ret = FSPDF_ViewerPref_SetUIVisibility(pdfDoc, &strItem[i], TRUE);
		if (FSCRT_ERRCODE_SUCCESS == ret)
			FSDK_OutputLog("Success: Set visibility of \"%s\": TRUE.\r\n", strItem[i].str);
		else
			FSDK_OutputLog("Failure: Fail to set visibility of \"%s\": TRUE, with error code %d.\r\n", strItem[i].str, ret);
	}

	//Set display mode to use outlines for non full-screen mode to viewer preferences
	ret = FSPDF_ViewerPref_SetNonFullScreenPageMode(pdfDoc, FSPDF_DISPLAYMODE_USEOUTLINES);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Set page mode in non-full screen: UseOutlines.\r\n");
	else
		FSDK_OutputLog("Failure: Fail to set page mode in non-full screen: UseOutlines, with error code %d.\r\n", ret);

	//Set reading direction to viewer preferences
	ret = FSPDF_ViewerPref_SetReadingDirection(pdfDoc, FSPDF_READDIRECT_R2L);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Set reading order: R2L.\r\n");
	else
		FSDK_OutputLog("Failure: Fail to set reading order: R2L, with error code %d.\r\n", ret);

	FSCRT_BStr_InitConstString(strItem[0], "ViewArea");
	FSCRT_BStr_InitConstString(strItem[1], "ViewClip");
	FSCRT_BStr_InitConstString(strItem[2], "PrintArea");
	FSCRT_BStr_InitConstString(strItem[3], "PrintClip");

	for (i = 0; i < 4; i++)
	{
		//Set the type of area item to viewer preferences
		ret = FSPDF_ViewerPref_SetAreaType(pdfDoc, &strItem[i], FSPDF_PAGEBOX_CROPBOX);
		if (FSCRT_ERRCODE_SUCCESS == ret)
			FSDK_OutputLog("Success: Set \"%s\" value: CropBox.\r\n", strItem[i].str);
		else
			FSDK_OutputLog("Failure: Fail to set \"%s\" value: CropBox, with error code %d.\r\n", strItem[i].str, ret);
	}
	
	//Set page scaling option to viewer preferences
	FSCRT_BStr_InitConstString(strItem[0], "AppDefault");
	ret = FSPDF_ViewerPref_SetPrintScale(pdfDoc, &strItem[0]);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Set page scaling value: AppDefault.\r\n");
	else
		FSDK_OutputLog("Failure: Fail to set page scaling value: AppDefault, with error code %d.\r\n", ret);

	//Set the number of copies to be printed
	ret = FSPDF_ViewerPref_SetPrintCopies(pdfDoc, 3);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Set print copies: 3.\r\n");
	else
		FSDK_OutputLog("Failure: Fail to set print copies: 3, with error code %d.\r\n", ret);

	//Set page ranges which allowed to print
	FS_INT32 pagearr[2] = {1, 5};
	ret = FSPDF_ViewerPref_SetPrintRanges(pdfDoc, pagearr, 2);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Set page ranges for printing: 1-5.\r\n");
	else
		FSDK_OutputLog("Failure: Fail to set page ranges for printing: 1-5, with error code %d.\r\n", ret);
}

//Set values in metadata
void SetMetadata(FSCRT_DOCUMENT pdfDoc)
{
	FSCRT_BSTR strItem[6];
	FSCRT_BStr_InitConstString(strItem[0], "Title");
	FSCRT_BStr_InitConstString(strItem[1], "Author");
	FSCRT_BStr_InitConstString(strItem[2], "Subject");
	FSCRT_BStr_InitConstString(strItem[3], "Keywords");
	FSCRT_BStr_InitConstString(strItem[4], "Creator");
	FSCRT_BStr_InitConstString(strItem[5], "Producer");

	FSCRT_BSTR strVal[6];
	FSCRT_BStr_InitConstString(strVal[0], "Tit");
	FSCRT_BStr_InitConstString(strVal[1], "Aut");
	FSCRT_BStr_InitConstString(strVal[2], "Sub");
	FSCRT_BStr_InitConstString(strVal[3], "Key");
	FSCRT_BStr_InitConstString(strVal[4], "Cre");
	FSCRT_BStr_InitConstString(strVal[5], "Pro");
	
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	int i = 0;
	for (i = 0; i < 6; i++)
	{
		//Set the value in metadata
		ret = FSPDF_Metadata_SetString(pdfDoc, &strItem[i], &strVal[i]);
		if (FSCRT_ERRCODE_SUCCESS == ret)
			FSDK_OutputLog("Success: Set metadata \"%s\" value: %s.\r\n", strItem[i].str, strVal[i].str);
		else
			FSDK_OutputLog("Failure: Fail to set metadata\"%s\" value: %s, with error code %d.\r\n", strItem[i].str, strVal[i].str, ret);
	}


	FSCRT_BStr_InitConstString(strItem[0], "CreationDate");
	FSCRT_BStr_InitConstString(strItem[1], "ModDate");

	//Init and set date zone
	FSCRT_DATETIMEZONE datezone;
	memset(&datezone, 0, sizeof(FSCRT_DATETIMEZONE));
	datezone.day = 2;
	datezone.hour = 3;
	datezone.minute = 4;
	datezone.month = 5;
	datezone.second = 6;
	datezone.year = 2013;
	for (i = 0; i < 2; i++)
	{
		//Set date time: 2013.05.02 03:04:06
		ret = FSPDF_Metadata_SetDateTime(pdfDoc, &strItem[i], &datezone);
		if (FSCRT_ERRCODE_SUCCESS == ret)
			FSDK_OutputLog("Success: Set \"%s\" value: 2013/05/02 03:04:06.\r\n", strItem[i].str);
		else
			FSDK_OutputLog("Failure: Fail to set \"%s\" value: 2013/05/02 03:04:06, with error code %d.\r\n", strItem[i].str, ret);
	}
}

//Set information of viewer preferences and metadata
void SetDocInfo(string pdfName, string inputFilesPath, string outputFilesPath)
{
	string strLog("\r\n======Set Info======\r\n");
	strLog += (inputFilesPath + "/" + pdfName + "\r\n");
	FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
	
	//Open PDF file
	CAutoDocument autoDoc((inputFilesPath + "/" + pdfName).c_str(), "rb");
	if (!autoDoc.GetFileHander())
	{
		return;
	}

	//Load PDF document
	if (FSCRT_ERRCODE_SUCCESS != autoDoc.LoadPDFDocument(NULL))
		return;

	//Set UI visibility status from viewer preferences
	SetViewerPref(autoDoc.GetDocument());

	//Set Metadata
	SetMetadata(autoDoc.GetDocument());

	//Save PDF to output files folder
	FSDK_SavePDFFile(autoDoc.GetDocument(), (outputFilesPath + "/" + pdfName).c_str());
}

//Get properties from viewer preferences
void GetViewerPref(FSCRT_DOCUMENT pdfDoc)
{
	FSCRT_BSTR strItem[6];
	FSCRT_BStr_InitConstString(strItem[0], "HideToolbar");
	FSCRT_BStr_InitConstString(strItem[1], "HideMenubar");
	FSCRT_BStr_InitConstString(strItem[2], "HideWindowUI");
	FSCRT_BStr_InitConstString(strItem[3], "FitWindow");
	FSCRT_BStr_InitConstString(strItem[4], "CenterWindow");
	FSCRT_BStr_InitConstString(strItem[5], "DisplayDocTitle");

	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	int i = 0;
	for ( i = 0; i < 6; i++)
	{
		FS_BOOL bVisible = TRUE;
		//Get UI visibility status to viewer preferences
		ret = FSPDF_ViewerPref_GetUIVisibility(pdfDoc, &strItem[i], &bVisible);
		if (FSCRT_ERRCODE_SUCCESS == ret){
			string strLog = bVisible ? "TRUE" : "FALSE";
			FSDK_OutputLog("Success: Get visibility of \"%s\": %s.\r\n", strItem[i].str, strLog.c_str());
		} else
			FSDK_OutputLog("Failure: Fail to get visibility of \"%s\" with error code %d.\r\n", strItem[i].str, ret);
	}

	//Get display mode for non full-screen mode from viewer preferences
	FS_INT32 nMode = 0;
	ret = FSPDF_ViewerPref_GetNonFullScreenPageMode(pdfDoc, &nMode);
	if (FSCRT_ERRCODE_SUCCESS == ret){
		string strLog;
		switch (nMode)
		{
		case FSPDF_DISPLAYMODE_USENONE:
			strLog = "UseNone";
			break;
		case FSPDF_DISPLAYMODE_USEOUTLINES:
			strLog = "UseOutlines";
			break;
		case FSPDF_DISPLAYMODE_USETHUMBS:
			strLog = "UseThumbs";
			break;
		case FSPDF_DISPLAYMODE_FULLSCREEN:
			strLog = "FullScreen";
			break;
		case FSPDF_DISPLAYMODE_USEOC:
			strLog = "UseOC";
			break;
		case FSPDF_DISPLAYMODE_USEATTACHMENTS:
			strLog = "UseAttachments";
			break;
		}
		FSDK_OutputLog("Success: Get page mode in non-full screen: %s.\r\n", strLog.c_str());
	}else
		FSDK_OutputLog("Failure: Fail to get page mode in non-full screen with error code %d.\r\n", ret);

	//Get reading direction from viewer preferences
	FS_INT32 nDir= 0;
	ret = FSPDF_ViewerPref_GetReadingDirection(pdfDoc, &nDir);
	if (FSCRT_ERRCODE_SUCCESS == ret){
		string strLog = "";
		switch (nDir)
		{
		case FSPDF_READDIRECT_L2R:
			strLog = "L2R";
			break;
		case FSPDF_READDIRECT_R2L:
			strLog = "R2L";
			break;
		}
		FSDK_OutputLog("Success: Get reading order: %s.\r\n", strLog.c_str());
	}
	else
		FSDK_OutputLog("Failure: Fail to get reading order with error code %d.\r\n", ret);

	FSCRT_BStr_InitConstString(strItem[0], "ViewArea");
	FSCRT_BStr_InitConstString(strItem[1], "ViewClip");
	FSCRT_BStr_InitConstString(strItem[2], "PrintArea");
	FSCRT_BStr_InitConstString(strItem[3], "PrintClip");

	for (i = 0; i < 4; i++)
	{
		FS_INT32 nType = 0;
		//Get the type of area item to viewer preferences
		FSPDF_ViewerPref_GetAreaType(pdfDoc, &strItem[i], &nType);
		if (FSCRT_ERRCODE_SUCCESS == ret){
			string strLog;
			switch (nMode)
			{
			case FSPDF_PAGEBOX_MEDIABOX:
				strLog = "MediaBox";
				break;
			case FSPDF_PAGEBOX_CROPBOX:
				strLog = "CropBox";
				break;
			case FSPDF_PAGEBOX_TRIMBOX:
				strLog = "TrimBox";
				break;
			case FSPDF_PAGEBOX_ARTBOX:
				strLog = "ArtBox";
				break;
			case FSPDF_PAGEBOX_BLEEDBOX:
				strLog = "BleedBox";
				break;
			}
			FSDK_OutputLog("Success: Get \"%s\" value: %s.\r\n", strItem[i].str, strLog.c_str());
		}
		else
			FSDK_OutputLog("Failure: Fail to get \"%s\" value with error code %d.\r\n", strItem[i].str, ret);
	}


	//Get page scaling option from viewer preferences
	FSCRT_BSTR strScaling;
	FSCRT_BStr_Init(&strScaling);
	ret = FSPDF_ViewerPref_GetPrintScale(pdfDoc, &strScaling);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Get page scaling value: %s.\r\n", strScaling.str);
	else
		FSDK_OutputLog("Failure: Fail to get page scaling value with error code %d.\r\n", ret);
	FSCRT_BStr_Clear(&strScaling);

	//Get the number of copies to be printed
	FS_INT32 nCopies = 0;
	ret = FSPDF_ViewerPref_GetPrintCopies(pdfDoc, &nCopies);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Get print copies: %d.\r\n",nCopies);
	else
		FSDK_OutputLog("Failure: Fail to get print copies with error code %d.\r\n", ret);


	//Get page ranges which allowed to print
	FS_INT32 nCount = 0;
	ret = FSPDF_ViewerPref_GetPrintRanges(pdfDoc, NULL, &nCount);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		FS_INT32* pRange = NULL;
		FSCRT_Memory_Alloc(sizeof(FS_INT32) * nCount, (FS_LPVOID*)&pRange);
		ret = FSPDF_ViewerPref_GetPrintRanges(pdfDoc, pRange, &nCount);
		if (FSCRT_ERRCODE_SUCCESS == ret)
		{
			FSDK_OutputLog("Success: Get page ranges for printing: ");
			//In this array, 2 numbers are a pair: the first integer is the starting page number, and the second integer is the last page number
			for (int i = 0; i < nCount; i+=2)
			{
				if(i != 0)
					FSDK_OutputLog(", ");
 				FSDK_OutputLog("%d-%d", pRange[i], pRange[i+1]);
			}
			FSDK_OutputLog(".\r\n");
		}
		else
			FSDK_OutputLog("Failure: Fail to get page ranges for printing with error code %d.\r\n", ret);
		FSCRT_Memory_Free(pRange);
	}
}

//Get values from metadata
void GetMetadata(FSCRT_DOCUMENT pdfDoc )
{
	FSCRT_BSTR strItem[6];
	FSCRT_BStr_InitConstString(strItem[0], "Title");
	FSCRT_BStr_InitConstString(strItem[1], "Author");
	FSCRT_BStr_InitConstString(strItem[2], "Subject");
	FSCRT_BStr_InitConstString(strItem[3], "Keywords");
	FSCRT_BStr_InitConstString(strItem[4], "Creator");
	FSCRT_BStr_InitConstString(strItem[5], "Producer");

	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	int i = 0;
	for ( i = 0; i < 6; i++)
	{
		FSCRT_BSTR strMeta;
		FSCRT_BStr_Init(&strMeta);
		//Get the value in metadata
		ret = FSPDF_Metadata_GetString(pdfDoc, &strItem[i], &strMeta);
		if (FSCRT_ERRCODE_SUCCESS == ret)
			FSDK_OutputLog("Success: Get metadata \"%s\" value: %s.\r\n", strItem[i].str, strMeta.str);
		else
			FSDK_OutputLog("Failure: Fail to get metadata\"%s\" value with error code %d.\r\n", strItem[i].str, ret);
		FSCRT_BStr_Clear(&strMeta);
	}
	
	FSCRT_BStr_InitConstString(strItem[0], "CreationDate");
	FSCRT_BStr_InitConstString(strItem[1], "ModDate");
	for (i = 0; i < 2; i++)
	{
		//Init data zone
		FSCRT_DATETIMEZONE datazone;
		memset(&datazone, 0, sizeof(FSCRT_DATETIMEZONE));
		//Get date time
		ret = FSPDF_Metadata_GetDateTime(pdfDoc, &strItem[i], &datazone);
		if (FSCRT_ERRCODE_SUCCESS == ret)
			FSDK_OutputLog("Success: Get \"%s\" value: %d/%d/%d %d:%d:%d.\r\n", strItem[i].str, 
						datazone.year, datazone.month, datazone.day, datazone.hour, datazone.minute, datazone.second);
		else
			FSDK_OutputLog("Failure: Fail to get \"%s\" value with error code %d.\r\n", strItem[i].str, ret);
	}
}

//Set information from viewer preferences and metadata
void GetDocInfo(string pdfName, string outputFilesPath)
{
	//Write log
	string strLog("\r\n======Get Info======\r\n");
	strLog += (outputFilesPath + "/" + pdfName + "\r\n");

	//Output log string to log.txt in output file folder
	FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
	//Open PDF file
	CAutoDocument autoDoc((outputFilesPath + "/" + pdfName).c_str(), "rb");
	if (!autoDoc.GetFileHander())
	{
		return;
	}

	//load document.
	if (FSCRT_ERRCODE_SUCCESS != autoDoc.LoadPDFDocument(NULL))
		return;

	//Get UI visibility status from viewer preferences
	GetViewerPref(autoDoc.GetDocument());

	//Get Metadata String
	GetMetadata(autoDoc.GetDocument());
}

void SetGetFolderDocInfo(void* folder, string inputFilesPath, string outputFilesPath)
{
	do
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if (strFile.size() > 0 && !isDir)
		{
			if (FSDK_CompareExtName(strFile.c_str(), ".pdf"))
			{
				//Set document information
				SetDocInfo(strFile, inputFilesPath, outputFilesPath);
				//Get document information
				GetDocInfo(strFile, outputFilesPath);
			}
		}

		if (strFile.size() < 1) break;

	} while (1);
}

int main(int argc, char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);
	string inputSrcFile = "";
	string outputFolder = "";
	//Judge if to command line arguments. 
	//usage: pdfdocinfo <srcfile> /o [destfolder]
	if (argc > 1)
	{
		inputSrcFile = argv[1];
		bool bArgumentValid = true;
		if (argc >= 3)
		{
			if (0 != strcmp(argv[2], "/o"))
				bArgumentValid = false;
			else if (argc > 3)
				outputFolder = argv[3];
		}
		if (false == bArgumentValid || !FSDK_FileExist(inputSrcFile.c_str()) || (outputFolder != "" && !FSDK_FolderExist(outputFolder.c_str())))
		{
			printf("\r\n");
			printf("usage: pdfdocinfo [<srcfile> /o [destfolder]] \r\n\r\n");
			printf("<srcfile>\t: Path of an image file as input file.\r\n");
			printf("/o [destfolder]\t: Path of a folder as output folder. If not set, use default\r\n\t\t  output folder.\r\n\r\n");
			return -1;
		}
	}

	string logPath ;
	if (outputFolder != "")
	{
		outputFolder += "/pdfdocinfo";
		FSDK_CreatFolder(outputFolder.c_str());
		logPath = outputFolder;
		logPath += "/log.txt";
	}
	else
	{
		outputFolder = FSDK_GetOutputFilesFolder("pdfdocinfo");
		logPath = FSDK_GetLogFile("pdfdocinfo");
	}
	
	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfdocinfo\r\n\r\n");

	//Initialize SDK library, or no method of SDK can be used
	bool bRet = FSDK_InitializeLibray(false);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module, or no method of PDF module can be used
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	string inputFolder;
	string pdfName;
	if (inputSrcFile != "")
	{
		pdfName = FSDK_GetFileName(inputSrcFile.c_str());
		inputFolder = FSDK_GetCustomInputPath(inputSrcFile.c_str());
		FSDK_OutputLog("Input from folder %s\r\n", inputFolder.c_str());
		FSDK_OutputLog("Output to folder %s \r\n\r\n", outputFolder.c_str());
		//Set document information
		SetDocInfo(pdfName, inputFolder, outputFolder);
		//Get document information
		GetDocInfo(pdfName, outputFolder);
	}
	else
	{
		//Get path of input file folder for this demo
		inputFolder = FSDK_GetInputFilesFolder("pdfdocinfo");
		void* folder = FSDK_OpenFolder(inputFolder.c_str(), FALSE);
		//Get output files folder path.
		if (folder)
		{
			FSDK_OutputLog("Input from folder %s\r\n", inputFolder.c_str());
			FSDK_OutputLog("Output to folder %s \r\n\r\n", outputFolder.c_str());
			FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfdocinfo: ");
			SetGetFolderDocInfo(folder, inputFolder, outputFolder);
			//Close access to specific folder.
			FSDK_CloseFolder(folder);
		}
		else
		{
			pdfName = "PageLabels.pdf";
			inputFolder = FSDK_GetFixFolder();
			FSDK_OutputLog("Input from folder %s\r\n", inputFolder.c_str());
			FSDK_OutputLog("Output to folder %s \r\n\r\n", outputFolder.c_str());
			//Set document information
			SetDocInfo(pdfName, inputFolder, outputFolder);
			//Get document information
			GetDocInfo(pdfName, outputFolder);
		}
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}
