/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This demo illustrates how to use Foxit PDF SDK to set password encryption, custom  encryption,  Foxit  DRM  encryption, 
 * certification  encryption  and user permission encryption for a PDF file. In addition, PDF decrypting will be included. 
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfencrypt
#endif

#if defined(_WIN32)|| defined(_WIN64)
#include "win32_crypt.h"
#include <wincrypt.h>
#else
#include "openssl/sha.h"
#include "openssl/pkcs7.h"
#include "openssl/objects.h"
#include "openssl/x509.h"
#include "openssl/bio.h"
#include "openssl/pem.h"
#include "openssl/pkcs12.h"
#include "openssl/evp.h"
#endif

#define PSL_PERMISSION_LENGTH	4
#define PSL_SEEDBUF_LENGTH		PSL_SEED_LENGTH+PSL_PERMISSION_LENGTH

//The encryptType
#define ENCRYPTIONTYPE_PASSWORD		0
#define ENCRYPTIONTYPE_CUSTOM		1
#define ENCRYPTIONTYPE_CERTIFICATE	2
#define ENCRYPTIONTYPE_FOXITDRM		3

class CPDFEncrypt
{
public:
	CPDFEncrypt();
	~CPDFEncrypt();
	void PDFEncryptFolder(void* folder, string inputFilesPath, string outputFilesPath);
	void PDFEncrypt(string fileName, string inputFilesPath, string outputFilesPath, int encryptType);
	FS_RESULT PasswrodEncryption(FSCRT_DOCUMENT pdfDoc, FSCRT_FILE encryptedFile);
	FS_RESULT CustomEncryption(FSCRT_DOCUMENT pdfDoc, FSCRT_FILE encryptedFile);
	FS_RESULT CertificateEncryption(FSCRT_DOCUMENT pdfDoc, FSCRT_FILE encryptedFile,string inputFilesPath);
	FS_RESULT FoxitDRMEncryption(FSCRT_DOCUMENT pdfDoc, FSCRT_FILE encryptedFile);

	void RemovePassword(string fileName, string inputFilesPath, string outputFilesPath, string password);
	void RemoveCustomEncryption(string fileName, string inputFilesPath, string outputFilesPath);
	void RemoveFoxitDRMEncryption(string fileName, string inputFilesPath, string outputFilesPath);
	FS_RESULT RemoveCertificateEncryption(string fileName, string inputFilesPath, string outputFilesPath);
public:
	string m_strInputSrcFile;
	string m_strInputFolder;
	string m_strOutputFolder;
	string m_strPdfFileName;

};

CPDFEncrypt::CPDFEncrypt()
	:m_strInputSrcFile("")
	,m_strInputFolder("")
	,m_strOutputFolder("")
	,m_strPdfFileName("")
{

}

CPDFEncrypt::~CPDFEncrypt()
{

}

static FS_RESULT     GetDecryptionKey(FS_LPVOID clientData,FS_LPCVOID envelopeBuf,FS_DWORD envelopeLen, FS_LPVOID keyBuf, FS_DWORD* keyLen)
#if defined(_WIN32)|| defined(_WIN64)
{
	CPSL_CryptLib cryptLib;

	HCRYPTMSG hMsg = cryptLib.CryptMsgOpenToDecode(0x00010000, 0, 0, 0, NULL, NULL);

	BOOL ret = cryptLib.CryptMsgUpdate(hMsg, (const unsigned char*)envelopeBuf, envelopeLen, TRUE);
	if (!ret)
	{
		cryptLib.CryptMsgClose(hMsg);
		return FALSE;
	}

	void* hStore = cryptLib.CertOpenSystemStoreA(NULL, "MY");
	if (hStore == NULL) return FALSE;

	// Go through all recipients on the envelop and check if any of those match local cert
	DWORD count = 0;
	envelopeLen = 4;
	ret = cryptLib.CryptMsgGetParam(hMsg, CMSG_RECIPIENT_COUNT_PARAM, 0, &count, (DWORD*)&envelopeLen);
	if (!ret)
	{
		cryptLib.CryptMsgClose(hMsg);
		cryptLib.CertCloseStore(hStore,0);
		return FALSE;
	}

	for (DWORD index = 0; index < count; index ++) {
		ret = cryptLib.CryptMsgGetParam(hMsg, CMSG_RECIPIENT_INFO_PARAM, index, NULL, (DWORD*)&envelopeLen);
		if (!ret) continue;
		CERT_INFO* cert_info = (CERT_INFO*)malloc(envelopeLen);
		ret = cryptLib.CryptMsgGetParam(hMsg, CMSG_RECIPIENT_INFO_PARAM, index, cert_info, (DWORD*)&envelopeLen);
		if (!ret) continue;

		PCCERT_CONTEXT cert_context = cryptLib.CertGetSubjectCertificateFromStore(hStore, 0x00000001|0x00010000, cert_info);
		free(cert_info);
		if (cert_context == NULL) continue;

		struct {
			DWORD size;
			HCRYPTPROV hcp;
			DWORD keyspec;
			DWORD index;
		} para;
		para.size = sizeof(para);
		para.index = index;

		BOOL b;
		ret = cryptLib.CryptAcquireCertificatePrivateKey(cert_context, 0, NULL, &para.hcp, &para.keyspec, &b);
		if (!ret)
		{
			cryptLib.CertFreeCertificateContext(cert_context);
			continue;
		}

		// Decrypt the message
		ret = cryptLib.CryptMsgControl(hMsg, 0, 2, &para);
		if (!ret)
		{
			cryptLib.CryptReleaseContext((HCRYPTPROV)para.hcp, 0);
			cryptLib.CertFreeCertificateContext(cert_context);
			continue;
		}

		*keyLen = 24;
		ret = cryptLib.CryptMsgGetParam(hMsg, 2, 0, keyBuf, (DWORD*)keyLen);
		cryptLib.CryptReleaseContext((HCRYPTPROV)para.hcp, 0);
		cryptLib.CertFreeCertificateContext(cert_context);
		if (!ret) continue;
		break;
	}
	cryptLib.CryptMsgClose(hMsg);
	cryptLib.CertCloseStore(hStore,0);
	if (ret)
		return FSCRT_ERRCODE_SUCCESS;
	else 
		return FSCRT_ERRCODE_ERROR;
}
#else
{
	if (!keyBuf && keyLen) *keyLen = 0;
	if (!clientData || !envelopeBuf || envelopeLen < 1 || !keyLen || (keyBuf && *keyLen < 1))
		return FSCRT_ERRCODE_PARAM;

	FS_LPSTR pfxFile = (FS_LPSTR)clientData;
	FS_LPSTR password = "123456";
	//Initialize the envrionment of the openssl. 
	OpenSSL_add_all_algorithms();  
	ERR_load_BIO_strings(); 

	//Open the certificate file.
	FILE* fp = NULL;
	if (!(fp = fopen(pfxFile, "rb")))   
		return FSCRT_ERRCODE_ERROR;

	//Retain PKCS12 object from certificate file.
	PKCS12* p12 = d2i_PKCS12_fp(fp, NULL);      
	fclose (fp);      
	if (!p12) return FSCRT_ERRCODE_ERROR;

	//Retain pkey, X509 certificate, certificate chains.  
	EVP_PKEY* pkey = NULL;       
	X509* x509 = NULL;  
	STACK_OF(X509) *ca = NULL;  
	if (!PKCS12_parse(p12, password, &pkey, &x509, &ca))
		return FSCRT_ERRCODE_ERROR;

	PKCS7* p7 = d2i_PKCS7(NULL, (const unsigned char**)&envelopeBuf, envelopeLen);
	BIO* p7bio = PKCS7_dataDecode(p7, pkey, NULL, x509);
	if (!p7bio) return FSCRT_ERRCODE_ERROR;

	*keyLen = (int)BIO_ctrl_pending(p7bio);
	if (*keyLen <= 0)
	{
		BIO_free(p7bio);
		return FSCRT_ERRCODE_ERROR;
	}

	if (!keyBuf)
	{
		BIO_free(p7bio);
		PKCS7_free(p7);
		return FSCRT_ERRCODE_SUCCESS;
	}

	FS_BYTE* buf = (FS_BYTE*)malloc(*keyLen);
	*keyLen = BIO_read(p7bio, buf, *keyLen);
	memcpy(keyBuf, buf, *keyLen);
	free(buf);

	BIO_free(p7bio);
	PKCS7_free(p7);

	return FSCRT_ERRCODE_SUCCESS;
}
#endif

static FS_RESULT GetDecryptionParams(FS_LPVOID clientData, FSCRT_FILE file, const FSCRT_BSTR* subFilter, FS_BOOL* isOwner, FS_DWORD* userPermissions, FS_INT32* cipher, FS_INT32* keyLen, FSCRT_BSTR* fileID, FSCRT_BSTR* initialKey)
{
	*isOwner = TRUE;
	*userPermissions = FSPDF_PERMISSION_PRINT;
	*cipher = FSCRT_CIPHER_AES;
	*keyLen = 16;
	FSCRT_BStr_Set(fileID, "fileID", strlen("fileID"));
	FSCRT_BStr_Set(initialKey, "initialKey", strlen("initialKey"));
	return FSCRT_ERRCODE_SUCCESS;
}

static FS_RESULT CreateContext(FS_LPVOID clientData, FSCRT_FILE file, const FSCRT_BSTR* filter, const FSCRT_BSTR* subFilter, const FSCRT_BSTR* encryptInfo, FS_LPVOID* context)
{
	//Sometimes, maybe only necessary parameters need to be checked.
	if (!file || !filter || !filter->str || 0 == filter->len || !encryptInfo )
		return FSCRT_ERRCODE_ERROR;
	//Context could not be NULL!!!!
	*context = (FS_LPVOID)1;
	return FSCRT_ERRCODE_SUCCESS;
}

static FS_RESULT ReleaseContext(FS_LPVOID clientData, FS_LPVOID context)
{
	if (!context)
		return FSCRT_ERRCODE_ERROR;
	return FSCRT_ERRCODE_SUCCESS;
}

static FS_RESULT GetUserPermissions(FS_LPVOID clientData, FS_LPVOID context, FS_DWORD userPermission, FS_DWORD* newPermission)
{
	*newPermission = FSPDF_PERMISSION_PRINT;
	return FSCRT_ERRCODE_SUCCESS;
}

static FS_RESULT IsOwner(FS_LPVOID clientData, FS_LPVOID context, FS_BOOL* isOwner)
{
	*isOwner = TRUE;
	return FSCRT_ERRCODE_SUCCESS;
}

static FS_RESULT GetCipherInfo(FS_LPVOID clientData, FS_LPVOID context, FS_INT32* cipher, FS_LPVOID keyBuf, FS_DWORD* keyLen)
{
	//No need for RMS encryption.
	if (!context || !cipher || !keyLen)
		return FSCRT_ERRCODE_ERROR;

	return FSCRT_ERRCODE_ERROR;
}

static FS_RESULT GetDecryptedSize(FS_LPVOID clientData, FS_LPVOID context, FS_DWORD dataSize, FS_DWORD* decryptedSize)
{
	*decryptedSize = 0;
	if (!context)
	{
		return FSCRT_ERRCODE_SUCCESS;
	}

	return FSCRT_ERRCODE_SUCCESS;
}

static FS_RESULT StartDecryptor(FS_LPVOID clientData, FS_LPVOID context, FS_INT32 objNum, FS_INT32 genNum, FS_LPVOID* decryptor)
{
	if (!context)
		return FSCRT_ERRCODE_ERROR;

	*decryptor = (FS_LPVOID)1;
	return FSCRT_ERRCODE_SUCCESS;
}

static FS_RESULT DecryptData(FS_LPVOID clientData, FS_LPVOID decryptor, FS_LPCVOID encDataBuf, FS_DWORD encDataLen, FS_LPVOID decDataBuf, FS_DWORD* decDataLen)
{
	if (!decryptor || !encDataBuf || !decDataLen)
		return FSCRT_ERRCODE_ERROR;
	FS_DWORD i=0;
	if(!decDataBuf)
	{
		*decDataLen = encDataLen;
		return FSCRT_ERRCODE_SUCCESS;
	}
	FS_BYTE* dec_data = (FS_BYTE*)malloc(encDataLen);
	while(i<encDataLen)
	{
		dec_data[i] = ((FS_BYTE*)encDataBuf)[i]+1;
		i++;
	}
	memcpy(decDataBuf, dec_data, *decDataLen);
	free(dec_data);

	return FSCRT_ERRCODE_SUCCESS;
}

static FS_RESULT FinishDecryptor(FS_LPVOID clientData, FS_LPVOID decryptor, FS_LPVOID decDataBuf, FS_DWORD* decDataLen)
{

	if (!decryptor /*|| !decDataBuf || !decDataLen*/)
		return FSCRT_ERRCODE_ERROR;

	return FSCRT_ERRCODE_SUCCESS;
}
static FS_RESULT GetEncryptedSize(FS_LPVOID clientData, FS_LPVOID context, FS_INT32 objNum, FS_INT32 genNum, FS_LPCVOID srcBuf, FS_DWORD srcLen, FS_DWORD* encryptedSize)
{
	if (!context || !srcBuf || !srcLen)
		return FSCRT_ERRCODE_ERROR;
	*encryptedSize = srcLen;
	return FSCRT_ERRCODE_SUCCESS;
}

static FS_RESULT EncryptData(FS_LPVOID clientData, FS_LPVOID context, FS_INT32 objNum, FS_INT32 genNum, FS_LPCVOID srcBuf, FS_DWORD srcLen, FS_LPVOID dstBuf, FS_DWORD* dstLen)
{
	if (!context || !srcBuf || !dstLen)
		return FSCRT_ERRCODE_ERROR;
	if (NULL == dstBuf)
	{
		*dstLen = srcLen;
		return FSCRT_ERRCODE_SUCCESS;
	}
	FS_DWORD i = 0;
	while(i < srcLen)
	{
		((FS_BYTE*)dstBuf)[i] = ((FS_BYTE*)srcBuf)[i]-1;
		i++;
	}
	*dstLen = srcLen;
	return FSCRT_ERRCODE_SUCCESS;
}

#if defined(_WIN32)|| defined(_WIN64)
void GetPDFKeyFromEnvelopes(CPSL_CryptLib* pCryptLib, FSCRT_BSTR* pEnvelopes, FS_INT32 envelopesCount, const FS_BYTE* pSeed, 
	FS_BOOL bEncryptMeta, FS_BYTE* pPDFKey)
{
	if(!pCryptLib) return;
	// calculate encryption key
	HCRYPTPROV hcp = NULL;

	// Acquire Context http://support.microsoft.com/kb/238187/zh-cn
	//pCryptLib->CryptAcquireContextA()
	if (!pCryptLib->CryptAcquireContextA(&hcp, 
		"Container", 
		NULL, 
		PROV_RSA_FULL, 0))
	{
		if (GetLastError() == NTE_BAD_KEYSET)
		{
			if (!pCryptLib->CryptAcquireContextA(&hcp, 
				("Container"), 
				NULL, 
				PROV_RSA_FULL, 
				CRYPT_NEWKEYSET))
			{
				// Error ...
				DWORD dwErrorCode = GetLastError();
				//				TRACE(_T("\nERROR CODE: %d\n") , dwErrorCode);
			}
		}
	}

	if (!hcp) return;

	HCRYPTHASH hash = NULL;
	pCryptLib->CryptCreateHash(hcp, (4 << 13) | 4, 0, 0, &hash);

	pCryptLib->CryptHashData(hash, pSeed, 20, 0);

	for (FS_INT32 i=0, sz=envelopesCount; i<sz; i++) 
	{
		pCryptLib->CryptHashData(hash, (BYTE*)pEnvelopes->str, pEnvelopes->len, 0);
	}

	if (!bEncryptMeta)
	{
		FS_DWORD tag = 0xffffffff;
		pCryptLib->CryptHashData(hash, (const BYTE*)&tag, 4, 0); 
	}


	DWORD size = 20;
	pCryptLib->CryptGetHashParam(hash, 2, pPDFKey, &size, 0);

	pCryptLib->CryptDestroyHash(hash);
	pCryptLib->CryptReleaseContext(hcp, 0);
}


FS_BOOL MakeEnvelope(CPSL_CryptLib* pCryptLib, PCERT_INFO* pCertArray, 
	int nCertCount, FS_DWORD nPermission, FS_BYTE* pSeed, FS_BYTE*& pbEncodedBlob, FS_DWORD& cbEncodedBlob)
{
	if(!pCryptLib) return false;

	HCRYPTPROV hcp = NULL;
	FS_BOOL bRet = pCryptLib->CryptAcquireContextA(&hcp, NULL, NULL, 1, 0);
	if(!bRet) return false;

	//-------------------------------------------------------------------
	// Initialize the symmetric-encryption algorithm identifier
	// structure.
	CRYPT_ALGORITHM_IDENTIFIER ContentEncryptAlgorithm;
	DWORD ContentEncryptAlgSize = sizeof(ContentEncryptAlgorithm);
	memset(&ContentEncryptAlgorithm, 0, ContentEncryptAlgSize); 

	//-------------------------------------------------------------------
	// Initialize the necessary members. This particular OID does not
	// need any parameters. Some OIDs, however, will require that
	// the other members be initialized.
	ContentEncryptAlgorithm.pszObjId = szOID_RSA_RC4;

	//-------------------------------------------------------------------
	// Initialize the CMSG_ENVELOPED_ENCODE_INFO structure.
	CMSG_ENVELOPED_ENCODE_INFO EnvelopedEncodeInfo;
	memset(&EnvelopedEncodeInfo, 0,sizeof(CMSG_ENVELOPED_ENCODE_INFO));
	EnvelopedEncodeInfo.cbSize = sizeof(CMSG_ENVELOPED_ENCODE_INFO);
	EnvelopedEncodeInfo.hCryptProv = (HCRYPTPROV)hcp;
	EnvelopedEncodeInfo.ContentEncryptionAlgorithm = ContentEncryptAlgorithm;
	EnvelopedEncodeInfo.pvEncryptionAuxInfo = NULL;
	EnvelopedEncodeInfo.cRecipients = nCertCount;
	EnvelopedEncodeInfo.rgpRecipients = pCertArray;

	//-------------------------------------------------------------------
	// Get the size of the encoded message BLOB.
	cbEncodedBlob = pCryptLib->CryptMsgCalculateEncodedLength(
		PSL_ENCODING_TYPE,  // message encoding type
		0,                       // flags
		CMSG_ENVELOPED,          // message type
		&EnvelopedEncodeInfo,	// pointer to structure
		NULL,				    // inner content OID
		24);           // size of content
	if (cbEncodedBlob == 0)
	{
		pCryptLib->CryptReleaseContext(hcp, 0);
		return false;
	}

	//-------------------------------------------------------------------
	// Allocate memory for the encoded BLOB.
	pbEncodedBlob = (FS_BYTE*)malloc(cbEncodedBlob);
	if (!pbEncodedBlob) return false;

	//-------------------------------------------------------------------
	// Open a message to encode.

	HCRYPTMSG hMsg = pCryptLib->CryptMsgOpenToEncode(
		PSL_ENCODING_TYPE,        // encoding type
		0,                       // flags
		CMSG_ENVELOPED,          // message type
		&EnvelopedEncodeInfo,    // pointer to structure
		NULL,						// inner content OID
		NULL);                // stream information (not used)

	if (hMsg == NULL) 
	{
		pCryptLib->CryptReleaseContext(hcp, 0);
		free(pbEncodedBlob);
		return false;
	}	

	//---------------------------------------------------------------
	//Data to be Encrypted
	FS_BYTE			szSeedBuf[PSL_SEEDBUF_LENGTH] = {0};
	memcpy( szSeedBuf, pSeed, PSL_SEED_LENGTH);

	*(szSeedBuf+PSL_SEED_LENGTH) = HIBYTE(HIWORD(nPermission));
	*(szSeedBuf+PSL_SEED_LENGTH+1) = LOBYTE(HIWORD(nPermission));
	*(szSeedBuf+PSL_SEED_LENGTH+2) = HIBYTE(LOWORD(nPermission));

	if (nPermission == PSL_ALL_PERMISSION)
		*(szSeedBuf+PSL_SEED_LENGTH+3) = LOBYTE(LOWORD(nPermission)) | 0x03;
	else
		*(szSeedBuf+PSL_SEED_LENGTH+3) = LOBYTE(LOWORD(nPermission)) | 0x01;


	int lasterr = GetLastError();
	//-------------------------------------------------------------------
	// Update the message with the data.
	bRet = pCryptLib->CryptMsgUpdate(hMsg, szSeedBuf, PSL_SEEDBUF_LENGTH, TRUE);
	if (!bRet)
	{
		pCryptLib->CryptMsgClose(hMsg);
		pCryptLib->CryptReleaseContext(hcp, 0);
		free(pbEncodedBlob);
		return false;
	}

	//-------------------------------------------------------------------
	// Get the resulting message.
	bRet = pCryptLib->CryptMsgGetParam(hMsg, CMSG_CONTENT_PARAM, 0, 
		pbEncodedBlob, (DWORD*)&cbEncodedBlob);

	if (!bRet)
	{
		pCryptLib->CryptMsgClose(hMsg);
		pCryptLib->CryptReleaseContext(hcp, 0);
		free(pbEncodedBlob);
		return false;
	}

	//-------------------------------------------------------------------
	// Clean up.
	pCryptLib->CryptMsgClose(hMsg);
	pCryptLib->CryptReleaseContext(hcp, 0);

	return true;
}


void CreateSeed(CPSL_CryptLib* pCryptLib, FS_BYTE* pSeed)
{
	HCRYPTPROV hcp = NULL;
	pCryptLib->CryptAcquireContextA(&hcp, NULL, NULL, 1, 0);

	DWORD dwErrCode = GetLastError();
	if (dwErrCode == NTE_BAD_KEYSET)
	{
		pCryptLib->CryptAcquireContextA(&hcp , NULL , NULL , 1 , CRYPT_DELETEKEYSET);
		pCryptLib->CryptAcquireContextA(&hcp , NULL , NULL , 1 , CRYPT_NEWKEYSET);
	}

	if (hcp)
	{
		pCryptLib->CryptGenRandom(hcp, PSL_SEED_LENGTH, pSeed);
		pCryptLib->CryptReleaseContext(hcp, 0);
	}
}

FS_BOOL GetCertInfo(FSCRT_BSTR* envelopes, FSCRT_BSTR* initialKey, FS_BOOL bEncryptMetaData, FS_INT32 len, string inputFilesPath)
{
	string cerFile = inputFilesPath + "/" + "foxit.cer";
	FILE* fpCert = fopen(cerFile.c_str(), "rb");
	if (!fpCert)
	{
		printf("Open certificate PDF file failed.\r\n");
		return FALSE;
	}
	fseek(fpCert, 0, SEEK_END);
	long cbData =  ftell(fpCert);
	FS_BYTE* pbData = new FS_BYTE[cbData];
	fseek(fpCert, 0, SEEK_SET);
	fread(pbData, sizeof(char), cbData, fpCert);
	fclose(fpCert);

	CPSL_CryptLib cryptLib;

	PCCERT_CONTEXT pTempCertContext = cryptLib.CertCreateCertificateContext(PSL_ENCODING_TYPE, pbData, cbData);
	if (!pTempCertContext) return FALSE;

	BYTE usage;
	cryptLib.CertGetIntendedKeyUsage(pTempCertContext->dwCertEncodingType, pTempCertContext->pCertInfo ,&usage, 1);
	if ((usage & CERT_DATA_ENCIPHERMENT_KEY_USAGE) != CERT_DATA_ENCIPHERMENT_KEY_USAGE && usage != 0)
	{
		cryptLib.CertFreeCertificateContext(pTempCertContext);
		return FALSE;
	}
	SYSTEMTIME st;
	FILETIME crtTime;
	::GetLocalTime(&st);
	::SystemTimeToFileTime(&st, &crtTime);
	if (::CompareFileTime(&crtTime, &(pTempCertContext->pCertInfo->NotBefore)) < 0 ||
		::CompareFileTime(&crtTime, &(pTempCertContext->pCertInfo->NotAfter)) > 0)
	{
		cryptLib.CertFreeCertificateContext(pTempCertContext);
		return FALSE;
	}


	FS_BYTE Seed[PSL_SEED_LENGTH];
	CreateSeed(&cryptLib, Seed);

	FS_BYTE* pbEncodedBlob = NULL; FS_DWORD cbEncodedBlob = 0;
	PCERT_INFO pCertInfo = pTempCertContext->pCertInfo;
	if(!MakeEnvelope(&cryptLib, &pCertInfo, 1, 0xfffff8c4, Seed, pbEncodedBlob, cbEncodedBlob))
		return FALSE;
	cryptLib.CertFreeCertificateContext(pTempCertContext);
	//FSCRT_BSTRC(tempEnvolops, (FS_LPSTR)pbEncodedBlob);
	FSCRT_BSTR tempEnvolops;
	FSCRT_BStr_Init(&tempEnvolops);
	FSCRT_BStr_SetLength(&tempEnvolops, cbEncodedBlob);
	memcpy(tempEnvolops.str, pbEncodedBlob, cbEncodedBlob);
	*envelopes = tempEnvolops;
	FS_BYTE PDFKey[128];	
	GetPDFKeyFromEnvelopes(&cryptLib, &tempEnvolops, 1, Seed, bEncryptMetaData, PDFKey);
	//FSCRT_BSTRC(key, (FS_LPSTR)PDFKey);
	FSCRT_BSTR key;
	FSCRT_BStr_Init(&key);
	FSCRT_BStr_SetLength(&key, len);
	memcpy(key.str, PDFKey, len);
	*initialKey = key;
	free(pbEncodedBlob);
	free(pbData);
	return TRUE;
}
#endif

FS_RESULT CPDFEncrypt::RemoveCertificateEncryption(string fileName, string inputFilesPath, string outputFilesPath)
{
	//Begin, output log 
	FSDK_OutputLog("\r\n====Remove Certificate Encryption====\r\n");
	unsigned int nStartTime = FSDK_GetClock();//Get current time
	//Get inputting file name and output log 
	string dstPDFPath = outputFilesPath + "/" + fileName;
	CAutoDocument autoDoc(dstPDFPath.c_str(), "rb");
	if (!autoDoc.GetFileHander()) return FSCRT_ERRCODE_ERROR;

	//Load PDF file
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	//not for windows because underlying code will decrypt automatically first when loading document.
#if !defined(_WIN32) && !defined(_WIN64)
	ret = autoDoc.LoadPDFDocument(NULL);
	if (FSCRT_ERRCODE_INVALIDCERTIFICATE == ret)
		FSDK_OutputLog("Because the document is encrypted by CertificateEncryption and not decrypted yet.\r\n");
#endif

	FSPDF_SECURITYHANDLER_CERT certificateHandler;
#if defined(_WIN32)|| defined(_WIN64)
	certificateHandler.clientData = NULL;
#else
	string pfxPath = inputFilesPath + "/foxit_all.pfx";
	certificateHandler.clientData = (FS_LPVOID)pfxPath.c_str();
#endif
	certificateHandler.GetDecryptionKey = GetDecryptionKey;
	ret = FSPDF_Security_SetCertificateHandler(&certificateHandler);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure. In function FSPDF_Security_SetCertificateHandler with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
		return ret;
	}

	ret = autoDoc.LoadPDFDocument(NULL);
	if (ret == FSCRT_ERRCODE_INVALIDCERTIFICATE) FSDK_OutputLog("Fail to decrypt the document!");
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	//Get outputting file name
	string filename(fileName.c_str(), fileName.length() - 4);
	outputFilesPath += "/";
	outputFilesPath += filename;
	outputFilesPath += "_removeCertificateEncryption";
	outputFilesPath += ".pdf";
	//Remove CertificateEncryption from PDF and save as a new file
	FS_BOOL bRet = FSDK_SavePDFFile(autoDoc.GetDocument(), outputFilesPath.data(), FSPDF_SAVEFLAG_REMOVESECURITY);
	if (!bRet) return FSCRT_ERRCODE_ERROR;

	//Get current time and output elapse time
	unsigned int nEndTime = FSDK_GetClock();
	FSDK_OutputLog("Success to remove CertificateEncryption.\tElapsed time: %dms.\r\n", nEndTime - nStartTime);
	return FSCRT_ERRCODE_SUCCESS;
}

void CPDFEncrypt::RemoveFoxitDRMEncryption(string fileName, string inputFilesPath, string outputFilesPath)
{
	//Begin, output log 
	FSDK_OutputLog("\r\n====Remove FoxitDRM Encryption====\r\n");
	unsigned int nStartTime = FSDK_GetClock();//Get current time
	//Get inputting file name and output log 
	inputFilesPath += "/" + fileName;
	CAutoDocument autoDoc(inputFilesPath.c_str(), "rb");
	FS_RESULT ret = autoDoc.LoadPDFDocument(NULL);
	if (ret != FSCRT_ERRCODE_UNKNOWNSECURITYHANDLER)
	{
		FSDK_OutputLog("Exit. The file isn't encrypted by FoxitDRM Encryption or any other error and the code is %d.\r\n", ret);
		return;
	}

	FSPDF_SECURITYHANDLER_FDRM foxitDRMHandler;
	foxitDRMHandler.clientData = NULL;
	foxitDRMHandler.GetDecryptionParams = GetDecryptionParams;
	ret = FSPDF_Security_SetFoxitDRMHandler(&foxitDRMHandler);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure. In function FSPDF_Security_SetFoxitDRMHandler with error code %d.\r\n", ret);
		return;
	}
	
	ret = autoDoc.LoadPDFDocument(NULL);
	if (ret != FSCRT_ERRCODE_SUCCESS) return;


	//Get outputting file name
	string filename(fileName.c_str(), fileName.length() - 4);
	outputFilesPath += "/";
	outputFilesPath += filename;
	outputFilesPath += "_removeFoxitDRMEncryption";
	outputFilesPath += ".pdf";
	//Remove FoxitDRMEncryption from PDF and save as a new file
	FS_BOOL bRet = FSDK_SavePDFFile(autoDoc.GetDocument(), outputFilesPath.data(), FSPDF_SAVEFLAG_REMOVESECURITY);
	if (!bRet) return;

	//Get current time and output elapse time
	unsigned int nEndTime = FSDK_GetClock();
	FSDK_OutputLog("Succeed to remove FoxitDRMEncryption.\tElapsed time: %dms.\r\n", nEndTime - nStartTime);
}

void CPDFEncrypt::RemoveCustomEncryption(string fileName, string inputFilesPath, string outputFilesPath)
{
	//Begin, output log 
	FSDK_OutputLog("\r\n====Remove Custom Encryption====\r\n");
	unsigned int nStartTime = FSDK_GetClock();//Get current time
	//Get inputting file name and output log 
	inputFilesPath += "/" + fileName;
	CAutoDocument autoDoc(inputFilesPath.c_str(), "rb");
	if (!autoDoc.GetFileHander()) return;
	FS_RESULT ret = autoDoc.LoadPDFDocument(NULL);
	if (ret != FSCRT_ERRCODE_UNKNOWNSECURITYHANDLER)
	{
		FSDK_OutputLog("Exit. The file isn't encrypted by CustomEncryption or any other error and the code is %d.\r\n", ret);
		return;
	}

	FSPDF_SECURITYHANDLER securityHandler;
	securityHandler.clientData = NULL;
	securityHandler.CreateContext = CreateContext;
	securityHandler.DecryptData = DecryptData;
	securityHandler.EncryptData = EncryptData;
	securityHandler.FinishDecryptor = FinishDecryptor;
	securityHandler.GetCipherInfo = GetCipherInfo;
	securityHandler.GetDecryptedSize = GetDecryptedSize;
	securityHandler.GetEncryptedSize = GetEncryptedSize;
	securityHandler.GetUserPermissions = GetUserPermissions;
	securityHandler.IsOwner = IsOwner;
	securityHandler.ReleaseContext = ReleaseContext;
	securityHandler.StartDecryptor = StartDecryptor;
	FSCRT_BSTR filter;
	FSCRT_BStr_Init(&filter);
	FSCRT_BStr_Set(&filter, "filter",FS_WORD(strlen("filter")));
	ret = FSPDF_Security_RegisterHandler(&filter, &securityHandler);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure. In function FSPDF_Security_RegisterHandler with error code %d.\r\n", ret);
		return;
	}
	//		Fail, PDF is encrypted by custom encryption.
	ret = autoDoc.LoadPDFDocument(NULL);
	if (ret != FSCRT_ERRCODE_SUCCESS) return;
	//Get outputting file name
	string filename(fileName.c_str(), fileName.length() - 4);
	outputFilesPath += "/";
	outputFilesPath += filename;
	outputFilesPath += "_removeCustomEncryption";
	outputFilesPath += ".pdf";
	//Remove CustomEncryption from PDF and save as a new file
	FS_BOOL bRet = FSDK_SavePDFFile(autoDoc.GetDocument(), outputFilesPath.data(), FSPDF_SAVEFLAG_REMOVESECURITY);
	if(!bRet) 
	{
		FSPDF_Security_UnregisterHandler(&filter);
		FSCRT_BStr_Clear(&filter);
		return;
	}

	FSPDF_Security_UnregisterHandler(&filter);
	FSCRT_BStr_Clear(&filter);

	//Get current time and output elapse time
	unsigned int nEndTime = FSDK_GetClock();
	FSDK_OutputLog("Succeed to remove CustomEncryption.\tElapsed time: %dms.\r\n", nEndTime - nStartTime);
}

FS_RESULT CPDFEncrypt::FoxitDRMEncryption(FSCRT_DOCUMENT pdfDoc, FSCRT_FILE encryptedFile)
{
	FSCRT_BSTRC(subFilter, "subFilter");
	FSCRT_BSTRC(fileID, "fileID");
	FSCRT_BSTRC(initialKey, "initialKey");
	FS_BOOL encryptMetadata = TRUE;
	FS_DWORD permissions = FSPDF_PERMISSION_PRINT ;
	FSCRT_PROGRESS encryptProgress = NULL;
	FS_RESULT ret = FSPDF_Security_StartFoxitDRMEncryption(pdfDoc, &subFilter,TRUE, permissions, FSCRT_CIPHER_AES,16, &fileID, &initialKey, encryptMetadata, encryptedFile, FSPDF_SAVEFLAG_OBJECTSTREAM, &encryptProgress);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure. In function FSPDF_Security_StartFoxitDRMEncryption with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
		return FSCRT_ERRCODE_ERROR;
	}
	ret = FSCRT_Progress_Continue(encryptProgress, NULL);
	if (FSCRT_ERRCODE_FINISHED != ret)
	{
		FSDK_OutputLog("Failure. In function FSCRT_Progress_Continue with error code %d.\r\n", ret);
	} else {

		FSDK_OutputLog("Success. Start Foxit DRM Encryption on a PDF document.\r\n");
	}

	FSCRT_Progress_Release(encryptProgress);//Release progress object
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT CPDFEncrypt::CertificateEncryption(FSCRT_DOCUMENT pdfDoc, FSCRT_FILE encryptedFile,string inputFilesPath)
#if defined(_WIN32)|| defined(_WIN64)
{
	FSCRT_BSTR envelopes;
	FSCRT_BStr_Init(&envelopes);
	FSCRT_BSTR encryptKey;
	FSCRT_BStr_Init(&encryptKey);
	FS_INT32 count = 1;
	FS_BOOL bEncryptMetaData = TRUE;
	FS_BOOL bret = GetCertInfo(&envelopes, &encryptKey, bEncryptMetaData, 16, inputFilesPath);
	if (!bret) return FSCRT_ERRCODE_ERROR;

	FSCRT_PROGRESS encryptProgress = NULL;
	FS_RESULT ret = FSPDF_Security_StartCertificateEncryption(pdfDoc, &envelopes, count, FSCRT_CIPHER_AES, &encryptKey, bEncryptMetaData,encryptedFile, FSPDF_SAVEFLAG_OBJECTSTREAM, &encryptProgress);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure. In function FSPDF_Security_StartCertificateEncryption with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
		return FSCRT_ERRCODE_ERROR;
	}
	ret = FSCRT_Progress_Continue(encryptProgress, NULL);
	if (FSCRT_ERRCODE_FINISHED != ret)
	{
		FSDK_OutputLog("Failure. In function FSCRT_Progress_Continue with error code %d.\r\n", ret);
	} else {

		FSDK_OutputLog("Success. Start Certification Encryption on a PDF document.\r\n");
	}

	FSCRT_Progress_Release(encryptProgress);//Release progress object
	
	FSCRT_BStr_Clear(&envelopes);
	FSCRT_BStr_Clear(&encryptKey);
	return FSCRT_ERRCODE_SUCCESS;
}
#else
{
	PKCS7				*p7;
	int					len;
	char				*p;
	FILE				*fp;
	PKCS7_RECIP_INFO	*inf;
	FS_BOOL bEncryptMetadata = TRUE;

	string cerFile = inputFilesPath + "/" + "foxit.cer";
	FILE* fpCert = fopen(cerFile.c_str(), "rb");
	if (!fpCert)
	{
		return FSCRT_ERRCODE_ERROR;
	}
	fseek(fpCert, 0, SEEK_END);
	long cbData =  ftell(fpCert);
	char* pbData = (char*)malloc(cbData);
	fseek(fpCert, 0, SEEK_SET);
	fread(pbData, sizeof(char), cbData, fpCert);
	fclose(fpCert);
	char random[24] = {0};
	memset(random,0xff,24);
	for(int i = 0; i<20; i++)
		random[i] = rand();
//		SSL_load_error_strings();
	ERR_load_BIO_strings();
	OpenSSL_add_all_algorithms();

	p=pbData;
	//d2i_PKCS7(&p7,(const unsigned char **)&p,cbData);
	X509 *x509_Cert=X509_new();

	d2i_X509(&x509_Cert,(const unsigned char **)&p,cbData);
	p7=PKCS7_new();
	PKCS7_set_type(p7,NID_pkcs7_enveloped);
	EVP_CIPHER *evp_cipher = (EVP_CIPHER *)EVP_des_ede3_cbc();

	PKCS7_set_cipher(p7,evp_cipher);
	inf=PKCS7_add_recipient(p7,x509_Cert);//PKCS7_RECIP_INFO_new();
	BIO *p7bio = PKCS7_dataInit(p7, NULL);
	BIO_write(p7bio,random,24);
	BIO_flush(p7bio);
	PKCS7_dataFinal(p7, p7bio);
	unsigned char* derTmp;
	int derLen = i2d_PKCS7(p7,NULL);
	unsigned char * der  = (unsigned char*)malloc(derLen);
	derTmp = der;
	derLen = i2d_PKCS7(p7,(unsigned char **)&derTmp);
	BIO_free(p7bio);
	PKCS7_free(p7);
	unsigned char* envelopmem = der;
	SHA_CTX c;
	unsigned char initmem[20]={0};   //ok
	SHA1_Init(&c);
	SHA1_Update(&c, random, 20);
	SHA1_Update(&c, envelopmem, derLen);
	if(!bEncryptMetadata)
	{
		FS_DWORD tag = 0xffffffff;
		SHA1_Update(&c, &tag, sizeof(FS_DWORD));
	}
	SHA1_Final(initmem,&c);
	FSCRT_PROGRESS progress;
	FSPDF_SECURITYHANDLER securityhandler;
	FSCRT_BSTR envolops;
	FSCRT_BSTR initialKey;
	
	len = sizeof(initmem)-1;
	FSCRT_BSTR tinitialKey;
	FSCRT_BStr_Init(&tinitialKey);
	FSCRT_BStr_SetLength(&tinitialKey, 16);	
	memcpy(tinitialKey.str, initmem, 16);
	initialKey = tinitialKey;
	
	len = sizeof(envelopmem);
	FSCRT_BSTR tEnvolops;
	FSCRT_BStr_Init(&tEnvolops);
	FSCRT_BStr_SetLength(&tEnvolops, derLen);	
	memcpy(tEnvolops.str, envelopmem, derLen);
	envolops = tEnvolops;
	FS_RESULT ret = FSPDF_Security_StartCertificateEncryption(pdfDoc, &envolops, 1, FSCRT_CIPHER_AES, &initialKey, bEncryptMetadata, encryptedFile, FSPDF_SAVEFLAG_NOORIGINAL, &progress);
	if (FSCRT_ERRCODE_SUCCESS != ret) return FSCRT_ERRCODE_ERROR;
	FS_INT32 pos = 0;
	while(pos < 100)
	{
		ret = FSCRT_Progress_Continue(progress, NULL);
		if (FSCRT_ERRCODE_FINISHED != ret) return FSCRT_ERRCODE_ERROR;
		ret = FSCRT_Progress_GetPercent(progress, &pos);
		if (FSCRT_ERRCODE_SUCCESS != ret) return FSCRT_ERRCODE_ERROR;
	}
	FSCRT_Progress_Release(progress);
	free(der);
	

/*
	FSPDF_SECURITYHANDLER_CERT security;
	security.clientData = NULL;
	security.GetDecryptionKey = MyGetDecryptionKey;
	FSCRT_BSTRC(filter, "Adobe.PubSec");
	ret = FSPDF_Security_SetCertificateHandler( &security);
	if (FSCRT_ERRCODE_SUCCESS != ret) return ;
	file = FSDK_OpenFile("cert_enc.pdf", "rb");
	if (!file)
	{
		return ;
	}
	
	ret = FSPDF_Doc_StartLoad(file, NULL, &document, NULL);
	
	FSPDF_Doc_Close(document);
*/

	FSCRT_BStr_Clear(&envolops);
	FSCRT_BStr_Clear(&initialKey);
	return FSCRT_ERRCODE_SUCCESS;
}
#endif

FS_RESULT CPDFEncrypt::CustomEncryption(FSCRT_DOCUMENT pdfDoc, FSCRT_FILE encryptedFile)
{
	FSCRT_BSTRC(filter,"filter");
	FSCRT_BSTRC(subfilter,"subfilter");
	FSCRT_BSTRC(encryptInfo,"encryptInfo");
	FSCRT_PROGRESS encryptProgress = NULL;
	FSPDF_SECURITYHANDLER securityHandler;
	securityHandler.clientData = NULL;
	securityHandler.CreateContext = CreateContext;
	securityHandler.DecryptData = DecryptData;
	securityHandler.EncryptData = EncryptData;
	securityHandler.FinishDecryptor = FinishDecryptor;
	securityHandler.GetCipherInfo = GetCipherInfo;
	securityHandler.GetDecryptedSize = GetDecryptedSize;
	securityHandler.GetEncryptedSize = GetEncryptedSize;
	securityHandler.GetUserPermissions = GetUserPermissions;
	securityHandler.IsOwner = IsOwner;
	securityHandler.ReleaseContext = ReleaseContext;
	securityHandler.StartDecryptor = StartDecryptor;
	FS_RESULT ret = FSPDF_Security_StartCustomEncryption(pdfDoc, &filter, &subfilter, &encryptInfo, &securityHandler, TRUE, encryptedFile, FSPDF_SAVEFLAG_OBJECTSTREAM, &encryptProgress);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure. In function FSPDF_Security_StartCustomEncryption with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
		return FSCRT_ERRCODE_ERROR;
	}
	ret = FSCRT_Progress_Continue(encryptProgress, NULL);
	if (FSCRT_ERRCODE_FINISHED != ret)
	{
		FSDK_OutputLog("Failure. In function FSCRT_Progress_Continue with error code %d.\r\n", ret);
	} else {

		FSDK_OutputLog("Success. Start custom Encryption on a PDF document.\r\n");
	}

	FSCRT_Progress_Release(encryptProgress);//Release progress object
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT CPDFEncrypt::PasswrodEncryption(FSCRT_DOCUMENT pdfDoc, FSCRT_FILE encryptedFile)
{
	//Set print permission, and user can set other permission
	FS_DWORD permissions = FSPDF_PERMISSION_PRINT ;
	//Define user and owner password, it not need to call ::FSCRT_BStr_Clear to clear
	FSCRT_BSTRC(userPwd,"123");
	FSCRT_BSTRC(ownerPwd,"456");
	//Encrypt the source PDF and save as a new file
	FSCRT_PROGRESS encryptProgress = NULL;
	FS_RESULT ret = FSPDF_Security_StartPasswordEncryption(pdfDoc, permissions, &userPwd, &ownerPwd, FSCRT_CIPHER_AES, 16, TRUE, encryptedFile, FSPDF_SAVEFLAG_OBJECTSTREAM, &encryptProgress);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure. In function FSPDF_Security_StartPasswordEncryption with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
		return FSCRT_ERRCODE_ERROR;
	}

	//Continue the progress of encrypting.
	ret = FSCRT_Progress_Continue(encryptProgress, NULL);
	if (FSCRT_ERRCODE_FINISHED != ret)
	{
		FSDK_OutputLog("Failure. In function FSCRT_Progress_Continue with error code %d.\r\n", ret);
	} else {

		FSDK_OutputLog("Success. Start standard password encryption on a PDF document.\r\n");
	}

	FSCRT_Progress_Release(encryptProgress);//Release progress object
	return FSCRT_ERRCODE_SUCCESS;

}


void CPDFEncrypt::PDFEncrypt(string fileName, string inputFilesPath, string outputFilesPath, int encryptType)
{
	string strFile = inputFilesPath + "/" + fileName;	
	//Begin, output log
	unsigned int nStartTime = FSDK_GetClock();//Get current time
	//Call function ::LoadPDF to open and Load a PDF
	CAutoDocument autoDoc(strFile.c_str(), "rb");
	FS_RESULT ret = autoDoc.LoadPDFDocument(NULL);
	if (ret != FSCRT_ERRCODE_SUCCESS) return;

	//Get outputting file name
	string filename(fileName.c_str(), fileName.length() - 4);
	outputFilesPath += "/";
	outputFilesPath += filename;
	
	switch (encryptType)
	{
	case ENCRYPTIONTYPE_PASSWORD:
		{
			outputFilesPath += "_Pwd(123).pdf";
		}
		break;
	case ENCRYPTIONTYPE_CUSTOM:
		{
			outputFilesPath += "_Custom.pdf";
		}
		break;
	case ENCRYPTIONTYPE_CERTIFICATE:
		{
			outputFilesPath += "_Certificate.pdf";
		}
		break;
	case  ENCRYPTIONTYPE_FOXITDRM:
		{
			outputFilesPath += "_FoxitDRM.pdf";
		}
		break;
	}
	//Open outputting file to write
	CAutoDocument encryptedDoc(outputFilesPath.data(), "wb");
	if (!encryptedDoc.GetFileHander()) return;
	switch (encryptType)
	{
	case ENCRYPTIONTYPE_PASSWORD:
		{
			PasswrodEncryption(autoDoc.GetDocument(), encryptedDoc.GetFileHander());
		}
		break;
	case ENCRYPTIONTYPE_CUSTOM:
		{
			CustomEncryption(autoDoc.GetDocument(), encryptedDoc.GetFileHander());
		}
		break;
	case ENCRYPTIONTYPE_CERTIFICATE:
		{
			CertificateEncryption(autoDoc.GetDocument(), encryptedDoc.GetFileHander(), inputFilesPath);
		}
		break;
	case ENCRYPTIONTYPE_FOXITDRM:
		{
			FoxitDRMEncryption(autoDoc.GetDocument(), encryptedDoc.GetFileHander());
		}
		break;
	}

	//Get current time and output elapse time
	unsigned int nEndTime = FSDK_GetClock();
	FSDK_OutputLog("Success. The PDF is encrypted.\tElapsed time: %dms.\r\n", nEndTime - nStartTime);
}

//This function is to save PDF with removing password "123".
void CPDFEncrypt::RemovePassword(string fileName, string inputFilesPath, string outputFilesPath, string password)
{
	//Begin, output log 
	FSDK_OutputLog("\r\n====Remove Password====\r\n");
	unsigned int nStartTime = FSDK_GetClock();//Get current time
	//Get inputting file name and output log 
	inputFilesPath += "/" + fileName;
	CAutoDocument autoDoc(inputFilesPath.c_str(), "rb");
	if (!autoDoc.GetFileHander()) return;
	FS_RESULT ret = autoDoc.LoadPDFDocument(NULL);
	if (ret != FSCRT_ERRCODE_PASSWORD)
	{
		FSDK_OutputLog("Failure: The file isn't encrypted by password or other error and the code is %d.\r\n", ret);
		return;
	}

	//Fail, PDF is encrypted by password.
	//Define user password, it not need to call ::FSCRT_BStr_Clear to clear
	FSCRT_BSTR userPwd = {(char*)password.c_str(), (FS_DWORD)password.length()};
	//Load PDF with user password 
	ret = autoDoc.LoadPDFDocument(&userPwd);
	if (ret != FSCRT_ERRCODE_SUCCESS) return;
	string filename(fileName.c_str(), fileName.length() - 4);
	outputFilesPath += "/";
	outputFilesPath += filename;
	outputFilesPath += "_removePwd.pdf";
	FS_BOOL bRet = FSDK_SavePDFFile(autoDoc.GetDocument(), outputFilesPath.data(), FSPDF_SAVEFLAG_REMOVESECURITY);
	if (!bRet) return;

	//Get current time and output elapse time
	unsigned int nEndTime = FSDK_GetClock();
	FSDK_OutputLog("Succeed to remove password.\tElapsed time: %dms.\r\n", nEndTime - nStartTime);
}

void CPDFEncrypt::PDFEncryptFolder(void* folder, string inputFilesPath, string outputFilesPath)
{
	//Success to enumerate
	do
	{
		//Get a file from folder
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if (strFile.size() > 0 && !isDir) {
			PDFEncrypt(strFile, inputFilesPath, outputFilesPath, ENCRYPTIONTYPE_PASSWORD);
			PDFEncrypt(strFile, inputFilesPath, outputFilesPath, ENCRYPTIONTYPE_CUSTOM);
			PDFEncrypt(strFile, inputFilesPath, outputFilesPath, ENCRYPTIONTYPE_FOXITDRM);
			PDFEncrypt(strFile, inputFilesPath, outputFilesPath, ENCRYPTIONTYPE_CERTIFICATE);
		}

		//If it's an invalid file name
		if (strFile.size() < 1) break;
	}
    while (1);
}

int main(int argc, char* argv[])
{
	//Set executing file path
	FSDK_SetExecuteFilePath(argc, argv);
	CPDFEncrypt pdfencrypt;
	//Judge if to command line arguments. 
	//usage: pdfencrypt <srcfile> /o [destfolder]
	if (argc > 1)
	{
		pdfencrypt.m_strInputSrcFile = argv[1];
		bool bArgumentValid = true;
		if (argc >= 3)
		{
			if (0 != strcmp(argv[2], "/o"))
				bArgumentValid = false;
			else if (argc > 3)
				pdfencrypt.m_strOutputFolder = argv[3];
		}
		if (false == bArgumentValid || !FSDK_FileExist(pdfencrypt.m_strInputSrcFile.c_str()) || (pdfencrypt.m_strOutputFolder != "" && !FSDK_FolderExist(pdfencrypt.m_strOutputFolder.c_str())))
		{
			printf("\r\n");
			printf("usage: pdfencrypt [<srcfile> /o [destfolder]] \r\n\r\n");
			printf("<srcfile>\t: Path of an image file as input file.\r\n");
			printf("/o [destfolder]\t: Path of a folder as output folder. If not set, use default\r\n\t\t  output folder.\r\n\r\n");
			return -1;
		}
	}
	// Locate the path of log file
	string logPath ;
	if (pdfencrypt.m_strOutputFolder != "")
	{
		pdfencrypt.m_strOutputFolder += "/pdfencrypt";
		FSDK_CreatFolder(pdfencrypt.m_strOutputFolder.c_str());
		logPath = pdfencrypt.m_strOutputFolder;
		logPath += "/log.txt";
	}
	else
	{
		pdfencrypt.m_strOutputFolder = FSDK_GetOutputFilesFolder("pdfencrypt");
		logPath = FSDK_GetLogFile("pdfencrypt");
	}
	
	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfencrypt\r\n");

	//Initialize Foxit SDK library
	if (!FSDK_InitializeLibray(true))
	{
		//Fail to initialize, output status and return
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	FSCRT_FILE gsdkLogFile = NULL;
	FS_BOOL bRet = FSDK_SetLibraryLogFile(pdfencrypt.m_strOutputFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get folder of input files for this example
	//string inputFolder;
	//string pdfName;
	bool bEnumFolder = false;

	if (pdfencrypt.m_strInputSrcFile != "")
	{
		pdfencrypt.m_strPdfFileName = FSDK_GetFileName(pdfencrypt.m_strInputSrcFile.c_str());
		pdfencrypt.m_strInputFolder = FSDK_GetCustomInputPath(pdfencrypt.m_strInputSrcFile.c_str());
		FSDK_OutputLog("Input from folder %s\r\n", pdfencrypt.m_strInputFolder.c_str());
		FSDK_OutputLog("Output to folder %s \r\n\r\n", pdfencrypt.m_strOutputFolder.c_str());
	}
	else
	{
		pdfencrypt.m_strInputFolder = FSDK_GetInputFilesFolder("pdfencrypt");
		//Open folder to enumerate files
		void* folder = FSDK_OpenFolder(pdfencrypt.m_strInputFolder.c_str());
		//Get output files folder path.
		string outputFilesPath = FSDK_GetOutputFilesFolder("pdfencrypt");
		if (folder)
		{
			FSDK_OutputLog("Input from folder %s\r\n", pdfencrypt.m_strInputFolder.c_str());
			FSDK_OutputLog("Output to folder %s \r\n\r\n", pdfencrypt.m_strOutputFolder.c_str());
			FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfencrypt: ");
			pdfencrypt.PDFEncryptFolder(folder, pdfencrypt.m_strInputFolder, pdfencrypt.m_strOutputFolder);
			//Close access to specific folder.
			FSDK_CloseFolder(folder);
			bEnumFolder = true;
		}
		else
		{
			pdfencrypt.m_strPdfFileName = "AboutFoxit.pdf";
			pdfencrypt.m_strInputFolder = FSDK_GetFixFolder();
			FSDK_OutputLog("Input from folder %s\r\n", pdfencrypt.m_strInputFolder.c_str());
			FSDK_OutputLog("Output to folder %s \r\n\r\n", pdfencrypt.m_strOutputFolder.c_str());
		}
	}
	if (false == bEnumFolder)
	{
		//Do encyprtion
		pdfencrypt.PDFEncrypt(pdfencrypt.m_strPdfFileName, pdfencrypt.m_strInputFolder, pdfencrypt.m_strOutputFolder, ENCRYPTIONTYPE_PASSWORD);
		pdfencrypt.PDFEncrypt(pdfencrypt.m_strPdfFileName, pdfencrypt.m_strInputFolder, pdfencrypt.m_strOutputFolder, ENCRYPTIONTYPE_CUSTOM);
		pdfencrypt.PDFEncrypt(pdfencrypt.m_strPdfFileName, pdfencrypt.m_strInputFolder, pdfencrypt.m_strOutputFolder, ENCRYPTIONTYPE_FOXITDRM);
		pdfencrypt.PDFEncrypt(pdfencrypt.m_strPdfFileName, pdfencrypt.m_strInputFolder, pdfencrypt.m_strOutputFolder, ENCRYPTIONTYPE_CERTIFICATE);

		//Do decryption
		string removeEncryptPrefixName = string(pdfencrypt.m_strPdfFileName.c_str(),pdfencrypt.m_strPdfFileName.length()-4);
		string removeEncryptName;
		removeEncryptName = removeEncryptPrefixName + "_Pwd(123).pdf";
		string password = "123";
		pdfencrypt.RemovePassword(removeEncryptName, pdfencrypt.m_strOutputFolder, pdfencrypt.m_strOutputFolder, password);
		removeEncryptName = removeEncryptPrefixName + "_Custom.pdf";
		pdfencrypt.RemoveCustomEncryption(removeEncryptName, pdfencrypt.m_strOutputFolder, pdfencrypt.m_strOutputFolder);
		removeEncryptName = removeEncryptPrefixName + "_FoxitDRM.pdf";
		pdfencrypt.RemoveFoxitDRMEncryption(removeEncryptName, pdfencrypt.m_strOutputFolder, pdfencrypt.m_strOutputFolder);
		removeEncryptName = removeEncryptPrefixName + "_Certificate.pdf";
		pdfencrypt.RemoveCertificateEncryption(removeEncryptName, pdfencrypt.m_strInputFolder, pdfencrypt.m_strOutputFolder);
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}

