/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a common source to implement some general functions which are used by digital signature.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(_MSC_VER) && _MSC_VER < 1300
	#include "wincrypt.h"
#endif
#include "win32_crypt.h"

#define PSL_PASSWORD_MAXLEN		32

/* ------------------------------ CPSL_CryptLib ------------------------------ */
CPSL_CryptLib::CPSL_CryptLib()
:m_hCrypt32(NULL),
m_hCryptui(NULL),
m_hAdvapi32(NULL),
m_bCrypt32Loaded(FALSE),
m_bCryptuiLoaded(FALSE),
m_bAdvapi32Loaded(FALSE)
{
}

CPSL_CryptLib::~CPSL_CryptLib()
{
	if (m_hCrypt32)
	{
		::FreeLibrary(m_hCrypt32);
		m_hCrypt32 = NULL;
	}
	
	if (m_hCryptui)
	{
		::FreeLibrary(m_hCryptui);
		m_hCryptui = NULL;
	}

	if (m_hAdvapi32)
	{
		::FreeLibrary(m_hAdvapi32);
		m_hAdvapi32 = NULL;
	}
}

HMODULE	CPSL_CryptLib::LoadCrypt32()
{
	if (!m_bCrypt32Loaded)
	{
		m_hCrypt32 = ::LoadLibraryW(L"crypt32.dll");
		m_bCrypt32Loaded = TRUE;
	}

	return m_hCrypt32;
}

HMODULE	CPSL_CryptLib::LoadCryptui()
{
	if (!m_bCryptuiLoaded)
	{
		m_hCryptui = ::LoadLibraryW(L"cryptui.dll");
		m_bCryptuiLoaded = TRUE;
	}

	return m_hCryptui;
}

HMODULE	CPSL_CryptLib::LoadAdvapi32()
{
	if (!m_bAdvapi32Loaded)
	{
		m_hAdvapi32 = ::LoadLibraryW(L"advapi32.dll");
		m_bAdvapi32Loaded = TRUE;
	}

	return m_hAdvapi32;
}

HMODULE	CPSL_CryptLib::GetCrypt32()
{
	return LoadCrypt32();
}

HMODULE	CPSL_CryptLib::GetAdvapi32()
{
	return LoadAdvapi32();
}


#define PSL_MAKEFUNC_CRYPTLIB(dllfunc, funcname, clare, params, rettype, defval) \
typedef rettype (WINAPI *Type_##funcname)##clare;\
rettype CPSL_CryptLib::funcname##clare \
{\
	if (HMODULE hModule = CPSL_CryptLib::##dllfunc())\
	{\
		if (Type_##funcname proc = (Type_##funcname)::GetProcAddress(hModule, #funcname))\
		{\
			return proc##params;\
		}\
	}\
	return defval;\
}

//Crypt32

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CryptMsgOpenToDecode,
			(
			DWORD dwMsgEncodingType,
			DWORD dwFlags,
			DWORD dwMsgType,
			HCRYPTPROV hCryptProv,
			PCERT_INFO pRecipientInfo,
			PCMSG_STREAM_INFO pStreamInfo
			),
			(
			dwMsgEncodingType, 
			dwFlags, 
			dwMsgType, 
			hCryptProv, 
			pRecipientInfo, 
			pStreamInfo
			),
			void*, NULL)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CryptMsgUpdate,
			(
			HCRYPTMSG hCryptMsg, 
			const BYTE *pbData,	
			DWORD cbData, 
			BOOL fFinal
			),
			(
			hCryptMsg, 
			pbData, 
			cbData, 
			fFinal
			),
			BOOL, FALSE)


PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CryptMsgGetParam,
			(
			void* hCryptMsg,
			DWORD dwParamType,
			DWORD dwIndex,
			void *pvData,
			DWORD *pcbData
			),
			(
			hCryptMsg,
			dwParamType,
			dwIndex,
			pvData,
			pcbData
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertOpenSystemStoreA,
			(
			HCRYPTPROV      hProv,
			LPCSTR            szSubsystemProtocol
			),
			(
			hProv,
			szSubsystemProtocol
			),
			void*,NULL)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertGetSubjectCertificateFromStore,
			(
			HCERTSTORE hCertStore,
			DWORD dwCertEncodingType,
			PCERT_INFO pCertId           // Only the Issuer and SerialNumber
			),
			(
			hCertStore,
			dwCertEncodingType,
			pCertId
			),
			PCCERT_CONTEXT, NULL)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CryptAcquireCertificatePrivateKey,
			(
			PCCERT_CONTEXT pCert,
			DWORD dwFlags,
			void *pvReserved,
			HCRYPTPROV *phCryptProv,
			DWORD *pdwKeySpec,
			BOOL *pfCallerFreeProv
			),
			(
			pCert,
			dwFlags,
			pvReserved,
			phCryptProv,
			pdwKeySpec,
			pfCallerFreeProv
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CryptMsgControl,
			(
			void* hCryptMsg,
			DWORD dwFlags,
			DWORD dwCtrlType,
			void const *pvCtrlPara
			),
			(
			hCryptMsg,
			dwFlags,
			dwCtrlType,
			pvCtrlPara
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CryptMsgClose,
			(
			HCRYPTMSG hCryptMsg
			),
			(
			hCryptMsg
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertCloseStore,
			(
			void* hCertStore,
			DWORD dwFlags
			),
			(
			hCertStore,
			dwFlags
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertOpenStore,
			(
			LPCSTR lpszStoreProvider,
			DWORD dwEncodingType,
			HCRYPTPROV hCryptProv,
			DWORD dwFlags,
			const void *pvPara
			),
			(
			lpszStoreProvider,
			dwEncodingType,
			hCryptProv,
			dwFlags,
			pvPara
			),
			HCERTSTORE, NULL)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CryptMsgCalculateEncodedLength,
			(
			DWORD dwMsgEncodingType,
			DWORD dwFlags,
			DWORD dwMsgType,
			void const *pvMsgEncodeInfo,
			OPTIONAL LPSTR pszInnerContentObjID,
			DWORD cbData
			),
			(
			dwMsgEncodingType,
			dwFlags,
			dwMsgType,
			pvMsgEncodeInfo,
			pszInnerContentObjID,
			cbData
			),
			DWORD, 0)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CryptMsgOpenToEncode,
			(
			DWORD dwMsgEncodingType,
			DWORD dwFlags,
			DWORD dwMsgType,
			void const *pvMsgEncodeInfo,
			OPTIONAL LPSTR pszInnerContentObjID,
			OPTIONAL PCMSG_STREAM_INFO pStreamInfo
			),
			(
			dwMsgEncodingType,
			dwFlags,
			dwMsgType,
			pvMsgEncodeInfo,
			pszInnerContentObjID,
			pStreamInfo
			),
			HCRYPTMSG, NULL)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertEnumCertificatesInStore,
			(
			HCERTSTORE hCertStore,
			PCCERT_CONTEXT pPrevCertContext
			),
			(
			hCertStore,
			pPrevCertContext
			),
			PCCERT_CONTEXT, NULL)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertAddCertificateContextToStore,
			(
			HCERTSTORE hCertStore,
			PCCERT_CONTEXT pCertContext,
			DWORD dwAddDisposition,
			OPTIONAL PCCERT_CONTEXT *ppStoreContext
			),
			(
			hCertStore,
			pCertContext,
			dwAddDisposition,
			ppStoreContext
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertDuplicateCertificateContext,
			(
			PCCERT_CONTEXT pCertContext
			),
			(
			pCertContext
			),
			PCCERT_CONTEXT, NULL)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertGetNameStringA,
			(
			PCCERT_CONTEXT pCertContext,
			DWORD dwType,
			DWORD dwFlags,
			void *pvTypePara,
			LPSTR pszNameString,
			DWORD cchNameString
			),
			(
			pCertContext,
			dwType,
			dwFlags,
			pvTypePara,
			pszNameString,
			cchNameString
			),
			DWORD, 0)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32 , CertGetIntendedKeyUsage,
			(
			DWORD dwCertEncodingType,
			PCERT_INFO pCertInfo,
			BYTE* pbKeyUsage,
			DWORD cbKeyUsage
			),
			(
			dwCertEncodingType,
			pCertInfo,
			pbKeyUsage,
			cbKeyUsage
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertSaveStore,
			(
			HCERTSTORE hCertStore,
			DWORD dwEncodingType,
			DWORD dwSaveAs,
			DWORD dwSaveTo,
			void *pvSaveToPara,
			DWORD dwFlags
			),
			(
			hCertStore,
			dwEncodingType,
			dwSaveAs,
			dwSaveTo,
			pvSaveToPara,
			dwFlags
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CryptBinaryToStringA,
			(
			const BYTE  *pbBinary,
			DWORD        cbBinary,
			DWORD        dwFlags,
			LPSTR      pszString,
			DWORD       *pcchString
			),
			(
			pbBinary,
			cbBinary,
			dwFlags,
			pszString,
			pcchString
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CryptStringToBinaryA,
			(
			LPCSTR  pszString,
			DWORD     cchString,
			DWORD     dwFlags,
			BYTE     *pbBinary,
			DWORD    *pcbBinary,
			DWORD    *pdwSkip,    //OPTIONAL
			DWORD    *pdwFlags    //OPTIONAL
			),
			(
			pszString,
			cchString,
			dwFlags,
			pbBinary,
			pcbBinary,
			pdwSkip,    //OPTIONAL
			pdwFlags    //OPTIONAL
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertCompareCertificate,
			(
			DWORD dwCertEncodingType,
			PCERT_INFO pCertId1,
			PCERT_INFO pCertId2
			),
			(
			dwCertEncodingType,
			pCertId1,
			pCertId2
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CryptEncryptMessage,
			(
			PCRYPT_ENCRYPT_MESSAGE_PARA pEncryptPara,
			DWORD cRecipientCert,
			PCCERT_CONTEXT rgpRecipientCert[],
			const BYTE *pbToBeEncrypted,
			DWORD cbToBeEncrypted,
			BYTE *pbEncryptedBlob,
			DWORD *pcbEncryptedBlob
			),
			(
			pEncryptPara,
			cRecipientCert,
			rgpRecipientCert,
			pbToBeEncrypted,
			cbToBeEncrypted,
			pbEncryptedBlob,
			pcbEncryptedBlob
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertFindCertificateInStore,
			(
			HCERTSTORE hCertStore,
			DWORD dwCertEncodingType,
			DWORD dwFindFlags,
			DWORD dwFindType,
			const void *pvFindPara,
			PCCERT_CONTEXT pPrevCertContext
			),
			(
			hCertStore,
			dwCertEncodingType,
			dwFindFlags,
			dwFindType,
			pvFindPara,
			pPrevCertContext
			),
			PCCERT_CONTEXT, NULL)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertDeleteCertificateFromStore,
			(
			IN PCCERT_CONTEXT pCertContext
			),
			(
			pCertContext
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertFreeCertificateContext,
			(
			IN PCCERT_CONTEXT pCertContext
			),
			(
			pCertContext
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetCrypt32, CertCreateCertificateContext,
			(
			IN DWORD dwCertEncodingType,
			IN const BYTE *pbCertEncoded,
			IN DWORD cbCertEncoded
			),
			(
			dwCertEncodingType,
			pbCertEncoded,
			cbCertEncoded
			),
			PCCERT_CONTEXT, NULL)

//Advapi32
PSL_MAKEFUNC_CRYPTLIB(GetAdvapi32, CryptAcquireContextA,
			(
			HCRYPTPROV *phProv,
			LPCSTR szContainer,
			LPCSTR szProvider,
			DWORD dwProvType,
			DWORD dwFlags
			),
			(
			phProv,
			szContainer,
			szProvider,
			dwProvType,
			dwFlags
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetAdvapi32, CryptCreateHash,
			(
			HCRYPTPROV hProv,
			ALG_ID Algid,
			HCRYPTKEY hKey,
			DWORD dwFlags,
			HCRYPTHASH *phHash
			),
			(
			hProv,
			Algid,
			hKey,
			dwFlags,
			phHash
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetAdvapi32, CryptHashData,
			(
			HCRYPTHASH hHash,
			CONST BYTE *pbData,
			DWORD dwDataLen,
			DWORD dwFlags
			),
			(
			hHash,
			pbData,
			dwDataLen,
			dwFlags
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetAdvapi32, CryptGetHashParam,
			(
			HCRYPTHASH hHash,
			DWORD dwParam,
			BYTE *pbData,
			DWORD *pdwDataLen,
			DWORD dwFlags
			),
			(
			hHash,
			dwParam,
			pbData,
			pdwDataLen,
			dwFlags
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetAdvapi32, CryptDestroyHash,
			(
			HCRYPTHASH hHash
			),
			(
			hHash
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetAdvapi32, CryptReleaseContext,
			(
			HCRYPTPROV hProv,
			DWORD dwFlags
			),
			(
			hProv,
			dwFlags
			),
			BOOL, FALSE)

PSL_MAKEFUNC_CRYPTLIB(GetAdvapi32, CryptGenRandom,
			(
			HCRYPTPROV hProv,
			DWORD dwLen,
			BYTE *pbBuffer
			),
			(
			hProv,
			dwLen,
			pbBuffer
			),
			BOOL, FALSE)





