#ifndef _FPSL_CRYPT_H_
#define _FPSL_CRYPT_H_

#define PSL_ALL_PERMISSION		4294967292
#define PSL_NO_PERMISSION		4294963392
#define PSL_ENCODING_TYPE		PKCS_7_ASN_ENCODING | X509_ASN_ENCODING
#define PSL_SEED_LENGTH			20
#define PSL_PDFKEY_LENGTH		16

class CPSL_CryptLib
{
public:
	CPSL_CryptLib();
	virtual ~CPSL_CryptLib();

public:
	HMODULE		GetCrypt32();
	HMODULE		GetAdvapi32();

	//Crypt32
	void* CryptMsgOpenToDecode(
					DWORD dwMsgEncodingType,
					DWORD dwFlags,
					DWORD dwMsgType,
					HCRYPTPROV hCryptProv,
					PCERT_INFO pRecipientInfo,
					PCMSG_STREAM_INFO pStreamInfo);

	BOOL CryptMsgUpdate(
					HCRYPTMSG hCryptMsg,
					const BYTE *pbData,
					DWORD cbData,
					BOOL fFinal
					);

	BOOL CryptMsgGetParam(
					void* hCryptMsg,
					DWORD dwParamType,
					DWORD dwIndex,
					void *pvData,
					DWORD *pcbData
					);

	void* CertOpenSystemStoreA(
					HCRYPTPROV      hProv,
					LPCSTR            szSubsystemProtocol
					);

	PCCERT_CONTEXT CertGetSubjectCertificateFromStore(
					HCERTSTORE hCertStore,
					DWORD dwCertEncodingType,
					PCERT_INFO pCertId           // Only the Issuer and SerialNumber
					);

	BOOL CryptAcquireCertificatePrivateKey(
					PCCERT_CONTEXT pCert,
					DWORD dwFlags,
					void *pvReserved,
					HCRYPTPROV *phCryptProv,
					DWORD *pdwKeySpec,
					BOOL *pfCallerFreeProv
					);

	BOOL CryptMsgControl(
					void* hCryptMsg,
					DWORD dwFlags,
					DWORD dwCtrlType,
					void const *pvCtrlPara
					);

	BOOL CryptMsgClose(
					HCRYPTMSG hCryptMsg
					);

	BOOL CertCloseStore(
					void* hCertStore,
					DWORD dwFlags
					);


	HCERTSTORE CertOpenStore(
					LPCSTR lpszStoreProvider,
					DWORD dwEncodingType,
					HCRYPTPROV hCryptProv,
					DWORD dwFlags,
					const void *pvPara
					);

	DWORD CryptMsgCalculateEncodedLength(
					DWORD dwMsgEncodingType,
					DWORD dwFlags,
					DWORD dwMsgType,
					void const *pvMsgEncodeInfo,
					OPTIONAL LPSTR pszInnerContentObjID,
					DWORD cbData
					);

	HCRYPTMSG CryptMsgOpenToEncode(
					DWORD dwMsgEncodingType,
					DWORD dwFlags,
					DWORD dwMsgType,
					void const *pvMsgEncodeInfo,
					OPTIONAL LPSTR pszInnerContentObjID,
					OPTIONAL PCMSG_STREAM_INFO pStreamInfo
					);

	PCCERT_CONTEXT CertEnumCertificatesInStore(
					HCERTSTORE hCertStore,
					PCCERT_CONTEXT pPrevCertContext
					);

	BOOL CertAddCertificateContextToStore(
					HCERTSTORE hCertStore,
					PCCERT_CONTEXT pCertContext,
					DWORD dwAddDisposition,
					OPTIONAL PCCERT_CONTEXT *ppStoreContext
					);

	PCCERT_CONTEXT CertDuplicateCertificateContext(
					PCCERT_CONTEXT pCertContext
					);

	DWORD CertGetNameStringA(
					PCCERT_CONTEXT pCertContext,
					DWORD dwType,
					DWORD dwFlags,
					void *pvTypePara,
					LPSTR pszNameString,
					DWORD cchNameString
					);

	BOOL CertSaveStore(
					HCERTSTORE hCertStore,
					DWORD dwEncodingType,
					DWORD dwSaveAs,
					DWORD dwSaveTo,
					void *pvSaveToPara,
					DWORD dwFlags
					);

	BOOL CryptBinaryToStringA(
					const BYTE  *pbBinary,
					DWORD        cbBinary,
					DWORD        dwFlags,
					LPSTR      pszString,
					DWORD       *pcchString
					);

	BOOL CryptStringToBinaryA(
					LPCSTR  pszString,
					DWORD     cchString,
					DWORD     dwFlags,
					BYTE     *pbBinary,
					DWORD    *pcbBinary,
					DWORD    *pdwSkip,    //OPTIONAL
					DWORD    *pdwFlags    //OPTIONAL
					);

	BOOL CertCompareCertificate(
					DWORD dwCertEncodingType,
					PCERT_INFO pCertId1,
					PCERT_INFO pCertId2
					);

	BOOL CryptEncryptMessage(
					PCRYPT_ENCRYPT_MESSAGE_PARA pEncryptPara,
					DWORD cRecipientCert,
					PCCERT_CONTEXT rgpRecipientCert[],
					const BYTE *pbToBeEncrypted,
					DWORD cbToBeEncrypted,
					BYTE *pbEncryptedBlob,
					DWORD *pcbEncryptedBlob
					);

	PCCERT_CONTEXT CertFindCertificateInStore(
					HCERTSTORE hCertStore,
					DWORD dwCertEncodingType,
					DWORD dwFindFlags,
					DWORD dwFindType,
					const void *pvFindPara,
					PCCERT_CONTEXT pPrevCertContext
					);

	BOOL CertGetIntendedKeyUsage(
					DWORD dwCertEncodingType,
					PCERT_INFO pCertInfo,
					BYTE* pbKeyUsage,
					DWORD cbKeyUsage
					);

	BOOL CertDeleteCertificateFromStore(
					IN PCCERT_CONTEXT pCertContext
					);

	BOOL CertFreeCertificateContext(
					IN PCCERT_CONTEXT pCertContext
					);

	PCCERT_CONTEXT CertCreateCertificateContext(
					IN DWORD dwCertEncodingType,
					IN const BYTE *pbCertEncoded,
					IN DWORD cbCertEncoded
					);

	//Advapi32
	BOOL CryptAcquireContextA(
					HCRYPTPROV *phProv,
					LPCSTR szContainer,
					LPCSTR szProvider,
					DWORD dwProvType,
					DWORD dwFlags
					);

	BOOL CryptCreateHash(
					HCRYPTPROV hProv,
					ALG_ID Algid,
					HCRYPTKEY hKey,
					DWORD dwFlags,
					HCRYPTHASH *phHash
					);

	BOOL CryptHashData(
					HCRYPTHASH hHash,
					CONST BYTE *pbData,
					DWORD dwDataLen,
					DWORD dwFlags
					);

	BOOL CryptGetHashParam(
					HCRYPTHASH hHash,
					DWORD dwParam,
					BYTE *pbData,
					DWORD *pdwDataLen,
					DWORD dwFlags
					);

	BOOL CryptDestroyHash(
					HCRYPTHASH hHash
					);

	BOOL CryptReleaseContext(
					HCRYPTPROV hProv,
					DWORD dwFlags
					);


	BOOL CryptGenRandom(
					HCRYPTPROV hProv,
					DWORD dwLen,
					BYTE *pbBuffer
					);

	//Cryptui
	BOOL CryptUIDlgViewContext(
					DWORD dwContextType,
					const void *pvContext,
					HWND hwnd,              // Defaults to the desktop window
					LPCWSTR pwszTitle,      // Defaults to the context type title
					DWORD dwFlags,
					void *pvReserved
					);

	PCCERT_CONTEXT CryptUIDlgSelectCertificateFromStore(
					HCERTSTORE hCertStore,
					OPTIONAL HWND hwnd,              
					OPTIONAL LPCWSTR pwszTitle,
					OPTIONAL LPCWSTR pwszDisplayString,
					DWORD dwDontUseColumn,
					DWORD dwFlags,
					void *pvReserved
					);

public:
	HMODULE					LoadCrypt32();
	HMODULE					LoadCryptui();
	HMODULE					LoadAdvapi32();

private:
//	CReader_App*			m_pApp;
	HMODULE					m_hCrypt32;
	HMODULE					m_hCryptui;
	HMODULE					m_hAdvapi32;

	BOOL					m_bCrypt32Loaded;
	BOOL					m_bCryptuiLoaded;
	BOOL					m_bAdvapi32Loaded;
};


#endif