/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This demo illustrates  how  to  use  Foxit PDF SDK  to flatten  a  PDF  page, annotations and form fields to be a part of page contents. 
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfflatten
#endif


FS_RESULT FlattenPage(string fileName, string inputFilesPath, string pageRange, string outputFilesPath, FS_DWORD flags)
{
	FSDK_OutputLog("\r\n====== Flatten a PDF page ======\r\n");	//Open PDF file
	CAutoDocument autoPDFDoc((inputFilesPath + "/" + fileName).c_str(), "rb");
	if (!autoPDFDoc.GetFileHander())
		return FSCRT_ERRCODE_ERROR;

	//Load PDF document
	if (FSCRT_ERRCODE_SUCCESS != autoPDFDoc.LoadPDFDocument(NULL))
		return FSCRT_ERRCODE_ERROR;

	//Count page number
	FS_INT32 pageCount = 0;
	FS_RESULT ret = FSPDF_Doc_CountPages(autoPDFDoc.GetDocument(), &pageCount);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("\r\nFailure: In function FSPDF_Doc_CountPages with error code %d.\r\n", ret); 
		return FSCRT_ERRCODE_ERROR;
	}

	FS_INT32 nPageCount = 0;
	FS_INT32 nPageNum = 0;
	FS_INT32* nPageIndexArray = NULL;
	if (pageRange != "")
	{
		if (FSDK_GetPageRange(pageRange.c_str(), nPageIndexArray, &nPageNum))
		{
			//Count page number
			ret = FSPDF_Doc_CountPages(autoPDFDoc.GetDocument(), &nPageCount);
			if (ret != FSCRT_ERRCODE_SUCCESS)
			{
				FSDK_OutputLog("\r\nFailure: In function FSPDF_Doc_CountPages with error code %d.\r\n", ret); 
				if (nPageIndexArray) 
					delete[] nPageIndexArray;
				nPageIndexArray = NULL;
				return FSCRT_ERRCODE_ERROR;
			}
		}
	}
	if (nPageNum < 1) nPageNum = 1;

	//Load page one by one and flatten page
	for (int i = 0; i<nPageNum; i++)
	{
		FS_INT32 pageIndex;
		if (pageRange != "" && nPageIndexArray)
		{
			if (nPageIndexArray[i] >= nPageCount || nPageIndexArray[i] < 0)
			{
				FSDK_OutputLog("Failure: Page index %d is out of valid range.\r\n", nPageIndexArray[i]);
				continue;
			}
			pageIndex = nPageIndexArray[i];
		}
		else
		{
			pageIndex = i;
		}

		char buff[128];
		memset(buff, 0, sizeof(char)*128);
		sprintf(buff, "%d",pageIndex);
		string tmp = buff;
		//Get a page based on the given page index
		CAutoPDFPage autoPDFPage;
		ret = autoPDFPage.LoadPDFPage(autoPDFDoc.GetDocument(), pageIndex);
		if (FSCRT_ERRCODE_SUCCESS != ret)
			return FSCRT_ERRCODE_ERROR;

		//flatten the page
		ret = FSPDF_Page_Flatten(autoPDFPage.GetPDFPage(),flags);
		if (ret!= FSCRT_ERRCODE_SUCCESS)
		{
			FSDK_OutputLog("Failure: In function FSPDF_Page_Flatten with error code %d.\r\n", ret); 
			FSDK_CHECK_LICENSE(ret);
		}else
			FSDK_OutputLog("Success: flatten page with index %d.\r\n", pageIndex); 
	}

	if (nPageIndexArray)
	{
		delete []nPageIndexArray;
		nPageIndexArray = NULL;
	}
	string saveName;
	saveName +=string(fileName.c_str(),fileName.length()-4);
	switch (flags)
	{
	case 0 : 
		saveName += "_display.pdf";
		break;
	case 1 : 
		saveName += "_print.pdf";
		break;
	}
	string savePath = outputFilesPath + "/" + saveName;
	FSDK_SavePDFFile(autoPDFDoc.GetDocument(), savePath.c_str(),1);
	return FSCRT_ERRCODE_SUCCESS;
}

void  BrowseFolder(void* folder, string inputFilesPath, string pageRange, string outputFilesPath)
{
	FSDK_OutputLog("Enter the input folder\r\n");

	do
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if(!isDir)
		{
			// Check whether a file is PDF or not
			if(FSDK_CompareExtName(strFile.c_str(), ".pdf")) 
			{
				FS_DWORD flags[]={FSPDF_FLATTENFLAG_DISPLAY,FSPDF_FLATTENFLAG_PRINT};
				for (int i =0;i<sizeof(flags)/sizeof(FS_DWORD);i++)
				{
					FlattenPage(strFile, inputFilesPath, pageRange, outputFilesPath,flags[i]);
				}
			}

				
		}
		if (strFile.size() < 1) break;

	} while (TRUE);
}
int main(int argc, char* argv[])
{
	//Set the current executing file path.
	FSDK_SetExecuteFilePath(argc, argv);
	string inputSrcFile = "";
	string outputFolder = "";
	string pageRange = "";
	//Judge if to command line arguments. 
	//usage: pdfflatten <srcfile> /r [pagerange] /o [destfolder]
	if (argc > 1)
	{
		inputSrcFile = argv[1];
		bool bArgumentValid = true;
		switch(argc)
		{
		case 2:
			break;
		case 3:
			{
				if (0 != strcmp(argv[2], "/r") && 0 != strcmp(argv[2], "/o"))
					bArgumentValid = false;
				break;
			}
		case 4:
			{//usage: pdfflatten <srcfile> XXX XXX
				if (0 != strcmp(argv[2], "/r") && 0 != strcmp(argv[2], "/o"))
					bArgumentValid = false;
				else if (0 == strcmp(argv[2], "/r"))
				{
					if (0 != strcmp(argv[3], "/o") && '/'!=argv[3][0])
						pageRange = argv[3]; //usage: pdfflatten <srcfile> /r [pagerange]
					else if (0 != strcmp(argv[3], "/o") && '/'==argv[3][0])
						bArgumentValid = false;
				}
				else if (0 == strcmp(argv[2], "/o"))
					outputFolder = argv[3]; //usage: pdfflatten <srcfile> /o [destfolder]
				break;
			}
		case 5:
			{//usage: pdfflatten <srcfile> /r XXX XXX
				if (0 != strcmp(argv[2], "/r"))
					bArgumentValid = false;
				else 
				{
					if (0 != strcmp(argv[3], "/o"))
					{
						if (0 != strcmp(argv[4], "/o"))
							bArgumentValid = false;
						else
							pageRange = argv[3]; //usage: pdfflatten <srcfile> /r [pagerange] /o
					}
					else
						outputFolder = argv[4];//usage: pdfflatten <srcfile> /r /o [destfolder]
				}
				break;
			}
		case 6:
			{//usage: pdfflatten <srcfile> /r [pagerange] /o [destfolder]
				if (0 != strcmp(argv[2], "/r") || 0 != strcmp(argv[4], "/o"))
					bArgumentValid = false;
				else
				{
					pageRange = argv[3];
					outputFolder = argv[5];
				}
				break;
			}
		default:
			bArgumentValid = false;
		}
		if (false == bArgumentValid || !FSDK_FileExist(inputSrcFile.c_str()) || (outputFolder != "" && !FSDK_FolderExist(outputFolder.c_str())))
		{
			printf("\r\n");
			printf("usage: pdfflatten [<srcfile> /r [pagerange] /o [destfolder]] \r\n\r\n");
			printf("<srcfile>\t: Path of a PDF file as input file.\r\n");
			printf("/r [pagerange]\t: Page range, used to specify indexes of PDF pages in input PDF\r\n ");
			printf("\t\t  file, such as \"0\", \"1-3,5\". Page index starts from 0 and\r\n\t\t  should be valid within page count. Invalid index will be\r\n\t\t  ignored. "\
					"If input page range is totally invalid, page range is\r\t\t  to be first page by default.\r\n");
			printf("/o [destfolder]\t: Path of a folder as output folder. If not set, use default\r\n\t\t  output folder.\r\n\r\n");
			return -1;
		}
	}
	// Locate the path of log file
	string logPath ;
	if (outputFolder != "")
	{
		outputFolder += "/pdfflatten";
		FSDK_CreatFolder(outputFolder.c_str());
		logPath = outputFolder;
		logPath += "/log.txt";
	}
	else
	{
		outputFolder = FSDK_GetOutputFilesFolder("pdfflatten");
		logPath = FSDK_GetLogFile("pdfflatten");
	}
	
	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfflatten\r\n\r\n");

	//Init library.
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Init PDF module.
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	string inputFolder;
	string pdfName;
	if(inputSrcFile != "")
	{
		pdfName = FSDK_GetFileName(inputSrcFile.c_str());
		inputFolder = FSDK_GetCustomInputPath(inputSrcFile.c_str());
		FSDK_OutputLog("Input from folder: %s\r\n", inputFolder.c_str());
		FSDK_OutputLog("Output to folder: %s \r\n\r\n", outputFolder.c_str());
		FS_DWORD flags[] = {FSPDF_FLATTENFLAG_DISPLAY,FSPDF_FLATTENFLAG_PRINT};
		for (int i = 0;i<sizeof(flags)/sizeof(FS_DWORD);i++)
		{
			FlattenPage(pdfName, inputFolder, pageRange, outputFolder, flags[i]);
		}
	}
	else
	{
		//Get input file path.
		inputFolder = FSDK_GetInputFilesFolder("pdfflatten");
		void* folder = FSDK_OpenFolder(inputFolder.c_str(), FALSE);
		if (folder)
		{
			FSDK_OutputLog("Input from folder: %s\r\n", inputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s \r\n\r\n", outputFolder.c_str());
			BrowseFolder(folder, inputFolder, pageRange, outputFolder);
			//Close access to specific folder.
			FSDK_CloseFolder(folder);
		}
		else
		{
			pdfName = "Annot_Mixed.pdf";
			inputFolder = FSDK_GetFixFolder();
			FSDK_OutputLog("Input from folder: %s\r\n", inputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s \r\n\r\n", outputFolder.c_str());
			FS_DWORD flags[] = {FSPDF_FLATTENFLAG_DISPLAY, FSPDF_FLATTENFLAG_PRINT};
			for (int i = 0;i<sizeof(flags)/sizeof(FS_DWORD);i++)
			{
				FlattenPage(pdfName, inputFolder, pageRange, outputFolder, flags[i]);
			}
		}
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}

