/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a source to load PDF document, reflow pages and save the reflow result as a bitmap.
 */
//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfreflow
#endif

enum e_param_flag{
	PARAM_FLAG_INVALID = -1,
	PARAM_FLAG_SRCFILE ,
	PARAM_FLAG_DSTFILE  ,
	PARAM_FLAG_SIZE,
	PARAM_FLAG_MARGIN,
	PARAM_FLAG_LINESPACE,
	PARAM_FLAG_SCALE,
	PARAM_FLAG_NOIMAGE,
	PARAM_FLAG_ISSPLITSCREEN,
	PARAM_FLAG_ISCONTINUOUS
};

typedef struct _Reflow_DataMgr{
	FS_FLOAT m_fScreenWidth ;
	FS_FLOAT m_fScreenHeight ;
	FS_FLOAT m_fLineSpace ;
	FS_FLOAT m_fTopSpace ;

	FS_FLOAT m_fMarginTop;
	FS_FLOAT m_fMarginBottom;
	FS_FLOAT m_fMarginLeft;
	FS_FLOAT m_fMarginRight;

	FS_DWORD m_dwReflowFlag;// For NoImage
	FS_FLOAT m_fScale;
	FS_BOOL  m_bNoImage;
	FS_BOOL  m_bContinuous;

	std::string m_strPageRange;
	std::string m_strInputSrcFile;
	std::string m_strInputFolder;
	std::string m_strOutputFolder;
	std::string m_strPdfName;

	_Reflow_DataMgr()
	{
		m_fScreenWidth = 480; // 1024*768
		m_fScreenHeight = 800;
		m_fLineSpace = 0;
		m_fTopSpace = 0;

		m_fMarginTop	= 50;   //50,30,30,30
		m_fMarginBottom = 30;
		m_fMarginLeft	= 30;
		m_fMarginRight	= 30;

		m_dwReflowFlag= FSPDF_REFLOWFLAG_NORMAL|FSPDF_REFLOWFLAG_IMAGE|FSPDF_REFLOWFLAG_NOTRUNCATE;
		m_fScale = 1;
		m_bNoImage = FALSE;
		m_bContinuous = FALSE;

		m_strPageRange = "";
		m_strInputSrcFile = "";
		m_strInputFolder = "";
		m_strOutputFolder = "";
		m_strPdfName = "";
	}
}Reflow_DataMgr;

e_param_flag ReflowCheckParamFlag(const char * param)
{
	e_param_flag ret = PARAM_FLAG_INVALID;

	if (0 == strcmp(param,"/o"))
	{
		ret = PARAM_FLAG_DSTFILE; 
	} 
	else if(0 == strcmp(param,"/size"))
	{
		ret = PARAM_FLAG_SIZE; 
	} 
	else if(0 == strcmp(param,"/m"))
	{
		ret = PARAM_FLAG_MARGIN; 
	} 
	else if(0 == strcmp(param,"/ls"))
	{
		ret = PARAM_FLAG_LINESPACE; 
	} 
	else if(0 == strcmp(param,"/s"))
	{
		ret = PARAM_FLAG_SCALE; 
	} 
	else if(0 == strcmp(param,"/noimage"))
	{
		ret = PARAM_FLAG_NOIMAGE; 
	} 
	else if(0 == strcmp(param,"/iscontinuous"))
	{
		ret = PARAM_FLAG_ISCONTINUOUS; 
	} 

	return ret;
}

const char * GetNextFloatParamString(const char * strValue)
{
	if (NULL == strValue)
	{
		return NULL;
	}

	int i = 0;
	while(strValue[i] != ',' && strValue[i] != '\0' )
		i++;
	const char * p = strValue+i;
	if (NULL != p && p[0] != '\0' && NULL != p+1 && (p+1)[0] != '\0')
	{
		return p+1;
	}

	return NULL;
}

bool isDigitString(const char* pString,int& commaCount)
{
	if(NULL == pString)return false;

	commaCount = 0;
	int len = strlen(pString);
	if(pString[0] == ',' || pString[len] == ',')
		return false;

	int i = 0;
	for (; i < len ;i++)
	{
		if( ( pString[i] >= '0' && pString[i] <= '9')|| pString[i] == '.' ||pString[i] == ',' )
		{
			if(pString[i] == ',')commaCount++;
		}
		else
			return false;
	}
	
	return true;
}

/*usage: pdfreflow [<srcfile> /o [destfolder] /size [width,height] 
							 /m [top,bottom,left,right] /ls [linespace] /s [scale]
							 /noimage /iscontinuous]
*		such as: pdfreflow E:/Works/product/fgsdk/branch/dev/sdk_examples/example/simple_sample/input_files 
							/o E:/Works/product/fgsdk/branch/dev/sdk_examples/example/simple_sample/output_files /size 400,800  
							/m 20,30,10,30 /ls 10 /s 1 /noimage /iscontinuous
							
*		if having <srcfile> param, the position of <srcfile> must be first.

*/
int ReflowParseArgvParam(int argc,char* argv[] ,Reflow_DataMgr &reflowDataMgr)
{
	if (argc <= 1)
	{
		return PARAM_FLAG_INVALID;
	}
	e_param_flag ret = PARAM_FLAG_INVALID;
	int i =1;
	if (argv[i][0] != '/')
	{
		reflowDataMgr.m_strInputSrcFile = argv[i];
		ret = PARAM_FLAG_SRCFILE;
		i++;
	}
	else
		return ret;

	bool bStringIsDigit = false;
	int  nCommaCount = 0;

	while (i < argc)
	{
		ret = ReflowCheckParamFlag(argv[i]);
		if(PARAM_FLAG_INVALID == ret)
			break;

		if(!( PARAM_FLAG_NOIMAGE == ret|| PARAM_FLAG_ISCONTINUOUS == ret) && ( (i+1 < argc && argv[i+1][0] == '/') || (i+1 >= argc) ) )
		{
			i++;
			continue;
		}			

		/*if ( !(PARAM_FLAG_NOIMAGE == ret|| PARAM_FLAG_ISCONTINUOUS == ret) && (i+1 >= argc))
		{
		ret	= PARAM_FLAG_INVALID;
		break;
		}*/

		switch (ret)
		{
		case PARAM_FLAG_DSTFILE:
			{
				if(!FSDK_FolderExist(argv[i+1]))
				{
					ret	= PARAM_FLAG_INVALID;
					break;
				}

				reflowDataMgr.m_strOutputFolder = argv[i+1];
				i += 2;
				break;
			}
		case PARAM_FLAG_SIZE:
			{				
				bStringIsDigit = isDigitString(argv[i+1],nCommaCount);
				if(!bStringIsDigit || nCommaCount != 1)
				{
					ret	= PARAM_FLAG_INVALID;
					break;
				}
				reflowDataMgr.m_fScreenWidth = FSDK_GetFloat(argv[i+1]);
				const char * nextString = GetNextFloatParamString(argv[i+1]);
				if(nextString == NULL)
				{
					ret	= PARAM_FLAG_INVALID;
					break;
				}
				reflowDataMgr.m_fScreenHeight = FSDK_GetFloat(nextString);
				i += 2;
				break;
			}
		case PARAM_FLAG_MARGIN:
			{
				bStringIsDigit = isDigitString(argv[i+1],nCommaCount);
				if(!bStringIsDigit || nCommaCount != 3)
				{
					ret	= PARAM_FLAG_INVALID;
					break;
				}
				const char * pNextParam = argv[i+1];
				reflowDataMgr.m_fMarginTop = FSDK_GetFloat(pNextParam);

				pNextParam = GetNextFloatParamString(pNextParam);
				reflowDataMgr.m_fMarginBottom = FSDK_GetFloat(pNextParam);

				pNextParam = GetNextFloatParamString(pNextParam);
				reflowDataMgr.m_fMarginLeft = FSDK_GetFloat(pNextParam);

				pNextParam = GetNextFloatParamString(pNextParam);
				reflowDataMgr.m_fMarginRight = FSDK_GetFloat(pNextParam);
				i += 2;
				break;
			}
		case PARAM_FLAG_LINESPACE:
			{
				bStringIsDigit = isDigitString(argv[i+1],nCommaCount);
				if(!bStringIsDigit || nCommaCount != 0)
				{
					ret	= PARAM_FLAG_INVALID;
					break;
				}
				reflowDataMgr.m_fLineSpace = FSDK_GetFloat(argv[i+1]);
				if (reflowDataMgr.m_fLineSpace < 0)
				{
					reflowDataMgr.m_fLineSpace = 0;
				}
				i += 2;
				break;
			}
		case PARAM_FLAG_SCALE:
			{
				bStringIsDigit = isDigitString(argv[i+1],nCommaCount);
				if(!bStringIsDigit || nCommaCount != 0)
				{
					ret	= PARAM_FLAG_INVALID;
					break;
				}
				reflowDataMgr.m_fScale = FSDK_GetFloat(argv[i+1]);
				if (reflowDataMgr.m_fScale <= 0)
				{
					reflowDataMgr.m_fScale = 1;
				}

				i += 2;
				break;
			}
		case PARAM_FLAG_NOIMAGE:
			{
				reflowDataMgr.m_dwReflowFlag = reflowDataMgr.m_dwReflowFlag&(~FSPDF_REFLOWFLAG_IMAGE) ;
				reflowDataMgr.m_bNoImage = TRUE;
				i += 1;
				break;
			}
		case PARAM_FLAG_ISCONTINUOUS:
			{
				reflowDataMgr.m_bContinuous = TRUE;
				i += 1;
				break;
			}
		default:
			ret	= PARAM_FLAG_INVALID;
			break;
		}
		if (PARAM_FLAG_INVALID == ret)
		{
			break;
		}
	}
	return ret;
}

class CPDFReflow
{
public:
	CPDFReflow();
	~CPDFReflow();
	void		BrowerFolder(void* folder);
	void		ReflowPDFFile(string strPdfFile);
	FS_RESULT	LoadPDFFile(string fileName);

	FS_RESULT	CalculateShowRegion(CAutoReflowPage* autoReflowPage,FSCRT_RECT &rectRegion);
	void		SetRestOfHeight(FS_FLOAT fRestOfHeight);
	FS_FLOAT	GetRestOfHeight();
	void		SetImageVisibleHeight(FS_FLOAT fImageVisibleHeight);
	FS_FLOAT	GetVisibleHeight();

public:
	string m_strPageRange;
	string m_strInputSrcFile;
	string m_strInputFolder;
	string m_strOutputFolder;
	string m_strPdfName;

	Reflow_DataMgr m_sReflowDataMgr;
	FS_FLOAT m_fRestOfHeight;
	FS_FLOAT m_fImageVisibleHeight;
};

CPDFReflow::CPDFReflow()
	:m_strPageRange("")
	,m_strInputSrcFile("")
	,m_strInputFolder("")
	,m_strOutputFolder("")
	,m_strPdfName("")
	,m_fRestOfHeight(0)
	,m_fImageVisibleHeight(0)
{

};

CPDFReflow::~CPDFReflow()
{

};

//Load PDF document. If load PDF document successfully, do reflow process for the first page.
FS_RESULT CPDFReflow::LoadPDFFile(string fileName)
{
	//Get path of input files folder of this demo.
	string strFile = m_strInputFolder;
	//Set path of input file.
	strFile += "/";
	strFile += fileName;

	//Open input file and create file handle.
	CAutoDocument autoDocument(strFile.c_str(), "rb");
	//Load PDF document handle.
	FS_RESULT ret = autoDocument.LoadPDFDocument(NULL);
	if (FSCRT_ERRCODE_SUCCESS != ret) return ret;

	// rewrite for reflow
	FS_INT32 nPageCount = 0;
	FS_INT32 nPageIndex = 0;
	FS_INT32 nImageIndex = 1;

	char buff[20] = "";
	string strOutBmpFileName  ;
	string strOutBmpFilePre = string(fileName.c_str(), fileName.length()-4);
	strOutBmpFilePre += "_";	

	nPageCount = autoDocument.GetPagesCount();

	if (0 == nPageCount)
	{
		return FSCRT_ERRCODE_ERROR;
	}  

	FS_FLOAT fReflowScreenWidth = 0;
	FS_FLOAT fReflowScreenHeight = 0;

	fReflowScreenWidth  = (m_sReflowDataMgr.m_fScreenWidth  - m_sReflowDataMgr.m_fMarginLeft - m_sReflowDataMgr.m_fMarginRight );
	fReflowScreenHeight = (m_sReflowDataMgr.m_fScreenHeight - m_sReflowDataMgr.m_fMarginTop  - m_sReflowDataMgr.m_fMarginBottom) ;

	FSCRT_RECT rectMargin;
	FSCRT_RECT rectRegion;

	rectMargin.left  = (FS_INT32)m_sReflowDataMgr.m_fMarginLeft;
	rectMargin.right = (FS_INT32)m_sReflowDataMgr.m_fMarginRight;
	rectMargin.top   = (FS_INT32)m_sReflowDataMgr.m_fMarginTop;
	rectMargin.bottom = (FS_INT32)m_sReflowDataMgr.m_fMarginBottom;

	// input param will with "/iscontinuous" 
	// this part code is wrote for displaying reflowed content on the splited screen
	do{					   
		CAutoPDFPage autoPage;
		CAutoReflowPage autoReflowpage;
		CAutoImage autoImage;				

		autoReflowpage.SetScreenParam(fReflowScreenWidth/m_sReflowDataMgr.m_fScale, fReflowScreenHeight/m_sReflowDataMgr.m_fScale);
		autoReflowpage.SetLineSpace( m_sReflowDataMgr.m_fLineSpace);
		autoReflowpage.SetLastYPixel( m_sReflowDataMgr.m_fMarginTop);
		autoReflowpage.SetReflowFlag(m_sReflowDataMgr.m_dwReflowFlag);

		ret = autoImage.CreateRenderResource(m_sReflowDataMgr.m_fScreenWidth,m_sReflowDataMgr.m_fScreenHeight,rectMargin);
		if (FSCRT_ERRCODE_SUCCESS != ret) 
			break;

		ret = autoImage.FillBitMapWithWhite();
		if (FSCRT_ERRCODE_SUCCESS != ret)
			break;

		this->SetImageVisibleHeight(fReflowScreenHeight);
		this->SetRestOfHeight(fReflowScreenHeight);

		do 
		{
			if (  autoReflowpage.GetLastYPixel() - m_sReflowDataMgr.m_fMarginTop >= 0 && nPageIndex >= nPageCount)
			{
				break;
			}

			if ( autoReflowpage.GetLastYPixel() - m_sReflowDataMgr.m_fMarginTop >= 0 && nPageIndex < nPageCount)
			{
				// before loading PDF Page, autoReflowpage object should release the resource of the last reflow page
				autoReflowpage.ReleaseReflowPage();
				ret = autoPage.LoadPDFPage(autoDocument.GetDocument(),nPageIndex++);
				if (FSCRT_ERRCODE_SUCCESS != ret) 
					break;
				//Before create a reflow page from a PDF page, the PDF page should be pared successfully first.
				ret = autoPage.ParsePage();
				if (FSCRT_ERRCODE_SUCCESS != ret) 
					break;

				if (m_sReflowDataMgr.m_bContinuous )
				{
					autoReflowpage.SetTopSpace( (GetVisibleHeight() - GetRestOfHeight())/m_sReflowDataMgr.m_fScale + m_sReflowDataMgr.m_fLineSpace);
				}

				autoReflowpage.SetLastYPixel(m_sReflowDataMgr.m_fMarginTop);
				this->SetRestOfHeight(fReflowScreenHeight);
				if (!m_sReflowDataMgr.m_bContinuous )
				{	
					// single page ,split screen, argv Param with  no /iscontinuous 
					nImageIndex = 1;
				}

				ret = autoReflowpage.CreateReflowPage(autoPage.GetPDFPage());
				if (FSCRT_ERRCODE_SUCCESS != ret) 
					break;
			}

			if ( ( autoReflowpage.GetContentHeight() )  <= 0 )
			{
				break;
			}

			this->CalculateShowRegion(&autoReflowpage,rectRegion);

			ret = autoImage.ReflowPageToBitmap(&autoReflowpage,rectRegion);
			if (FSCRT_ERRCODE_SUCCESS != ret) 
				break;

			if (!m_sReflowDataMgr.m_bContinuous )
			{
				// single page ,split screen,argv Param with  no /iscontinuous
				this->SetRestOfHeight(0);
			}

			if (0 == this->GetRestOfHeight() || (this->GetRestOfHeight() > 0 && nPageIndex >= nPageCount))
			{
				//Set output file path.						
				strOutBmpFileName = strOutBmpFilePre;
				if (!m_sReflowDataMgr.m_bContinuous)
				{
					// single page ,split screen OR no split screen
					strOutBmpFileName += "s&";
					strOutBmpFileName += "screen_";

					sprintf(buff,"%d_",nPageIndex);
					strOutBmpFileName += buff;
				}
				else
				{
					// continuous page ,split screen
					strOutBmpFileName += "c&";
					strOutBmpFileName += "screen_";
				}

				sprintf(buff,"%d",nImageIndex++);
				strOutBmpFileName += buff;
				string strOutfile = m_strOutputFolder + "/" + strOutBmpFileName + ".bmp";
				//Save reflow result to a .bmp file.
				FSDK_SaveImageFile(autoImage.GetBitmap(), FSCRT_IMAGETYPE_BMP,(FS_LPSTR)strOutfile.c_str());	

				this->SetRestOfHeight(fReflowScreenHeight);
				ret = autoImage.FillBitMapWithWhite();
				if (FSCRT_ERRCODE_SUCCESS != ret) 
					break;

			}

		} while (1);		  

	}while(0);

	return ret;	 
}


void CPDFReflow::ReflowPDFFile(string pdfFile)
{

	if (pdfFile.size() > 0 && FSDK_CompareExtName(pdfFile.c_str(), ".pdf"))
	{
		//Load PDF file and if load successfully, do reflow process.
		LoadPDFFile(pdfFile);
	}
}

void CPDFReflow::BrowerFolder(void* folder)
{
	FSDK_OutputLog("success\r\n");
	do
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if(!isDir)
		{
			ReflowPDFFile(strFile);
		}
		if (strFile.size() < 1) break;

	} while (1);
}

void CPDFReflow::SetRestOfHeight(FS_FLOAT fRestOfHeight)
{
	m_fRestOfHeight = fRestOfHeight;
}

FS_FLOAT CPDFReflow::GetRestOfHeight()
{
	return m_fRestOfHeight;
}

void CPDFReflow::SetImageVisibleHeight(FS_FLOAT fImageVisibleHeight)
{
	m_fImageVisibleHeight = fImageVisibleHeight;
}

FS_FLOAT CPDFReflow::GetVisibleHeight()
{
	return m_fImageVisibleHeight;
}

FS_RESULT CPDFReflow::CalculateShowRegion(CAutoReflowPage* autoReflowPage,FSCRT_RECT &rectRegion)
{
	FS_INT32 nX = (FS_INT32)(m_sReflowDataMgr.m_fMarginLeft);   
	FS_INT32 nY = (FS_INT32)(autoReflowPage->GetLastYPixel() );
	FS_FLOAT fWidth  = autoReflowPage->GetContentWidth()  * m_sReflowDataMgr.m_fScale;
	FS_FLOAT fHeight = autoReflowPage->GetContentHeight() * m_sReflowDataMgr.m_fScale;

	FS_FLOAT fDifValue = 0;

	fDifValue = this->GetVisibleHeight() - (autoReflowPage->GetLastYPixel() - m_sReflowDataMgr.m_fMarginTop ) - fHeight ;

	//calculate Height
	if ( fDifValue > 0)
	{
		this->m_fRestOfHeight = fDifValue;
		autoReflowPage->SetLastYPixel(autoReflowPage->GetLastYPixel() + fHeight );
	} 
	else if( fDifValue < 0)
	{
		if (autoReflowPage->GetLastYPixel() - m_sReflowDataMgr.m_fMarginTop > 0)
		{
			autoReflowPage->SetLastYPixel( m_sReflowDataMgr.m_fMarginTop);
		}
		autoReflowPage->SetLastYPixel( autoReflowPage->GetLastYPixel() - this->m_fRestOfHeight  );
		this->m_fRestOfHeight = 0;		
	}
	else
	{
		this->m_fRestOfHeight = 0;
		autoReflowPage->SetLastYPixel( m_sReflowDataMgr.m_fMarginTop);
	}

	rectRegion.left = nX;
	rectRegion.top  = nY;
	rectRegion.right = (FS_INT32)(nX + fWidth);
	rectRegion.bottom = (FS_INT32)(nY + fHeight);

	return FSCRT_ERRCODE_SUCCESS;
}

/*usage: pdfreflow [<srcfile> /o [destfolder] /size [width,height]  
							 /m [top,bottom,left,right] /ls [linespace] /s [scale]
							 /noimage  /iscontinuous]
*		such as: pdfreflow E:/Works/product/fgsdk/branch/dev/sdk_examples/example/simple_sample/input_files 
							/o E:/Works/product/fgsdk/branch/dev/sdk_examples/example/simple_sample/output_files /size 400,800  
							/m 20,30,10,30 /ls 10 /s 1 /noimage /iscontinuous
							
*		if having <srcfile> param, the position of <srcfile> must be first.

*/
int main(int argc, char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);
	CPDFReflow pdfReflow;
	//Judge if to command line arguments. 
	if (argc > 1)
	{
		// Parse the input  parameters
		int ret = ReflowParseArgvParam(argc,argv,pdfReflow.m_sReflowDataMgr);
		if (PARAM_FLAG_INVALID != ret)
		{
			// comment out this line , will ignore the input param "input_files" , in next part of code , will use default value
			pdfReflow.m_strInputSrcFile = pdfReflow.m_sReflowDataMgr.m_strInputSrcFile; 
			pdfReflow.m_strOutputFolder = pdfReflow.m_sReflowDataMgr.m_strOutputFolder;
		}
		if (PARAM_FLAG_INVALID == ret 
			|| (pdfReflow.m_strInputSrcFile != "" && !FSDK_FileExist(pdfReflow.m_strInputSrcFile.c_str())  )
			|| (pdfReflow.m_strOutputFolder != "" && !FSDK_FolderExist(pdfReflow.m_strOutputFolder.c_str())) )
		{
			printf("\r\n");
			printf("Input parameters make error.Please read the following descriptions.\r\n");
			printf("usage: pdfreflow [<srcfile> /o [destfolder] /size [width,height] \r\n\t\t");
			printf("/m [top,bottom,left,right] /ls [linespace] \r\n\t\t");
			printf("/s [scale] /noimage /iscontinuous] \r\n\r\n");
			printf("<srcfile> \t: Path of a PDF file as input file,at the first position.\r\n");
			printf("/o [destfolder]\t: Path of a folder as output folder. If not set, use default\r\n\t\t  output folder.\r\n");
			printf("/size [width,height] : the size of the screen, width and height, default value\r\n\t\t  is 480*800.\r\n");
			printf("/m [top,bottom,left,right] : the margin of screen, default value is \r\n\t\t  [50,30,30,30]. also can set other values.\r\n");
			printf("/ls [linespace] : the space between the lines, default value is input by sdk. \r\n\t\t  you can set the value more than 1.\r\n");
			printf("/s [scale] \t: the size of scaling, default value is 1. suggest set the\r\n\t\t  value between 0.6 and 5, ");
			printf("in the range, the size will be\r\n\t\t  suitable for reading.\r\n");
			printf("/noimage \t: With the parameter, all images in the pages won't be showed;\r\n\t\t  default without this value.\r\n");
			printf("/iscontinuous \t: With the parameter, if the contents of the page only render\r\n\t\t  a part of the screen,");
			printf(" than will render the contents of the\r\n\t\t  next page on the rest of the screen; default without this \r\n\t\t  value.\r\n\r\n");
			return -1;
		}
	}
	// Locate the path of log file
	string logPath ;
	if (pdfReflow.m_strOutputFolder != "")
	{
		pdfReflow.m_strOutputFolder += "/pdfreflow";
		FSDK_CreatFolder(pdfReflow.m_strOutputFolder.c_str());

		if (pdfReflow.m_sReflowDataMgr.m_bNoImage)
		{
			pdfReflow.m_strOutputFolder += "/NoImage";
			FSDK_CreatFolder(pdfReflow.m_strOutputFolder.c_str());
		}
		else
		{
			pdfReflow.m_strOutputFolder += "/HasImage";
			FSDK_CreatFolder(pdfReflow.m_strOutputFolder.c_str());
		}
		logPath = pdfReflow.m_strOutputFolder;
		logPath += "/log.txt";
	}
	else
	{
		pdfReflow.m_strOutputFolder = FSDK_GetOutputFilesFolder("pdfreflow");

		if (pdfReflow.m_sReflowDataMgr.m_bNoImage)
		{
			pdfReflow.m_strOutputFolder += "/NoImage";
			FSDK_CreatFolder(pdfReflow.m_strOutputFolder.c_str());
		}
		else
		{
			pdfReflow.m_strOutputFolder += "/HasImage";
			FSDK_CreatFolder(pdfReflow.m_strOutputFolder.c_str());
		}

		logPath = FSDK_GetLogFile("pdfreflow");
	}
	
	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfreflow\r\n\r\n");

	//Initialize SDK library, or no method of SDK can be used.
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}
	//Initialize PDF module, or no method of PDF module can be used.
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(pdfReflow.m_strOutputFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	if (pdfReflow.m_strInputSrcFile != "")
	{
		pdfReflow.m_strPdfName = FSDK_GetFileName(pdfReflow.m_strInputSrcFile.c_str());
		if (0 == pdfReflow.m_strPdfName.size())
		{
			pdfReflow.m_strInputFolder = pdfReflow.m_strInputSrcFile;
			void* folder = FSDK_OpenFolder(pdfReflow.m_strInputFolder.c_str(), FALSE);
			if (folder)
			{
				FSDK_OutputLog("Input from folder: %s\r\n", pdfReflow.m_strInputFolder.c_str());
				FSDK_OutputLog("Output to folder: %s\r\n", pdfReflow.m_strOutputFolder.c_str());
				FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfreflow: ");
				pdfReflow.BrowerFolder(folder);
				//Close access to specific folder.
				FSDK_CloseFolder(folder);
			}
		} 
		else
		{
			pdfReflow.m_strInputFolder = FSDK_GetCustomInputPath(pdfReflow.m_strInputSrcFile.c_str());
			FSDK_OutputLog("Input from folder: %s\r\n", pdfReflow.m_strInputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s\r\n", pdfReflow.m_strOutputFolder.c_str());
			pdfReflow.ReflowPDFFile(pdfReflow.m_strPdfName);
		}

	}
	else
	{
		//Get path of input file folder for this demo.
		pdfReflow.m_strInputFolder = FSDK_GetInputFilesFolder("pdfreflow");
		void* folder = FSDK_OpenFolder(pdfReflow.m_strInputFolder.c_str(), FALSE);
		if (folder)
		{
			FSDK_OutputLog("Input from folder: %s\r\n", pdfReflow.m_strInputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s\r\n", pdfReflow.m_strOutputFolder.c_str());
			FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfreflow: ");
			pdfReflow.BrowerFolder(folder);
			//Close access to specific folder.
			FSDK_CloseFolder(folder);
		}
		else
		{
			pdfReflow.m_strInputFolder = FSDK_GetFixFolder();
			FSDK_OutputLog("Input from folder: %s\r\n", pdfReflow.m_strInputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s\r\n", pdfReflow.m_strOutputFolder.c_str());
			pdfReflow.m_strPdfName = "testfile.pdf";
			pdfReflow.ReflowPDFFile(pdfReflow.m_strPdfName);
		}
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}
