/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This demo illustrates how  to  use  Foxit PDF SDK  to create  a  wrapper PDF document and open a wrapper document.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfwrapper
#endif

typedef struct _WRAPPERP_FILE
{
	FILE*				fileStream;
	FSCRT_FILEHANDLER	fileHandler;
	FSCRT_FILESIZE		offset;
}WRAPPERP_FILE;

class CAutoWrapper
{
public:
	CAutoWrapper();

	static FS_RESULT	ReadBlock(FS_LPVOID clientData, FS_DWORD offset, FS_LPVOID buffer, FS_DWORD size);
	static void			Release(FS_LPVOID clientData);
	static FS_DWORD		GetSize(FS_LPVOID clientData);
	static FS_RESULT	WriteBlock(FS_LPVOID clientData, FS_DWORD offset, FS_LPCVOID buffer, FS_DWORD size);
	static FS_RESULT	Flush(FS_LPVOID clientData);
	static FS_RESULT	Truncate(FS_LPVOID clientData, FS_DWORD size);

	FS_RESULT			RenderDoc(FSCRT_DOCUMENT pdfDocument,string outputFilesPath,int flags);
	FS_RESULT			CreateWrapperDoc(string wrapperFileName, string realFileName, string inputFilesPath, string outputFilesPath);
	void				OpenWrapperDoc(string warpperfile, string inputFilesPath, string outputFilesPath);

	int m_Warpper;
	int m_Real;
};

CAutoWrapper::CAutoWrapper()
{
	m_Warpper = 0;
	m_Real = 1;
}

FS_RESULT CAutoWrapper::ReadBlock(FS_LPVOID clientData, FS_DWORD offset, FS_LPVOID buffer, FS_DWORD size)
{//Check parameter
	if (!clientData) return FSCRT_ERRCODE_FILE;

	//Change file pointer to offset
	fseek(((WRAPPERP_FILE*)clientData)->fileStream, (long)offset, SEEK_SET);
	//Read data
	size_t readSize = fread(buffer, 1, size, ((WRAPPERP_FILE*)clientData)->fileStream);
	return readSize == size ? FSCRT_ERRCODE_SUCCESS : FSCRT_ERRCODE_FILE;
}

void CAutoWrapper::Release(FS_LPVOID clientData)
{
	if (!clientData) return;
	//Close file stream
	fclose(((WRAPPERP_FILE*)clientData)->fileStream);
	//Free file data
	free((WRAPPERP_FILE*)clientData);
}
FS_DWORD CAutoWrapper::GetSize(FS_LPVOID clientData)
{
	return (FS_DWORD)((WRAPPERP_FILE*)clientData)->offset.loSize;
}

FS_RESULT CAutoWrapper::WriteBlock(FS_LPVOID clientData, FS_DWORD offset, FS_LPCVOID buffer, FS_DWORD size)
{
	//Check parameter
	if (!clientData) return FSCRT_ERRCODE_FILE;

	//Change file pointer to offset
	fseek(((WRAPPERP_FILE*)clientData)->fileStream, (long)offset, SEEK_SET);
	//Write data
	size_t writeSize = fwrite(buffer, 1, size, ((WRAPPERP_FILE*)clientData)->fileStream);
	return writeSize == size ? FSCRT_ERRCODE_SUCCESS : FSCRT_ERRCODE_FILE;
}

FS_RESULT CAutoWrapper::Flush(FS_LPVOID clientData)
{
	//Check parameter
	if (!clientData) return FSCRT_ERRCODE_FILE;

	//Flush file
	fflush(((WRAPPERP_FILE*)clientData)->fileStream);

	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT CAutoWrapper::Truncate(FS_LPVOID clientData, FS_DWORD size)
{
    return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT CAutoWrapper::RenderDoc(FSCRT_DOCUMENT pdfDocument,string outputFilesPath,int flags)
{
	CAutoPDFPage autoPDFPage;
	FS_RESULT ret = autoPDFPage.LoadPDFPage(pdfDocument, 0);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		return ret;

	ret = autoPDFPage.ParsePage();
	if (FSCRT_ERRCODE_SUCCESS != ret)
		return ret;
	CAutoImage Image;
	Image.PageToBitmap(autoPDFPage.GetPDFPage());

	string imageFile;
	if (flags == m_Warpper)
		imageFile = "Wrapper.bmp";
	else if (flags == m_Real)
		imageFile += "RealPage.bmp";

	ret = FSDK_SaveImageFile(Image.GetBitmap() ,FSCRT_IMAGETYPE_BMP,(outputFilesPath+"/"+ imageFile).c_str());
	return ret;
}

void CAutoWrapper::OpenWrapperDoc(string warpperfile, string inputFilesPath, string outputFilesPath)
{
	FSDK_OutputLog("\r\n====== Start to open wrapper document ======\r\n");
	FS_RESULT ret;
	{
		//for normal open
		CAutoDocument pdfDoc((outputFilesPath + "/" + warpperfile).c_str(), "rb");
		if (!pdfDoc.GetFileHander())
			return;

		if (FSCRT_ERRCODE_SUCCESS != pdfDoc.LoadPDFDocument(NULL))
			return;

		ret = RenderDoc(pdfDoc.GetDocument(), outputFilesPath, m_Warpper);
	}

	FSDK_OutputLog("\r\n====== Create the wrapper document ======\r\n");

	//open really document
	CAutoDocument wrapperdocument((outputFilesPath + "/" + warpperfile).c_str(), "rb");
	if (!wrapperdocument.GetFileHander())
		return;

	if (FSCRT_ERRCODE_SUCCESS != wrapperdocument.LoadPDFDocument(NULL))
		return;

	FSPDF_WRAPPERDATA wrapperData;
	FS_BOOL isWrapper = false;
	ret = FSPDF_Doc_IsWrapper(wrapperdocument.GetDocument(), &isWrapper);
	if (false == isWrapper)
	{
		FSDK_OutputLog("Failure: This's not a wrapper document.\r\n");
		return;
	}
	FSCRT_FILESIZE offset;
	ret = FSPDF_Doc_GetWrapperOffset(wrapperdocument.GetDocument(), &offset);
	ret = FSPDF_Doc_GetWrapperData(wrapperdocument.GetDocument(), &wrapperData);
	if (FSCRT_ERRCODE_SUCCESS == ret){
		FSDK_OutputLog("Success: The wrapper data information:\r\n");
		FSDK_OutputLog("\tversion = %d\r\n", wrapperData.version);
		FSDK_OutputLog("\tappID = %s\r\n", wrapperData.appID.str);
		FSDK_OutputLog("\tdescription = %s\r\n", wrapperData.description.str);
		FSDK_OutputLog("\ttype = %s\r\n", wrapperData.type.str);
		FSDK_OutputLog("\turi = %s\r\n", wrapperData.uri.str);
	}
	FILE* fileStream = fopen((outputFilesPath + "/" + warpperfile).c_str(), "rb");
	WRAPPERP_FILE* fileData = (WRAPPERP_FILE*)malloc(sizeof(WRAPPERP_FILE));
	fileData->fileHandler.clientData = fileData;
	fileData->fileHandler.ReadBlock = ReadBlock;
	fileData->fileHandler.Release = Release;
	fileData->fileHandler.GetSize = GetSize;
	fileData->fileHandler.Flush = Flush;
	fileData->fileHandler.WriteBlock = WriteBlock;
	fileData->fileHandler.Truncate = Truncate;
	fileData->fileStream = fileStream;
	fileData->offset = offset;

	FSCRT_FILE file = NULL;
	ret = FSCRT_File_Create(&fileData->fileHandler, &file);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure:In function FSCRT_File_Create with error code %d.\r\n", ret);
		return;
	}
	FSDK_OutputLog("Success: In function FSCRT_File_Create.\r\n"); 

	FSCRT_DOCUMENT realDoc = NULL;
	ret = FSPDF_Doc_StartLoad(file, NULL, &realDoc, NULL);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Doc_StartLoad with error code %d.\r\n", ret);
		return;
	}
	FSDK_OutputLog("Success: In function FSPDF_Doc_StartLoad.\r\n"); 

	ret = RenderDoc(realDoc, outputFilesPath, m_Real);

	FSPDF_WrapperData_Clear(&wrapperData);
	FSPDF_Doc_Close(realDoc);
	FSCRT_File_Release(file);
}

FS_RESULT CAutoWrapper::CreateWrapperDoc(string wrapperFileName, string realFileName, string inputFilesPath, string outputFilesPath)
{
	FSDK_OutputLog("\r\n====== Create the wrapper document ======\r\n");
	//Open PDF file
	CAutoDocument pdfDoc((inputFilesPath + "/" + realFileName).c_str(), "rb");
	if (!pdfDoc.GetFileHander())
		return FSCRT_ERRCODE_ERROR;

	//Load PDF document
	if (FSCRT_ERRCODE_SUCCESS != pdfDoc.LoadPDFDocument(NULL))
		return FSCRT_ERRCODE_ERROR;

	string savePath = outputFilesPath + "/" + "FoxitWrapper.pdf";
	FS_BOOL bRet = FSDK_SavePDFFile(pdfDoc.GetDocument(), savePath.c_str());
	if (!bRet)
		return FSCRT_ERRCODE_ERROR;

	//open wrapper file
	CAutoDocument wrapDoc((inputFilesPath + "/" + wrapperFileName).c_str(), "rb");
	if (!wrapDoc.GetFileHander())
		return FSCRT_ERRCODE_ERROR;

	//Load PDF document
	if (FSCRT_ERRCODE_SUCCESS != wrapDoc.LoadPDFDocument(NULL))
		return FSCRT_ERRCODE_ERROR;

	string realfilePath = outputFilesPath + "/" + "FoxitWrapper.pdf";
	FSCRT_FILE pdfFile = FSDK_OpenFile(realfilePath.c_str(), "rb+");
	if (!pdfFile)
		return FSCRT_ERRCODE_ERROR;

	FSDK_OutputLog("Success: Create a wrapper document.\r\n");
    FSCRT_BSTRC(bstr, "Foxit");
	FSPDF_WRAPPERDATA wrapperData;
	wrapperData.appID = bstr;
	wrapperData.description = bstr;
	wrapperData.type = bstr;
    FSCRT_BSTRC(bstr2, "www.foxitsoftware.com");
	wrapperData.uri = bstr2;
	wrapperData.version = 10;
	FS_RESULT ret = FSPDF_Doc_SaveAsWrapperFile(wrapDoc.GetDocument(), pdfFile, &wrapperData, 0, NULL);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Doc_SaveAsWrapperFile with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
	}
	else
		FSDK_OutputLog("Succeed. Save the wrapper document to %s.\r\n", FSDK_GetFileName(realfilePath.c_str()).c_str());
	FSCRT_File_Release(pdfFile);
	return ret;
}

void WrapperFolder(void* folder, string inputFilesPath, string outputFilesPath)
{
	do
	{//Enumerate files in input file folder and choose PDF file to load
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if (strFile.size() > 0 && !isDir)
		{
			if (FSDK_CompareExtName(strFile.c_str(), ".pdf"))
			{
				CAutoWrapper autoWrapper;
				string WrapperDoc = "wrapper.pdf";
				autoWrapper.CreateWrapperDoc(WrapperDoc, strFile, inputFilesPath, outputFilesPath);
				autoWrapper.OpenWrapperDoc(WrapperDoc, inputFilesPath, outputFilesPath);
			}
		}

		if (strFile.size() < 1) break;
	} while (1);
}

int main(int argc,char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);
	string logPath = FSDK_GetLogFile("pdfwrapper");
	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfwrapper\r\n\r\n");

	//Initialize SDK library, or no method of SDK can be used
	bool bRet = FSDK_InitializeLibray(false);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module, or no method of PDF module can be used
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}
	//Get path of input file folder for this demo
	string inputFilesPath = FSDK_GetInputFilesFolder("pdfwrapper");
	string outputFilesPath = FSDK_GetOutputFilesFolder("pdfwrapper");

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFilesPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	void* folder = FSDK_OpenFolder(inputFilesPath.c_str(), FALSE);
	if (folder)
	{
		FSDK_OutputLog("Input from folder: %s\r\n", inputFilesPath.c_str());
		FSDK_OutputLog("Output to folder: %s \r\n\r\n", outputFilesPath.c_str());
		WrapperFolder(folder, inputFilesPath, outputFilesPath);
		//Close access to specific folder
		FSDK_CloseFolder(folder);
	}
	else
	{
			CAutoWrapper autoWrapper;
			string inputFilesPath = FSDK_GetFixFolder();
			FSDK_OutputLog("Input from folder: %s\r\n", inputFilesPath.c_str());
			FSDK_OutputLog("Output to folder: %s \r\n\r\n", outputFilesPath.c_str());
			string pdfName = "AboutFoxit.pdf";
			string WrapperDoc = "wrapper.pdf";
			autoWrapper.CreateWrapperDoc(WrapperDoc, pdfName, inputFilesPath, outputFilesPath);
			string wrapperfileName = "FoxitWrapper.pdf";
			autoWrapper.OpenWrapperDoc(wrapperfileName, inputFilesPath, outputFilesPath);
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}


