#include "StdAfx.h"
#include "TreeView.h"

//////////////////////////////////////////////////////////////////////////
BEGIN_MESSAGE_MAP(CBookmark_TreeCtrl, CTreeCtrl)	
	ON_WM_SIZE()
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_CREATE()
END_MESSAGE_MAP()


void CBookmark_TreeCtrl::OnSize(UINT nType, int cx, int cy)
{
	CTreeCtrl::OnSize(nType, cx, cy);
}

void CBookmark_TreeCtrl::OnDestroy()
{
	CTreeCtrl::OnDestroy();
}

void CBookmark_TreeCtrl::OnPaint()
{
	CTreeCtrl::OnPaint();
}

void CBookmark_TreeCtrl::OnMouseMove(UINT nFlags, CPoint point)
{
	//	AfxMessageBox(L"Mousemove");
}

void CBookmark_TreeCtrl::OnLButtonDown(UINT nFlags, CPoint point)
{
	CTreeCtrl::OnLButtonDown(nFlags, point);
	m_pPage->OnLButtonDown(nFlags, point);
}

void CBookmark_TreeCtrl::OnLButtonUp(UINT nFlags, CPoint point)
{
	
}

int  CBookmark_TreeCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if(-1 == CTreeCtrl::OnCreate(lpCreateStruct))
		return -1;
	
	return 0;
	
}
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
CBookmark_TreeView::CBookmark_TreeView(IReader_App* pApp, FSCRT_DOCUMENT doc)
{
	m_pApp = pApp;
	m_pPDFDoc = doc;
	m_iteratorBookmark = NULL;
	FSPDF_Doc_CreateBookmarkIterator(m_pPDFDoc, &m_iteratorBookmark);

	m_pTreeCtrl = new CBookmark_TreeCtrl;
}

CBookmark_TreeView::~CBookmark_TreeView()
{
	delete m_pTreeCtrl;
	if (m_iteratorBookmark != NULL)
	{
		FSPDF_Bookmark_ReleaseIterator(m_iteratorBookmark);
		m_iteratorBookmark = NULL;
	}
}

BEGIN_MESSAGE_MAP(CBookmark_TreeView, CWnd)	
	ON_WM_SIZE()
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_CREATE()
	
END_MESSAGE_MAP()


void CBookmark_TreeView::OnSize(UINT nType, int cx, int cy)
{
//	this->MoveWindow(0, 0, cx, cy);
	m_pTreeCtrl->MoveWindow(0, 0, cx, cy);
	CWnd::OnSize(nType, cx, cy)	;
}

void CBookmark_TreeView::OnDestroy()
{
	m_pTreeCtrl->DestroyWindow();
	CWnd::OnDestroy();
}

void CBookmark_TreeView::OnPaint()
{
	CPaintDC dc(this);
	CRect rc;
	this->GetClientRect(&rc);
	dc.FillSolidRect(&rc, RGB(100,0,100));
}

void CBookmark_TreeView::OnMouseMove(UINT nFlags, CPoint point)
{

}

void CBookmark_TreeView::OnLButtonDown(UINT nFlags, CPoint point)
{	
		HTREEITEM hItem = m_pTreeCtrl->GetSelectedItem();
	    int index = (int)m_pTreeCtrl->GetItemData(hItem);
		IReader_Document* pReaderDocument = m_pApp->GetCurrentDocument();
		if(pReaderDocument && index >= 0)
		{
			pReaderDocument->GotoPage(index);
		}
		return;
}

void CBookmark_TreeView::OnLButtonUp(UINT nFlags, CPoint point)
{

}

int  CBookmark_TreeView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if(-1 == CWnd::OnCreate(lpCreateStruct))
		return -1;

	m_pTreeCtrl->Create(WS_VISIBLE | WS_TABSTOP | WS_CHILD | WS_BORDER
		| TVS_HASBUTTONS | TVS_LINESATROOT | TVS_HASLINES
		| TVS_DISABLEDRAGDROP,
		CRect(0, 0, 0, 0), this, 0x1005);

	m_pTreeCtrl->SetTreeView(this);
	
	if(PDDocHasBookmark())
	{
		FSPDF_Bookmark_MoveToRoot(m_iteratorBookmark);
		this->BuildBookmark(NULL);
	}
	else
	{
		this->BuildPageTree();
	}
	
	return 0;

}

BOOL CBookmark_TreeView::PDDocHasBookmark()
{
	FS_RESULT hResult = FSPDF_Bookmark_MoveToFirstChild(m_iteratorBookmark);
	if (hResult == FSCRT_ERRCODE_SUCCESS)
	{
		return TRUE;
	}
	return FALSE;
}

void CBookmark_TreeView::BuildPageTree()
{
	TVINSERTSTRUCT tvInsert;
	tvInsert.hParent = NULL;
	tvInsert.hInsertAfter = NULL;
	tvInsert.item.mask = TVIF_TEXT;
	tvInsert.item.pszText = _T("Bookmark");
	HTREEITEM hRoot = m_pTreeCtrl->InsertItem(&tvInsert);

	int nCount = 0;
	FSPDF_Doc_CountPages(m_pPDFDoc, &nCount);
	for (int i=0; i<nCount; i++)
	{
		CString str;
		str.Format(L"Page %d", i+1);
		HTREEITEM hItem = m_pTreeCtrl->InsertItem(TVIF_TEXT, str, 0, 0, 0, 0, 0, hRoot, NULL);
		m_pTreeCtrl->SetItemData(hItem, i);

	}
	m_pTreeCtrl->Expand(hRoot, TVE_EXPAND);

}

void CBookmark_TreeView::BuildBookmark(HTREEITEM hParent)
{	
	/*if(NULL == hParent)
	{
		TVINSERTSTRUCT tvInsert;
		tvInsert.hParent = NULL;
		tvInsert.hInsertAfter = NULL;
		tvInsert.item.mask = TVIF_TEXT;
		tvInsert.item.pszText = _T("Bookmark");
		
		hParent = m_pTreeCtrl->InsertItem(&tvInsert);
	}*/

	FS_RESULT hResult = FSPDF_Bookmark_MoveToFirstChild(m_iteratorBookmark);
	if(hResult == FSCRT_ERRCODE_SUCCESS)
	{
		FSPDF_BOOKMARKDATA bmData;
		memset(&bmData, 0, sizeof(FSPDF_BOOKMARKDATA));
		FSPDF_Bookmark_GetData(m_iteratorBookmark, &bmData);

		wchar_t* buf = NULL;
		FS_DWORD length = 0;
		hResult = FSCRT_UTF8_ToUTF16LE(&(bmData.title), NULL, &length);
		if(hResult == FSCRT_ERRCODE_SUCCESS)
		{
			buf = new wchar_t[length+1];
			memset(buf, 0, (length+1)*sizeof(wchar_t));
			FSCRT_UTF8_ToUTF16LE(&(bmData.title), (FS_WORD*)buf, &length);
		}
		CString csTitle = buf;
		delete [] buf;
		HTREEITEM hItem = m_pTreeCtrl->InsertItem(TVIF_TEXT,
			csTitle, 0, 0, 0, 0, 0, hParent, NULL);

		int nPage = 0;
		FSPDF_ACTIONDATA actionData;
		memset(&actionData, 0, sizeof(FSPDF_ACTIONDATA));
		FSPDF_Bookmark_GetAction(m_iteratorBookmark, 0, &actionData);
		if (actionData.actionType == FSPDF_ACTION_GOTO)
		{
			nPage = ((FSPDF_ACTIONDATA_GOTO*)actionData.actionData)->destData.pageIndex;
		}
		m_pTreeCtrl->SetItemData(hItem, nPage);		

		this->BuildBookmark(hItem);
	}
	else
	{
		return;
	}
	
	hResult = FSCRT_ERRCODE_SUCCESS;
	while (hResult == FSCRT_ERRCODE_SUCCESS)
	{		
		hResult = FSPDF_Bookmark_MoveToNextSibling(m_iteratorBookmark);
		if(hResult == FSCRT_ERRCODE_SUCCESS)
		{
			FSPDF_BOOKMARKDATA bmData;
			memset(&bmData, 0, sizeof(FSPDF_BOOKMARKDATA));
			FSPDF_Bookmark_GetData(m_iteratorBookmark, &bmData);

			wchar_t* buf = NULL;
			FS_DWORD length = 0;
			hResult = FSCRT_UTF8_ToUTF16LE(&(bmData.title), NULL, &length);
			if(hResult == FSCRT_ERRCODE_SUCCESS)
			{
				buf = new wchar_t[length+1];
				memset(buf, 0, (length+1)*sizeof(wchar_t));
				FSCRT_UTF8_ToUTF16LE(&(bmData.title), (FS_WORD*)buf, &length);
			}
			CString csTitle = buf;
			delete [] buf;
			HTREEITEM hParent2 = m_pTreeCtrl->InsertItem(TVIF_TEXT,
			csTitle, 0, 0, 0, 0, 0, hParent, NULL);

			int nPage = 0;
			FSPDF_ACTIONDATA actionData;
			memset(&actionData, 0, sizeof(FSPDF_ACTIONDATA));
			FSPDF_Bookmark_GetAction(m_iteratorBookmark, 0, &actionData);
			if (actionData.actionType == FSPDF_ACTION_GOTO)
			{
				nPage = ((FSPDF_ACTIONDATA_GOTO*)actionData.actionData)->destData.pageIndex;
			}
			m_pTreeCtrl->SetItemData(hParent2, nPage);

			this->BuildBookmark(hParent2);
		}
		else
		{
			break;
		}
	}
	FSPDF_Bookmark_MoveToParent(m_iteratorBookmark);

	m_pTreeCtrl->Expand(hParent, TVE_EXPAND);
}


