#include "StdAfx.h"
#include "TreeView.h"
#include "../PDFReader/resource.h"



#define ITEMIMAGE_SELECT 1
#define ITEMIMAGE_UNSELECT 2
#define ITEMIMAGE_CHECK 3
#define ITEMIMAGE_UNCHECK 4
#define ITEMIMAGE_LAYERSET 5

//////////////////////////////////////////////////////////////////////////
BEGIN_MESSAGE_MAP(CLayer_TreeCtrl, CTreeCtrl)	
	ON_WM_SIZE()
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_CREATE()
END_MESSAGE_MAP()


void CLayer_TreeCtrl::OnSize(UINT nType, int cx, int cy)
{
	CTreeCtrl::OnSize(nType, cx, cy);
}

void CLayer_TreeCtrl::OnDestroy()
{
	CTreeCtrl::OnDestroy();
}

void CLayer_TreeCtrl::OnPaint()
{
	CTreeCtrl::OnPaint();
}

void CLayer_TreeCtrl::OnMouseMove(UINT nFlags, CPoint point)
{
	//	AfxMessageBox(L"Mousemove");
}

void CLayer_TreeCtrl::OnLButtonDown(UINT nFlags, CPoint point)
{
	CTreeCtrl::OnLButtonDown(nFlags, point);
	m_pPage->OnLButtonDown(nFlags, point);
}

void CLayer_TreeCtrl::OnLButtonUp(UINT nFlags, CPoint point)
{
	
}

int  CLayer_TreeCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if(-1 == CTreeCtrl::OnCreate(lpCreateStruct))
		return -1;
	
	return 0;
	
}
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
CLayer_TreeView::CLayer_TreeView(IReader_App* pApp, FSCRT_DOCUMENT doc)
{
	m_pApp = pApp;
	m_pPDFDoc = doc;

	FSPDF_LayerNode_Init(&m_Layers);
	FS_RESULT nRet = FSPDF_Doc_EnumLayers(doc, &m_Layers);

	nRet = FSPDF_LayerContext_Create(m_pPDFDoc, FSPDF_LAYERUSAGETYPE_VIEW, &m_ViewContext);


	m_pTreeCtrl = new CLayer_TreeCtrl;
}

CLayer_TreeView::~CLayer_TreeView()
{
	delete m_pTreeCtrl;
	FSPDF_LayerNode_Clear(&m_Layers);

	if (m_ViewContext)
	{
		FSPDF_LayerContext_Release(m_ViewContext);
	}

	if (m_pImageList4Tree)
	{
		m_pImageList4Tree->DeleteImageList();
		delete m_pImageList4Tree;
	}
}

BEGIN_MESSAGE_MAP(CLayer_TreeView, CWnd)	
	ON_WM_SIZE()
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_CREATE()
	
END_MESSAGE_MAP()


void CLayer_TreeView::OnSize(UINT nType, int cx, int cy)
{
	m_pTreeCtrl->MoveWindow(0, 0, cx, cy);
	CWnd::OnSize(nType, cx, cy)	;
}

void CLayer_TreeView::OnDestroy()
{
	m_pTreeCtrl->DestroyWindow();
	CWnd::OnDestroy();
}

void CLayer_TreeView::OnPaint()
{
	CPaintDC dc(this);
	CRect rc;
	this->GetClientRect(&rc);
	dc.FillSolidRect(&rc, RGB(100,0,100));
}

void CLayer_TreeView::OnMouseMove(UINT nFlags, CPoint point)
{

}

void CLayer_TreeView::OnLButtonDown(UINT nFlags, CPoint point)
{	
	//click +/- button  
	UINT uFlags;
	HTREEITEM hItem = m_pTreeCtrl->HitTest(point, &uFlags);  
	if (hItem)
	{
		if(TVHT_ONITEMBUTTON & uFlags)
		{
			return;
		}
	}
	
	hItem = m_pTreeCtrl->GetSelectedItem();
	FSPDF_LAYER pLayer =(FSPDF_LAYER) (DWORD)m_pTreeCtrl->GetItemData(hItem);

	HTREEITEM hFirstItem = m_pTreeCtrl->GetFirstVisibleItem();

	if (hItem == hFirstItem)
	{
		ShowLayerSetup(point);
		return;
	}

	if (!pLayer)
	{
		return;
	}

	IReader_Document* pReaderDocument = m_pApp->GetCurrentDocument();
	if (IsDesign(pLayer))
	{
		return;
	}

	if (!CheckParentItem(hItem))
	{
		return;
	}
	

	FS_BOOL bCheck = m_pTreeCtrl->GetCheck(hItem);
	m_pTreeCtrl->SetCheck(hItem,!bCheck);
	if (!bCheck)
		SetItemImageStatus(hItem,ITEMIMAGE_SELECT);
	else
		SetItemImageStatus(hItem,ITEMIMAGE_UNSELECT);

	SetChildItem(hItem,!bCheck);
	
	FS_BOOL bRet = !bCheck; 
	FS_RESULT nRet  = FSPDF_LayerContext_SetVisible(m_ViewContext,pLayer,bRet);

	//Set LayerContext for render.
	pReaderDocument->SetLayerContext(m_ViewContext);

	//refresh
	CWnd* pDocView = CWnd::FromHandle(pReaderDocument->GetDocViewHwnd());
	pDocView->Invalidate();

	return;
}

void CLayer_TreeView::OnLButtonUp(UINT nFlags, CPoint point)
{

}

int  CLayer_TreeView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if(-1 == CWnd::OnCreate(lpCreateStruct))
		return -1;

	m_pTreeCtrl->Create(WS_VISIBLE | WS_TABSTOP | WS_CHILD | WS_BORDER
		| TVS_HASBUTTONS | TVS_LINESATROOT | TVS_HASLINES  |TVS_EX_MULTISELECT
		| TVS_DISABLEDRAGDROP,
		CRect(0, 0, 0, 0), this, 0x1005);

	HICON icon[6];

	icon[0]=AfxGetApp()->LoadIcon(IDI_ICON5);
	icon[1]=AfxGetApp()->LoadIcon(IDI_ICON2);
 	icon[2]=AfxGetApp()->LoadIcon(IDI_ICON3);
 	icon[3]=AfxGetApp()->LoadIcon(IDI_ICON4);
	icon[4]=AfxGetApp()->LoadIcon(IDI_ICON1);
	icon[5]=AfxGetApp()->LoadIcon(IDI_ICON6);
	

	m_pImageList4Tree = new CImageList;

	m_pImageList4Tree->Create(16,16,32,6,6); 

	for(int i=0;i<6;i++)
	{
		m_pImageList4Tree->Add(icon[i]); //load icon
	}

	m_pTreeCtrl->SetImageList(m_pImageList4Tree,TVSIL_NORMAL);
	m_pTreeCtrl->SetTreeView(this);

	if(PDDocHasLayer())
	{
		this->BuildLayer(&m_Layers,NULL,FALSE);
	}
	else
	{
		this->BuildPageTree();
	}
	return 0;
}

BOOL CLayer_TreeView::PDDocHasLayer()
{
	if (m_Layers.children)
	{
		return TRUE;
	}
	return FALSE;
}

void CLayer_TreeView::BuildPageTree()
{
	TVINSERTSTRUCT tvInsert;
	tvInsert.hParent = NULL;
	tvInsert.hInsertAfter = NULL;
	tvInsert.item.mask = TVIF_TEXT;
	tvInsert.item.pszText = _T("Layer");
	HTREEITEM hRoot = m_pTreeCtrl->InsertItem(&tvInsert);
	m_pTreeCtrl->SetItemState(hRoot ,INDEXTOSTATEIMAGEMASK(0),   TVIS_STATEIMAGEMASK);//remove checkbox
}

void CLayer_TreeView::BuildLayer(FSPDF_LAYERNODE *pLayer,HTREEITEM hParent,FS_BOOL bParentLayer)
{	
	if(NULL == hParent)
	{
		TVINSERTSTRUCT tvInsert;
		tvInsert.hParent = NULL;
		tvInsert.hInsertAfter = NULL;
		tvInsert.item.mask = TVIF_TEXT;
		tvInsert.item.pszText = _T("Layer");
		
		hParent = m_pTreeCtrl->InsertItem(&tvInsert);
		//m_pTreeCtrl->SetItemState(hParent ,INDEXTOSTATEIMAGEMASK(0),   TVIS_STATEIMAGEMASK);//remove checkbox
		SetItemImageStatus(hParent,ITEMIMAGE_LAYERSET);
	}

	FSPDF_LAYERNODE pChildlayer;
	FS_BOOL bRet = FALSE; 
	FSPDF_LayerContext_IsVisible(m_ViewContext,pLayer->layer,&bRet);

	for(FS_INT32 i = 0; i < pLayer->count; i++)
	{
		pChildlayer = pLayer->children[i];
		if (pLayer->layer)
			AddLayer(&pChildlayer,hParent,bParentLayer,bRet);
		else
			AddLayer(&pChildlayer,hParent,bParentLayer,TRUE);
		
	}
	m_pTreeCtrl->Expand(hParent, TVE_EXPAND);
}

void	CLayer_TreeView::AddLayer(FSPDF_LAYERNODE *pLayer, HTREEITEM hParent,FS_BOOL bParentLayer ,FS_BOOL bParentVisable)
{
	wchar_t* buf = NULL;
	FS_DWORD length = 0;
	FS_RESULT  	hResult = FSCRT_UTF8_ToUTF16LE(&pLayer->name, NULL, &length);
	if(hResult == FSCRT_ERRCODE_SUCCESS)
	{
		buf = new wchar_t[length+1];
		memset(buf, 0, (length+1)*sizeof(wchar_t));
		FSCRT_UTF8_ToUTF16LE(&pLayer->name, (FS_WORD*)buf, &length);
	}
	CString csTitle = buf;
	delete [] buf;
	HTREEITEM hParent2 = m_pTreeCtrl->InsertItem(TVIF_TEXT,
		csTitle, 0, 0, 0, 0, 0, hParent, NULL);

	FS_BOOL bRet = FALSE; 
	FSPDF_LayerContext_IsVisible(m_ViewContext,pLayer->layer,&bRet);
	m_pTreeCtrl->SetCheck(hParent2,bRet);

	if (!pLayer->layer || IsDesign(pLayer->layer))
	{
		m_pTreeCtrl->SetItemState(hParent2 ,INDEXTOSTATEIMAGEMASK(0),   TVIS_STATEIMAGEMASK);//remove checkbox
		if (pLayer->layer)
			BuildLayer(pLayer,hParent2,FALSE);
		else
			BuildLayer(pLayer,hParent2,TRUE);
		return;
	}

	if (bParentVisable)
	{
		if (bRet)
			SetItemImageStatus(hParent2,ITEMIMAGE_SELECT);
		else
			SetItemImageStatus(hParent2,ITEMIMAGE_UNSELECT);
 	}
 	else
 	{
		if (bRet)
			SetItemImageStatus(hParent2,ITEMIMAGE_CHECK);
		else
			SetItemImageStatus(hParent2,ITEMIMAGE_UNCHECK);
 		
 	}

	m_pTreeCtrl->SetItemData(hParent2,(DWORD)pLayer->layer);

	if (pLayer->layer)
		BuildLayer(pLayer,hParent2,FALSE);
	else
		BuildLayer(pLayer,hParent2,TRUE);

}

FS_BOOL	CLayer_TreeView::IsDesign(FSPDF_LAYER layer)
{
	FS_BOOL  bDesignIntent= TRUE;
	const FSCRT_BSTR intentDesign = {"Design",sizeof("Design")-1};
	FSPDF_Layer_HasIntent(layer, &intentDesign, &bDesignIntent);

	FS_BOOL bViewIntent = TRUE;
	const FSCRT_BSTR intentView = {"View",sizeof("View")-1};
	FSPDF_Layer_HasIntent(layer, &intentView, &bViewIntent);

	if (bDesignIntent && !bViewIntent)
	{
		return TRUE;
	}
	return FALSE;
}

FS_BOOL	CLayer_TreeView::CheckParentItem(HTREEITEM hItem)
{
	HTREEITEM hParentItem = m_pTreeCtrl->GetParentItem(hItem);

	FSPDF_LAYER pLayer =(FSPDF_LAYER) (DWORD)m_pTreeCtrl->GetItemData(hParentItem);
	if (!pLayer)
	{
		return TRUE;
	}

	FS_BOOL bCheck = m_pTreeCtrl->GetCheck(hParentItem);

	if (bCheck)
	{
		return TRUE;
	}

	MessageBox(_T("This layer has been disable because its parent has been disable"));

	return FALSE;
}

void	CLayer_TreeView::SetChildItem(HTREEITEM hItem, FS_BOOL bParentCheck)
{
	HTREEITEM hChildItem = m_pTreeCtrl->GetChildItem(hItem);
	HTREEITEM hNextItem  = hChildItem;
	BOOL bCheck = FALSE;
	if (bParentCheck)
	{
		while(hNextItem)
		{
			bCheck = m_pTreeCtrl->GetCheck(hNextItem);
			if (bCheck)
				SetItemImageStatus(hNextItem,ITEMIMAGE_SELECT);
			else
				SetItemImageStatus(hNextItem,ITEMIMAGE_UNSELECT);
			FSPDF_LAYER pLayer =(FSPDF_LAYER) (DWORD)m_pTreeCtrl->GetItemData(hNextItem);
			if (!pLayer)
			{
				continue;
			}
			FS_BOOL bRet = bCheck; 
			FS_RESULT nRet  = FSPDF_LayerContext_SetVisible(m_ViewContext,pLayer,bRet);
			hNextItem = m_pTreeCtrl->GetNextSiblingItem(hNextItem);
		}
	}
	else
	{
		while(hNextItem)
		{
			bCheck = m_pTreeCtrl->GetCheck(hNextItem);
			if (bCheck)
				SetItemImageStatus(hNextItem,ITEMIMAGE_CHECK);
			else
				SetItemImageStatus(hNextItem,ITEMIMAGE_UNCHECK);
			FSPDF_LAYER pLayer =(FSPDF_LAYER) (DWORD)m_pTreeCtrl->GetItemData(hNextItem);
			if (!pLayer)
			{
				continue;
			}
			FS_RESULT nRet  = FSPDF_LayerContext_SetVisible(m_ViewContext,pLayer,FALSE);
			hNextItem = m_pTreeCtrl->GetNextSiblingItem(hNextItem);
		}
	}
}

void CLayer_TreeView::SetItemImageStatus(HTREEITEM hItem, int nStatus)
{
	switch(nStatus)
	{
	case ITEMIMAGE_SELECT :
		m_pTreeCtrl->SetItemImage(hItem,1,1);
		break;
	case ITEMIMAGE_UNSELECT :
		m_pTreeCtrl->SetItemImage(hItem,2,2);
		break;
	case ITEMIMAGE_CHECK :
		m_pTreeCtrl->SetItemImage(hItem,3,3);
		break;
	case  ITEMIMAGE_UNCHECK :
		m_pTreeCtrl->SetItemImage(hItem,4,4);
		break;
	case  ITEMIMAGE_LAYERSET :
		m_pTreeCtrl->SetItemImage(hItem,5,5);
		break;
	default:
		break;
	}
}


void CLayer_TreeView::ShowLayerSetup(CPoint point)
{

	CMenu menu;
	menu.CreatePopupMenu();
	menu.AppendMenu(MF_STRING,1,_T("Reset Layer"));
	CPoint pt= point;

	CWnd* pwnd = CWnd::FromHandle(this->GetSafeHwnd()) ;
	pwnd->ClientToScreen(&pt);
	int ret=menu.TrackPopupMenu(TPM_LEFTALIGN|TPM_RETURNCMD,pt.x,pt.y,pwnd);
	if (ret)
	{
		IReader_Document* pReaderDocument = m_pApp->GetCurrentDocument();
		FSPDF_LayerContext_Reset(m_ViewContext);
		//refresh
		CWnd* pDocView = CWnd::FromHandle(pReaderDocument->GetDocViewHwnd());
		HTREEITEM hFirstItem =	m_pTreeCtrl->GetFirstVisibleItem();
		HTREEITEM	hItem = m_pTreeCtrl->GetChildItem(hFirstItem);
		ResetItemState(hItem);
		pDocView->Invalidate();
		
	}

	menu.DestroyMenu();
}

void CLayer_TreeView::ResetItemState(HTREEITEM hItem)
{

	HTREEITEM hParentItem = m_pTreeCtrl->GetParentItem(hItem);
	HTREEITEM hFirstItem = m_pTreeCtrl->GetFirstVisibleItem();

	if(hItem)
	{
		FSPDF_LAYER Layer =(FSPDF_LAYER) (DWORD)m_pTreeCtrl->GetItemData(hItem);
		FSPDF_LAYER ParentLayer =(FSPDF_LAYER) (DWORD)m_pTreeCtrl->GetItemData(hParentItem);
		FS_BOOL bVisible = FALSE;
		FS_BOOL bParentVisible = FALSE;
		if (!Layer)
		{
			HTREEITEM hChildItem = m_pTreeCtrl->GetChildItem(hItem);
			if (hChildItem)
			{
				ResetItemState(hChildItem);
			}
		}
		else
		{
			FSPDF_LayerContext_IsVisible(m_ViewContext,ParentLayer,&bParentVisible);
			FSPDF_LayerContext_IsVisible(m_ViewContext,Layer,&bVisible);
			
			if (hFirstItem ==  hParentItem || !ParentLayer)
			{
				bParentVisible = TRUE;
			}

			if (bParentVisible)
			{
				if (bVisible)
					SetItemImageStatus(hItem,ITEMIMAGE_SELECT);
				else
					SetItemImageStatus(hItem,ITEMIMAGE_UNSELECT);
			}else
			{
				if (bVisible)
					SetItemImageStatus(hItem,ITEMIMAGE_CHECK);
				else
					SetItemImageStatus(hItem,ITEMIMAGE_UNCHECK);
			}

			HTREEITEM hChildItem = m_pTreeCtrl->GetChildItem(hItem);
			if (hChildItem)
			{
				ResetItemState(hChildItem);
			}
		}
	}

	HTREEITEM hNextItem = m_pTreeCtrl->GetNextSiblingItem(hItem);
	if(hNextItem)
	{
		ResetItemState(hNextItem);
	}
}

