
// ChildFrm.cpp : implementation of the CChildFrame class
//

#include "stdafx.h"
#include "PDFReader.h"

#include "ChildFrm.h"
#include "PDFReaderView.h"
#include "PDFReaderDoc.h"
#include "PDFDisplayView.h"
#include "./FrameworkImpl/PDFReaderApp.h"
#include "./FrameworkImpl/PDFReaderDocument.h"
#include "../FrameworkInterface/include/IPanelTool.h"
#include "./NavigationPanel/FX_DockContainer.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CChildFrame

IMPLEMENT_DYNCREATE(CChildFrame, CMDIChildWnd)

BEGIN_MESSAGE_MAP(CChildFrame, CMDIChildWnd)
	ON_WM_CREATE()
	ON_WM_MDIACTIVATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_SYSCOMMAND()
END_MESSAGE_MAP()

// CChildFrame construction/destruction

CChildFrame::CChildFrame()
{
	// TODO: add member initialization code here
}

CChildFrame::~CChildFrame()
{
}


BOOL CChildFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying the CREATESTRUCT cs

	cs.style |= WS_VISIBLE|WS_MAXIMIZE;
	if( !CMDIChildWnd::PreCreateWindow(cs) )
		return FALSE;

	return TRUE;
}

int CChildFrame::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{

	if (CMDIChildWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndDockPageBar.Create(L"DockContainer", this, CSize(230, 240), TRUE, AFX_IDW_BOOKMARKBAND, FALSE))
	{
		TRACE0("Failed to create DockContainer.\n");
		return -1;
	}
	m_wndDockPageBar.SetBarStyle(m_wndDockPageBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	m_wndDockPageBar.EnableDocking (CBRS_ALIGN_LEFT);
	
	EnableDocking(CBRS_ALIGN_ANY);
	////////////////////////////////////////////////////////////////////////////////////
	DockControlBar(&m_wndDockPageBar, AFX_IDW_DOCKBAR_LEFT);
	RecalcLayout();
	return 0;
}

void CChildFrame::OnMDIActivate(BOOL bActivate, CWnd* pActivateWnd, CWnd* pDeactivateWnd) 
{
	CReader_App* pReaderApp = theApp.m_pReaderApp;
	CMDIChildWnd::OnMDIActivate(bActivate, pActivateWnd, pDeactivateWnd);	
	
	if (bActivate && pActivateWnd != NULL)
	{
		CChildFrame* pActivateChildFrm = (CChildFrame*)pActivateWnd;
		CPDFDisplayView* pPDFDisplayView = (CPDFDisplayView*)pActivateChildFrm->GetActiveView();
		if (pPDFDisplayView)
		{
			CPDFReaderView*		pPDFReaderView = 	pPDFDisplayView->GetPDFReaderView();
			if (pPDFReaderView)
			{
				CPDFReaderDoc* pPDFReaderDoc = pPDFReaderView->GetDocument();
				if(pPDFReaderDoc)
				{
					CReader_Document* pReaderDocument = pPDFReaderDoc->GetReaderDocument();
					pReaderDocument->SetPDFDisplayView(pPDFDisplayView);
					pReaderApp->SetCurrentDocument(pReaderDocument);	
				}
			}
		}	
	}
	else if(!bActivate && pDeactivateWnd != NULL)
	{
		
	}
}

void CChildFrame::OnDestroy() 
{
	CReader_App* pReaderApp = theApp.m_pReaderApp;
	CReader_Document* pReaderDocument = (CReader_Document*)pReaderApp->GetCurrentDocument();

	CPDFDisplayView* pPDFDisplayView = (CPDFDisplayView*)GetActiveView();
	if (pPDFDisplayView)
	{
		CPDFReaderView*		pPDFReaderView = 	pPDFDisplayView->GetPDFReaderView();
		if (pPDFReaderView)
		{
			CPDFReaderDoc* pPDFReaderDoc = pPDFReaderView->GetDocument();
			if (pPDFReaderDoc)
			{
				CReader_Document* pReaderDocument = pPDFReaderDoc->GetReaderDocument();
				int nCount = pReaderApp->GetPanelToolHandlerSize();
				for(int i = 0; i<nCount; i++)
				{
					IReader_PanelToolHandler* pToolHandler = pReaderApp->GetPanelToolHandler(i);
					if(pToolHandler)
					{
						pToolHandler->Tool_CloseView(pReaderDocument->GetDocument(), m_wndDockPageBar.m_hWnd);
					}
				}
			}
		}
	}
	pReaderApp->m_bSetDocument = FALSE;
	CMDIChildWnd::OnDestroy();
}

// CChildFrame diagnostics

#ifdef _DEBUG
void CChildFrame::AssertValid() const
{
	CMDIChildWnd::AssertValid();
}

void CChildFrame::Dump(CDumpContext& dc) const
{
	CMDIChildWnd::Dump(dc);
}
#endif //_DEBUG

BOOL CChildFrame::AddPagesToDockWindow(CPDFReaderDoc* pPDFReaderDoc)
{
	CReader_App* pReaderApp = theApp.m_pReaderApp;
	CReader_Document* pReaderDocument = (CReader_Document*)pReaderApp->GetCurrentDocument();
	if (!pReaderDocument)
	{
		return FALSE;
	}
	int nCount = pReaderApp->GetPanelToolHandlerSize();
	for(int i = 0; i<nCount; i++)
	{
		IReader_PanelToolHandler* pToolHandler = pReaderApp->GetPanelToolHandler(i);
		if(pToolHandler) 
		{
			IReader_PanelView* pPanelView = pToolHandler->Tool_CreateNewView(pReaderDocument->GetDocument(), m_wndDockPageBar.m_hWnd);
			
			m_wndDockPageBar.AddPage(CWnd::FromHandle(pPanelView->GetHwnd()), pToolHandler->Tool_GetToolTip(), pToolHandler->Tool_GetLabel(), 
				pToolHandler->Tool_GetName(), pToolHandler->Tool_GetIcon(), TAB_LEFT, 
				pPanelView, pPanelView->GetBDockBottom());

			pToolHandler->Tool_SetActiveView(pReaderDocument->GetDocument(), m_wndDockPageBar.m_hWnd);
		}

	}
	
	m_wndDockPageBar.ShowALLVisibleContainer(TRUE, TRUE);
	if(m_wndDockPageBar.m_bUsedDocHistory == FALSE)
		m_wndDockPageBar.SetOnlyTabBar();

	return TRUE;
}


BOOL CChildFrame::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle , const RECT& rect , CMDIFrameWnd* pParentWnd , CCreateContext* pContext)
{
	// TODO: Add your specialized code here and/or call the base class

	return CMDIChildWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, pContext);
}
