#include "stdafx.h"
#include "AnnotHandlerMgr.h"

CReader_AnnotHandlerMgr::CReader_AnnotHandlerMgr()
{
	m_pAnnotPropertyDlg = NULL;
	m_bLButtonDown = FALSE;
}

CReader_AnnotHandlerMgr::~CReader_AnnotHandlerMgr()
{
	POSITION pos = m_mapType2Handler.GetStartPosition();
	while(pos != NULL)
	{
		CString rKey;
		void* rValue;
		m_mapType2Handler.GetNextAssoc(pos, rKey, rValue);
		IReader_AnnotHandler *pAnnotHandler = (IReader_AnnotHandler *)rValue;
		delete pAnnotHandler;
	}
	m_mapType2Handler.RemoveAll();

	if (m_pAnnotPropertyDlg)
	{
		m_pAnnotPropertyDlg->DestroyWindow();
		delete m_pAnnotPropertyDlg;
		m_pAnnotPropertyDlg = NULL;
	}
}

IReader_Annot* CReader_AnnotHandlerMgr::NewAnnot(FSCRT_ANNOT pdfAnnot, IReader_Page* pPage)
{
	CString csType;
	FSCRT_BSTR annotType = {NULL, 0};
	FSPDF_Annot_GetType(pdfAnnot, &annotType);

	wchar_t* buf = NULL;
	FS_DWORD length = 0;
	FS_RESULT hResult = FSCRT_UTF8_ToUTF16LE(&annotType, NULL, &length);
	if(hResult == FSCRT_ERRCODE_SUCCESS)
	{
		buf = new wchar_t[length+1];
		memset(buf, 0, (length+1)*sizeof(wchar_t));
		FSCRT_UTF8_ToUTF16LE(&annotType, (FS_WORD*)buf, &length);
		csType = buf;
		delete [] buf;
	}

	//FSCRT_Memory_Free(&annotType);

	if (IReader_AnnotHandler* pAnnotHandler = GetAnnotHandler(csType))
	{
		return pAnnotHandler->NewAnnot(pdfAnnot, pPage);
	}

	return NULL;
}

void CReader_AnnotHandlerMgr::Annot_OnLoad(IReader_Annot* pAnnot)
{
}

CRect CReader_AnnotHandlerMgr::Annot_OnGetViewBBox(IReader_Page* pPage, IReader_Annot* pAnnot)
{
	if (IReader_AnnotHandler* pAnnotHandler = GetAnnotHandler(pAnnot))
	{
		return pAnnotHandler->GetViewBBox(pPage, pAnnot);
	}
	return CRect(0, 0, 0, 0);
}

// Mouse events
BOOL CReader_AnnotHandlerMgr::Annot_OnHitTest(IReader_Page* pPage, IReader_Annot* pAnnot, const CPoint& point)
{
	if (IReader_AnnotHandler* pAnnotHandler = GetAnnotHandler(pAnnot))
	{
		return pAnnotHandler->HitTest(pPage, pAnnot, point);
	}

	return FALSE;
}

void CReader_AnnotHandlerMgr::Annot_OnDraw(IReader_Page* pPage, HDC hDC, IReader_Annot* pAnnot, FSCRT_MATRIX matrix, const CRect& rcPage)
{
	if (IReader_AnnotHandler* pAnnotHandler = GetAnnotHandler(pAnnot))
	{
		pAnnotHandler->Annot_OnDraw(pPage, hDC, pAnnot, matrix, rcPage);
	}
}

BOOL CReader_AnnotHandlerMgr::Annot_OnLButtonDown(IReader_Page* pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	m_bLButtonDown = TRUE;
	m_curPos = point;
	IReader_Document* pReaderDocument = pPage->GetDocument();
	CWnd* pDocView = CWnd::FromHandle(pReaderDocument->GetDocViewHwnd());
	if (IReader_AnnotHandler* pAnnotHandler = GetAnnotHandler(pAnnot))
	{
		BOOL bRet = pAnnotHandler->OnLButtonDown(pPage, pAnnot, nFlags, point);
		if (bRet)
		{
			FS_DWORD dwFlags;
			FSPDF_Annot_GetFlags(pAnnot->GetPDFAnnot(), &dwFlags);
			BOOL bLock = dwFlags & FSPDF_ANNOTFLAG_LOCKED ? TRUE : FALSE;

			if (!pAnnot->IsCaptured() && !bLock)
			{
				pReaderDocument->SetCapturedAnnot(pAnnot);
			}

			if (!pAnnot->IsSelected())
			{
				//deselect pre annot
				IReader_Annot* pSelectedAnnot = pReaderDocument->GetSelectedAnnot();

				pReaderDocument->SetSelectedAnnot(pAnnot);

				if (!m_pAnnotPropertyDlg)
				{
					m_pAnnotPropertyDlg = new CAnnotPropertyDlg;
					m_pAnnotPropertyDlg->Create(IDD_ANNOT_PROPERTY_DLG, pDocView);
				}

				m_pAnnotPropertyDlg->UpdateDialog(pAnnot, pDocView);
				m_pAnnotPropertyDlg->EnableWindow(TRUE);

				// refresh annot rectangular area
				CRect rcWindow = pAnnot->GetViewBBox();
				rcWindow.InflateRect(3, 3);
				pDocView->InvalidateRect(rcWindow);
			}
		}
	
		return bRet;
	}
	else
	{
		if (m_pAnnotPropertyDlg)
		{
			m_pAnnotPropertyDlg->UpdateDialog(pAnnot, pDocView);
		}

	}

	return FALSE;
}

BOOL CReader_AnnotHandlerMgr::Annot_OnLButtonDblClk(IReader_Page* pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	if (IReader_AnnotHandler* pAnnotHandler = GetAnnotHandler(pAnnot))
	{
		return pAnnotHandler->OnLButtonDblClk(pPage, pAnnot, nFlags, point);
	}
	return FALSE;
}


BOOL CReader_AnnotHandlerMgr::Annot_OnLButtonUp(IReader_Page* pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	m_bLButtonDown = FALSE;
	IReader_Document* pReaderDocument = pPage->GetDocument();
	IReader_Annot* pCapturedAnnot = pReaderDocument->GetCapturedAnnot();
	IReader_Annot* pAnnotTemp = NULL;
	
	CRect rcWindow;
	if (pCapturedAnnot)
	{
		pAnnotTemp = pCapturedAnnot;
		rcWindow = this->Annot_OnGetViewBBox(pPage, pAnnotTemp);
		rcWindow.InflateRect(3, 3);
	}
	else
	{
		pAnnotTemp = pAnnot;
	}

	BOOL bLock = FALSE;
	
	if (pAnnotTemp)
	{
		FS_DWORD dwFlags;
		FSPDF_Annot_GetFlags(pAnnotTemp->GetPDFAnnot(), &dwFlags);
		bLock = dwFlags & FSPDF_ANNOTFLAG_LOCKED ? TRUE : FALSE;
	}
 	
 	IReader_AnnotHandler* pAnnotHandler = GetAnnotHandler(pAnnotTemp);

	if (pAnnotHandler && !bLock)
	{
		BOOL bRet = pAnnotHandler->OnLButtonUp(pPage, pAnnotTemp, nFlags, point);
		if (bRet)
		{
			pReaderDocument->SetCapturedAnnot(NULL);
			if (pCapturedAnnot && m_curPos != point)
			{
				//refresh annot rectangular area
				CWnd* pDocView = CWnd::FromHandle(pReaderDocument->GetDocViewHwnd());
				pDocView->InvalidateRect(rcWindow);
				int x = point.x-m_curPos.x;
				int y = point.y -m_curPos.y;
				rcWindow.OffsetRect(x, y);
				pDocView->InvalidateRect(rcWindow);
			}
		}
		return bRet;
	}
	return FALSE;
}

BOOL CReader_AnnotHandlerMgr::Annot_OnMouseMove(IReader_Page* pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	IReader_Document* pReaderDocument = pPage->GetDocument();
	IReader_Annot* pCapturedAnnot = pReaderDocument->GetCapturedAnnot();
	FS_FLOAT x = 0;
	FS_FLOAT y = 0;
	FSCRT_ANNOT annot = NULL;
	pPage->WindowToDoc((int)point.x, (int)point.y, x, y);
	FSPDF_Annot_GetAtPos(pPage->GetPage(), NULL, x, y, 1, &annot);

	if (m_bLButtonDown)
	{
		::SetCursor(AfxGetApp()->LoadCursor(IDC_CUR_SELECTANNOT));
	}
	else
	{
		if (annot)
			::SetCursor(AfxGetApp()->LoadCursor(IDC_CUR_SELECTANNOT));
		else
			::SetCursor(AfxGetApp()->LoadCursor(IDC_CUR_HAND));
	}

	if (!pCapturedAnnot)
	{
		pCapturedAnnot = pAnnot;
	}

	if (IReader_AnnotHandler* pAnnotHandler = GetAnnotHandler(pCapturedAnnot))
	{
		return pAnnotHandler->OnMouseMove(pPage, pCapturedAnnot, nFlags, point);
	}

	return TRUE;
}

void CReader_AnnotHandlerMgr::RegisterAnnotHandler(IReader_AnnotHandler* pAnnotHandler)
{
	m_mapType2Handler.SetAt(pAnnotHandler->GetType(), (void*)pAnnotHandler);
}

IReader_AnnotHandler* CReader_AnnotHandlerMgr::GetAnnotHandler(IReader_Annot* pAnnot)
{
	FSCRT_ANNOT pdfAnnot = NULL;
	if (pAnnot)
	{
		pdfAnnot = pAnnot->GetPDFAnnot();
	}
	if (pdfAnnot)
	{
		CString csType;
		FSCRT_BSTR annotType = {NULL, 0};
		FSPDF_Annot_GetType(pdfAnnot, &annotType);

		wchar_t* buf = NULL;
		FS_DWORD length = 0;
		FS_RESULT hResult = FSCRT_UTF8_ToUTF16LE(&annotType, NULL, &length);
		if(hResult == FSCRT_ERRCODE_SUCCESS)
		{
			buf = new wchar_t[length+1];
			memset(buf, 0, (length+1)*sizeof(wchar_t));
			FSCRT_UTF8_ToUTF16LE(&annotType, (FS_WORD*)buf, &length);
			csType = buf;
			delete [] buf;
		}
		FSCRT_BStr_Clear(&annotType);
		IReader_AnnotHandler* pAnnotHandler = GetAnnotHandler(csType);		
		return pAnnotHandler;
	}
	return NULL;
}

IReader_AnnotHandler* CReader_AnnotHandlerMgr::GetAnnotHandler(CString csType)
{
	void* pRet = NULL;
	m_mapType2Handler.Lookup(csType, pRet);	
	return (IReader_AnnotHandler*)pRet;
}

BOOL CReader_AnnotHandlerMgr::Annot_OnRButtonUp(IReader_Page* pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	if (!pPage || ! pAnnot)
	{
		return FALSE;
	}

	if (pAnnot->IsSelected())
	{
		CMenu menu;
		menu.CreatePopupMenu();
		menu.AppendMenu(MF_STRING, Annot_Item_Properties, _T("Properties"));
		FS_DWORD dwFlags = 0 ;
		FSPDF_Annot_GetFlags(pAnnot->GetPDFAnnot(), &dwFlags);
		BOOL bLock = dwFlags & FSPDF_ANNOTFLAG_LOCKED ? TRUE : FALSE;
		if (!bLock)
		{
			menu.AppendMenu(MF_STRING, Annot_Item_Delete, _T("Delete"));
		}
		CPoint pt = point;
		CWnd* pWnd = CWnd::FromHandle(pPage->GetDocument()->GetDocViewHwnd()) ;
		CRect winRect = 	pAnnot->GetViewBBox();
		winRect.InflateRect(3, 3);
		pWnd->ClientToScreen(&pt);
		int ret = menu.TrackPopupMenu(TPM_LEFTALIGN|TPM_RETURNCMD, pt.x, pt.y, pWnd);
		switch (ret)
		{
		case Annot_Item_Properties:
			if (!m_pAnnotPropertyDlg)
			{
				m_pAnnotPropertyDlg = new CAnnotPropertyDlg;
				m_pAnnotPropertyDlg->Create(IDD_ANNOT_PROPERTY_DLG, pWnd);
			}
			m_pAnnotPropertyDlg->UpdateDialog(pAnnot, pWnd);
			m_pAnnotPropertyDlg->ShowWindow(TRUE);
			break;
		case Annot_Item_Delete:
			pPage->DeleteAnnot(pAnnot);
			pWnd->InvalidateRect(winRect);
			break;
		default:
			break;
		}
		menu.DestroyMenu();
	}
	
	return TRUE;
}

void CReader_AnnotHandlerMgr::ValidAnnotPropertyDlg()
{
	if (m_pAnnotPropertyDlg)
	{
		m_pAnnotPropertyDlg->UpdateDialog(NULL, NULL);
	}
}
