#include "StdAfx.h"
#include "PDFReaderApp.h"
#include "../../FrameworkInterface/include/IPanelTool.h"
#include "../../FrameworkInterface/include/ITool.h"
#include "./PDFReaderTool.h"
#include "./AnnotHandlerMgr.h"
#include "./Reader_FreeTextAnnotHandler.h"
#include "./Reader_HighLightAnnotHandler.h"
#include "./Reader_UnderLineAnnotHandler.h"
#include "./Reader_LinkAnnotHandler.h"
#include "./Reader_NoteAnnotHandler.h"

extern IReader_Module* Reader_CreateBookmarkPanelTool(HMODULE hModule, IReader_App* pApp);
extern IReader_Module* Reader_CreateLayerPanelTool(HMODULE hModule, IReader_App* pApp);
extern IReader_Module* Reader_CreateSnapshotTool(HMODULE hModule, IReader_App* pApp);
extern IReader_Module* Reader_CreateLineAnnot(HMODULE hModule, IReader_App* pApp);

const extern IReader_CreateModuleProc g_ModuleList[] = 
{
	Reader_CreateBookmarkPanelTool,
	Reader_CreateLayerPanelTool
};

CReader_App::CReader_App()
{
	m_pCurrentDocument = NULL;
	m_pCurrentTool = NULL;
	m_mapModuleList.InitHashTable(100);
	m_pAnnotHandlerMgr = NULL;
}

CReader_App::~CReader_App()
{
	POSITION pos = m_mapModuleList.GetStartPosition();
	while(pos != NULL)
	{
		CString rKey;
		void* rValue;
		m_mapModuleList.GetNextAssoc(pos, rKey, rValue);
		IReader_Module *pModule = (IReader_Module *)rValue;
		pModule->Destroy();
		delete pModule;
	}
	m_mapModuleList.RemoveAll();

	int i = 0;
	int nCounts = this->m_pPaneHandlerList.GetSize();
	for(i = nCounts-1; i>= 0; i--)
	{
		IReader_PanelToolHandler* pToolHandler = m_pPaneHandlerList.GetAt(i);
		if(pToolHandler) 
			pToolHandler->Tool_Destroy();
		delete pToolHandler;
		m_pPaneHandlerList.RemoveAt(i);
	}

	nCounts = this->m_pToolHandlerList.GetSize();
	for(i = nCounts-1; i>= 0; i--)
	{
		IReader_ToolHandler* pToolHandler = m_pToolHandlerList.GetAt(i);
		if(pToolHandler) 
			pToolHandler->Tool_Destroy();
		delete pToolHandler;
		m_pToolHandlerList.RemoveAt(i);
	}

	nCounts = this->m_pMenuToolbarCommandHandlerList.GetSize();
	for(i = nCounts-1; i>= 0; i--)
	{
		IReader_MenuToolbarCommandHandler* pHandler = m_pMenuToolbarCommandHandlerList.GetAt(i);
		if(pHandler) 
			pHandler->Destroy();
		delete pHandler;
		m_pMenuToolbarCommandHandlerList.RemoveAt(i);
	}

	if (m_pCurrentDocument)
	{
		delete m_pCurrentDocument;
		m_pCurrentDocument = NULL;
	}

	if (m_pAnnotHandlerMgr != NULL)
	{
		delete m_pAnnotHandlerMgr;
		m_pAnnotHandlerMgr = NULL;
	}
}

// Register a tool handler
BOOL CReader_App::RegisterToolHandler(IReader_ToolHandler* pToolHandler)
{
	m_pToolHandlerList.Add(pToolHandler);
	pToolHandler->Tool_OnInit(this);
	return TRUE;
}

// Register a Panel Tool handler
BOOL CReader_App::RegisterPanelToolHandler(IReader_PanelToolHandler* pPanelToolHandler)
{
	BOOL bInit = pPanelToolHandler->Tool_OnInit();
	if (bInit)
	{
		m_pPaneHandlerList.Add(pPanelToolHandler);
		return TRUE;
	}
	return FALSE;
}

// Register a App Event handle
BOOL CReader_App::RegisterAppEventHandler(IReader_AppEventHandler* pAppEventHandler)
{
	return FALSE;
}

// Register A Doc Event handle
BOOL CReader_App::RegisterDocEventHandler(IReader_DocEventHandler* pDocEventHandler)
{
	return FALSE;
}

//Page Event handle
BOOL CReader_App::RegisterPageEventHandler(IReader_PageEventHandler* pPageEventHandler)
{
	return FALSE;
}

BOOL CReader_App::RegisterMenuToolbarCommandHandler(IReader_MenuToolbarCommandHandler* pCommandHandler)
{
	m_pMenuToolbarCommandHandlerList.Add(pCommandHandler);
	pCommandHandler->OnInit(this);
	return TRUE;
}
IReader_MenuToolbarCommandHandler* CReader_App::GetMenuToolbarCommandHandler(UINT nID)
{
	int nCount = m_pMenuToolbarCommandHandlerList.GetSize();
	for (int i = 0; i<nCount; i++)
	{
		IReader_MenuToolbarCommandHandler* pHandler = m_pMenuToolbarCommandHandlerList.GetAt(i);
		CArray<UINT, UINT> arrayID;
		pHandler->GetIDs(arrayID);
		int nIDCount = arrayID.GetSize();
		for (int j = 0; j<nIDCount; j++)
		{
			if (nID == arrayID.GetAt(j))
			{
				return pHandler;
			}
		}
	}
	return NULL;
}

// Get a panel tool handler from name
IReader_PanelToolHandler* CReader_App::GetPanelToolHandler(LPCWSTR pToolName)
{
	int nCount = m_pPaneHandlerList.GetSize();
	for(int i = 0; i<nCount; i++)
	{
		IReader_PanelToolHandler* pToolHandler = m_pPaneHandlerList.GetAt(i);
		if(pToolHandler && pToolHandler->Tool_GetName().Compare(pToolName) == 0) 
		{
			return pToolHandler;
		}
	}
	return NULL;
}

int CReader_App::GetPanelToolHandlerSize()
{
	return m_pPaneHandlerList.GetSize();
}
IReader_PanelToolHandler* CReader_App::GetPanelToolHandler(int nIndex)
{
	return m_pPaneHandlerList.GetAt(nIndex);
}

int CReader_App::CountToolHandlers()
{
	return m_pToolHandlerList.GetSize();
}

IReader_ToolHandler* CReader_App::GetToolHandlerByIndex(int index)
{
	IReader_ToolHandler* pToolHandler = m_pToolHandlerList.GetAt(index);
	return pToolHandler;
}

IReader_ToolHandler* CReader_App::GetToolHandlerByName(LPCWSTR pToolName)
{
	int nSize = m_pToolHandlerList.GetSize();
	for (int i = 0; i<nSize; i++)
	{
		IReader_ToolHandler* pToolHandler = m_pToolHandlerList.GetAt(i);
		if(pToolHandler && pToolHandler->Tool_GetName().Compare(pToolName) == 0) 
		{
			return pToolHandler;
		}
	}
	return NULL;
}

IReader_ToolHandler* CReader_App::GetCurrentToolHandler()
{
	return m_pCurrentTool;
}

void CReader_App::SetCurrentToolHandler(IReader_ToolHandler* pToolHandler, BOOL bPersistent, BOOL bDelay/* = FALSE*/)
{
	IReader_ToolHandler *pTemp = m_pCurrentTool;
	if(m_pCurrentTool != pToolHandler)
	{
		if (IReader_Document* pDocument = this->GetCurrentDocument())
		{
			CString csHanderName = pToolHandler->Tool_GetName();
		}

	}

	m_pCurrentTool = NULL; //during Deactivate, g_pCurToolHandler set NULL.

	if (pTemp && pTemp != pToolHandler)
	{
		pTemp->Tool_OnDeactivate();
	}

	m_pCurrentTool = pToolHandler;//during Activate, g_pCurToolHandler have been assignmented.

	if (pTemp != pToolHandler && m_pCurrentTool != NULL)
		m_pCurrentTool->Tool_OnActivate(bPersistent);
}

void CReader_App::SetCurrentToolHandler(LPCWSTR csToolname, BOOL bPersistent)
{
	IReader_ToolHandler* pToolHandler = GetToolHandlerByName(csToolname);
	this->SetCurrentToolHandler(pToolHandler, bPersistent, FALSE);
}

IReader_Document* CReader_App::GetCurrentDocument()
{
	return m_pCurrentDocument;
}

BOOL CReader_App::SetCurrentDocument(IReader_Document* pDocument)
{
	m_pCurrentDocument = pDocument;
	if (m_pCurrentDocument)
	{
		m_bSetDocument = TRUE;
	}
	return TRUE;
}

IReader_Document* CReader_App::OpenDocument(LPCWSTR wsFilePath, BOOL bAttachment, BOOL bMakeVisible/* = TRUE*/, LPCWSTR wsPassword/* = L""*/, BOOL bAddToMRU/* = TRUE*/)
{
	return NULL;
}

BOOL CReader_App::CloseDocument(IReader_Document* pDocument, BOOL bPromptToSave/* = TRUE*/, BOOL bDelay/* = TRUE*/)
{
	return FALSE;
}

IReader_Document* CReader_App::GetReaderDocument(FSCRT_DOCUMENT pDoc)
{
	return NULL;
}

int CReader_App::CountDocuments()
{
	return 0;
}

IReader_Document* CReader_App::GetReaderDocument(int index)
{
	return NULL;
}

void CReader_App::AddDocument(IReader_Document* pDocument)
{
	m_pDocumentList.Add(pDocument);
}

void CReader_App::RemoveDocument(IReader_Document* pDocument)
{

	int nCount = this->m_pDocumentList.GetSize();
	for(int i = 0; i<nCount; i++)
	{
		IReader_Document* pDoc = (IReader_Document*)this->m_pDocumentList.GetAt(i);
		if(pDoc && pDoc == pDocument){			
			this->m_pDocumentList.RemoveAt(i);
			break;
		}	
	}
	return;
}

//HandlerMgr
IReader_AnnotHandlerMgr* CReader_App::GetAnnotHandlerMgr()
{
	return (IReader_AnnotHandlerMgr*)m_pAnnotHandlerMgr;
}

// MainFrame/Menu Operator
HWND CReader_App::GetMainFrameWnd()
{
	return NULL;
}

//Get Module
IReader_Module*	CReader_App::GetModuleByName(LPCWSTR wsName)
{
	void* pValue = NULL;
	m_mapModuleList.Lookup(wsName, pValue);
	IReader_Module * pModule = (IReader_Module *)pValue;
	return pModule;
}

BOOL CReader_App::LoadLibrarys()
{
	CReader_HandToolHandler* pHandToolHandler = new CReader_HandToolHandler;
	this->RegisterToolHandler(pHandToolHandler);
	this->SetCurrentToolHandler(pHandToolHandler, TRUE, FALSE);

	m_pAnnotHandlerMgr = new CReader_AnnotHandlerMgr;

	CReader_FreeTextAnnotHandler *pTextAnnotHandler = new CReader_FreeTextAnnotHandler;
	m_pAnnotHandlerMgr->RegisterAnnotHandler(pTextAnnotHandler);

	CReader_HighLightAnnotHandler* pHighlightHandler = new CReader_HighLightAnnotHandler;
	m_pAnnotHandlerMgr->RegisterAnnotHandler(pHighlightHandler);

	CReader_UnderLineAnnotHandler* pUnderLineHandler = new CReader_UnderLineAnnotHandler;
	m_pAnnotHandlerMgr->RegisterAnnotHandler(pUnderLineHandler);

	CReader_LinkAnnotHandler* pLinkHandler = new CReader_LinkAnnotHandler;
	m_pAnnotHandlerMgr->RegisterAnnotHandler(pLinkHandler);

	CReader_NotesAnnotHandler* pNotesHandler = new CReader_NotesAnnotHandler;
	m_pAnnotHandlerMgr->RegisterAnnotHandler(pNotesHandler);

	HINSTANCE hInstance = AfxGetInstanceHandle();
	int nModuleCount = sizeof(g_ModuleList)/sizeof(IReader_CreateModuleProc);
	for(int i = 0; i < nModuleCount ; i++)
	{
		IReader_Module* pModule = g_ModuleList[i](hInstance, this);
		if(pModule)
		{
			m_mapModuleList.SetAt(pModule->GetModuleName(), pModule);
		}
	}
	return TRUE;
}