#include "StdAfx.h"
#include "PDFReaderDocument.h"
#include "PDFReaderPage.h"
#include "../PDFDisplayView.h"
#include "../../FrameworkInterface/include/IAnnot.h"
#include "../PasswordDlg.h"



static void WideStringToUTF8(CString csStr, FSCRT_BSTR* bstr)
{
	char *buffer = NULL;  
	int  length;  
	length = WideCharToMultiByte(CP_UTF8, 0, csStr, -1, NULL, 0, NULL, NULL);  

	FSCRT_BStr_SetLength(bstr, length);
	WideCharToMultiByte(CP_UTF8, 0, csStr, -1, bstr->str, length, NULL, NULL);  
}

//callback function of FSPDF_ACTION_HANDLER
FS_RESULT	InvalidateWindowRect(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_INT32 pageIndex, FSCRT_RECTF* pdfRect)
{
	if(!clientData)
		return FSCRT_ERRCODE_ERROR;

	CReader_Document* pDoc = (CReader_Document*)clientData;
	CPDFDisplayView* pDisplayView = pDoc->GetPDFDisplayView();
	if(!pDisplayView)
		return FSCRT_ERRCODE_ERROR;

	FSCRT_MATRIX matrix;
	pDisplayView->GetPageMatrix(pageIndex, &matrix);
	FSCRT_Matrix_TransformRectF(&matrix, pdfRect);
	RECT rc = {(LONG)pdfRect->left, (LONG)pdfRect->top, (LONG)pdfRect->right, (LONG)pdfRect->bottom};
	::InvalidateRect(pDisplayView->m_hWnd, &rc, FALSE);
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	GetCurrentPageIndex(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_INT32* pageIndex)
{
	wchar_t str[MAX_PATH];
	CReader_Document* pDoc = (CReader_Document*)clientData;
	*pageIndex = pDoc->GetPDFDisplayView()->GetCurVisiblePageIndex();

	swprintf(str, _T("GetCurrentPageIndex : %d"), *pageIndex);
	if (IDOK == AfxMessageBox(str))
	{
		CReader_Document::SetMouseFlag(TRUE);
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	SetCurrentPageIndex(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_INT32 pageIndex)
{
	CReader_Document* pDoc = (CReader_Document*)clientData;
	pDoc->GotoPage(pageIndex);

	return FSCRT_ERRCODE_SUCCESS;
}

void AppendUTF8ToWideString(CString& csMessage, const FSCRT_BSTR* bstrMsg, int type)
{
	wchar_t* msgType = NULL;
	switch(type)
	{
	case 2:
		msgType = L"To";
		break;
	case 3:
		msgType = L"Subject";
		break;
	case 4:
		msgType = L"CC";
		break;
	case 5:
		msgType = L"BCC";
		break;
	case 6:
		msgType = L"Msg";
		break;
	}
	CString csTemp;

	FS_DWORD len = 0;
	FSCRT_UTF8_ToUTF16LE(bstrMsg, NULL, &len);
	FS_WORD* pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(bstrMsg, pBuff, &len);
	csTemp.Format(L"\n%s\t:\t%s",msgType, pBuff);
	delete pBuff;

	csMessage.Append(csTemp);
}

FS_RESULT	MailData(FS_LPVOID clientData, FS_LPVOID data, FS_DWORD dataType, FS_BOOL bUI, const FSCRT_BSTR* To, 
	const FSCRT_BSTR* Subject, const FSCRT_BSTR* CC, const FSCRT_BSTR* BCC, const FSCRT_BSTR* Msg)
{
	CString csMessageType;
	switch(dataType)
	{
	case FSPDF_MAILTYPE_DOC:
		csMessageType = L"call MailData() to mail doc";
		break;
	case FSPDF_MAILTYPE_FORM:
		csMessageType = L"call MailData() to mail form";
		break;
	case FSPDF_MAILTYPE_MSG:
		csMessageType = L"call MailData() to mail msg";
		break;
	}
	AppendUTF8ToWideString(csMessageType, To, 2);
	AppendUTF8ToWideString(csMessageType, Subject, 3);
	AppendUTF8ToWideString(csMessageType, CC, 4);
	AppendUTF8ToWideString(csMessageType, BCC, 5);
	AppendUTF8ToWideString(csMessageType, Msg, 6);
	AfxMessageBox(csMessageType);
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	GetAppInfo(FS_LPVOID clientData, FS_INT32 type, FSCRT_BSTR* info)
{
	CString csInfo;
	switch(type)
	{
	case FSPDF_APPINFOTYPE_FORMSVERSION:
		csInfo = L"7.3";
		break;
	case FSPDF_APPINFOTYPE_VIEWERTYPE:
		csInfo = L"foxit sdk ui demo for windows";
		break;
	case FSPDF_APPINFOTYPE_VIEWERVARIATION:
		csInfo = L"full expired";
		break;
	case FSPDF_APPINFOTYPE_VIEWERVERSION:
		csInfo = L"6.5";
		break;
	case FSPDF_APPINFOTYPE_APPVERSION:
		csInfo = L"5.6";
		break;
	}
	WideStringToUTF8(csInfo, info);
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	MyGetChangeMark(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_BOOL* changeMark)
{
	CReader_Document* pUIDocument = (CReader_Document*)clientData;

	*changeMark = pUIDocument->GetChangeMark();

	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	Alert(FS_LPVOID clientData, const FSCRT_BSTR* Msg, const FSCRT_BSTR* Title, FS_INT32 Type, FS_INT32 Icon, FS_INT32* retCode)
{
	FS_DWORD len = 0;
	FSCRT_UTF8_ToUTF16LE(Msg, NULL, &len);
	FS_WORD* pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(Msg, pBuff, &len);
	if (2 == Type) Type = MB_YESNO;
	int ret = AfxMessageBox((LPWSTR)pBuff, Type);
	if (IDOK == ret)
	{
		CReader_Document::SetMouseFlag(TRUE);
	}
	else if (IDYES == ret)
		*retCode = 4;

	delete[] pBuff;
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	Beep(FS_LPVOID clientData, FS_INT32 nType)
{
	wstring funname(_T("Beep\n\n"));
	wstring strType;
	switch (nType)
	{
	case 0: 
		strType = _T("Error");
		break;
	case 1: 
		strType = _T("Warning");
		break;
	case 2: 
		strType = _T("Question");
		break;
	case 3:
		strType = _T("Status");
		break;
	case 4:
		strType = _T("Default");
		break;
	default:
		strType = _T("not defined");
	}
	wstring str = funname + strType;

	if (IDOK == AfxMessageBox(str.c_str()))
	{
		CReader_Document::SetMouseFlag(TRUE);
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	GetRotation(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_INT32 pageIndex, FS_INT32* rotation)
{
	wchar_t temp[MAX_PATH];
	swprintf(temp, _T("page index : %d\n\nrotation : %d\n\n"), pageIndex, rotation);
	wstring strRotate(temp);
	wstring str = _T("GetRotation\n\n") + strRotate;
	CReader_Document::SetMouseFlag(TRUE);
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT GetOpenedDocs(FS_LPVOID clientData, FSCRT_DOCUMENT* document, FS_INT32* count)
{
	if (document)
	{
		*document = ((CReader_Document*)clientData)->GetDocument();
		*count = 1;
	} 
	else
	{
		*count = 1;
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	ExecuteNamedAction(FS_LPVOID clientData, FSCRT_DOCUMENT document, const FSCRT_BSTR* namedAction)
{
	FS_DWORD len = 0;
	FSCRT_UTF8_ToUTF16LE(namedAction, NULL, &len);
	FS_WORD* pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(namedAction, pBuff, &len);
	wstring strNamedAction((wchar_t*)pBuff);
	wstring str = _T("ExecuteNamedAction\n\n") + strNamedAction;
	if (IDOK == AfxMessageBox(str.c_str()))
	{
		CReader_Document::SetMouseFlag(TRUE);
	}
	delete[] pBuff;

	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	GetFilePath(FS_LPVOID clientData, FSCRT_DOCUMENT document, FSCRT_BSTR* filePath)
{
	CReader_Document* pUIDocument = (CReader_Document*)clientData;

	WideStringToUTF8(pUIDocument->GetFilePathStr(), filePath);

	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	Mail(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_BOOL bUI, const FSCRT_BSTR* To, 
	const FSCRT_BSTR* Subject, const FSCRT_BSTR* CC, const FSCRT_BSTR* BCC, const FSCRT_BSTR* Msg)
{
	FS_DWORD len = 0;
	FSCRT_UTF8_ToUTF16LE(To, NULL, &len);
	FS_WORD* pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(To, pBuff, &len);
	wstring wstrTo((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	FSCRT_UTF8_ToUTF16LE(Subject, NULL, &len);
	pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(Subject, pBuff, &len);
	wstring wstrSubject((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	FSCRT_UTF8_ToUTF16LE(CC, NULL, &len);
	pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(CC, pBuff, &len);
	wstring wstrCC((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	FSCRT_UTF8_ToUTF16LE(BCC, NULL, &len);
	pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(BCC, pBuff, &len);
	wstring wstrBCC((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	FSCRT_UTF8_ToUTF16LE(Msg, NULL, &len);
	pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(Msg, pBuff, &len);
	wstring wstrMsg((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	wstring funname(_T("Mail\n\n"));
	wstring n1(_T("To:")); 
	wstring n2(_T("Subject:")); 
	wstring n3(_T("CC:")); 
	wstring n4(_T("BCC:")); 
	wstring n5(_T("Msg:")); 
	wstring CL(_T("\n"));
	wstring str = funname + n1 + wstrTo + CL + n2 + wstrSubject + CL + n3 + wstrCC + CL + 
		n4 + wstrBCC + CL + n5 + wstrMsg + CL;

	if (IDOK == AfxMessageBox(str.c_str()))
	{
		CReader_Document::SetMouseFlag(TRUE);
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	Print(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_BOOL bUI, FS_INT32 nStart, FS_INT32 nEnd, FS_BOOL bSilent , 
	FS_BOOL bShrinkToFit, FS_BOOL bPrintAsImage, FS_BOOL bReverse, FS_BOOL bAnnotations)
{
	wchar_t strPrint[1024];
	swprintf(strPrint, _T("bUI: %d\nrange: from %d to %d\nbSilent: %d\nbShrinkToFit: %d\nbPrintAsImage: %d\nbReverse: %d\nbAnnotations: %d"), 
		bUI, nStart, nEnd, bSilent, bShrinkToFit, bPrintAsImage, bReverse, bAnnotations);

	wstring funname(_T("Print\n\n"));
	wstring str = funname + strPrint;
	if (IDOK == AfxMessageBox(str.c_str()))
	{
		CReader_Document::SetMouseFlag(TRUE);
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	Release(FS_LPVOID clientData)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	Response(FS_LPVOID clientData, const FSCRT_BSTR* Question, const FSCRT_BSTR* Title, const FSCRT_BSTR* Default, 
	const FSCRT_BSTR* cLabel, FS_BOOL bPassword, FSCRT_BSTR* response)
{
	FSCRT_BStr_Set(response, "no answers.", strlen("no answers."));
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	SubmitForm(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_LPVOID formData, FS_DWORD length, const FSCRT_BSTR* URL)
{
	FS_DWORD len = 0;
	FSCRT_UTF8_ToUTF16LE(URL, NULL, &len);
	FS_WORD* pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(URL, pBuff, &len);

	wstring strURL((wchar_t*)pBuff);
	wstring funname(_T("SubmitForm URL\n\n"));
	wstring str = funname + strURL;
	if (IDOK == AfxMessageBox(str.c_str()))
	{
		CReader_Document::SetMouseFlag(TRUE);
	}
	delete[] pBuff;
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	LaunchURL(FS_LPVOID clientData, const FSCRT_BSTR* URL)
{
	FS_DWORD len = 0;
	FSCRT_UTF8_ToUTF16LE(URL, NULL, &len);
	FS_WORD* pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(URL, pBuff, &len);

	wstring strURL((wchar_t*)pBuff);
	wstring str = _T("LaunchURL\n\n") + strURL;
	if (IDOK == AfxMessageBox(str.c_str()))
	{
		CReader_Document::SetMouseFlag(TRUE);
	}

	delete[] pBuff;

	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	BrowseFile(FS_LPVOID clientData, FSCRT_BSTR* filePath)
{
	CFileDialog  Dlg(TRUE,L"fdf",NULL,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,L"fdf Files(*.fdf)|All Files(*.*)|*.*");
	CString FilePathName;

	if(Dlg.DoModal() == IDOK)
		FilePathName =  Dlg.GetPathName();

	WideStringToUTF8(FilePathName, filePath);

	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	MySetChangeMark(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_BOOL changeMark)
{
	CReader_Document* pUIDocument = (CReader_Document*)clientData;
	if (changeMark)
		pUIDocument->SetChangeMark();

	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	OutputSelectedRect(FS_LPVOID clientData, FS_INT32 pageIndex, FSCRT_RECTF* pdfRect)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	SetCursor(FS_LPVOID clientData, FS_INT32 nCursorType)
{
	LPWSTR uiCusortype = 0;
	switch(nCursorType)
	{
	case FSCRT_CURSOR_ARROW:
		uiCusortype = IDC_ARROW;
		break;
	case FSCRT_CURSOR_NESW:
		uiCusortype = IDC_SIZENESW;
		break;
	case FSCRT_CURSOR_NWSE:
		uiCusortype = IDC_SIZENWSE;
		break;
	case FSCRT_CURSOR_VBEAM:
		uiCusortype = IDC_IBEAM;
		break;
	case FSCRT_CURSOR_HBEAM:
		uiCusortype = IDC_IBEAM;
		break;
	case FSCRT_CURSOR_HAND:
		uiCusortype = IDC_HAND;
	break;
	}
	HCURSOR hCursor = (HCURSOR)AfxGetApp()->LoadStandardCursor(uiCusortype);
	::SetCursor(hCursor);
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	MySetTimer(FS_LPVOID clientData, FS_INT32 uElapse, FSCRT_CALLBACK_TIMER lpTimerFunc, FS_INT32* timeID)
{
	CReader_Document* pDoc = 	(CReader_Document*) clientData;

	*timeID = (FS_INT32)clientData;
	if (pDoc->GetPDFDisplayView())
	pDoc->GetPDFDisplayView()->CreateTimer(clientData, uElapse, lpTimerFunc);
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	MyKillTimer(FS_LPVOID clientData, FS_INT32 timerID)
{
	CReader_Document* pDoc = 	(CReader_Document*) clientData;
	if(pDoc->GetPDFDisplayView())
		pDoc->GetPDFDisplayView()->DestroyTime(timerID);
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	GotFocusOnControl(FS_LPVOID clientData, FSPDF_FORMCONTROL control, const FSCRT_BSTR* filedValue)
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;

	FSCRT_ANNOT widgetAnnot = NULL;
	FSPDF_FormControl_GetWidgetAnnot( control, &widgetAnnot);
	if (!widgetAnnot) return ret;

	FSCRT_PAGE page = NULL;
	FSCRT_Annot_GetPage(widgetAnnot, &page);
	if (!page) return ret;

	FS_INT32 index = 0;
	FSPDF_Page_GetIndex(page, &index);

	CReader_Document* pDoc = 	(CReader_Document*) clientData;
	CReader_Page* pPage = (CReader_Page *)	pDoc->GetPage(index);

	if (pPage)
	{
		pPage->SetFocusAnnotFlag(TRUE);
	}

	if (!pDoc->IsVisiblePage(pDoc->m_nLostFocusIndex))
	{
		CReader_Page* pRealesePage = (CReader_Page *)	pDoc->GetPage(pDoc->m_nLostFocusIndex);
		pRealesePage->ReleaseTempData();
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	LostFocusOnControl(FS_LPVOID clientData, FSPDF_FORMCONTROL control, const FSCRT_BSTR* filedValue)
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;

	FSCRT_ANNOT widgetAnnot = NULL;
	FSPDF_FormControl_GetWidgetAnnot( control, &widgetAnnot);

	if (!widgetAnnot) return ret;
	
	FSCRT_PAGE page = NULL;
	FSCRT_Annot_GetPage(widgetAnnot, &page);

	if (!page) return ret;
	
	FS_INT32 index = 0;
	FSPDF_Page_GetIndex(page, &index);

	CReader_Document* pDoc = (CReader_Document*) clientData;
	CReader_Page* pPage = (CReader_Page *)	pDoc->GetPage(index);

	if (!pDoc->IsVisiblePage(pDoc->m_nLostFocusIndex))
	{
		CReader_Page* pRealesePage = (CReader_Page *)pDoc->GetPage(pDoc->m_nLostFocusIndex);
		if (pRealesePage)
		{
			pRealesePage->ReleaseTempData();
		}			
	}

	if (pPage)
	{	
		pPage->SetFocusAnnotFlag(FALSE);
		pDoc->m_nLostFocusIndex = index;
	}
	return FSCRT_ERRCODE_SUCCESS;
}

//CReader_Document
BOOL CReader_Document::m_bReleaseMouse = FALSE;

CReader_Document::CReader_Document(IReader_App* pApp)
	: m_bInitPages(FALSE)
	, m_pdfDocument(NULL)
	, m_hFSCRTFile(NULL)
	, m_nTotalPage(0)
	, m_fRealHeight(0)
	, m_fRealWidth(0)
	, m_nPageSpace(10)
	, m_nPageShadowWidth(4)
	, m_pPDFDisplayView(NULL)
	, m_pApp(pApp)
	, m_pSelectedAnnot(NULL)
	, m_pCapturedAnnot(NULL)
	, m_bChangeMark(FALSE)
	, m_nLostFocusIndex(0)
	, m_bLayer(FALSE)
	, m_FormFiller(NULL)
	, m_form(NULL)
	, m_bLoadAnnot(TRUE)
	, m_bLoadText(TRUE)
	, m_bRender(TRUE)
{
	FSCRT_BStr_Init(&m_strPassword);
}

CReader_Document::~CReader_Document()
{
	if (NULL != m_FormFiller)
		FSPDF_FormFiller_End(m_FormFiller);
	if (NULL != m_form)
		FSPDF_Form_Release(m_form);
	int nPageCount = m_pPageList.GetCount();
	for(int i = 0; i<nPageCount; i++)
	{
		IReader_Page* pReaderPage = (IReader_Page*)(m_pPageList.GetAt(i));

// 		if (pReaderPage)
// 		{
 		((CReader_Page*)pReaderPage)->ReleaseTempData();
//  		}
		delete pReaderPage;
	}
	m_pPageList.RemoveAll();

	if (m_pdfDocument != NULL)
	{
		FSPDF_Doc_Close(m_pdfDocument);
		m_pdfDocument = NULL;
	}	

	if (m_hFSCRTFile != NULL)
	{
		FSCRT_File_Release(m_hFSCRTFile);
		m_hFSCRTFile = NULL;
	}
	FSCRT_BStr_Clear(&m_strPassword);
}

IReader_App* CReader_Document::GetApp()
{
	return m_pApp;
}

FSCRT_DOCUMENT CReader_Document::GetDocument()
{
	return m_pdfDocument;
}

int CReader_Document::GetPageCount()
{
	return m_pPageList.GetCount();
}

IReader_Page* CReader_Document::GetPage(int nIndex)
{
	int nCount = m_pPageList.GetCount();
	if (nIndex < 0 || nIndex > nCount - 1)
	{
		return NULL;
	}
	IReader_Page* pReaderPage = (IReader_Page*)m_pPageList.GetAt(nIndex);
	return pReaderPage;
}

IReader_Page* CReader_Document::GetPage(CPoint point)
{
	return (IReader_Page*)m_pPDFDisplayView->GetPage(point);;
}

BOOL CReader_Document::GotoPage(int nIndex)
{
	if (m_pPDFDisplayView)
	{
		m_pPDFDisplayView->SetVisiblePageByIndex(nIndex);
		m_pPDFDisplayView->Invalidate();
		return TRUE;
	}
	return FALSE;
}

BOOL CReader_Document::MovePage(CPoint curPoint, CPoint toPoint)
{
	if (m_pPDFDisplayView)
	{
		m_pPDFDisplayView->MovePage(curPoint, toPoint);
		m_pPDFDisplayView->ResetCurrentPageIndex();
		m_pPDFDisplayView->Invalidate();
		return TRUE;
	}
	return FALSE;
}

BOOL CReader_Document::MovePage(short zDelta)
{
	if (m_pPDFDisplayView)
	{
		m_pPDFDisplayView->MovePage(zDelta);
		m_pPDFDisplayView->ResetCurrentPageIndex();
		m_pPDFDisplayView->Invalidate();
		return TRUE;
	}
	return FALSE;
}

void CReader_Document::SetChangeMark()
{
	m_bChangeMark = TRUE;
}

BOOL CReader_Document::GetChangeMark()
{
	return m_bChangeMark;
}

void CReader_Document::ClearChangeMark()
{
	m_bChangeMark = FALSE;
}

HWND CReader_Document::GetDocViewHwnd()
{
	if (m_pPDFDisplayView != NULL)
	{
		return m_pPDFDisplayView->GetSafeHwnd();
	}
	return NULL;
}

void CReader_Document::SetSelectedAnnot(IReader_Annot* pReaderAnnot)
{
	if(m_pSelectedAnnot == pReaderAnnot) return;
	if (m_pSelectedAnnot)
	{
		m_pSelectedAnnot->SetSelected(FALSE);
	}
	m_pSelectedAnnot = pReaderAnnot;
	if (m_pSelectedAnnot)
	{
		m_pSelectedAnnot->SetSelected(TRUE);
	}
}

IReader_Annot* CReader_Document::GetSelectedAnnot()
{
	return m_pSelectedAnnot;
}

void CReader_Document::SetCapturedAnnot(IReader_Annot* pReaderAnnot)
{
	if(m_pCapturedAnnot == pReaderAnnot) return;

	if (m_pCapturedAnnot)
	{
		m_pCapturedAnnot->SetCaptured(FALSE);
	}
	m_pCapturedAnnot = pReaderAnnot;
	if (m_pCapturedAnnot)
	{
		m_pCapturedAnnot->SetCaptured(TRUE);
	}
}

IReader_Annot* CReader_Document::GetCapturedAnnot()
{
	return m_pCapturedAnnot;
}

void CReader_Document::SetDocPassword(CString csPassword)
{
	int len = csPassword.GetLength();
	FSCRT_UTF8_FromUTF16LE(&m_strPassword, (FS_WORD*)csPassword.GetBuffer(), len);
}

BOOL CReader_Document::LoadPDFDocument(LPCWSTR lpwsFilePath)
{
	if(NULL != m_form)
		FSPDF_Form_Release(m_form);
	m_form = NULL;
	m_pApp->SetCurrentDocument(this);
	m_csFilePath = lpwsFilePath;
	size_t length = wcslen(m_csFilePath);

	wchar_t* buf = new wchar_t[length+1];
	memset(buf, 0, sizeof(char)*length*sizeof(WCHAR));

	wcscpy(buf, m_csFilePath);
	//Open the input file
	m_hFSCRTFile = FSDK_OpenFileW_Win(buf);
	delete [] buf;
	if (!m_hFSCRTFile)
	{
		return FALSE;
	}

	FS_RESULT ret = FSPDF_Doc_StartLoad(m_hFSCRTFile, NULL, &m_pdfDocument, NULL);
	if (FSCRT_ERRCODE_PASSWORD == ret)
	{
		PasswordDlg pwDlg;
		while (TRUE)
		{
			int rtValue = pwDlg.DoModal();
			if (IDOK == rtValue && m_strPassword.str != NULL && m_strPassword.len > 0)
			{
				ret = FSPDF_Doc_StartLoad(m_hFSCRTFile, &m_strPassword, &m_pdfDocument, NULL);
				FSCRT_BStr_Clear(&m_strPassword);
				if (FSCRT_ERRCODE_SUCCESS == ret)
					break;
			}
			if (IDCANCEL == rtValue)
				break;
			AfxMessageBox(_T("Invalid password, please enter password again!"), MB_OK | MB_ICONINFORMATION);	
		}
	}
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		if (m_pdfDocument) FSPDF_Doc_Close(m_pdfDocument);
		if (ret == FSCRT_ERRCODE_INVALIDCERTIFICATE) {
			AfxMessageBox(_T("The certificate is invalid!"), MB_OK | MB_ICONSTOP);
		} else {
			AfxMessageBox(_T("You don't have the right of \"FSPDF_Doc_StartLoad\" Functions, please purchase \"Standard\" Modules!"), MB_OK | MB_ICONSTOP);
		}
		return FALSE;
	}
	//for formfill
	//FSPDF_FORM form = NULL;
	ret = FSPDF_Form_Load(m_pdfDocument, &m_form);
	if(ret != FSCRT_ERRCODE_SUCCESS && ret == FSCRT_ERRCODE_NOTFOUND)
	{
		ret = FSPDF_Doc_CreateForm(m_pdfDocument, &m_form);
	}
	if (ret != FSCRT_ERRCODE_SUCCESS && ret != FSCRT_ERRCODE_NOTFOUND)
	{
		AfxMessageBox(_T("You don't have the right of \"FSPDF_Form_Load\" Functions, please purchase \"Form Add-on\" Modules!"));
	}

	FSPDF_JAVASCRIPTACTION_HANDLER actionHandler;
	memset(&actionHandler, 0, sizeof(FSPDF_JAVASCRIPTACTION_HANDLER));
	actionHandler.actionHandleSize = sizeof(FSPDF_JAVASCRIPTACTION_HANDLER);

	actionHandler.Alert = Alert;
	actionHandler.Beep = Beep;
	actionHandler.BrowseFile = BrowseFile;
	actionHandler.ExecuteNamedAction = ExecuteNamedAction;
	actionHandler.GetChangeMark = MyGetChangeMark;
	actionHandler.GetCurrentPageIndex = GetCurrentPageIndex;
	actionHandler.GetFilePath = GetFilePath;
	actionHandler.GetRotation = GetRotation;
	actionHandler.GetOpenedDocs = GetOpenedDocs;
	actionHandler.InvalidateRect = InvalidateWindowRect;
	actionHandler.LaunchURL = LaunchURL;
	actionHandler.Mail = Mail;
	actionHandler.Print = Print;
	actionHandler.Release = Release;
	actionHandler.Response = Response;
	actionHandler.SubmitForm = SubmitForm;
	actionHandler.SetChangeMark = MySetChangeMark;	
	actionHandler.SetCurrentPageIndex = SetCurrentPageIndex;
	actionHandler.MailData = MailData;
	actionHandler.GetAppInfo = GetAppInfo;
	actionHandler.clientData = this;

	FSPDF_FORMINTERACTION_WINDOWLESS forminterAction;
	forminterAction.clientData = this;
	forminterAction.lStructSize = sizeof(FSPDF_FORMINTERACTION_WINDOWLESS);
	forminterAction.GotFocusOnControl = GotFocusOnControl;
	forminterAction.LostFocusOnControl = LostFocusOnControl;
	forminterAction.KillTimer = MyKillTimer;
	forminterAction.SetCursor = SetCursor;
	forminterAction.OutputSelectedRect = OutputSelectedRect;
	forminterAction.SetTimer = MySetTimer;

	ret = FSPDF_Doc_SetJavaScriptActionHandler(m_pdfDocument, &actionHandler);
	ret = FSPDF_FormFiller_Begin(m_form, &forminterAction, &m_FormFiller);
//	FSPDF_FormFiller_Begin(form, NULL, &m_FormFiller);
	ret = FSPDF_FormFiller_SetHighlightColor(m_FormFiller, FSPDF_FORMFIELDTYPE_PUSHBUTTON, 0x8000ff00);
	ret = FSPDF_FormFiller_SetHighlightColor(m_FormFiller, FSPDF_FORMFIELDTYPE_CHECKBOX, 0x8000ff00);
	ret = FSPDF_FormFiller_SetHighlightColor(m_FormFiller, FSPDF_FORMFIELDTYPE_RADIOBUTTON, 0x8000ff00);
	ret = FSPDF_FormFiller_SetHighlightColor(m_FormFiller, FSPDF_FORMFIELDTYPE_COMBOBOX, 0x8000ff00);
	ret = FSPDF_FormFiller_SetHighlightColor(m_FormFiller, FSPDF_FORMFIELDTYPE_LISTBOX, 0x8000ff00);
	ret = FSPDF_FormFiller_SetHighlightColor(m_FormFiller, FSPDF_FORMFIELDTYPE_TEXTFIELD, 0x8000ff00);

	ret = FSPDF_FormFiller_ShowHighlight(m_FormFiller, TRUE);

	ret = FSPDF_Doc_InitiateJavaScript(m_pdfDocument);
	if (FSCRT_ERRCODE_INVALIDLICENSE == ret)
	{
		AfxMessageBox(_T("You don't have the right of \"FSPDF_Doc_InitiateJavaScript\" Functions, please purchase \"Standard\" Modules!"));
	}
	FSPDF_Doc_DoJavaScriptAction(m_pdfDocument, FSPDF_TRIGGER_DOC_DO);
	FSPDF_Doc_DoJavaScriptAction(m_pdfDocument, FSPDF_TRIGGER_DOC_WC);
	FSPDF_Doc_DoJavaScriptAction(m_pdfDocument, FSPDF_TRIGGER_DOC_WS);
	FSPDF_Doc_DoJavaScriptAction(m_pdfDocument, FSPDF_TRIGGER_DOC_DS);
	FSPDF_Doc_DoJavaScriptAction(m_pdfDocument, FSPDF_TRIGGER_DOC_WP);
	FSPDF_Doc_DoJavaScriptAction(m_pdfDocument, FSPDF_TRIGGER_DOC_DP);

	

	return TRUE;
}

BOOL CReader_Document::ReLoadPDFDocument()
{
	Save();
	if (NULL != m_form)
		FSPDF_Form_Release(m_form);
	m_form = NULL;
	for(int i = 0; i<m_nTotalPage; i++)
	{
		IReader_Page* pReaderPage = (IReader_Page*)m_pPageList.GetAt(i);
		delete pReaderPage;
	}
	m_bInitPages = 0;
	m_pPageList.RemoveAll();
	
	if (m_pdfDocument) 
		FSPDF_Doc_Close(m_pdfDocument);
	FS_RESULT ret = FSPDF_Doc_StartLoad(m_hFSCRTFile, NULL, &m_pdfDocument, NULL);

	if (FSCRT_ERRCODE_PASSWORD == ret)
	{
		ret = FSPDF_Doc_StartLoad(m_hFSCRTFile, &m_strPassword, &m_pdfDocument, NULL);
	}

	//for formfill
	//FSPDF_FORM form = NULL;
	ret = FSPDF_Form_Load(m_pdfDocument, &m_form);
	if(ret != FSCRT_ERRCODE_SUCCESS && ret == FSCRT_ERRCODE_NOTFOUND)
	{
		ret = FSPDF_Doc_CreateForm(m_pdfDocument, &m_form);
	}

	FSPDF_JAVASCRIPTACTION_HANDLER actionHandler;
	memset(&actionHandler, 0, sizeof(FSPDF_JAVASCRIPTACTION_HANDLER));
	actionHandler.actionHandleSize = sizeof(FSPDF_JAVASCRIPTACTION_HANDLER);

	actionHandler.Alert = Alert;
	actionHandler.Beep = Beep;
	actionHandler.BrowseFile = BrowseFile;
	actionHandler.ExecuteNamedAction = ExecuteNamedAction;
	actionHandler.GetChangeMark = MyGetChangeMark;
	actionHandler.GetCurrentPageIndex = GetCurrentPageIndex;
	actionHandler.GetFilePath = GetFilePath;
	actionHandler.GetRotation = GetRotation;
	actionHandler.GetOpenedDocs = GetOpenedDocs;
	actionHandler.InvalidateRect = InvalidateWindowRect;
	actionHandler.LaunchURL = LaunchURL;
	actionHandler.Mail = Mail;
	actionHandler.Print = Print;
	actionHandler.Release = Release;
	actionHandler.Response = Response;
	actionHandler.SubmitForm = SubmitForm;
	actionHandler.SetChangeMark = MySetChangeMark;	
	actionHandler.SetCurrentPageIndex = SetCurrentPageIndex;
	actionHandler.MailData = MailData;
	actionHandler.GetAppInfo = GetAppInfo;
	actionHandler.clientData = this;

	ret = FSPDF_Doc_SetJavaScriptActionHandler(m_pdfDocument, &actionHandler);
	FSPDF_FormFiller_Begin(m_form, NULL, &m_FormFiller);

	FSPDF_FormFiller_SetHighlightColor(m_FormFiller, FSPDF_FORMFIELDTYPE_PUSHBUTTON, 0x8000ff00);
	FSPDF_FormFiller_SetHighlightColor(m_FormFiller, FSPDF_FORMFIELDTYPE_CHECKBOX, 0x8000ff00);
	FSPDF_FormFiller_SetHighlightColor(m_FormFiller, FSPDF_FORMFIELDTYPE_RADIOBUTTON, 0x8000ff00);
	FSPDF_FormFiller_SetHighlightColor(m_FormFiller, FSPDF_FORMFIELDTYPE_COMBOBOX, 0x8000ff00);
	FSPDF_FormFiller_SetHighlightColor(m_FormFiller, FSPDF_FORMFIELDTYPE_LISTBOX, 0x8000ff00);
	FSPDF_FormFiller_SetHighlightColor(m_FormFiller, FSPDF_FORMFIELDTYPE_TEXTFIELD, 0x8000ff00);

	FSPDF_FormFiller_ShowHighlight(m_FormFiller, TRUE);
	return TRUE;
}

BOOL CReader_Document::InitPDFPages(int nLogPixelX, int nLogPixelY, int nRotate)
{
	m_fRealHeight = 0;
	m_fRealWidth = 0;

	if (m_bInitPages)
	{
		for(int i = 0; i < m_nTotalPage; i++)
		{
			CReader_Page* pReaderPage = (CReader_Page*)m_pPageList.GetAt(i);

			FSCRT_PAGE pdfPage = NULL;
			FSPDF_Doc_GetPage(m_pdfDocument, i, &pdfPage);

			FS_FLOAT fPageWidth = 0;
			FS_FLOAT fPageHeight = 0;
			FSPDF_Page_GetSize(pdfPage, &fPageWidth, &fPageHeight);

			float nWidth = fPageWidth / 72.0f * nLogPixelX + 0.5f; 
			float nHeight = fPageHeight / 72.0f * nLogPixelY + 0.5f;

			if(nRotate == 1 || nRotate == 3)
			{
				m_fRealHeight = m_fRealHeight + nWidth + m_nPageSpace;
				if(nHeight > m_fRealWidth)
					m_fRealWidth = nHeight;
			}
			else
			{
				m_fRealHeight = m_fRealHeight + nHeight + m_nPageSpace;
				if(nWidth > m_fRealWidth)
					m_fRealWidth = nWidth;
			}

			pReaderPage->SetPageHeight(nHeight);
			pReaderPage->SetPageWidth(nWidth);
			pReaderPage->SetPageTotalHeight(m_fRealHeight);
			pReaderPage->SetPageRealWidth(nWidth);
		}
		return TRUE;
	}

	FSPDF_Doc_CountPages(m_pdfDocument, &m_nTotalPage);
	for(int i = 0; i<m_nTotalPage; i++)
	{
		CReader_Page* pReaderPage = new CReader_Page(this);

		FSCRT_PAGE pdfPage = NULL;
		FSPDF_Doc_GetPage(m_pdfDocument, i, &pdfPage);

		FS_FLOAT fPageWidth = 0;
		FS_FLOAT fPageHeight = 0;
		FSPDF_Page_GetSize(pdfPage, &fPageWidth, &fPageHeight);
		
		float nWidth = fPageWidth / 72.0f * nLogPixelX + 0.5f; 
		float nHeight = fPageHeight / 72.0f * nLogPixelY + 0.5f;

		if(nRotate == 1 || nRotate == 3)
		{
			m_fRealHeight = m_fRealHeight + nWidth + m_nPageSpace;
			if(nHeight > m_fRealWidth)
				m_fRealWidth = nHeight;
		}
		else
		{
			m_fRealHeight = m_fRealHeight + nHeight + m_nPageSpace;
			if(nWidth > m_fRealWidth)
				m_fRealWidth = nWidth;
		}
		
		pReaderPage->SetPageHeight(nHeight);
		pReaderPage->SetPageWidth(nWidth);
		pReaderPage->SetPageTotalHeight(m_fRealHeight);
		pReaderPage->SetPageRealWidth(nWidth);
		pReaderPage->SetPageIndex(i);
		
	
		m_pPageList.Add(pReaderPage);
	}
	
	m_bInitPages = TRUE;
	return TRUE;
}

void CReader_Document::FindVisiblePage(float& StartPos, float& EndPos, int& StartPage, int& EndPage)
{
	int nPageCount = m_pPageList.GetSize();
	for (int i = 0; i<nPageCount; i++)
	{
		CReader_Page* pReaderPage = (CReader_Page*)m_pPageList.GetAt(i);
		float fPageTotalHeight = pReaderPage->GetPageTotalHeight();
		if (StartPos < fPageTotalHeight)
		{
			StartPage = pReaderPage->GetPageIndex();
			if (i != 0)
			{
				CReader_Page* pReaderPage2 = (CReader_Page*)m_pPageList.GetAt(i-1);
				StartPos = pReaderPage2->GetPageTotalHeight();
			}
			else 
				StartPos = 0;
			for(int j = i; j<nPageCount; j++)
			{
				CReader_Page* pReaderPage3 = (CReader_Page*)m_pPageList.GetAt(j);
				float fPageTotalHeight3 = pReaderPage3->GetPageTotalHeight();
				if (EndPos < fPageTotalHeight3)
				{
					EndPage = pReaderPage3->GetPageIndex();
					
					break;
				}
				else
				{
					EndPage = nPageCount - 1;
				}
			}
			break;
		}
	}

	if(EndPage < StartPage)
	{
		EndPage = m_nTotalPage - 1;
	}
}

BOOL CReader_Document::IsVisiblePage(FS_INT32 index)
{
	CPDFDisplayView *pDisplayView = GetPDFDisplayView();
	return pDisplayView->IsVisiblePage(index);
}

float CReader_Document::GetRealHeight()
{
	return m_fRealHeight;
}
float CReader_Document::GetRealWidth()
{
	return m_fRealWidth;
}

int CReader_Document::GetPageShadowWidth()
{
	return m_nPageShadowWidth;
}

BOOL CReader_Document::SetPDFDisplayView(CPDFDisplayView* pPDFDisplayView)
{
	m_pPDFDisplayView = pPDFDisplayView;
	return TRUE;
}

BOOL CReader_Document::DocToWindow(int nPageIndex, FSCRT_RECTF rcDoc, CRect& rcWindow)
{
	return m_pPDFDisplayView->DocToWindow(nPageIndex, rcDoc, rcWindow);
}

BOOL CReader_Document::WindowToDoc(int nPageIndex, int window_x, int window_y, FS_FLOAT& doc_x, FS_FLOAT& doc_y)
{
	return m_pPDFDisplayView->WindowToDoc(nPageIndex, window_x, window_y, doc_x, doc_y);
}

BOOL CReader_Document::WindowToDoc(int nPageIndex, CRect rcWindow, FSCRT_RECTF& rcDoc)
{
	return m_pPDFDisplayView->WindowToDoc(nPageIndex, rcWindow, rcDoc);
}

BOOL CReader_Document::WindowToDocForPrint(int nPageIndex, CRect rcWindow, FSCRT_RECTF& rcDoc)
{
	return m_pPDFDisplayView->WindowToDocForPrint(nPageIndex, rcWindow, rcDoc);
}
BOOL CReader_Document::Save()
{
	//[make up temporary file name]
    m_pPDFDisplayView->KillFormfillerFocus();
	CString szTempSaveFile = m_csFilePath;
	int ret = szTempSaveFile .ReverseFind(L'\\');
	if(ret != -1)
	{
		szTempSaveFile.Insert(ret+1, L"~$");
	}

	FS_BOOL bSavePro = FSDK_SavePDFFile(m_pdfDocument, szTempSaveFile.GetBuffer());
	if (bSavePro)
	{
		bSavePro = CopyFile(szTempSaveFile, m_csFilePath, FALSE);
		if(bSavePro)
		{
			this->ClearChangeMark();
		}
		else
		{
			AfxMessageBox(_T("Save Failed!"));
		}
	}

	BOOL bDel = DeleteFile(szTempSaveFile);//[delete temporary file name]
	return bSavePro;
}

CString CReader_Document::GetFilePathStr()
{
	return m_csFilePath;
}

BOOL CReader_Document::SaveAs()
{
	FS_BOOL bSavePro = FALSE;
	CFileDialog filedlg(FALSE); 
	filedlg.m_ofn.lpstrFilter = _T("*.pdf"); 
	filedlg.m_ofn.lpstrDefExt = _T("pdf");
	if (filedlg.DoModal() == IDOK)
	{
		CString path = filedlg.GetPathName();

		FS_BOOL bSavePro = FSDK_SavePDFFile(m_pdfDocument, path.GetBuffer());
	}
	return bSavePro;
}

void CReader_Document::SetLayerContext(FSPDF_LAYERCONTEXT LayerContext)
{
	m_bLayer = TRUE;
	m_layerContext = LayerContext;
}

FSPDF_LAYERCONTEXT CReader_Document::GetLayerContext()
{
	return m_layerContext;
}

FS_BOOL CReader_Document::IsChangeLayerContext()
{
	return m_bLayer;
}

