#ifndef _PDFREADERDOCUMENT_H
#define _PDFREADERDOCUMENT_H

#include "../../FrameworkInterface/include/IApp.h"

/************************************************************************/
/*class  IReader_Document                                               */
/************************************************************************/
class CPDFDisplayView;
class CReader_Document : public IReader_Document
{
public:
	CReader_Document(IReader_App* pApp);
	virtual ~CReader_Document();

public:
	virtual IReader_App* GetApp();

	virtual FSCRT_DOCUMENT GetDocument();

	virtual int GetPageCount();

	virtual IReader_Page* GetPage(int nIndex);

	virtual IReader_Page* GetPage(CPoint point);

	virtual BOOL GotoPage(int nIndex);

	virtual BOOL MovePage(CPoint curPoint, CPoint toPoint);

	virtual BOOL MovePage(short zDelta);

	virtual void SetChangeMark();

	virtual BOOL GetChangeMark();

	virtual void ClearChangeMark();

	virtual HWND GetDocViewHwnd();

	virtual void SetSelectedAnnot(IReader_Annot* pReaderAnnot);
	virtual IReader_Annot* GetSelectedAnnot();

	virtual void SetCapturedAnnot(IReader_Annot* pReaderAnnot);
	IReader_Annot* GetCapturedAnnot();

	virtual BOOL Save();
	virtual BOOL SaveAs();

	virtual void SetLayerContext(FSPDF_LAYERCONTEXT LayerContext);
	

public:
	FSCRT_FILE GetFSCRTFILE(const wchar_t* fileName, const char* fileMode, bool useDef);
	BOOL LoadPDFDocument(LPCWSTR lpwsFilePath);
	BOOL ReLoadPDFDocument();
	BOOL InitPDFPages(int nLogPixelX, int nLogPixelY, int nRotate);
	BOOL SetPDFDisplayView(CPDFDisplayView* pPDFDisplayView);
	CPDFDisplayView* GetPDFDisplayView() {return m_pPDFDisplayView;}
	FSPDF_LAYERCONTEXT GetLayerContext();
	FS_BOOL IsChangeLayerContext();


	float GetRealHeight();
	float GetRealWidth();
	int GetPageShadowWidth();
	void FindVisiblePage(float& StartPos, float& EndPos, int& StartPage, int& EndPage);
	BOOL IsVisiblePage(FS_INT32 index);

	BOOL DocToWindow(int nPageIndex, FSCRT_RECTF rcDoc, CRect& rcWindow);
	BOOL WindowToDoc(int nPageIndex, int window_x, int window_y, FS_FLOAT& doc_x, FS_FLOAT& doc_y);
	BOOL WindowToDoc(int nPageIndex, CRect rcWindow, FSCRT_RECTF& rcDoc);
	BOOL WindowToDocForPrint(int nPageIndex, CRect rcWindow, FSCRT_RECTF& rcDoc);

	CString GetFilePathStr();
	FS_INT32 m_nLostFocusIndex;
protected:
	FSCRT_DOCUMENT m_pdfDocument;
	CString m_csFilePath;
	FSCRT_FILE m_hFSCRTFile;

	BOOL m_bInitPages;
	int m_nPageSpace;
	int m_nTotalPage;
	float m_fRealHeight;
	float m_fRealWidth;
	CTypedPtrArray<CPtrArray, IReader_Page*> m_pPageList;

	int m_nPageShadowWidth;

	CPDFDisplayView* m_pPDFDisplayView;
	IReader_App* m_pApp;
	IReader_Annot* m_pSelectedAnnot;
	IReader_Annot* m_pCapturedAnnot;
	BOOL m_bChangeMark;
	//when call AfxMessageBox in callback function, the mouse won't be released, it doesn't response LButtonUp. So, set the flag to send message.
	static BOOL m_bReleaseMouse;
	//for load password encrypted document
	FSCRT_BSTR m_strPassword;

	FSPDF_LAYERCONTEXT m_layerContext;
	FS_BOOL  m_bLayer;
public:
	FSPDF_FORMFILLER GetFormFiller(){return m_FormFiller;}
	static BOOL GetMouseFlag(){ return m_bReleaseMouse; }
	static void SetMouseFlag(BOOL bReleaseMouse){ m_bReleaseMouse = bReleaseMouse; }
	//for load password encrypted document
	void SetDocPassword(CString csPassword);
	BOOL m_bLoadAnnot;//for License check
	BOOL m_bLoadText;
	BOOL m_bRender;
protected:
	FSPDF_FORMFILLER m_FormFiller;
	FSPDF_FORM m_form ;
};

#endif