#ifndef _PDFREADERPAGE_H
#define _PDFREADERPAGE_H

#include "../../FrameworkInterface/include/IApp.h"

/////////////////////////////////////////////////////////////////////////////////////////////////////
//class  IReader_Page
// A class representing a page
// This is an interface class implemented by Reader

class CReader_Document;
class CReader_Page : public IReader_Page
{
public:
	CReader_Page(CReader_Document* pReaderDocument);
	virtual ~CReader_Page();

public:
	virtual IReader_Document* GetDocument();

	virtual FSCRT_PAGE GetPage();

	virtual FSPDF_TEXTLINK GetTextLink();

    virtual BOOL ParsePage();

	virtual BOOL IsParseTextLink();

	virtual void SetPDFPageTextLink(FSPDF_TEXTLINK textLink);

	virtual void SetPDFTextPage(FSPDF_TEXTPAGE textPage);

	//for Text Serch
	FS_BOOL		LoadPDFTextPage();
	void		ReleasePDFTextPage();
	FS_BOOL     StartSearch(const FSCRT_BSTR* searchPattern, FS_DWORD flags, FS_INT32 startIndex);
	FS_BOOL		FindNext();
	FS_BOOL		FindPrev();
	FS_BOOL		ContinueSearch(FS_INT32 nDirection,const FSCRT_BSTR* searchPattern, FS_DWORD flags, FS_INT32 startIndex,FS_INT32* pFindPageIndex);

	FSPDF_TEXTPAGE GetPDFTextPage();
	FSPDF_TEXTSEARCH GetPDFTextSearch();

	virtual int GetPageIndex();

	virtual BOOL DocToWindow(FSCRT_RECTF rcDoc, CRect& rcWindow);

	virtual BOOL WindowToDoc(int window_x, int window_y, FS_FLOAT& doc_x, FS_FLOAT& doc_y);

	virtual BOOL WindowToDoc(CRect rcWindow, FSCRT_RECTF& rcDoc);

	virtual BOOL WindowToDocForPrint(int nPageIndex, CRect rcWindow, FSCRT_RECTF& rcDoc);

	virtual IReader_Annot* AddAnnot(LPCSTR lpSubType, FSCRT_RECTF rect);

	virtual BOOL DeleteAnnot(IReader_Annot* pAnnot);

	virtual	void MoveAnnot(IReader_Page* pDestPage, IReader_Annot *pAnnot);

	virtual int CountAnnots();

	virtual IReader_Annot* GetAnnot(int index);

	// there can be only one focus annotation in the page, and actually in one document
	virtual IReader_Annot* GetFocusAnnot();

	virtual void SetFocusAnnot(IReader_Annot* pAnnot);
	void  SetFocusAnnotFlag(BOOL bFlag);

public:
	void SetPageIndex(int nIndex);
	void SetPageHeight(float fHeight);
	void SetPageWidth(float fWidth);
	void SetPageTotalHeight(float fTotalHeight);
	void SetPageRealWidth(float fPageRealWidth);

	BOOL LoadAnnots();


	IReader_Annot* GetAnnotAtPoint(CPoint point, CString csType = L"");

	float GetPageTotalHeight();
	float GetPageRealWidth();
	float GetPageHeight();
	float GetPageWidth();
	BOOL  IsHasTransparency();
	void ReleaseTempData();

	//Annot Set
	void setAnnotProperties(FSCRT_ANNOT annot, char* sAnnotType);
	void setHighlightAnnotProperties(FSCRT_ANNOT annot, char* sAnnotType,FSCRT_RECTF rect);
	void setUnderLineAnnotProperties(FSCRT_ANNOT annot, char* sAnnotType,FSCRT_RECTF rect);
	void setFreeTextAnnotProperties(FSCRT_ANNOT annot, char* sAnnotType,FSCRT_RECTF rect);
	void setTextAnnotProperties(FSCRT_ANNOT annot, char* sAnnotType);
	void setLinkAnnotProperties(FSCRT_ANNOT annot, char* sAnnotType);
	void setAnnotBorder(FSCRT_ANNOT annot);
protected:
	int m_nIndex;
	float m_fHeight;
	float m_fWidth;
	float m_fTotalHeight;
	float m_fPageRealWidth;
	FSCRT_PAGE m_pdfPage;
	
	BOOL m_bHasTransparency;
	BOOL m_bIsParserTextLink;
	BOOL m_bIsLoadAnnot;
	BOOL m_bIsParserPage;

	BOOL m_bAnnotFocus;

	CReader_Document* m_pReaderDocument;
	FSPDF_TEXTPAGE m_TextPage;
	FSPDF_TEXTLINK m_TextLink;
	FSPDF_TEXTSEARCH m_TextSearch;
	CTypedPtrArray<CPtrArray, IReader_Annot*> m_pAnnotList;
};

#endif