#include "stdafx.h"
#include "../../FrameworkInterface/include/IApp.h"
#include "PDFReaderTool.h"
#include "../resource.h"
#include "PDFReaderPage.h"

CReader_HandToolHandler::CReader_HandToolHandler()
:m_bLButtonDown(FALSE)
,m_nTextLink(-1)
{
}
CReader_HandToolHandler::~CReader_HandToolHandler()
{
}

// Initialization
void CReader_HandToolHandler::Tool_OnInit(IReader_App* pApp)
{
	m_pApp = pApp;
}

// Destory
void CReader_HandToolHandler::Tool_Destroy()
{
}

// Get tool basic info. Note: all strings should be in host encoding
CString CReader_HandToolHandler::Tool_GetName()
{
	return TOOLHANDLER_NAME_HAND;
}

BOOL CReader_HandToolHandler::Tool_IsProcessing()
{
	return FALSE;
}

// UI Events
void CReader_HandToolHandler::Tool_OnActivate(BOOL bPersistent)
{
	::SetCursor(AfxGetApp()->LoadCursor(IDC_CUR_HAND));
}

void CReader_HandToolHandler::Tool_OnDeactivate()
{
}

// Mouse events
BOOL CReader_HandToolHandler::Tool_OnLButtonDown(IReader_Page* pPage, UINT nFlags, const CPoint& point)
{
	m_bLButtonDown = TRUE;
	m_curPos = point;

	if (-1 != m_nTextLink )
	{
		FSPDF_TEXTLINK	textLink = pPage->GetTextLink();
		FSCRT_BSTR linkURI;
		FSCRT_BStr_Init(&linkURI);
		FS_RESULT ret = FSPDF_TextLink_GetLink(textLink, m_nTextLink, &linkURI);

		if (FSCRT_ERRCODE_SUCCESS == ret)
		{
			FS_DWORD len = 0;
			FSCRT_UTF8_ToUTF16LE(&linkURI, NULL, &len);
			FS_WORD* pBuff = new FS_WORD[len+1];
			pBuff[len] = 0;
			FSCRT_UTF8_ToUTF16LE(&linkURI, pBuff, &len);
			AfxMessageBox((LPWSTR)pBuff);
			delete[] pBuff;
			m_bLButtonDown = FALSE;
		}
		FSCRT_BStr_Clear(&linkURI);
	}
	return TRUE;
}

BOOL CReader_HandToolHandler::Tool_OnLButtonUp(IReader_Page* pPage, UINT nFlags, const CPoint& point)
{
	m_bLButtonDown = FALSE;
	m_curPos = 0;
	return TRUE;
}

BOOL CReader_HandToolHandler::Tool_OnLButtonDblClk(IReader_Page* pPage, UINT nFlags, const CPoint& point)
{
	return FALSE;
}

BOOL CReader_HandToolHandler::Tool_OnMouseMove(IReader_Page* pPage, UINT nFlags, const CPoint& point)
{
	IReader_Document* pReaderDocument = m_pApp->GetCurrentDocument();
	if(!pReaderDocument) return FALSE;

	if(m_bLButtonDown)
	{
		CWnd* pDocView = CWnd::FromHandle(pReaderDocument->GetDocViewHwnd());
		pDocView->SetCapture();
		::SetCursor(AfxGetApp()->LoadCursor(IDC_CUR_CATCH));
		pReaderDocument->MovePage(m_curPos, point);
		
		m_curPos = point;
		
		CRect rsClient;
		pDocView->GetClientRect(&rsClient);
		if(rsClient.PtInRect(point) == 0)
		{
			::ReleaseCapture();
			::SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));
			m_bLButtonDown = FALSE;
			m_curPos = 0;
		}
	}
	else
	{
		if (IsInTextLinkRect(pPage, point))
			::SetCursor(AfxGetApp()->LoadCursor(IDC_CUR_SELECTANNOT));
		else
			::SetCursor(AfxGetApp()->LoadCursor(IDC_CUR_HAND));
	}
	return TRUE;
}

BOOL CReader_HandToolHandler::Tool_OnRButtonDown(IReader_Page* pPage, UINT nFlags, const CPoint& point)
{
	return FALSE;
}

BOOL CReader_HandToolHandler::Tool_OnRButtonUp(IReader_Page* pPage, UINT nFlags, const CPoint& point)
{
	return FALSE;
}

BOOL CReader_HandToolHandler::Tool_OnRButtonDblClk(IReader_Page* pPage, UINT nFlags, const CPoint& point)
{
	return FALSE;
}

BOOL CReader_HandToolHandler::Tool_OnMouseWheel(IReader_Page* pPage, UINT nFlags, short zDelta, const CPoint& point)
{
	IReader_Document* pReaderDocument = m_pApp->GetCurrentDocument();
	if(!pReaderDocument) return FALSE;

	pReaderDocument->MovePage(zDelta);

	return TRUE;
}

//[Draw]
BOOL CReader_HandToolHandler::Tool_OnDraw(IReader_Document* pDoc, HDC hDC)
{
	return FALSE;
}

BOOL CReader_HandToolHandler::IsInTextLinkRect(IReader_Page* pPage, const CPoint& point)
{
	if (!pPage){
		m_nTextLink = -1;
		return FALSE;
	}
	FSPDF_TEXTPAGE textpage = NULL;
	FSPDF_TEXTLINK textLink = NULL; 

	if (!pPage->IsParseTextLink())
	{
		BOOL bRet = pPage->ParsePage();
		if (!bRet)
		{
			return TRUE;
		}
		CReader_Page* page = (CReader_Page*)pPage;
		if (!page->GetPDFTextPage())
		{
			page->LoadPDFTextPage();
		}
		textpage = page->GetPDFTextPage();
		FS_RESULT ret = FSPDF_TextPage_ExtractLinks(textpage, &textLink);
		pPage->SetPDFPageTextLink(textLink);
	}
	else
		textLink = pPage->GetTextLink();

	if (!textLink)
	{
		m_nTextLink = -1;
		return FALSE;
	}
	FS_INT32 nLinkCount = 0;
	FS_RESULT ret = FSPDF_TextLink_CountLinks(textLink, &nLinkCount);
	FSPDF_TEXTSELECTION textSelect;
	if ( FSCRT_ERRCODE_SUCCESS != ret)
	{
		m_nTextLink = -1;
		return FALSE;
	}
	for (FS_INT32 i = 0; i < nLinkCount; i ++)
	{
		ret = FSPDF_TextLink_GetSelection(textLink, i, &textSelect);
		if (FSCRT_ERRCODE_SUCCESS != ret)
			continue;
		FS_INT32 nPieceCount = 0;
		FSPDF_TextSelection_CountPieces(textSelect, &nPieceCount);
		FSCRT_RECTF rect;
		for (FS_INT32 j = 0; j < nPieceCount; j++)
		{
			ret = FSPDF_TextSelection_GetPieceRect(textSelect, j, &rect);
			if (FSCRT_ERRCODE_SUCCESS != ret)
			{
				continue;
			}
			CRect winRect;
			float x;
			float y;	 
			pPage->WindowToDoc(point.x, point.y, x, y);
			if (x > rect.left && x < rect .right && y < rect.top && y > rect.bottom)
			{
				m_nTextLink = i;
				return TRUE;
			}
		}
		FSPDF_TextSelection_Release(textSelect);
	}
	m_nTextLink = -1;
	return FALSE;
}

