#include "stdafx.h"
#include "Reader_FreeTextAnnotHandler.h"

#define FREETEXTANNOTTYPE L"FreeText"
#define FREETEXTANNOTHANDLER L"FreeTextAnnotHandler"


///////////////////////////////////////////////////////////////////////////////////////////
// CReader_FreeTextAnnot
///////////////////////////////////////////////////////////////////////////////////////////

CReader_FreeTextAnnot::CReader_FreeTextAnnot(FSCRT_ANNOT Annot, IReader_Page* pPgae)
	: m_FreeTextAnnot(Annot)
	, m_pPage(pPgae)
	, m_bSelect(FALSE)
	, m_bCapture(FALSE)
{
}

CReader_FreeTextAnnot::~CReader_FreeTextAnnot()
{

}

FSCRT_ANNOT CReader_FreeTextAnnot::GetPDFAnnot()
{
	return m_FreeTextAnnot;
}

CString	CReader_FreeTextAnnot::GetType() 
{
	return NULL;
}

CString CReader_FreeTextAnnot::GetSubType()
{
	return NULL;
}

BOOL CReader_FreeTextAnnot::IsSelected()
{
	return m_bSelect;
}

void CReader_FreeTextAnnot::SetSelected(BOOL bSelected)
{
	m_bSelect = bSelected;
}

BOOL CReader_FreeTextAnnot::IsCaptured()
{
	return m_bCapture;
}

void CReader_FreeTextAnnot::SetCaptured(BOOL bCaptured)
{
	m_bCapture = bCaptured;
}

CRect CReader_FreeTextAnnot::GetViewBBox()
{
	FSCRT_RECTF rect;
	CRect WinRect;
	FSPDF_Annot_GetRect(m_FreeTextAnnot, &rect);

	m_pPage->DocToWindow(rect, WinRect);
	WinRect.NormalizeRect();
	// WinRect.top += 3;
	return WinRect;
}

CRect CReader_FreeTextAnnot::GetInnerRectMargin()
{
	FSCRT_RECTF rect;
	CRect WinRect;
	FS_FLOAT left = 0;
	FS_FLOAT top = 0;
	FS_FLOAT right = 0;
	FS_FLOAT bottom = 0;
	FSPDF_Annot_GetInnerRectMargin(m_FreeTextAnnot, &left, &top, &right, &bottom);
	FSPDF_Annot_GetRect(m_FreeTextAnnot, &rect);
	rect.top -= top;
	rect.right -= right;
	rect.bottom += bottom;
	rect.left += left;

	m_pPage->DocToWindow(rect, WinRect);
	WinRect.NormalizeRect();
	return WinRect;
}

BOOL CReader_FreeTextAnnot:: IsLock()
{
	FS_DWORD flags;
	FSPDF_Annot_GetFlags(m_FreeTextAnnot, &flags);
	if (FSPDF_ANNOTFLAG_LOCKED == flags)
	{
		return TRUE;
	}
	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////
// CReader_FreeTextAnnotHandler
///////////////////////////////////////////////////////////////////////////////////////////
CReader_FreeTextAnnotHandler::CReader_FreeTextAnnotHandler(void)
{
	m_bLButtonDown = FALSE;
}

CReader_FreeTextAnnotHandler::~CReader_FreeTextAnnotHandler(void)
{
	
}

IReader_Annot* CReader_FreeTextAnnotHandler::NewAnnot(FSCRT_ANNOT pdfAnnot, IReader_Page* pPage)
{
	return (IReader_Annot*) new CReader_FreeTextAnnot(pdfAnnot, pPage);
}

//return annotation type name, for example: "Line"
CString	CReader_FreeTextAnnotHandler::GetType() 
{
	return FREETEXTANNOTTYPE;
}

//return AnnotHandler unique identification, for example:"LineAnnotHandler"
CString CReader_FreeTextAnnotHandler::GetName() 
{
	return FREETEXTANNOTHANDLER;
}

//return position of annotation, it can be real position, also can be zoomed in properly, this position is also UpdateAllViews's reference position.
CRect CReader_FreeTextAnnotHandler::GetViewBBox(IReader_Page *pPage, IReader_Annot* pAnnot)
{
	return	pAnnot->GetViewBBox();
}

BOOL CReader_FreeTextAnnotHandler::HitTest(IReader_Page *pPage, IReader_Annot* pAnnot, const CPoint& point)
{
	return TRUE;
}

//return TRUE means have done, framework doesn't deal with .
BOOL CReader_FreeTextAnnotHandler::OnLButtonDown(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	m_bLButtonDown = TRUE;
	m_StartPoint = m_EndPoint = point;
	return TRUE;
}

BOOL CReader_FreeTextAnnotHandler::OnLButtonUp(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	m_bLButtonDown = FALSE;
	m_EndPoint = point;

	if (m_StartPoint == m_EndPoint)
	{
		return TRUE;
	}

	CRect winRect = pAnnot->GetViewBBox();

	int x = m_EndPoint.x-m_StartPoint.x;
	int y = m_EndPoint.y -m_StartPoint.y;
	winRect.OffsetRect(x, y);
	FSCRT_RECTF rect;
	pPage->WindowToDoc(winRect, rect);

	if (rect.left > rect.right)
	{
		FS_FLOAT tem = rect.left;
		rect.left = rect.right;
		rect.right = tem;
	}

	if (rect.top < rect.bottom)
	{
		FS_FLOAT tem = rect.top;
		rect.top = rect.bottom;
		rect.bottom = tem;
	}


	FS_RESULT ret = FSPDF_Annot_Move(pAnnot->GetPDFAnnot(), &rect);

	if ( FSCRT_ERRCODE_SUCCESS == ret)
	{
		FSPDF_Annot_ResetAppearance(pAnnot->GetPDFAnnot());

		CWnd* pDocView = CWnd::FromHandle(pPage->GetDocument()->GetDocViewHwnd());
		
		if (pDocView)
		{
			pDocView->InvalidateRect(winRect);
		}

		IReader_Document* pReaderDocument = pPage->GetDocument();
		if (NULL != pReaderDocument)
		{
			pReaderDocument->SetChangeMark();
		}
	}

	return TRUE;
}

 BOOL CReader_FreeTextAnnotHandler::OnLButtonDblClk(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	CRect winRect = pAnnot->GetViewBBox();
	return FALSE;
}

 BOOL CReader_FreeTextAnnotHandler::OnMouseMove(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	if (m_bLButtonDown)
	{
		m_EndPoint = point;
	}
	return TRUE;
}

 BOOL CReader_FreeTextAnnotHandler::OnMouseWheel(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, short zDelta, const CPoint& point) 
{
	return FALSE;
}

 BOOL CReader_FreeTextAnnotHandler::OnRButtonDown(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	return FALSE;
}

 BOOL CReader_FreeTextAnnotHandler::OnRButtonUp(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return FALSE;
}

 BOOL CReader_FreeTextAnnotHandler::OnRButtonDblClk(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return FALSE;
}

void CReader_FreeTextAnnotHandler::Annot_OnDraw(IReader_Page* pPage, HDC hDC, IReader_Annot* pAnnot, FSCRT_MATRIX matrix, const CRect& rcPage)
{
	DWORD crBorder = RGB(255, 0, 0);
	DWORD crRect = RGB(255, 0, 0);
	CRect winRect = pAnnot->GetInnerRectMargin();
	if (pAnnot->IsSelected())	
		DrawSelectRect(pPage, hDC, pAnnot, winRect, crBorder, crRect, TRUE);


	CRect viewRect = pAnnot->GetViewBBox();
	if (pAnnot->IsCaptured() && m_bLButtonDown)
	{
		DWORD crBorder = RGB(100, 162, 226);
		DWORD crRect = RGB(100, 25, 125);
		int x = m_EndPoint.x-m_StartPoint.x;
		int y = m_EndPoint.y -m_StartPoint.y;
		viewRect.OffsetRect(x, y);
		DrawSelectRect(pPage, hDC, pAnnot, viewRect, crBorder, crRect, FALSE);
	}
}

void CReader_FreeTextAnnotHandler::DrawSelectRect(IReader_Page* pPage, HDC hDC, IReader_Annot* pAnnot, const CRect& winRect, DWORD crBorder, DWORD crRect, BOOL bCorner)
{
	HPEN hPen = ::CreatePen(PS_SOLID, 1, crBorder);
	HPEN hOldPen = (HPEN)::SelectObject(hDC, hPen);
	LOGBRUSH lb;
	lb.lbStyle = BS_HOLLOW;
	lb.lbColor = crRect;
	lb.lbHatch = 0;
	HBRUSH hBrush = ::CreateBrushIndirect(&lb);
	HBRUSH hOldBrush = (HBRUSH)::SelectObject(hDC, hBrush);
	::Rectangle(hDC, winRect.left, winRect.top, winRect.right, winRect.bottom);
	::SelectObject(hDC, hOldBrush);
	::DeleteObject(hBrush);
	::SelectObject(hDC, hOldPen);
	::DeleteObject(hPen);

	CRect rcArray[11];
	if (bCorner)
	{
		GetSelectRects(winRect, rcArray);
		GetCalloutLinePoints(pPage, pAnnot, rcArray);
		LOGBRUSH lb;
		lb.lbStyle = BS_SOLID;
		lb.lbColor = crRect;
		lb.lbHatch = 0;
		HBRUSH hBrush = ::CreateBrushIndirect(&lb);

		for (int i = 0; i<11; i++)
		{
			::FillRect(hDC, &rcArray[i], hBrush);
		}

		::DeleteObject(hBrush);
	}
}

void CReader_FreeTextAnnotHandler::GetSelectRects(CRect winRect, CRect* rcArray)
{
	CPoint ptCenter[8];
	ptCenter[0] = CPoint(winRect.left, winRect.top);
	ptCenter[1] = CPoint((winRect.left + winRect.right) / 2, winRect.top);
	ptCenter[2] = CPoint(winRect.right, winRect.top);
	ptCenter[3] = CPoint(winRect.right, (winRect.top + winRect.bottom) / 2);
	ptCenter[4] = CPoint(winRect.right, winRect.bottom);
	ptCenter[5] = CPoint((winRect.left + winRect.right) / 2, winRect.bottom);
	ptCenter[6] = CPoint(winRect.left, winRect.bottom);
	ptCenter[7] = CPoint(winRect.left, (winRect.top + winRect.bottom) / 2);


	for (int i = 0; i<8; i++)
	{
		rcArray[i] = CRect(ptCenter[i].x - 3, ptCenter[i].y - 3, ptCenter[i].x + 3, ptCenter[i].y + 3);
	}
}

void CReader_FreeTextAnnotHandler::GetCalloutLinePoints(IReader_Page* pPage, IReader_Annot* pAnnot, CRect* rcArray)

{
	FS_FLOAT Docx1;
	FS_FLOAT Docy1;
	FS_FLOAT Docx2;
	FS_FLOAT Docy2;
	FS_FLOAT Docx3;
	FS_FLOAT Docy3;

	FSPDF_Annot_GetCalloutLinePoints(pAnnot->GetPDFAnnot(), &Docx1, &Docy1, &Docx2, &Docy2, &Docx3, &Docy3);
	CRect winRect;
	FSCRT_RECTF rect;
	rect.left = 0;
	rect.top = Docy1;
	rect.right = Docx1;
	rect.bottom = 0;
	pPage->DocToWindow(rect, winRect);

	winRect.NormalizeRect();
	rcArray[8] = CRect(winRect.right - 3, winRect.top - 3, winRect.right + 3, winRect.top + 3);
	
	rect.top = Docy2;
	rect.right = Docx2;
	pPage->DocToWindow(rect, winRect);
	winRect.NormalizeRect();

	rcArray[9] = CRect(winRect.right - 3, winRect.top - 3, winRect.right + 3, winRect.top + 3);

	rect.top = Docy3;
	rect.right = Docx3;
	pPage->DocToWindow(rect, winRect);
	winRect.NormalizeRect();

	rcArray[10] = CRect(winRect.right - 3, winRect.top - 3, winRect.right + 3, winRect.top + 3);
}
