#include "stdafx.h"
#include "Reader_HighLightAnnotHandler.h"

#define HIGHLIGHTANNOTTYPE L"Highlight"
#define HIGHLIGHTANNOTHANDLER L"HighlightAnnotHandler"


///////////////////////////////////////////////////////////////////////////////////////////
// CReader_HighLightAnnot
///////////////////////////////////////////////////////////////////////////////////////////
CReader_HighLightAnnot::CReader_HighLightAnnot(FSCRT_ANNOT Annot, IReader_Page* pPgae)
	: m_HighlightAnnot(Annot)
	, m_pPage(pPgae)
	, m_bSelect(FALSE)
{
}

CReader_HighLightAnnot::~CReader_HighLightAnnot()
{

}

FSCRT_ANNOT CReader_HighLightAnnot::GetPDFAnnot()
{
	return m_HighlightAnnot;
}

CString	CReader_HighLightAnnot::GetType() 
{
	return NULL;
}

CString CReader_HighLightAnnot::GetSubType()
{
	return NULL;
}

BOOL CReader_HighLightAnnot::IsSelected()
{
	return m_bSelect;
}

void CReader_HighLightAnnot::SetSelected(BOOL bSelected)
{
	m_bSelect = bSelected;
}

BOOL CReader_HighLightAnnot::IsCaptured()
{
	return FALSE;
}

void CReader_HighLightAnnot::SetCaptured(BOOL bCaptured)
{
	
}

CRect CReader_HighLightAnnot::GetViewBBox()
{
	FSCRT_RECTF rect;
	CRect WinRect;
	FSPDF_Annot_GetRect(m_HighlightAnnot, &rect);

	m_pPage->DocToWindow(rect, WinRect);
	WinRect.NormalizeRect();
	return WinRect;
}

CRect CReader_HighLightAnnot::GetInnerRectMargin()
{
	FSCRT_RECTF rect;
	CRect WinRect;
	FS_FLOAT left = 0;
	FS_FLOAT top = 0;
	FS_FLOAT right = 0;
	FS_FLOAT bottom = 0;

	FSPDF_Annot_GetInnerRectMargin(m_HighlightAnnot, &left, &top, &right, &bottom);
	FSPDF_Annot_GetRect(m_HighlightAnnot, &rect);
	rect.top -= top;
	rect.right -= right;
	rect.bottom += bottom;
	rect.left += left;

	m_pPage->DocToWindow(rect, WinRect);
	WinRect.NormalizeRect();
	return WinRect;
}

BOOL CReader_HighLightAnnot:: IsLock()
{
	FS_DWORD flags;
	FSPDF_Annot_GetFlags(m_HighlightAnnot, &flags);
	if (FSPDF_ANNOTFLAG_LOCKED == flags)
	{
		return TRUE;
	}
	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////
// CReader_HighLightAnnotHandler
///////////////////////////////////////////////////////////////////////////////////////////
CReader_HighLightAnnotHandler::CReader_HighLightAnnotHandler(void)
{
}

CReader_HighLightAnnotHandler::~CReader_HighLightAnnotHandler(void)
{
}

IReader_Annot* CReader_HighLightAnnotHandler::NewAnnot(FSCRT_ANNOT pdfAnnot, IReader_Page* pPage)
{
	return (IReader_Annot*) new CReader_HighLightAnnot(pdfAnnot, pPage) ;
}

//return annotation type name, for example: "Line"
CString	CReader_HighLightAnnotHandler::GetType() 
{
	return HIGHLIGHTANNOTTYPE;
}

//return AnnotHandler unique identification, for example:"LineAnnotHandler"
CString CReader_HighLightAnnotHandler::GetName() 
{
	return HIGHLIGHTANNOTHANDLER;
}

//return position of annotation, it can be real position, also can be zoomed in properly, this position is also UpdateAllViews's reference position.
CRect CReader_HighLightAnnotHandler::GetViewBBox(IReader_Page *pPage, IReader_Annot* pAnnot)
{
	return	pAnnot->GetViewBBox();
}

 BOOL CReader_HighLightAnnotHandler::HitTest(IReader_Page *pPage, IReader_Annot* pAnnot, const CPoint& point)
{
	return TRUE;
}

//return TRUE means have done, framework doesn't deal with .
 BOOL CReader_HighLightAnnotHandler::OnLButtonDown(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return TRUE;
}

 BOOL CReader_HighLightAnnotHandler::OnLButtonUp(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return FALSE;
}
 
 BOOL CReader_HighLightAnnotHandler::OnLButtonDblClk(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	return FALSE;
}
 
 BOOL CReader_HighLightAnnotHandler::OnMouseMove(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	return FALSE;
}

 BOOL CReader_HighLightAnnotHandler::OnMouseWheel(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, short zDelta, const CPoint& point) 
{
	return FALSE;
}

 BOOL CReader_HighLightAnnotHandler::OnRButtonDown(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return FALSE;
}

 BOOL CReader_HighLightAnnotHandler::OnRButtonUp(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return FALSE;
}

 BOOL CReader_HighLightAnnotHandler::OnRButtonDblClk(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return FALSE;
}

void CReader_HighLightAnnotHandler::Annot_OnDraw(IReader_Page* pPage, HDC hDC, IReader_Annot* pAnnot, FSCRT_MATRIX matrix, const CRect& rcPage)
{
	DWORD crBorder = RGB(255, 0, 0);
	DWORD crRect = RGB(0, 0, 0);
	CRect winRect = pAnnot->GetInnerRectMargin();
	if (pAnnot->IsSelected())
		DrawSelectRect(pPage, hDC, pAnnot, winRect, crBorder, crRect, TRUE);
}

void CReader_HighLightAnnotHandler::DrawSelectRect(IReader_Page* pPage, HDC hDC, IReader_Annot* pAnnot, const CRect& winRect, DWORD crBorder, 
	DWORD crRect, BOOL bCorner)
{
	HPEN hPen = ::CreatePen(PS_SOLID, 1, crBorder);
	HPEN hOldPen = (HPEN)::SelectObject(hDC, hPen);
	LOGBRUSH lb;
	lb.lbStyle = BS_HOLLOW;
	lb.lbColor = crRect;
	lb.lbHatch = 0;
	HBRUSH hBrush = ::CreateBrushIndirect(&lb);
	HBRUSH hOldBrush = (HBRUSH)::SelectObject(hDC, hBrush);
	::Rectangle(hDC, winRect.left, winRect.top, winRect.right, winRect.bottom);
	::SelectObject(hDC, hOldBrush);
	::DeleteObject(hBrush);
	::SelectObject(hDC, hOldPen);
	::DeleteObject(hPen);
}
