#include "stdafx.h"
#include "Reader_LinkAnnotHandler.h"

#define LINKANNOTTYPE L"Link"
#define LINKANNOTHANDLER L"LinkAnnotHandler"



///////////////////////////////////////////////////////////////////////////////////////////
//			CReader_LinkAnnot
///////////////////////////////////////////////////////////////////////////////////////////

CReader_LinkAnnot::CReader_LinkAnnot(FSCRT_ANNOT Annot, IReader_Page* pPgae)
	: m_LinkAnnot(Annot)
	, m_pPage(pPgae)
	, m_bSelect(FALSE)
	, m_bCapture(FALSE)
{
}

CReader_LinkAnnot::~CReader_LinkAnnot()
{

}


FSCRT_ANNOT CReader_LinkAnnot::GetPDFAnnot()
{
	return m_LinkAnnot;
}

CString	CReader_LinkAnnot::GetType() 
{
	return NULL;
}

CString CReader_LinkAnnot::GetSubType()
{
	return NULL;
}

BOOL CReader_LinkAnnot::IsSelected()
{
	return m_bSelect;
}

void CReader_LinkAnnot::SetSelected(BOOL bSelected)
{
	m_bSelect = bSelected;
}

BOOL CReader_LinkAnnot::IsCaptured()
{
	return m_bCapture;
}

void CReader_LinkAnnot::SetCaptured(BOOL bCaptured)
{
	m_bCapture = bCaptured;
}

CRect CReader_LinkAnnot::GetViewBBox()
{
	FSCRT_RECTF rect;
	CRect WinRect;
	FSPDF_Annot_GetRect(m_LinkAnnot, &rect);

	m_pPage->DocToWindow(rect, WinRect);
	WinRect.NormalizeRect();
	return WinRect;
}

CRect CReader_LinkAnnot::GetInnerRectMargin()
{
	FSCRT_RECTF rect;
	CRect WinRect;
	FS_FLOAT left = 0;
	FS_FLOAT top = 0;
	FS_FLOAT right = 0;
	FS_FLOAT bottom = 0;

	FSPDF_Annot_GetInnerRectMargin(m_LinkAnnot, &left, &top, &right, &bottom);
	FSPDF_Annot_GetRect(m_LinkAnnot, &rect);
	rect.top -= top;
	rect.right -= right;
	rect.bottom += bottom;
	rect.left += left;

	m_pPage->DocToWindow(rect, WinRect);
	WinRect.NormalizeRect();
	return WinRect;
}

BOOL CReader_LinkAnnot:: IsLock()
{
	FS_DWORD flags;
	FSPDF_Annot_GetFlags(m_LinkAnnot, &flags);
	if (FSPDF_ANNOTFLAG_LOCKED == flags)
	{
		return TRUE;
	}
	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////
// CReader_LinkAnnotHandler
///////////////////////////////////////////////////////////////////////////////////////////

CReader_LinkAnnotHandler::CReader_LinkAnnotHandler(void)
{
	m_bLButtonDown = FALSE;
}

CReader_LinkAnnotHandler::~CReader_LinkAnnotHandler(void)
{
}

IReader_Annot* CReader_LinkAnnotHandler::NewAnnot(FSCRT_ANNOT pdfAnnot, IReader_Page* pPage)
{
	return (IReader_Annot*) new CReader_LinkAnnot(pdfAnnot, pPage) ;
}

//return annotation type name, for example: "Line"
CString	CReader_LinkAnnotHandler::GetType() 
{
	return LINKANNOTTYPE;
}

//return AnnotHandler unique identification, for example:"LineAnnotHandler"
CString CReader_LinkAnnotHandler::GetName() 
{
	return LINKANNOTHANDLER;
}

//return position of annotation, it can be real position, also can be zoomed in properly, this position is also UpdateAllViews's reference position.
CRect CReader_LinkAnnotHandler::GetViewBBox(IReader_Page *pPage, IReader_Annot* pAnnot)
{
	return	pAnnot->GetViewBBox();
}

BOOL CReader_LinkAnnotHandler::HitTest(IReader_Page *pPage, IReader_Annot* pAnnot, const CPoint& point)
{
	return TRUE;
}

//return TRUE means have done, framework doesn't deal with .
BOOL CReader_LinkAnnotHandler::OnLButtonDown(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	m_bLButtonDown = TRUE;
	m_StartPoint = m_EndPoint = point;
	return TRUE;
}

BOOL CReader_LinkAnnotHandler::OnLButtonUp(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	m_bLButtonDown = FALSE;
	m_EndPoint = point;

	CRect winRect = pAnnot->GetInnerRectMargin();

	int x = m_EndPoint.x-m_StartPoint.x;
	int y = m_EndPoint.y -m_StartPoint.y;
	winRect.OffsetRect(x, y);
	FSCRT_RECTF rect;
	pPage->WindowToDoc(winRect, rect);

	if (rect.left > rect.right)
	{
		FS_FLOAT tem = rect.left;
		rect.left = rect.right;
		rect.right = tem;
	}

	if (rect.top < rect.bottom)
	{
		FS_FLOAT tem = rect.top;
		rect.top = rect.bottom;
		rect.bottom = tem;
	}

	FS_RESULT ret = FSPDF_Annot_Move(pAnnot->GetPDFAnnot(), &rect);

	if ( FSCRT_ERRCODE_SUCCESS == ret)
	{
		FSPDF_Annot_ResetAppearance(pAnnot->GetPDFAnnot());
		CWnd* pDocView = CWnd::FromHandle(pPage->GetDocument()->GetDocViewHwnd());

		if (pDocView)
		{
			pDocView->InvalidateRect(winRect);
		}

		IReader_Document* pReaderDocument = pPage->GetDocument();
		if (NULL != pReaderDocument)
		{
			pReaderDocument->SetChangeMark();
		}
	}
	return TRUE;
}

BOOL CReader_LinkAnnotHandler::OnLButtonDblClk(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	FSPDF_ACTIONDATA data;
	for (int j = 0;j< 10;j++)
	{
		FS_INT32 actionCount = 0;
		FS_WORD* pBuff = NULL;
		CString strOutPut;
		
		FS_RESULT	ret = FSPDF_Annot_CountActions(pAnnot->GetPDFAnnot(), j, &actionCount);
		if (!actionCount)
		{
			continue;
		}
		for (int k = 0;k<actionCount;k++ )
		{
			strOutPut.Empty();
			FSPDF_Annot_GetAction(pAnnot->GetPDFAnnot(), j, k, &data);
			switch(data.actionType)
			{
			case FSPDF_ACTION_UNSUPPORTED:
				strOutPut += _T("ActionType: UNSUPPORTED ");
				break;
			case FSPDF_ACTION_GOTO:
				{
					FSPDF_ACTIONDATA_GOTO* gotoData = (FSPDF_ACTIONDATA_GOTO*)(data.actionData);
					strOutPut += _T("ActionType: GOTO ");
				}
				break;
			case FSPDF_ACTION_URI:				
				{
					FSCRT_BSTR bStr;
					FSCRT_BStr_Init(&bStr);

					FSPDF_ACTIONDATA_URI *UrlData = (FSPDF_ACTIONDATA_URI*)(data.actionData);
					strOutPut += _T("ActionType: URL ");
					wchar_t* buf = NULL;
					FS_DWORD length = 0;
					FS_RESULT hResult = FSCRT_UTF8_ToUTF16LE(&(UrlData->url), NULL, &length);
					if(hResult == FSCRT_ERRCODE_SUCCESS)
					{
						buf = new wchar_t[length+1];
						memset(buf, 0, (length+1)*sizeof(wchar_t));
						FSCRT_UTF8_ToUTF16LE(&(UrlData->url), (FS_WORD*)buf, &length);
					}
					CString tem = buf;
					delete [] buf;
					buf = NULL; 
					strOutPut += tem;
					FSCRT_BStr_Clear(&bStr);
				}
				break;
			case FSPDF_ACTION_LAUNCH:
				{
					FSPDF_ACTIONDATA_LAUNCH * launchData = (FSPDF_ACTIONDATA_LAUNCH*)(data.actionData);
		
					FSPDF_ATTACHMENT attach = launchData->attachment;
					FSCRT_BSTR filename;
					FSCRT_BStr_Init(&filename);
					FSPDF_Attachment_GetFileName(attach, &filename);
					
					strOutPut += _T("ActionType: ATTACHMENT ");
					wchar_t* buf = NULL;
					FS_DWORD length = 0;
					FS_RESULT hResult = FSCRT_UTF8_ToUTF16LE(&(filename), NULL, &length);
					if(hResult == FSCRT_ERRCODE_SUCCESS)
					{
						buf = new wchar_t[length+1];
						memset(buf, 0, (length+1)*sizeof(wchar_t));
						FSCRT_UTF8_ToUTF16LE(&(filename), (FS_WORD*)buf, &length);
					}
					CString tem = buf;
					delete [] buf;
					buf = NULL; 
					strOutPut += tem;
					FSCRT_BStr_Clear(&filename);
				}
				break;
			case FSPDF_ACTION_JAVASCRIPT :
				{

					FSPDF_ACTIONDATA_JAVASCRIPT* javaData = (FSPDF_ACTIONDATA_JAVASCRIPT*)data.actionData;
					
					strOutPut += _T("ActionType: JAVASCRIPT ");
					wchar_t* buf = NULL;
					FS_DWORD length = 0;
					FS_RESULT hResult = FSCRT_UTF8_ToUTF16LE(&(javaData->javascript), NULL, &length);
					if(hResult == FSCRT_ERRCODE_SUCCESS)
					{
						buf = new wchar_t[length+1];
						memset(buf, 0, (length+1)*sizeof(wchar_t));
						FSCRT_UTF8_ToUTF16LE(&(javaData->javascript), (FS_WORD*)buf, &length);
					}
					CString tem = buf;
					delete [] buf;
					buf = NULL; 
					strOutPut += tem;
				}
				break;
			case FSPDF_ACTION_HIDE:
				{
					strOutPut.Format(_T("ActionType: %s"), FSPDF_ACTION_HIDE);
					FSPDF_ACTIONDATA_HIDE* hideData = (FSPDF_ACTIONDATA_HIDE*)(data.actionData);

					strOutPut += _T("ActionType: HIDE ");
				}
				break;
			case FSPDF_ACTION_NAMED:
				{
					FSPDF_ACTIONDATA_NAMED* namedData = (FSPDF_ACTIONDATA_NAMED*)(data.actionData);
					strOutPut += _T("ActionType: NAMED ");
					wchar_t* buf = NULL;
					FS_DWORD length = 0;
					FS_RESULT hResult = FSCRT_UTF8_ToUTF16LE(&(namedData->namedAction), NULL, &length);
					if(hResult == FSCRT_ERRCODE_SUCCESS)
					{
						buf = new wchar_t[length+1];
						memset(buf, 0, (length+1)*sizeof(wchar_t));
						FSCRT_UTF8_ToUTF16LE(&(namedData->namedAction), (FS_WORD*)buf, &length);
					}
					CString tem = buf;
					delete [] buf;
					buf = NULL; 
					strOutPut += tem;
				}
				break;
			case FSPDF_ACTION_SUBMITFORM:
				{
					FSPDF_ACTIONDATA_SUBMITFORM* submitData = (FSPDF_ACTIONDATA_SUBMITFORM*)(data.actionData);
			
					strOutPut += _T("ActionType: SUBMITFORM ");
					if (submitData->attachment)
					{
						FSCRT_BSTR fileName;
						FSCRT_BStr_Init(&fileName);
						FSPDF_Attachment_GetFileName(submitData->attachment, &fileName);
								
						wchar_t* buf = NULL;
						FS_DWORD length = 0;
						FS_RESULT hResult = FSCRT_UTF8_ToUTF16LE(&(fileName), NULL, &length);
						if(hResult == FSCRT_ERRCODE_SUCCESS)
						{
							buf = new wchar_t[length+1];
							memset(buf, 0, (length+1)*sizeof(wchar_t));
							FSCRT_UTF8_ToUTF16LE(&(fileName), (FS_WORD*)buf, &length);
						}
						CString tem = buf;
						delete [] buf;
						buf = NULL; 
						strOutPut += tem;
						FSCRT_BStr_Clear(&fileName);
					}
				}
				break;
			case FSPDF_ACTION_RESETFORM:
				{
					FSPDF_ACTIONDATA_RESETFORM* resetData = (FSPDF_ACTIONDATA_RESETFORM*)(data.actionData);
					strOutPut += _T("ActionType: RESETFORM ");
				}
				break;
			case FSPDF_ACTION_IMPORTDATA:
				{
					FSPDF_ACTIONDATA_IMPORTDATA* importData = (FSPDF_ACTIONDATA_IMPORTDATA*)(data.actionData);
					strOutPut += _T("ActionType: IMPORTDATA ");
					break; 
				}
			case FSPDF_ACTION_RENDITION:
				strOutPut += _T("ActionType: RENDITION ");
				break;
			}
			if (!strOutPut.IsEmpty())
			{
				AfxMessageBox(strOutPut);
			}
		}
	}
	return FALSE;
}

BOOL CReader_LinkAnnotHandler::OnMouseMove(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	if (m_bLButtonDown)
	{
		m_EndPoint = point;
	}
	return FALSE;
}

BOOL CReader_LinkAnnotHandler::OnMouseWheel(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, short zDelta, const CPoint& point) 
{
	return FALSE;
}

BOOL CReader_LinkAnnotHandler::OnRButtonDown(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return FALSE;
}

BOOL CReader_LinkAnnotHandler::OnRButtonUp(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return FALSE;
}

BOOL CReader_LinkAnnotHandler::OnRButtonDblClk(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return FALSE;
}

void CReader_LinkAnnotHandler::Annot_OnDraw(IReader_Page* pPage, HDC hDC, IReader_Annot* pAnnot, FSCRT_MATRIX matrix, const CRect& rcPage)
{
	DWORD crBorder = RGB(255, 0, 0);
	DWORD crRect = RGB(0, 0, 0);
	CRect winRect = pAnnot->GetInnerRectMargin();
	if (pAnnot->IsSelected())
		DrawSelectRect(pPage, hDC, pAnnot, winRect, crBorder, crRect, TRUE);

	if (pAnnot->IsCaptured() && m_bLButtonDown)
	{
		DWORD crBorder = RGB(100, 162, 226);
		DWORD crRect = RGB(100, 25, 125);
		int x = m_EndPoint.x-m_StartPoint.x;
		int y = m_EndPoint.y -m_StartPoint.y;
		winRect.OffsetRect(x, y);
		DrawSelectRect(pPage, hDC, pAnnot, winRect, crBorder, crRect, TRUE);
	}
}

void CReader_LinkAnnotHandler::DrawSelectRect(IReader_Page* pPage, HDC hDC, IReader_Annot* pAnnot, const CRect winRect, DWORD crBorder, 
	DWORD crRect, BOOL bCorner)
{
	HPEN hPen = ::CreatePen(PS_SOLID, 1, crBorder);
	HPEN hOldPen = (HPEN)::SelectObject(hDC, hPen);
	LOGBRUSH lb;
	lb.lbStyle = BS_HOLLOW;
	lb.lbColor = crRect;
	lb.lbHatch = 0;
	HBRUSH hBrush = ::CreateBrushIndirect(&lb);
	HBRUSH hOldBrush = (HBRUSH)::SelectObject(hDC, hBrush);
	::Rectangle(hDC, winRect.left, winRect.top, winRect.right, winRect.bottom);
	::SelectObject(hDC, hOldBrush);
	::DeleteObject(hBrush);
	::SelectObject(hDC, hOldPen);
	::DeleteObject(hPen);

	CRect rcArray[8];
	if (bCorner)
	{
		GetSelectRects(winRect, rcArray);
		LOGBRUSH lb;
		lb.lbStyle = BS_SOLID;
		lb.lbColor = crBorder;
		lb.lbHatch = 0;
		HBRUSH hBrush = ::CreateBrushIndirect(&lb);

		for (int i = 0; i<8; i++)
		{
			::FillRect(hDC, &rcArray[i], hBrush);		
		}

		::DeleteObject(hBrush);
	}
}

void CReader_LinkAnnotHandler::GetSelectRects(CRect winRect, CRect* rcArray)
{
	CPoint ptCenter[8];
	ptCenter[0] = CPoint(winRect.left, winRect.top);
	ptCenter[1] = CPoint((winRect.left + winRect.right) / 2, winRect.top);
	ptCenter[2] = CPoint(winRect.right, winRect.top);
	ptCenter[3] = CPoint(winRect.right, (winRect.top + winRect.bottom) / 2);
	ptCenter[4] = CPoint(winRect.right, winRect.bottom);
	ptCenter[5] = CPoint((winRect.left + winRect.right) / 2, winRect.bottom);
	ptCenter[6] = CPoint(winRect.left, winRect.bottom);
	ptCenter[7] = CPoint(winRect.left, (winRect.top + winRect.bottom) / 2);

	for (int i = 0; i<8; i++)
	{
		rcArray[i] = CRect(ptCenter[i].x - 3, ptCenter[i].y - 3, ptCenter[i].x + 3, ptCenter[i].y + 3);
	}
}
