#include "stdafx.h"
#include "Reader_NoteAnnotHandler.h"

#define TEXTANNOTTYPE L"Text"
#define TEXTANNOTHANDLER L"TextAnnotHandler"


///////////////////////////////////////////////////////////////////////////////////////////
// CReader_NotesAnnot
///////////////////////////////////////////////////////////////////////////////////////////
CReader_NotesAnnot::CReader_NotesAnnot(FSCRT_ANNOT Annot, IReader_Page* pPgae)
	: m_NoteAnnot(Annot)
	, m_pPage(pPgae)
	, m_bSelect(FALSE)
	, m_bCapture(FALSE)
{

}

CReader_NotesAnnot::~CReader_NotesAnnot()
{

}

FSCRT_ANNOT CReader_NotesAnnot::GetPDFAnnot()
{
	return m_NoteAnnot;
}

CString	CReader_NotesAnnot::GetType() 
{
	return NULL;
}

CString CReader_NotesAnnot::GetSubType()
{
	return NULL;
}

BOOL CReader_NotesAnnot::IsSelected()
{
	return m_bSelect;
}

void CReader_NotesAnnot::SetSelected(BOOL bSelected)
{
	m_bSelect = bSelected;
}

BOOL CReader_NotesAnnot::IsCaptured()
{
	return m_bCapture;
}

void CReader_NotesAnnot::SetCaptured(BOOL bCaptured)
{
	m_bCapture = bCaptured;
}

CRect CReader_NotesAnnot::GetViewBBox()
{
	FSCRT_RECTF rect;
	CRect WinRect;
	FSPDF_Annot_GetRect(m_NoteAnnot, &rect);

	m_pPage->DocToWindow(rect, WinRect);
	WinRect.NormalizeRect();
	return WinRect;
}


CRect CReader_NotesAnnot::GetInnerRectMargin()
{
	FSCRT_RECTF rect;
	CRect WinRect;
	FS_FLOAT left = 0;
	FS_FLOAT top = 0;
	FS_FLOAT right = 0;
	FS_FLOAT bottom = 0;

	FSPDF_Annot_GetInnerRectMargin(m_NoteAnnot, &left, &top, &right, &bottom);
	FSPDF_Annot_GetRect(m_NoteAnnot, &rect);
	rect.top -= top;
	rect.right -= right;
	rect.bottom += bottom;
	rect.left += left;

	m_pPage->DocToWindow(rect, WinRect);
	WinRect.NormalizeRect();
	return WinRect;
}

BOOL CReader_NotesAnnot:: IsLock()
{
	FS_DWORD flags;
	FSPDF_Annot_GetFlags(m_NoteAnnot, &flags);
	if (FSPDF_ANNOTFLAG_LOCKED == flags)
	{
		return TRUE;
	}
	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////
// CReader_NotesAnnotHandler
///////////////////////////////////////////////////////////////////////////////////////////
CReader_NotesAnnotHandler::CReader_NotesAnnotHandler(void)
{
	m_bLButtonDown = FALSE;
}

CReader_NotesAnnotHandler::~CReader_NotesAnnotHandler(void)
{
}

IReader_Annot* CReader_NotesAnnotHandler::NewAnnot(FSCRT_ANNOT pdfAnnot, IReader_Page* pPage)
{
	return (IReader_Annot*) new CReader_NotesAnnot(pdfAnnot, pPage) ;
}

//return annotation type name, for example: "Line"
CString	CReader_NotesAnnotHandler::GetType() 
{
	return TEXTANNOTTYPE;
}

//return AnnotHandler unique identification, for example:"LineAnnotHandler"
CString CReader_NotesAnnotHandler::GetName() 
{
	return TEXTANNOTHANDLER;
}

//return position of annotation, it can be real position, also can be zoomed in properly, this position is also UpdateAllViews's reference position.
CRect CReader_NotesAnnotHandler::GetViewBBox(IReader_Page *pPage, IReader_Annot* pAnnot)
{
	return	pAnnot->GetViewBBox();
}

BOOL CReader_NotesAnnotHandler::HitTest(IReader_Page *pPage, IReader_Annot* pAnnot, const CPoint& point)
{
	return TRUE;
}

//return TRUE means have done, framework doesn't deal with .
BOOL CReader_NotesAnnotHandler::OnLButtonDown(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{

	m_bLButtonDown = TRUE;
	m_StartPoint = m_EndPoint = point;
	return TRUE;
}

BOOL CReader_NotesAnnotHandler::OnLButtonUp(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	m_bLButtonDown = FALSE;
	m_EndPoint = point;

	CRect winRect = pAnnot->GetInnerRectMargin();

	int x = m_EndPoint.x-m_StartPoint.x;
	int y = m_EndPoint.y -m_StartPoint.y;
	winRect.OffsetRect(x, y);
	FSCRT_RECTF rect;
	pPage->WindowToDoc(winRect, rect);

	if (rect.left > rect.right)
	{
		FS_FLOAT tem = rect.left;
		rect.left = rect.right;
		rect.right = tem;
	}

	if (rect.top < rect.bottom)
	{
		FS_FLOAT tem = rect.top;
		rect.top = rect.bottom;
		rect.bottom = tem;
	}

	FS_RESULT ret = FSPDF_Annot_Move(pAnnot->GetPDFAnnot(), &rect);
	if ( FSCRT_ERRCODE_SUCCESS == ret)
	{
		FSPDF_Annot_ResetAppearance(pAnnot->GetPDFAnnot());
		CWnd* pDocView = CWnd::FromHandle(pPage->GetDocument()->GetDocViewHwnd());

		if (pDocView)
		{
			pDocView->InvalidateRect(winRect);
		}
		IReader_Document* pReaderDocument = pPage->GetDocument();
		if (NULL != pReaderDocument)
		{
			pReaderDocument->SetChangeMark();
		}
	}

	return TRUE;
}
 
BOOL CReader_NotesAnnotHandler::OnLButtonDblClk(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	CRect winRect = 	pAnnot->GetViewBBox();
	return FALSE;
}
 
BOOL CReader_NotesAnnotHandler::OnMouseMove(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	if (m_bLButtonDown)
	{
		m_EndPoint = point;
	}
	return FALSE;
}

BOOL CReader_NotesAnnotHandler::OnMouseWheel(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, short zDelta, const CPoint& point)
{
	return FALSE;
}

BOOL CReader_NotesAnnotHandler::OnRButtonDown(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	return FALSE;
}

BOOL CReader_NotesAnnotHandler::OnRButtonUp(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	return FALSE;
}

BOOL CReader_NotesAnnotHandler::OnRButtonDblClk(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	return FALSE;
}

void CReader_NotesAnnotHandler::Annot_OnDraw(IReader_Page* pPage, HDC hDC, IReader_Annot* pAnnot, FSCRT_MATRIX matrix, const CRect& rcPage)
{
	DWORD crBorder = RGB(255, 0, 0);
	DWORD crRect = RGB(0, 0, 0);
	CRect winRect = pAnnot->GetInnerRectMargin();
	if (pAnnot->IsSelected())
		DrawSelectRect(pPage, hDC, pAnnot, winRect, crBorder, crRect, TRUE);

	if (pAnnot->IsCaptured() && m_bLButtonDown)
	{
		DWORD crBorder = RGB(100, 162, 226);
		DWORD crRect = RGB(100, 25, 125);;
		int x = m_EndPoint.x - m_StartPoint.x;
		int y = m_EndPoint.y - m_StartPoint.y;
		winRect.OffsetRect(x, y);
		DrawSelectRect(pPage, hDC, pAnnot, winRect, crBorder, crRect, TRUE);
	}
}

void CReader_NotesAnnotHandler::DrawSelectRect(IReader_Page* pPage, HDC hDC, IReader_Annot* pAnnot, const CRect& winRect, DWORD crBorder, 
	DWORD crRect,BOOL bCorner)
{
	HPEN hPen = ::CreatePen(PS_SOLID, 1, crBorder);
	HPEN hOldPen = (HPEN)::SelectObject(hDC, hPen);
	LOGBRUSH lb;
	lb.lbStyle = BS_HOLLOW;
	lb.lbColor = crRect;
	lb.lbHatch = 0;
	HBRUSH hBrush = ::CreateBrushIndirect(&lb);
	HBRUSH hOldBrush = (HBRUSH)::SelectObject(hDC, hBrush);
	::Rectangle(hDC, winRect.left, winRect.top, winRect.right, winRect.bottom);
	::SelectObject(hDC, hOldBrush);
	::DeleteObject(hBrush);
	::SelectObject(hDC, hOldPen);
	::DeleteObject(hPen);
}
