#include "stdafx.h"
#include "Reader_UnderLineAnnotHandler.h"

#define UNDERLINEANNOTTYPE L"Underline"
#define UNDERLINEANNOTHANDLER L"UnderlineAnnotHandler"


///////////////////////////////////////////////////////////////////////////////////////////
//						CReader_UnderLineAnnot
///////////////////////////////////////////////////////////////////////////////////////////
CReader_UnderLineAnnot::CReader_UnderLineAnnot(FSCRT_ANNOT Annot, IReader_Page* pPgae)
	: m_UnderLineAnnot(Annot)
	, m_pPage(pPgae)
	, m_bSelect(FALSE)
{
}

CReader_UnderLineAnnot::~CReader_UnderLineAnnot()
{

}

FSCRT_ANNOT CReader_UnderLineAnnot::GetPDFAnnot()
{
	return m_UnderLineAnnot;
}

CString	CReader_UnderLineAnnot::GetType() 
{
	return NULL;
}

CString CReader_UnderLineAnnot::GetSubType()
{
	return NULL;
}

BOOL CReader_UnderLineAnnot::IsSelected()
{
	return m_bSelect;
}

void CReader_UnderLineAnnot::SetSelected(BOOL bSelected)
{
	m_bSelect = bSelected;
}

BOOL CReader_UnderLineAnnot::IsCaptured()
{
	return FALSE;
}

void CReader_UnderLineAnnot::SetCaptured(BOOL bCaptured)
{
	
}

CRect CReader_UnderLineAnnot::GetViewBBox()
{
	FSCRT_RECTF rect;
	CRect WinRect;
	FSPDF_Annot_GetRect(m_UnderLineAnnot, &rect);

	m_pPage->DocToWindow(rect, WinRect);
	WinRect.NormalizeRect();
	return WinRect;
}

CRect CReader_UnderLineAnnot::GetInnerRectMargin()
{
	FSCRT_RECTF rect;
	CRect WinRect;
	FS_FLOAT left = 0;
	FS_FLOAT top = 0;
	FS_FLOAT right = 0;
	FS_FLOAT bottom = 0;

	FSPDF_Annot_GetInnerRectMargin(m_UnderLineAnnot, &left, &top, &right, &bottom);
	FSPDF_Annot_GetRect(m_UnderLineAnnot, &rect);
	rect.top -= top;
	rect.right -= right;
	rect.bottom += bottom;
	rect.left += left;

	m_pPage->DocToWindow(rect, WinRect);
	WinRect.NormalizeRect();
	return WinRect;
}

BOOL CReader_UnderLineAnnot:: IsLock()
{
	FS_DWORD flags;
	FSPDF_Annot_GetFlags(m_UnderLineAnnot, &flags);
	if (FSPDF_ANNOTFLAG_LOCKED == flags)
	{
		return TRUE;
	}
	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////
//						CReader_UnderLineAnnotHandler
///////////////////////////////////////////////////////////////////////////////////////////
CReader_UnderLineAnnotHandler::CReader_UnderLineAnnotHandler(void)
{
}

CReader_UnderLineAnnotHandler::~CReader_UnderLineAnnotHandler(void)
{
}

IReader_Annot* CReader_UnderLineAnnotHandler::NewAnnot(FSCRT_ANNOT pdfAnnot, IReader_Page* pPage)
{
	return (IReader_Annot*) new CReader_UnderLineAnnot(pdfAnnot, pPage) ;
}

//return annotation type name, for example: "Line"
CString	CReader_UnderLineAnnotHandler::GetType() 
{
	return UNDERLINEANNOTTYPE;
}

//return AnnotHandler unique identification, for example:"LineAnnotHandler"
CString CReader_UnderLineAnnotHandler::GetName() 
{
	return UNDERLINEANNOTHANDLER;
}

//return position of annotation, it can be real position, also can be zoomed in properly, this position is also UpdateAllViews's reference position.
CRect CReader_UnderLineAnnotHandler::GetViewBBox(IReader_Page *pPage, IReader_Annot* pAnnot)
{
	return	pAnnot->GetViewBBox();
}

BOOL CReader_UnderLineAnnotHandler::HitTest(IReader_Page *pPage, IReader_Annot* pAnnot, const CPoint& point)
{
	return TRUE;
}

//return TRUE means have done, framework doesn't deal with .
BOOL CReader_UnderLineAnnotHandler::OnLButtonDown(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return TRUE;
}

BOOL CReader_UnderLineAnnotHandler::OnLButtonUp(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	return FALSE;
}
 
BOOL CReader_UnderLineAnnotHandler::OnLButtonDblClk(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	CRect winRect = 	pAnnot->GetViewBBox();
	return FALSE;
}

BOOL CReader_UnderLineAnnotHandler::OnMouseMove(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point)
{
	return FALSE;
}

BOOL CReader_UnderLineAnnotHandler::OnMouseWheel(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, short zDelta, const CPoint& point) 
{
	return FALSE;
}

BOOL CReader_UnderLineAnnotHandler::OnRButtonDown(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return FALSE;
}

BOOL CReader_UnderLineAnnotHandler::OnRButtonUp(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return FALSE;
}

BOOL CReader_UnderLineAnnotHandler::OnRButtonDblClk(IReader_Page *pPage, IReader_Annot* pAnnot, UINT nFlags, const CPoint& point) 
{
	return FALSE;
}

void CReader_UnderLineAnnotHandler::Annot_OnDraw(IReader_Page* pPage, HDC hDC, IReader_Annot* pAnnot, FSCRT_MATRIX matrix, const CRect& rcPage)
{
	DWORD crBorder = RGB(255, 0, 0);
	DWORD crRect = RGB(0, 0, 0);
	CRect winRect = pAnnot->GetInnerRectMargin();
	if (pAnnot->IsSelected())
		DrawSelectRect(pPage, hDC, pAnnot, winRect, crBorder, crRect, TRUE);
}

void CReader_UnderLineAnnotHandler::DrawSelectRect(IReader_Page* pPage, HDC hDC, IReader_Annot* pAnnot, const CRect& winRect, DWORD crBorder, 
	DWORD crRect,BOOL bCorner)
{
	HPEN hPen = ::CreatePen(PS_SOLID, 1, crBorder);
	HPEN hOldPen = (HPEN)::SelectObject(hDC, hPen);
	LOGBRUSH lb;
	lb.lbStyle = BS_HOLLOW;
	lb.lbColor = crRect;
	lb.lbHatch = 0;
	HBRUSH hBrush = ::CreateBrushIndirect(&lb);
	HBRUSH hOldBrush = (HBRUSH)::SelectObject(hDC, hBrush);
	::Rectangle(hDC, winRect.left, winRect.top, winRect.right, winRect.bottom);
	::SelectObject(hDC, hOldBrush);
	::DeleteObject(hBrush);
	::SelectObject(hDC, hOldPen);
	::DeleteObject(hPen);
}
