// FX_ControlBar.cpp: implementation of the CFX_ControlBar class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FX_ControlBar.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

/////////////////////////////////////////////////////////////////////////
// CFX_DockBar
BOOL CFX_DockBar::PreTranslateMessage(MSG *pMsg)
{
	if (pMsg->message == WM_KEYDOWN && (pMsg->wParam == 13 || pMsg->wParam == 27 
		||pMsg->wParam == VK_UP
		||pMsg->wParam == VK_DOWN
		||pMsg->wParam == VK_LEFT
		||pMsg->wParam == VK_RIGHT))
		return FALSE;
	return CDockBar::PreTranslateMessage(pMsg);
}
////////////////////////////////////////////////////////////////////////////


#define NMAX_SIZE 32767


/////////////////////////////////////////////////////////////////////////
// CFX_ControlBar

IMPLEMENT_DYNAMIC(CFX_ControlBar, CControlBar);


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFX_ControlBar::CFX_ControlBar()
{
	m_szMinHorzBar = CSize(30, 32);
	m_szMinVertBar = CSize(20, 30);
	m_szMinFloatBar = CSize(37, 32);
	m_szHorzBar = CSize(200, 200);
	m_szVertBar = CSize(200, 200);
	m_szFloatBar = CSize(200, 200);
	m_bTracking = FALSE;
	m_bKeepSize = FALSE;
	m_bParentSizing = FALSE;
	m_ncxEdge = 5;
	m_nDockBarState = 0;
	m_dwBarStyle = 0;
}

CFX_ControlBar::CFX_ControlBar(CSize szFloatBar, CSize szMinFloatBar )
{
	m_szMinHorzBar = m_szMinVertBar = m_szMinFloatBar = m_szMinFloatBar;
	m_szHorzBar = m_szVertBar = m_szFloatBar = szFloatBar;
	m_bTracking = FALSE;
	m_bKeepSize = FALSE;
	m_bParentSizing = FALSE;
	m_ncxEdge = 5;
	m_nDockBarState = 0;
	m_dwBarStyle = 0;
}

CFX_ControlBar::~CFX_ControlBar()
{

}

BEGIN_MESSAGE_MAP(CFX_ControlBar, CControlBar)
 //{{AFX_MSG_MAP(CFX_ControlBar)
	ON_WM_CREATE()
	ON_WM_PAINT()
	ON_WM_NCPAINT()
	ON_WM_NCCALCSIZE()
	ON_WM_WINDOWPOSCHANGING()
	ON_WM_CAPTURECHANGED()
	ON_WM_SETTINGCHANGE()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_NCLBUTTONDOWN()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_RBUTTONDOWN()
	ON_WM_NCHITTEST()
	ON_WM_CLOSE()
	ON_WM_SIZE()
	ON_WM_DESTROY()
 //}}AFX_MSG_MAP
 ON_MESSAGE(WM_SETTEXT, OnSetText)
END_MESSAGE_MAP()


BOOL CFX_ControlBar::Create(LPCWSTR lpszWindowName, CWnd* pParentWnd, 
					CSize sizeDefault, BOOL bHasGripper, 
					UINT nID, DWORD dwStyle)
{
	m_szHorzBar = m_szVertBar = m_szFloatBar = sizeDefault;
	return Create(lpszWindowName, pParentWnd, nID, dwStyle);
}

BOOL CFX_ControlBar::Create(LPCWSTR lpszWindowName, CWnd* pParentWnd, UINT nID, DWORD dwStyle )
{
	ASSERT_VALID(pParentWnd);
	m_dwStyle = dwStyle & CBRS_ALL;
	
	CString wndclass = ::AfxRegisterWndClass(CS_DBLCLKS, ::LoadCursor(NULL, IDC_ARROW), 
			::GetSysColorBrush(COLOR_BTNFACE), 0);
	dwStyle &= ~CBRS_ALL;
	dwStyle |= WS_CLIPCHILDREN|CBRS_GRIPPER;

	if(!CWnd::Create(wndclass, lpszWindowName, dwStyle, CRect(0, 0, 0, 0), pParentWnd, nID))
		return FALSE;
	return TRUE;
}

void CFX_ControlBar::OnDestroy() 
{
}

const BOOL	CFX_ControlBar::IsFloating() const
{
	return !IsHorzDocked() && !IsVertDocked();
}

const BOOL	CFX_ControlBar::IsHorzDocked() const
{
	return (m_nDockBarState == AFX_IDW_DOCKBAR_TOP || m_nDockBarState == AFX_IDW_DOCKBAR_BOTTOM);
}

const BOOL	CFX_ControlBar::IsVertDocked() const
{
	return (m_nDockBarState == AFX_IDW_DOCKBAR_LEFT || m_nDockBarState == AFX_IDW_DOCKBAR_RIGHT);
}

const BOOL	CFX_ControlBar::IsSideTracking() const
{
	ASSERT(m_bTracking && !IsFloating());
	return (m_htEdge == HTLEFT || m_htEdge == HTRIGHT) ? IsHorzDocked() : IsVertDocked();
}


CSize CFX_ControlBar::CalcFixedLayout(BOOL bStretch, BOOL bHorz)
{
	if (bStretch) // the bar is stretched 
		if (bHorz)
			return CSize(NMAX_SIZE, m_szHorzBar.cy);
		else
			return CSize(m_szVertBar.cx, NMAX_SIZE);

	CPtrCtrlBarList pCtrlBarList;
	GetCtrlBarList(pCtrlBarList);
	// dirty cast - we need access to protected CDockBar members
	CFX_DockBar* pDockBar = (CFX_DockBar*) m_pDockBar;
	AFX_SIZEPARENTPARAMS layout;

	if(pDockBar->m_bLayoutQuery)
	{
		layout.hDWP = NULL;
	}
	else
	{
		layout.hDWP = ::BeginDeferWindowPos(pCtrlBarList.GetSize());
	}

	for (int i = 0; i < pCtrlBarList.GetSize(); i++)
		if (pCtrlBarList[i]->m_nStateFlags & (delayHide|delayShow))
			pCtrlBarList[i]->RecalcDelayShow(&layout);

	if (layout.hDWP != NULL)
		::EndDeferWindowPos(layout.hDWP);

 // get available length
	CRect rc = pDockBar->m_rectLayout;
	if (rc.IsRectEmpty())	
		m_pDockSite->GetClientRect(&rc);
	int nLengthTotal = bHorz ? rc.Width() + 2 : rc.Height() - 2;

	if (IsVisible() && !IsFloating() && m_bParentSizing && pCtrlBarList[0] == this)
	{	
		NegotiateSpace(nLengthTotal, (bHorz != FALSE));
	}

	m_bParentSizing = FALSE;

	if (bHorz)
		return CSize(max(m_szMinHorzBar.cx, m_szHorzBar.cx), m_szHorzBar.cy);
	return CSize(m_szVertBar.cx, max(m_szMinVertBar.cy, m_szVertBar.cy));
}
CSize CFX_ControlBar::CalcDynamicLayout(int nLength, DWORD dwMode)
{
	if (dwMode & (LM_HORZDOCK | LM_VERTDOCK))
	{
		if (nLength == -1)
			m_bParentSizing = TRUE;
		return CControlBar::CalcDynamicLayout(nLength, dwMode);
	}
	if	(dwMode & LM_MRUWIDTH) return m_szFloatBar;
	if	(dwMode & LM_COMMIT) return m_szFloatBar;

	int nHitTest = m_pDockContext->m_nHitTest;
	if (IsFloating() && 
		(nHitTest == HTTOPLEFT || nHitTest == HTBOTTOMLEFT ||
		nHitTest == HTTOPRIGHT || nHitTest == HTBOTTOMRIGHT))
	{
		CPoint ptCursor;
		::GetCursorPos(&ptCursor);

		CRect rFrame, rBar;
		GetParentFrame()->GetWindowRect(&rFrame);
		GetWindowRect(&rBar);
  
		if (nHitTest == HTTOPLEFT || nHitTest == HTBOTTOMLEFT)
		{
			m_szFloatBar.cx = rFrame.left + rBar.Width() - ptCursor.x;
			m_pDockContext->m_rectFrameDragHorz.left = 
				min(ptCursor.x, rFrame.left + rBar.Width() - m_szMinFloatBar.cx);
		}
		
		if (nHitTest == HTTOPLEFT || nHitTest == HTTOPRIGHT)
		{
			m_szFloatBar.cy = rFrame.top + rBar.Height() - ptCursor.y;
			m_pDockContext->m_rectFrameDragHorz.top = 
				min(ptCursor.y, rFrame.top + rBar.Height() - m_szMinFloatBar.cy);
		}
		
		if (nHitTest == HTTOPRIGHT || nHitTest == HTBOTTOMRIGHT)
			m_szFloatBar.cx = rBar.Width() + ptCursor.x - rFrame.right;

		if (nHitTest == HTBOTTOMLEFT || nHitTest == HTBOTTOMRIGHT)
			m_szFloatBar.cy = rBar.Height() + ptCursor.y - rFrame.bottom;
	}
	else
	{
		((dwMode & LM_LENGTHY) ? m_szFloatBar.cy : m_szFloatBar.cx) = nLength;
	}
	m_szFloatBar.cx = max(m_szFloatBar.cx, m_szMinFloatBar.cx);
	m_szFloatBar.cy = max(m_szFloatBar.cy, m_szMinFloatBar.cy);
	return m_szFloatBar;
}


void CFX_ControlBar::OnUpdateCmdUI(CFrameWnd* pTarget, BOOL bDisableIfNoHndler)
{
	UNUSED_ALWAYS(bDisableIfNoHndler);
	UNUSED_ALWAYS(pTarget);
}

UINT CFX_ControlBar::GetEdgeHTCode(int nEdge)
{
	switch(nEdge)
	{
	case 0: 
		return HTLEFT;
		break;
	case 1:
		return HTTOP;
		break;
	case 2:
		return HTRIGHT;
		break;
	case 3:
		return HTBOTTOM;
		break;
	default:
		return HTNOWHERE;
	}	
}

BOOL CFX_ControlBar::GetEdgeRect(CRect rcWnd, UINT nHitTest, CRect& rcEdge)
{
	rcEdge = rcWnd;
	if (m_dwBarStyle & FX_CNTRLBARS_SHOWEDGES)
	rcEdge.DeflateRect(1, 1);
	BOOL bHorz = IsHorzDocked();

	switch (nHitTest)
	{
	case HTLEFT:
		if (!(m_dwBarStyle & FX_CNTRLBARS_EDGELEFT)) return FALSE;
		rcEdge.right = rcEdge.left + m_ncxEdge;
		rcEdge.DeflateRect(0, 0);
		break;
	case HTTOP:
		if (!(m_dwBarStyle & FX_CNTRLBARS_EDGETOP)) return FALSE;
		rcEdge.bottom = rcEdge.top + m_ncxEdge;
		rcEdge.DeflateRect(bHorz ? 0 : m_ncxEdge, 0);
		break;
	case HTRIGHT:
		if (!(m_dwBarStyle & FX_CNTRLBARS_EDGERIGHT)) return FALSE;
		rcEdge.left = rcEdge.right - m_ncxEdge;
		rcEdge.DeflateRect(0, 0);
		break;
	case HTBOTTOM:
		if (!(m_dwBarStyle & FX_CNTRLBARS_EDGEBOTTOM)) return FALSE;
		rcEdge.top = rcEdge.bottom - m_ncxEdge;
		rcEdge.DeflateRect(bHorz ? 0 : m_ncxEdge, 0);
		break;
	default:
		ASSERT(FALSE); // invalid hit test code
	}
	return TRUE;
}

void CFX_ControlBar::StartTracking(UINT nHitTest, CPoint point)
{
	SetCapture();

	// make sure no updates are pending
	RedrawWindow(NULL, NULL, RDW_ALLCHILDREN | RDW_UPDATENOW);

	m_htEdge = nHitTest;
	m_bTracking = TRUE;

	BOOL bHorz = IsHorzDocked();
	BOOL bHorzTracking = m_htEdge == HTLEFT || m_htEdge == HTRIGHT;

	m_nTrackPosOld = bHorzTracking ? point.x : point.y;

	CRect rcBar, rcEdge;
	GetWindowRect(rcBar);
	GetEdgeRect(rcBar, m_htEdge, rcEdge);
	m_nTrackEdgeOfs = m_nTrackPosOld -
	(bHorzTracking ? rcEdge.CenterPoint().x : rcEdge.CenterPoint().y);
 
	CPtrCtrlBarList pCtrlBarList;
	int nThisPos;
	GetCtrlBarList(pCtrlBarList, nThisPos);

	m_nTrackPosMin = m_nTrackPosMax = m_nTrackPosOld;
	if (!IsSideTracking())
	{
		// calc minwidth as the max minwidth of the sizing bars on row
		int nMinWidth = bHorz ? m_szMinHorzBar.cy : m_szMinVertBar.cx;
		for (int i = 0; i < pCtrlBarList.GetSize(); i++)
		nMinWidth = max(nMinWidth, bHorz ? 
		pCtrlBarList[i]->m_szMinHorzBar.cy :
		pCtrlBarList[i]->m_szMinVertBar.cx);
		int nExcessWidth = (bHorz ? m_szHorzBar.cy : m_szVertBar.cx) - nMinWidth;

		// the control bar cannot grow with more than the width of
		// remaining client area of the mainframe
		CRect rcT;
		::SendMessage(this->m_hWnd, WM_SETREDRAW, FALSE, 0L);
		m_pDockSite->RepositionBars(0, 0xFFFF, AFX_IDW_PANE_FIRST, 
		reposQuery, &rcT, NULL, TRUE);		
		::SendMessage(this->m_hWnd, WM_SETREDRAW, TRUE, 0L);
		
		int nMaxWidth = bHorz ? rcT.Height() - 2 : rcT.Width() - 2;

		BOOL bTopOrLeft = m_htEdge == HTTOP || m_htEdge == HTLEFT;

		m_nTrackPosMin -= bTopOrLeft ? nMaxWidth : nExcessWidth;
		m_nTrackPosMax += bTopOrLeft ? nExcessWidth : nMaxWidth;
	}
	else
	{
		// side tracking:
		// max size is the actual size plus the amount the other
		// sizing bars can be decreased until they reach their minsize
		if (m_htEdge == HTBOTTOM || m_htEdge == HTRIGHT)
			nThisPos++;

		for (int i = 0; i < pCtrlBarList.GetSize(); i++)
		{
			CFX_ControlBar* pBar = pCtrlBarList[i];

			int nExcessWidth = bHorz ? 
				pBar->m_szHorzBar.cx - pBar->m_szMinHorzBar.cx :
			pBar->m_szVertBar.cy - pBar->m_szMinVertBar.cy;

			if (i < nThisPos)
				m_nTrackPosMin -= nExcessWidth;
			else
				m_nTrackPosMax += nExcessWidth;
		}
	}	
	OnTrackInvertTracker(); // draw tracker
}
void CFX_ControlBar::StopTracking()
{
	OnTrackInvertTracker();
	m_bTracking = FALSE;
	ReleaseCapture();
	m_pDockSite->DelayRecalcLayout();
}
void CFX_ControlBar::OnTrackUpdateSize(CPoint& point)
{
	ASSERT(!IsFloating());
	BOOL bHorzTrack = m_htEdge == HTLEFT || m_htEdge == HTRIGHT;
	int nTrackPos = bHorzTrack ? point.x : point.y;
	nTrackPos = max(m_nTrackPosMin, min(m_nTrackPosMax, nTrackPos));
	int nDelta = nTrackPos - m_nTrackPosOld;
	if (nDelta == 0)
		return; // no pos change
	
	OnTrackInvertTracker(); // erase tracker
	m_nTrackPosOld = nTrackPos;
	BOOL bHorz = IsHorzDocked();
	CSize sizeNew = bHorz ? m_szHorzBar : m_szVertBar;
	switch (m_htEdge)
	{
	case HTLEFT: 
		sizeNew -= CSize(nDelta, 0); 
		break;
	case HTTOP: 
		sizeNew -= CSize(0, nDelta); 
		break;
	case HTRIGHT: 
		sizeNew += CSize(nDelta, 0); 
		break;
	case HTBOTTOM: 
		sizeNew += CSize(0, nDelta);
		break;
	}
	
	CPtrCtrlBarList pCtrlBarList;
	int nThisPos;
	GetCtrlBarList(pCtrlBarList, nThisPos);
	
	if (!IsSideTracking())
	{
		for (int i = 0; i < pCtrlBarList.GetSize(); i++)
		{
			CFX_ControlBar* pBar = pCtrlBarList[i];
			// make same width (or height)
			(bHorz ? pBar->m_szHorzBar.cy : pBar->m_szVertBar.cx) = 
			bHorz ? sizeNew.cy : sizeNew.cx;
		}
	}
	else
	{
		int nGrowingBar = nThisPos;
		BOOL bBefore = m_htEdge == HTTOP || m_htEdge == HTLEFT;
		if (bBefore && nDelta > 0)
			nGrowingBar--;
		if (!bBefore && nDelta < 0)
			nGrowingBar++;
		if (nGrowingBar != nThisPos)
			bBefore = !bBefore;
		
		// nGrowing is growing
		nDelta = abs(nDelta);
		CFX_ControlBar* pBar = pCtrlBarList[nGrowingBar];
		(bHorz ? pBar->m_szHorzBar.cx : pBar->m_szVertBar.cy) += nDelta;
		
		// the others are shrinking
		int nFirst = bBefore ? nGrowingBar - 1 : nGrowingBar + 1;
		int nLimit = bBefore ? -1 : pCtrlBarList.GetSize();
		
		for (int i = nFirst; nDelta != 0 && i != nLimit; i += (bBefore ? -1 : 1))
		{
			CFX_ControlBar* pBar = pCtrlBarList[i];

			int nDeltaT = min(nDelta, 
				(bHorz ? pBar->m_szHorzBar.cx : pBar->m_szVertBar.cy) -
				(bHorz ? pBar->m_szMinHorzBar.cx : pBar->m_szMinVertBar.cy));
			(bHorz ? pBar->m_szHorzBar.cx : pBar->m_szVertBar.cy) -= nDeltaT;
			nDelta -= nDeltaT;
		}
	}		
	OnTrackInvertTracker(); // redraw tracker at new pos
}

void CFX_ControlBar::OnTrackInvertTracker()
{
	if(m_bTracking == FALSE) return;
	BOOL bHorz = IsHorzDocked();
	CRect rc, rcBar, rcDock, rcFrame;
	GetWindowRect(rcBar);
	m_pDockBar->GetWindowRect(rcDock);
	m_pDockSite->GetWindowRect(rcFrame);
	VERIFY(GetEdgeRect(rcBar, m_htEdge, rc));
	if (!IsSideTracking())
		rc = bHorz ? 
			CRect(rcDock.left + 1, rc.top, rcDock.right - 1, rc.bottom) :
			CRect(rc.left, rcDock.top + 1, rc.right, rcDock.bottom - 1);

	BOOL bHorzTracking = m_htEdge == HTLEFT || m_htEdge == HTRIGHT;
	int nOfs = m_nTrackPosOld - m_nTrackEdgeOfs;
	nOfs -= bHorzTracking ? rc.CenterPoint().x : rc.CenterPoint().y;
	rc.OffsetRect(bHorzTracking ? nOfs : 0, bHorzTracking ? 0 : nOfs);
	rc.OffsetRect(-rcFrame.TopLeft());

	CDC *pDC = m_pDockSite->GetDCEx(NULL, 
	DCX_WINDOW | DCX_CACHE | DCX_LOCKWINDOWUPDATE);
	CBrush* pBrush = CDC::GetHalftoneBrush();
	CBrush* pBrushOld = pDC->SelectObject(pBrush);
	pDC->PatBlt(rc.left, rc.top, rc.Width(), rc.Height(), PATINVERT);
	pDC->SelectObject(pBrush);
	m_pDockSite->ReleaseDC(pDC);
}

void CFX_ControlBar::NcCalcClient(LPRECT pRc, UINT nDockBarID)
{
	CRect rc(pRc);

	switch(nDockBarID)	
	{
	case AFX_IDW_DOCKBAR_TOP:
		m_dwBarStyle |= FX_CNTRLBARS_EDGEBOTTOM;
		break;
	case AFX_IDW_DOCKBAR_BOTTOM:
		m_dwBarStyle |= FX_CNTRLBARS_EDGETOP;
		break;
	case AFX_IDW_DOCKBAR_LEFT:
		m_dwBarStyle |= FX_CNTRLBARS_EDGERIGHT;
		break;
	case AFX_IDW_DOCKBAR_RIGHT:
		m_dwBarStyle |= FX_CNTRLBARS_EDGELEFT;
		break;	
	}	
	*pRc = rc;
}

BOOL CFX_ControlBar::GetCtrlBarListInfo(int& nFirst, int& nLast, int& nThis)
{
	ASSERT_VALID(m_pDockBar);

	nThis = m_pDockBar->FindBar(this);
	ASSERT(nThis != -1);

	int nBars = m_pDockBar->m_arrBars.GetSize();
	int i;
	for (nFirst = -1, i = nThis - 1; i >= 0 && nFirst == -1; i--)
		if (m_pDockBar->m_arrBars[i] == NULL)
			nFirst = i + 1;
	for (nLast = -1, i = nThis + 1; i < nBars && nLast == -1; i++)
		if (m_pDockBar->m_arrBars[i] == NULL)
			nLast = i - 1;
	
	if((nLast != -1) && (nFirst != -1)) 
		return FALSE;
	return TRUE;
}

void CFX_ControlBar::GetCtrlBarList(CPtrCtrlBarList& pCtrlBarList)
{
	int nThis; // dummy
 GetCtrlBarList(pCtrlBarList, nThis);
}

void CFX_ControlBar::GetCtrlBarList(CPtrCtrlBarList& pCtrlBarList, int& nThisPos)
{
	pCtrlBarList.RemoveAll();	
	int nFirst, nLast, nThis;
	GetCtrlBarListInfo(nFirst, nLast, nThis);
	//int nThisPos = -1;
	for (int i = nFirst; i <= nLast; i++)
	{
		CFX_ControlBar* pBar = (CFX_ControlBar*) m_pDockBar->m_arrBars[i];
		if (HIWORD(pBar) == 0) continue; // placeholder
		if (!pBar->IsVisible()) continue;
		if (pBar->IsKindOf(RUNTIME_CLASS(CFX_ControlBar)))
		{
			if (pBar == this)
				nThisPos = pCtrlBarList.GetSize();
			pCtrlBarList.Add(pBar);
		}
	}
}
BOOL CFX_ControlBar::NegotiateSpace(int nLengthTotal, BOOL bHorz)
{
	if(bHorz != IsHorzDocked()) return FALSE;

	int nFirst, nLast, nThis;
	GetCtrlBarListInfo(nFirst, nLast, nThis);

	int nLengthAvail = nLengthTotal;
	int nLengthActual = 0;
	int nLengthMin = 2;
	int nWidthMax = 0;
	CFX_ControlBar* pBar;

	for (int i = nFirst; i <= nLast; i++)
	{
		pBar = (CFX_ControlBar*) m_pDockBar->m_arrBars[i];
		if (HIWORD(pBar) == 0) continue; // placeholder
		if (!pBar->IsVisible()) continue;
		BOOL bIsCtrlBar = pBar->IsKindOf(RUNTIME_CLASS(CFX_ControlBar));

		int nLengthBar; // minimum length of the bar
		if (bIsCtrlBar)
			nLengthBar = bHorz ? pBar->m_szMinHorzBar.cx - 2 : pBar->m_szMinVertBar.cy - 2;
		else
		{
			CRect rcBar;
			pBar->GetWindowRect(&rcBar);
			nLengthBar = bHorz ? rcBar.Width() - 2 : rcBar.Height() - 2;
		}

		nLengthMin += nLengthBar;
		if (nLengthMin > nLengthTotal)
		{
			// split the row after fixed bar
			if (i < nThis)
			{
				m_pDockBar->m_arrBars.InsertAt(i + 1, (CControlBar*) NULL);
				return FALSE;
			}
   
			// only this sizebar remains on the row, adjust it to minsize
			if (i == nThis)
			{
				if (bHorz)
					m_szHorzBar.cx = m_szMinHorzBar.cx;
				else
					m_szVertBar.cy = m_szMinVertBar.cy;
				return TRUE; // the dockbar will split the row for us
			}

		   // we have enough bars - go negotiate with them
			m_pDockBar->m_arrBars.InsertAt(i, (CControlBar*) NULL);
			nLast = i - 1;
			break;
		}

		if (bIsCtrlBar)
		{
			nLengthActual += bHorz ? pBar->m_szHorzBar.cx - 2 : pBar->m_szVertBar.cy - 2;
			nWidthMax = max(nWidthMax, bHorz ? pBar->m_szHorzBar.cy :
				pBar->m_szVertBar.cx);
		}
		else
			nLengthAvail -= nLengthBar;
	}

	CPtrCtrlBarList pCtrlBarList;
	GetCtrlBarList(pCtrlBarList);
	int nNumBars = pCtrlBarList.GetSize();
	int nDelta = nLengthAvail - nLengthActual;

 // return faster when there is only one sizing bar per row (this one)
	if (nNumBars == 1)
	{
		ASSERT(pCtrlBarList[0] == this);

		if (nDelta == 0)
			return TRUE;

		m_bKeepSize = FALSE;
		(bHorz ? m_szHorzBar.cx : m_szVertBar.cy) += nDelta;

		return TRUE;
	}

	// make all the bars the same width
	for (int i = 0; i < nNumBars; i++)
		if (bHorz)
			pCtrlBarList[i]->m_szHorzBar.cy = nWidthMax;
		else
			pCtrlBarList[i]->m_szVertBar.cx = nWidthMax;

	// distribute the difference between the bars, 
	// but don't shrink them below their minsizes
	while (nDelta != 0)
	{
		int nDeltaOld = nDelta;
		for (int i = 0; i < nNumBars; i++)
		{
			pBar = pCtrlBarList[i];
			int nLMin = bHorz ?
			pBar->m_szMinHorzBar.cx : pBar->m_szMinVertBar.cy;
			int nL = bHorz ? pBar->m_szHorzBar.cx : pBar->m_szVertBar.cy;

			if ((nL == nLMin) && (nDelta < 0) || // already at min length
			pBar->m_bKeepSize) // or wants to keep its size
			continue;

			// sign of nDelta
			int nDelta2 = (nDelta == 0) ? 0 : ((nDelta < 0) ? -1 : 1);

			(bHorz ? pBar->m_szHorzBar.cx : pBar->m_szVertBar.cy) += nDelta2;
			nDelta -= nDelta2;
			if (nDelta == 0) break;
		}
		// clear m_bKeepSize flags
		if ((nDeltaOld == nDelta) || (nDelta == 0))
		for (int i = 0; i < nNumBars; i++)
		pCtrlBarList[i]->m_bKeepSize = FALSE;
	}

	return TRUE;
}

int	 CFX_ControlBar::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CControlBar :: OnCreate(lpCreateStruct) == -1)
		return -1;
	return 0;
}
void CFX_ControlBar::OnNcPaint()
{
	CWindowDC dc(this);
	CRect rcClient, rcBar, rcDockSite;
	GetClientRect(&rcClient);
	ClientToScreen(rcClient);
	GetWindowRect(rcBar);
	rcClient.OffsetRect(-rcBar.TopLeft());
	rcBar.OffsetRect(-rcBar.TopLeft());
	
	CDC mdc;
	mdc.CreateCompatibleDC(&dc);
 
	CBitmap bm;
	bm.CreateCompatibleBitmap(&dc, rcBar.Width(), rcBar.Height());
	CBitmap* pOldBm = mdc.SelectObject(&bm);
	

	CRect rcDraw = rcBar;
	
 // erase the NC background
#ifdef _WIN64
	mdc.FillRect(rcDraw, CBrush::FromHandle((HBRUSH) GetClassLong(m_hWnd, GCLP_HBRBACKGROUND)));
#else
	mdc.FillRect(rcDraw, CBrush::FromHandle((HBRUSH) GetClassLong(m_hWnd, GCL_HBRBACKGROUND)));
#endif
	
 if (m_dwBarStyle & FX_CNTRLBARS_SHOWEDGES)
 {
	CRect rcEdge; // paint the sizing edges
	for (int i = 0; i < 4; i++)
		if (GetEdgeRect(rcBar, GetEdgeHTCode(i), rcEdge))
			mdc.Draw3dRect(rcEdge, ::GetSysColor(COLOR_BTNHIGHLIGHT), 
				::GetSysColor(COLOR_BTNSHADOW));
 }
	
	NcPaintGripper(&mdc, rcClient);
	// client area is not our bussiness :)
	dc.IntersectClipRect(rcBar);
	if(rcClient.bottom>rcClient.top)
		dc.ExcludeClipRect(rcClient);

	
	dc.BitBlt(0, 0, rcBar.Width(), rcBar.Height(), &mdc, 0, 0, SRCCOPY);

	ReleaseDC(&dc);

	mdc.SelectObject(pOldBm);
	bm.DeleteObject();
	mdc.DeleteDC();
}

void CFX_ControlBar::NcPaintGripper(CDC* pDC, CRect rcClient)
{
	UNUSED_ALWAYS(pDC);
	UNUSED_ALWAYS(rcClient);
}

void CFX_ControlBar::OnNcCalcSize(BOOL bCalcValidRects, NCCALCSIZE_PARAMS FAR* lpncsp)
{
	UNUSED_ALWAYS(bCalcValidRects);
	if (IsFloating())
	{
		CFrameWnd* pFrame = GetParentFrame();
		if (pFrame != NULL && 
			pFrame->IsKindOf(RUNTIME_CLASS(CMiniFrameWnd)))
		{
			DWORD dwStyle = ::GetWindowLong(pFrame->m_hWnd, GWL_STYLE);
			if ((dwStyle & MFS_4THICKFRAME) != 0)
			{
				pFrame->ModifyStyle(MFS_4THICKFRAME, 0); // clear
				GetParent()->ModifyStyle(0, WS_CLIPCHILDREN);
			}
		}
	}
	m_dwBarStyle &= ~FX_CNTRLBARS_EDGEALL;
 // add resizing edges between bars on the same row
	if (!IsFloating() && m_pDockBar != NULL)
	{
		CPtrCtrlBarList pCtrlBarList;
		int nThisPos;
		GetCtrlBarList(pCtrlBarList, nThisPos);
		BOOL bHorz = IsHorzDocked();
		if (nThisPos > 0)
			m_dwBarStyle |= bHorz ? FX_CNTRLBARS_EDGELEFT : FX_CNTRLBARS_EDGETOP;

		if (nThisPos < pCtrlBarList.GetUpperBound())
			m_dwBarStyle |= bHorz ? FX_CNTRLBARS_EDGERIGHT : FX_CNTRLBARS_EDGEBOTTOM;
	}
	NcCalcClient(&lpncsp->rgrc[0], m_nDockBarState);
}

LRESULT CFX_ControlBar::OnNcHitTest(CPoint point)
{
	CRect rcBar, rcEdge;
	GetWindowRect(rcBar);	
	if (!IsFloating())
		for (int i = 0; i < 4; i++)
			if (GetEdgeRect(rcBar, GetEdgeHTCode(i), rcEdge))
				if (rcEdge.PtInRect(point))
					return GetEdgeHTCode(i);			
	return HTCLIENT;
}

void CFX_ControlBar::OnCaptureChanged(CWnd *pWnd)
{
	if (m_bTracking && (pWnd != this))
		StopTracking();
	CControlBar::OnCaptureChanged(pWnd);
}

void CFX_ControlBar::OnSettingChange(UINT uFlags, LPCTSTR lpszSection)
{
	CControlBar::OnSettingChange(uFlags, lpszSection);
}

void CFX_ControlBar::OnLButtonUp(UINT nFlags, CPoint point)
{
	if (m_bTracking)
		StopTracking();
	CControlBar::OnLButtonUp(nFlags, point);
}

void CFX_ControlBar::OnMouseMove(UINT nFlags, CPoint point)
{
	if (m_bTracking)
	{
		CPoint ptScreen = point;
		ClientToScreen(&ptScreen);	
		OnTrackUpdateSize(ptScreen);
	}	
	CControlBar::OnMouseMove(nFlags, point);
}

void CFX_ControlBar::OnNcLButtonDown(UINT nHitTest, CPoint point)
{
	UNUSED_ALWAYS(point);
	if (m_bTracking || IsFloating())
		return;

	if ((nHitTest >= HTSIZEFIRST) && (nHitTest <= HTSIZELAST))
		StartTracking(nHitTest, point);
}

void CFX_ControlBar::OnLButtonDown(UINT nFlags, CPoint point)
{
	m_pDockBar->m_pDockSite->SetActiveWindow();
	if (m_pDockBar != NULL)
	{
		// start the drag
		ASSERT(m_pDockContext != NULL);
		ClientToScreen(&point);
		m_pDockContext->StartDrag(point);
	}
	else
		CWnd::OnLButtonDown(nFlags, point);
}

void CFX_ControlBar::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	if (m_pDockBar != NULL)
	{
		// toggle docking
		ASSERT(m_pDockContext != NULL);
		m_pDockContext->ToggleDocking();
	}
	else
		CWnd::OnLButtonDblClk(nFlags, point);
}

void CFX_ControlBar::OnRButtonDown(UINT nFlags, CPoint point)
{
	if (m_bTracking)
		StopTracking();

	CControlBar::OnRButtonDown(nFlags, point);
}

void CFX_ControlBar::OnWindowPosChanging(WINDOWPOS FAR* lpwndpos)
{
	lpwndpos->flags |= SWP_FRAMECHANGED;
	CControlBar::OnWindowPosChanging(lpwndpos);
	// find on which side are we docked
	m_nDockBarState = GetParent()->GetDlgCtrlID();
	if (!IsFloating())
	if (lpwndpos->flags & SWP_SHOWWINDOW)
		m_bKeepSize = TRUE;
}
void CFX_ControlBar::OnPaint()
{
	CPaintDC dc(this);
}
void CFX_ControlBar::OnClose()
{
	return;
}
void CFX_ControlBar::OnSize(UINT nType, int cx, int cy)
{
	UNUSED_ALWAYS(nType);
	if ((m_dwBarStyle & FX_CNTRLBARS_SIZECHILD) != 0)
	{
		// automatic child resizing - only one child is allowed
		CWnd* pWnd = GetWindow(GW_CHILD);
		if (pWnd != NULL)
		{
			pWnd->MoveWindow(0, 0, cx, cy);
		}
	}
}
LRESULT CFX_ControlBar::OnSetText(WPARAM wParam, LPARAM lParam)
{
	UNUSED_ALWAYS(wParam);	
	LRESULT lResult = CWnd::Default();	
	if (IsFloating() && 
		GetParentFrame()->IsKindOf(RUNTIME_CLASS(CMiniDockFrameWnd)))
	{
		m_pDockBar->SetWindowText((LPCWSTR) lParam); 
		GetParentFrame()->DelayRecalcLayout(); 
	}	
	return lResult;
}
