// FX_ControlBar.h: interface for the CFX_ControlBar class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FX_CONTROLBAR_H__INCLUDED_)
#define AFX_FX_CONTROLBAR_H__INCLUDED_

#include <afxpriv.h>    // for CDockContext
#include <afxtempl.h>   // for CTypedPtrArray

#if _MSC_VER > 1000
#pragma once
#endif 

//////////////////////////////////////////////////////////////////////
//CFX_DockBar dummy class for access CFX_ControlBar's protected members.

class CFX_DockBar : public CDockBar
{
	
public:	
	
	// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CChildFrame)
public:
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	//}}AFX_VIRTUAL
	
	// Implementation
	friend class CFX_ControlBar;
	friend class CFX_DockContainer;
};

/////////////////////////////////////////////////////////////////////////
// CFX_ControlBar control bar styles

#define FX_CNTRLBARS_EDGELEFT       0x00000001
#define FX_CNTRLBARS_EDGERIGHT      0x00000002
#define FX_CNTRLBARS_EDGETOP        0x00000004
#define FX_CNTRLBARS_EDGEBOTTOM     0x00000008
#define FX_CNTRLBARS_EDGEALL        0x0000000F
#define FX_CNTRLBARS_SHOWEDGES      0x00000010
#define FX_CNTRLBARS_SIZECHILD      0x00000020


/////////////////////////////////////////////////////////////////////////
// CFX_ControlBar control bar

class CFX_ControlBar;
typedef CTypedPtrArray <CPtrArray, CFX_ControlBar*> CPtrCtrlBarList;


class CFX_ControlBar : public CControlBar  
{
	DECLARE_DYNAMIC(CFX_ControlBar);
// Construction
public:
	CFX_ControlBar();
	CFX_ControlBar(CSize szFloatBar,CSize szMinFloatBar = (50,50));
	

	virtual BOOL Create(LPCWSTR lpszWindowName, CWnd* pParentWnd,
        CSize sizeDefault, BOOL bHasGripper,
        UINT nID, DWORD dwStyle = WS_CHILD | WS_VISIBLE | CBRS_TOP | WS_CLIPCHILDREN);
	virtual BOOL Create(LPCWSTR lpszWindowName, CWnd* pParentWnd, 
		UINT nID, DWORD dwStyle = WS_CHILD | WS_VISIBLE | CBRS_TOP| WS_CLIPCHILDREN);
// Attributes
public:
	const BOOL IsFloating() const;
	const BOOL IsHorzDocked() const;
	const BOOL IsVertDocked() const;
	const BOOL IsSideTracking() const;

// Operations
public:

	const BOOL GetFXBarStyle() const{ return m_dwBarStyle;};
	const CSize GetHorzBarSize() {return m_szHorzBar;}
	const CSize GetVertBarSize() {return m_szVertBar;}
	const CSize GetFloatBarSize() {return m_szFloatBar;}
	
	const CSize GetMinHorzBarSize() {return m_szMinHorzBar;}
	const CSize GetMinVertBarSize() {return m_szMinVertBar;}
	const CSize GetMinFloatBarSize() {return m_szMinFloatBar;}
	
	void SetHorzBarSize(CSize szHorzBar) { m_szHorzBar = szHorzBar;}
	void SetVertBarSize(CSize szVertBar) { m_szVertBar = szVertBar;}
	void setFloatBarSize(CSize szFloatBar) { m_szFloatBar = szFloatBar;}
	
	void SetMinHorzBarSize(CSize szMinHorzBar) { m_szMinHorzBar = szMinHorzBar;}
	void SetMinVertBarSize(CSize szMinVertBar) { m_szMinVertBar = szMinVertBar;}
	void setMinFloatBarSize(CSize szMinFloatBar) { m_szMinFloatBar = szMinFloatBar;}

	void SetFXBarStyle(DWORD dwBarStyle) {m_dwBarStyle = (dwBarStyle & ~FX_CNTRLBARS_EDGEALL);}
	UINT GetDockBarState(){return m_nDockBarState;}

//	CWnd* GetDockParentWnd() {return m_pDockParentWnd;}
//Overridables
	virtual void OnUpdateCmdUI(CFrameWnd* pTarget, BOOL bDisableIfNoHndler);
/*	virtual void EnableDocking(DWORD dwDockStyle);*/
//Overrides
public:
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CFX_ControlBar)
    virtual CSize CalcFixedLayout(BOOL bStretch, BOOL bHorz);
    virtual CSize CalcDynamicLayout(int nLength, DWORD dwMode);
    //}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CFX_ControlBar();


protected:
	UINT GetEdgeHTCode(int nEdge);
	BOOL GetEdgeRect(CRect rcWnd,UINT nHitTest, CRect& rcEdge);
	virtual void StartTracking(UINT nHitTest, CPoint point);
	virtual void StopTracking();
	virtual void OnTrackUpdateSize(CPoint& point);
	virtual void OnTrackInvertTracker();
	virtual void NcPaintGripper(CDC* pDC,CRect rcClient);
    virtual void NcCalcClient(LPRECT pRc, UINT nDockBarID);	
	
	void GetCtrlBarList(CPtrCtrlBarList& pCtrlBarList);
    void GetCtrlBarList(CPtrCtrlBarList& pCtrlBarList, int& nThisPos);
    BOOL GetCtrlBarListInfo(int& nFirst, int& nLast, int& nThis);
    
    BOOL NegotiateSpace(int nLengthTotal, BOOL bHorz);

protected:
    DWORD   m_dwBarStyle;
    UINT    m_htEdge;
	UINT    m_nDockBarState;
    CSize   m_szHorzBar;
    CSize   m_szVertBar;
    CSize   m_szFloatBar;
    CSize   m_szMinHorzBar;
    CSize   m_szMinVertBar;
    CSize   m_szMinFloatBar;
    int     m_nTrackPosMin;
    int     m_nTrackPosMax;
    int     m_nTrackPosOld;
    int     m_nTrackEdgeOfs;
    BOOL    m_bTracking;
    BOOL    m_bKeepSize;
    BOOL    m_bParentSizing;
//    BOOL    m_bDragShowContent;
    int     m_ncxEdge;

	// Generated message map functions
protected:
    //{{AFX_MSG(CFX_ControlBar )
    afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
    afx_msg void OnNcPaint();
    afx_msg void OnNcCalcSize(BOOL bCalcValidRects, NCCALCSIZE_PARAMS FAR* lpncsp);
    afx_msg LRESULT OnNcHitTest(CPoint point);
    afx_msg void OnCaptureChanged(CWnd *pWnd);
    afx_msg void OnSettingChange(UINT uFlags, LPCTSTR lpszSection);
    afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
    afx_msg void OnMouseMove(UINT nFlags, CPoint point);
    afx_msg void OnNcLButtonDown(UINT nHitTest, CPoint point);
    afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
    afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
    afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
    afx_msg void OnWindowPosChanging(WINDOWPOS FAR* lpwndpos);
    afx_msg void OnPaint();
    afx_msg void OnClose();
    afx_msg void OnSize(UINT nType, int cx, int cy);
   	afx_msg void OnDestroy();
    //}}AFX_MSG
    afx_msg LRESULT OnSetText(WPARAM wParam, LPARAM lParam);
	
    DECLARE_MESSAGE_MAP()
	
};


/////////////////////////////////////////////////////////////////////////
// CFX_MiniDockFrameWnd 

 
#endif // !defined(AFX_FX_CONTROLBAR_H__INCLUDED_)
