// NotesAnnotAP.cpp : implementation file
//

#include "stdafx.h"
#include "PDFReader.h"
#include "NotesAnnotAP.h"
#include "afxdialogex.h"


// CNotesAnnotAP dialog

IMPLEMENT_DYNAMIC(CNotesAnnotAP, CDialogEx)

CNotesAnnotAP::CNotesAnnotAP(CWnd* pParent /* = NULL*/)
	: CDialogEx(CNotesAnnotAP::IDD, pParent)
	, m_nNoteOpacity(0)
{
	m_pAnnot = NULL;
	m_pWnd = NULL;
}

CNotesAnnotAP::~CNotesAnnotAP()
{
}

void CNotesAnnotAP::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_ANNOT_NOTE_FILLCOLOR, m_FillColorctl);
	DDX_Text(pDX, IDC_ANNOT_TEXT_OPACITY, m_nNoteOpacity);
	DDX_Control(pDX, IDC_ANNOT_NOTES_SLIDER, m_NoteOpacitySlider);
	DDX_Control(pDX, IDC_TEXT_NOTETYPE, m_NoteTypeCtl);
}

BEGIN_MESSAGE_MAP(CNotesAnnotAP, CDialogEx)
	ON_BN_CLICKED(IDC_ANNOT_NOTE_FILLCOLOR, &CNotesAnnotAP::OnBnClickedAnnotNoteFillcolor)
	ON_EN_CHANGE(IDC_ANNOT_TEXT_OPACITY, &CNotesAnnotAP::OnEnChangeAnnotTextOpacity)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_ANNOT_NOTES_SLIDER, &CNotesAnnotAP::OnNMCustomdrawAnnotNotesSlider)
	ON_CBN_SELCHANGE(IDC_TEXT_NOTETYPE, &CNotesAnnotAP::OnCbnSelchangeTextNotetype)
END_MESSAGE_MAP()

// CNotesAnnotAP message handlers

void CNotesAnnotAP::OnBnClickedAnnotNoteFillcolor()
{
	CColorDialog colorDlg;
	if( colorDlg.DoModal() == IDOK )
	{
		COLORREF crRGB;
		FS_ARGB crArgb;

		crRGB = colorDlg.GetColor();
		
		FS_RESULT ret = FSPDF_Annot_GetColor(m_pAnnot->GetPDFAnnot(), FALSE, &crArgb);
		
		if (FSCRT_ERRCODE_SUCCESS == ret)
		{
			BYTE R = GetRValue(crRGB);
			BYTE G = GetGValue(crRGB);
			BYTE B = GetBValue(crRGB);
			crArgb = crArgb & 0xff000000;
			COLORREF temp = RGB(B, G, R);
			temp = temp | 0xff000000;
			crArgb = temp | crArgb;
		}

		ret = FSPDF_Annot_SetColor(m_pAnnot->GetPDFAnnot(), FALSE, crArgb);
		if (FSCRT_ERRCODE_SUCCESS == ret)
		{
			FSPDF_Annot_ResetAppearance(m_pAnnot->GetPDFAnnot());
			m_FillColorctl.SetFaceColor(crRGB);
			CRect winRect = m_pAnnot->GetViewBBox();
			m_pWnd->InvalidateRect(winRect);
		}
	}
}

void CNotesAnnotAP::UpdateDialog(IReader_Annot* pAnnot, CWnd* pWnd)
{
	if (!pAnnot && !pWnd)
	{
		return;
	}
	m_pAnnot = pAnnot;
	m_pWnd = pWnd;
	FS_ARGB crArgb;
	int nSel;

	//Color
	FS_RESULT ret = FSPDF_Annot_GetColor(m_pAnnot->GetPDFAnnot(), FALSE, &crArgb);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		COLORREF crRGB = RGB((crArgb >>16) &0xff, (crArgb >>8) &0xff, (crArgb) &0xff);
		m_FillColorctl.SetFaceColor(crRGB);
	}

	//Opacity
	FS_FLOAT fOpacity = 0.0f;
	ret = FSPDF_Annot_GetOpacity(m_pAnnot->GetPDFAnnot(), &fOpacity);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		m_nNoteOpacity = (int)(fOpacity * 100.0f + 0.5);
		m_NoteOpacitySlider.SetPos(m_nNoteOpacity);
		CString strNoteOpacity;
		strNoteOpacity.Format(_T("%d"), m_nNoteOpacity);
		GetDlgItem(IDC_ANNOT_TEXT_OPACITY)->SetWindowTextW(strNoteOpacity);
		UpdateData(FALSE);
	}

	//NoteTypele
	FSCRT_BSTR TypeBstr;
	FSCRT_BStr_Init(&TypeBstr);
	FSPDF_Annot_GetType(pAnnot->GetPDFAnnot(), &TypeBstr);

	if (0 == strcmp(FSPDF_ANNOTTYPE_TEXT, TypeBstr.str))
	{
		FSCRT_BStr_Clear(&TypeBstr);
		FSCRT_BSTR NoteType;
		FSCRT_BStr_Init(&NoteType);
		FSPDF_Annot_GetIconName(m_pAnnot->GetPDFAnnot(), &NoteType);
		GetSelFromNoteStyle(&NoteType, &nSel);
		m_NoteTypeCtl.SetCurSel(nSel);
		m_NoteTypeCtl.ShowWindow(SW_SHOW);
		GetDlgItem(IDC_STATIC_NOTETYPE)->ShowWindow(SW_SHOW);
		FSCRT_BStr_Clear(&NoteType);
	}
	else
	{
		m_NoteTypeCtl.ShowWindow(SW_HIDE);
		GetDlgItem(IDC_STATIC_NOTETYPE)->ShowWindow(SW_HIDE);
	}

	FSCRT_BStr_Clear(&TypeBstr);
	
	UpdateData(FALSE);
}


void CNotesAnnotAP::OnEnChangeAnnotTextOpacity()
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialogEx::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO: Add your control notification handler code here
	UpdateData(TRUE);

	if (m_nNoteOpacity > 100)
	{
		m_nNoteOpacity = 100;
		UpdateData(FALSE);
	}

	FS_FLOAT fOpacity = (FS_FLOAT)m_nNoteOpacity;

	fOpacity = fOpacity / 100;

	FS_RESULT ret = FSPDF_Annot_SetOpacity(m_pAnnot->GetPDFAnnot(), fOpacity);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		FSPDF_Annot_ResetAppearance(m_pAnnot->GetPDFAnnot());
		CRect winRect = m_pAnnot->GetViewBBox();
		m_pWnd->InvalidateRect(winRect);
		m_NoteOpacitySlider.SetPos(m_nNoteOpacity);
	}
}

BOOL CNotesAnnotAP::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	// TODO: Add extra initialization here
	m_NoteOpacitySlider.SetRange(0, 100);

	return TRUE; // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CNotesAnnotAP::OnNMCustomdrawAnnotNotesSlider(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMCUSTOMDRAW pNMCD = reinterpret_cast<LPNMCUSTOMDRAW>(pNMHDR);
	// TODO: Add your control notification handler code here
	int nPos = 	m_NoteOpacitySlider.GetPos();
	m_nNoteOpacity = nPos;
	UpdateData(FALSE);
	OnEnChangeAnnotTextOpacity();
	*pResult = 0;
}

void CNotesAnnotAP::GetSelFromNoteStyle(FSCRT_BSTR * NoteStyle, int *nSel)
{

	if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_CHECK, NoteStyle->str))
	{
		*nSel = 0;
	}
	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_CIRCLE , NoteStyle->str))
	{
		*nSel = 1;
	}
	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_COMMENT , NoteStyle->str))
	{
		*nSel = 2;
	}
	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_CROSS , NoteStyle->str))
	{
		*nSel = 3;
	}
	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_HELP , NoteStyle->str))
	{
		*nSel = 4;
	}
	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_INSERT , NoteStyle->str))
	{
		*nSel = 5;
	}
	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_KEY , NoteStyle->str))
	{
		*nSel = 6;
	}
	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_NEWPARAGRAPH, NoteStyle->str))
	{
		*nSel = 7;
	}
	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_NOTE, NoteStyle->str))
	{
		*nSel = 8;
	}
	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_PARAGRAPH , NoteStyle->str))
	{
		*nSel = 9;
	}

	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_RIGHARROW , NoteStyle->str))
	{
		*nSel = 10;
	}
	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_RIGHPOINTER, NoteStyle->str))
	{
		*nSel = 11;
	}
	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_STAR, NoteStyle->str))
	{
		*nSel = 12;
	}
	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_UPARROW , NoteStyle->str))
	{
		*nSel = 13;
	}
	else if ( 0 == strcmp(FSPDF_ANNOT_ICONNAME_TEXT_UPLEFTARROW , NoteStyle->str))
	{
		*nSel = 13;
	}
}

void CNotesAnnotAP::GetNoteStyleFromSel(int nSel, FSCRT_BSTR& NoteStyle)
{
	switch(nSel)
	{
	case 0:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_CHECK, strlen(FSPDF_ANNOT_ICONNAME_TEXT_CHECK));
		break;
	case 1:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_CIRCLE, strlen(FSPDF_ANNOT_ICONNAME_TEXT_CIRCLE));
		break;
	case 2:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_COMMENT, strlen(FSPDF_ANNOT_ICONNAME_TEXT_COMMENT));
		break;
	case 3:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_CROSS, strlen(FSPDF_ANNOT_ICONNAME_TEXT_CROSS));
		break;
	case 4:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_HELP, strlen(FSPDF_ANNOT_ICONNAME_TEXT_HELP));
		break;
	case 5:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_INSERT, strlen(FSPDF_ANNOT_ICONNAME_TEXT_INSERT));
		break;
	case 6:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_KEY, strlen(FSPDF_ANNOT_ICONNAME_TEXT_KEY));
		break;
	case 7:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_NEWPARAGRAPH, strlen(FSPDF_ANNOT_ICONNAME_TEXT_NEWPARAGRAPH));
		break;
	case 8:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_NOTE, strlen(FSPDF_ANNOT_ICONNAME_TEXT_NOTE));
		break;
	case 9:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_PARAGRAPH, strlen(FSPDF_ANNOT_ICONNAME_TEXT_PARAGRAPH));
		break;
	case 10:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_RIGHARROW, strlen(FSPDF_ANNOT_ICONNAME_TEXT_RIGHARROW));
		break;
	case 11:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_RIGHPOINTER, strlen(FSPDF_ANNOT_ICONNAME_TEXT_RIGHPOINTER));
		break;
	case 12:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_STAR, strlen(FSPDF_ANNOT_ICONNAME_TEXT_STAR));
		break;
	case 13:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_UPARROW, strlen(FSPDF_ANNOT_ICONNAME_TEXT_UPARROW));
		break;
	case 14:
		FSCRT_BStr_Set(&NoteStyle, FSPDF_ANNOT_ICONNAME_TEXT_UPLEFTARROW, strlen(FSPDF_ANNOT_ICONNAME_TEXT_UPLEFTARROW));
		break;
	}
}

void CNotesAnnotAP::OnCbnSelchangeTextNotetype()
{
	// TODO: Add your control notification handler code here
	int nSel = m_NoteTypeCtl.GetCurSel();
	FSCRT_BSTR NoteType;
	FSCRT_BStr_Init(&NoteType);
	GetNoteStyleFromSel(nSel, NoteType);
	FS_RESULT ret = FSPDF_Annot_SetIconName(m_pAnnot->GetPDFAnnot(), &NoteType);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		FSPDF_Annot_ResetAppearance(m_pAnnot->GetPDFAnnot());

		CRect winRect = m_pAnnot->GetViewBBox();
		winRect.InflateRect(3, 3);
		m_pWnd->InvalidateRect(winRect);
	}
	FSCRT_BStr_Clear(&NoteType);
}

void CNotesAnnotAP::EnableWindowDlg(BOOL bEnable)
{
	GetDlgItem(IDC_ANNOT_NOTE_FILLCOLOR)->EnableWindow(bEnable);
	GetDlgItem(IDC_ANNOT_TEXT_OPACITY)->EnableWindow(bEnable);
	GetDlgItem(IDC_ANNOT_NOTES_SLIDER)->EnableWindow(bEnable);
	GetDlgItem(IDC_TEXT_NOTETYPE)->EnableWindow(bEnable);
}
