// PDFDisplayView.cpp : implementation file
//

#include "stdafx.h"
#include "PDFReader.h"
#include "PDFDisplayView.h"
#include "PDFReaderDoc.h"
#include "./FrameworkImpl/PDFReaderApp.h"
#include "./FrameworkImpl/PDFReaderDocument.h"
#include "./FrameworkImpl/PDFReaderPage.h"
#include "./FrameworkImpl/PDFReaderTool.h"
#include "PDFReaderView.h"
#include "./FrameworkImpl/AnnotHandlerMgr.h"
#include <math.h>
#define IDC_ANNOT_TEXT_EDIT		2550

#define  SUPPORT_UNICODE	 1
#define  NOT_SUPPORT_UNICODE 0
#define  NOT_CHK_SUPPORT	 -1	 

//app handler callback
static FS_RESULT _AppEvent(FS_LPVOID clientData, FS_LPVOID senderObject, FS_DWORD senderObjectType, FS_DWORD eventType, FS_LPVOID eventData)
{
	if(senderObjectType != FSCRT_OBJECTTYPE_PSI)
		return FSCRT_ERRCODE_ERROR;

	FSCRT_RECTF* rect = (FSCRT_RECTF*)eventData;
	return ((CPDFDisplayView*)clientData)->PsiFlush(rect);
}
static FS_RESULT _MyOnRecover(FS_LPVOID clientData, FS_LPVOID senderObject, FS_DWORD senderObjectType)
{
	return FSCRT_ERRCODE_SUCCESS;
}
static FS_RESULT	_MyCallFunction(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_INT32 function, FSCRT_VAR* parameters, FS_INT32 count, FSCRT_VAR* ret)
{
	return FSCRT_ERRCODE_SUCCESS;
}
static void		_MyOnOutOfMemory(FS_LPVOID clientData)
{
}

/////////////////////////////////////////////////////////////////////////////
// CPDFDisplayView

IMPLEMENT_DYNCREATE(CPDFDisplayView, CView)

CPDFDisplayView::CPDFDisplayView()
{
	m_bAnnotTool = FALSE;
	m_bHand = TRUE;
	m_bAnnotIsEdit = FALSE;
	m_pEdit = NULL;
	m_brMine = NULL;
	m_pAnnotType = NULL;
	m_bDrawAnnot = FALSE; 
	m_bScrollFlag = TRUE;
	m_bRatateChange = TRUE;
	m_nRotate = 0;
	m_fZoom = 1;
	m_fLastZoom = m_fZoom;
	m_nCurPageIndex = 0;
	m_nLogPixelX = 0;
	m_nLogPixelY = 0;
	
	m_bFitPage = TRUE;
	m_bFitHeight = FALSE;
	m_bFitWidth = FALSE;

	m_pReaderDocument = NULL;
	m_pPDFReaderView = NULL;
	m_pTextSelection = NULL;
	m_strSearch = _T("");

	m_nSearchFlag = 0;
	m_nSearchPageIndex = 0;
	m_bClick = FALSE;
	m_bLButtonDown = FALSE;
	m_pdfPage = NULL;
	//for select
	m_bSelect = FALSE;
	m_bSelecting = FALSE;
	m_nSelectPage = -1;
	m_bHasChar = FALSE;
	m_nStartCharIndex = -1;
	m_nEndCharIndex = -1;
	m_bStarSelect = FALSE;

	//for psi
	m_hPSI = NULL;
	m_hRenderer = NULL;
	m_hBitmap = NULL;
	m_pBitmapInfo = NULL;
	m_dInkColor = 0;
	m_nInkDia = 0;
	m_fOpacity = 0;
	m_bPSI = FALSE;
	m_bDrawing = FALSE;
	m_curPsiIndex = -1;

	//for psi callback function
	m_hAppHandler.clientData = this;
	m_hAppHandler.OnEvent = _AppEvent;
	m_hAppHandler.OnRecover = _MyOnRecover;
	m_hAppHandler.OnOutOfMemory = _MyOnOutOfMemory;
	m_hAppHandler.CallFunction = _MyCallFunction;
	FSCRT_Library_SetAppHandler(&m_hAppHandler);

	//for pagebox
	m_bDrawBox = FALSE;
	m_nBoxPage = -1;
	//for renderoption
	m_bRenderOption = FALSE;
	m_nFlags = 0;
	m_nTextSelectPage = 0;
	

	m_pSearchDlg = NULL;

	//for annot
	m_nAnnotCur = -1;

	m_bSupportUnicode = NOT_CHK_SUPPORT;

	m_pFreeTextAnnotEditFont = NULL;

	m_lpTimerFunc = NULL;
}

CPDFDisplayView::~CPDFDisplayView()
{
	CReader_AnnotHandlerMgr* pAnnotHandlerMgr = theApp.m_pReaderApp->m_pAnnotHandlerMgr;
	if (pAnnotHandlerMgr)
	{
		if (pAnnotHandlerMgr->m_pAnnotPropertyDlg)
		{
			pAnnotHandlerMgr->m_pAnnotPropertyDlg->DestroyWindow();
			delete pAnnotHandlerMgr->m_pAnnotPropertyDlg;
			pAnnotHandlerMgr->m_pAnnotPropertyDlg = NULL;
		}
	}

	if (m_pSearchDlg)
	{
		m_pSearchDlg->DestroyWindow();

		delete m_pSearchDlg;
		m_pSearchDlg = NULL;
	}
	if (m_pEdit)
	{
		m_pEdit->DestroyWindow();

		delete m_pEdit;
		m_pEdit = NULL;
	}

	if (NULL != m_brMine)
	{
		DeleteObject(m_brMine);
	}

	if(m_pReaderDocument->GetChangeMark())
	{
		CString szSaveFile = m_pReaderDocument->GetFilePathStr();
		int ret = szSaveFile.ReverseFind(L'\\');
		CString szTem = szSaveFile.Right(szSaveFile.GetLength() - ret - 1);
		szTem = _T("Do you want to save changes to '") + szTem + _T("' before closing?");
		if (IDYES == AfxGetApp()->ShowAppMessageBox(AfxGetApp(), szTem , MB_YESNO, NULL))
		{
			m_pReaderDocument->Save();
		}
	}
	if (m_pFreeTextAnnotEditFont)
	{
		delete m_pFreeTextAnnotEditFont;
		m_pFreeTextAnnotEditFont = NULL;
	}
}

BEGIN_MESSAGE_MAP(CPDFDisplayView, CView)
	//{{AFX_MSG_MAP(CPDFDisplayView)
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
	ON_WM_MOUSEWHEEL()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_RBUTTONDBLCLK()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_MOUSEMOVE()
	ON_WM_CHAR()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_COMMAND(ID_TOOL_SEARCH, OnSearch)
	ON_COMMAND(ID_TOOL_SELECT, OnSelect)
	ON_COMMAND(ID_TOOL_PSI, OnPsi)
	ON_COMMAND(ID_TOOL_ZOOMIN, OnZoomIn)
	ON_COMMAND(ID_TOOL_ZOOMOUT, OnZoomOut)
	ON_COMMAND(ID_FILE_PRINT, OnFilePrint)
	ON_COMMAND(ID_TOOL_ANNOTSELECT, OnAnnotSelect)

	ON_COMMAND(ID_ANNOT_UNDERLINE, OnAnnotUnderLine)
	ON_COMMAND(ID_ANNOT_FREETEXT, OnAnnotFreeText)
	ON_COMMAND(ID_ANNOT_LINK, OnAnnotLink)
	ON_COMMAND(ID_ANNOT_NOTES, OnAnnotNotes)
	ON_COMMAND(ID_ANNOT_HIGHTLIGHT, OnAnnotHightLight)

	//ON_EN_KILLFOCUS(IDC_ANNOT_TEXT_EDIT, OnAnnotAnnotTextEditKillFocus)

	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	//}}AFX_MSG_MAP
	ON_WM_CTLCOLOR()
	
	ON_COMMAND(ID_ROTATION_LEFT, &CPDFDisplayView::OnRotationLeft)
	ON_COMMAND(ID_ROTATION_RIGHT, &CPDFDisplayView::OnRotationRight)
	ON_WM_TIMER()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPDFDisplayView drawing
void CPDFDisplayView::OnDraw(CDC* pDC)
{
	if(m_bScrollFlag)
	{
		m_nLogPixelX = pDC->GetDeviceCaps(LOGPIXELSX); //screen resolution
		m_nLogPixelY = pDC->GetDeviceCaps(LOGPIXELSY);	
		InitScrollSize(m_nLogPixelX, m_nLogPixelY);
		SetVisiblePageByScrollPos();
		m_bScrollFlag = FALSE;
	}
	if(!m_vecPageToDisplay.empty())
	{		
		vector<PAGETODISPLAY>::iterator i;
		CDC dcMem;
		CBitmap bm;		
		CRect rc;

		dcMem.CreateCompatibleDC(pDC);
		this->GetClientRect(&rc);

		bm.CreateCompatibleBitmap(pDC, rc.Width(), rc.Height());
		HBITMAP hOldbmp  = (HBITMAP)::SelectObject(dcMem.GetSafeHdc(), (HBITMAP)bm.GetSafeHandle());

		dcMem.FillSolidRect(0, 0, rc.Width(), rc.Height(), RGB(120, 120, 120));	

		for(i = m_vecPageToDisplay.begin(); i!= m_vecPageToDisplay.end(); i++)
		{
			CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(i->index);
			float width = 0, height = 0;		
			if(m_nRotate == 0 || m_nRotate == 2)
			{					
				height = pReaderPage->GetPageHeight() * m_fZoom;
				width = pReaderPage->GetPageWidth() * m_fZoom;				
			}
			else
			{
				height = pReaderPage->GetPageWidth() * m_fZoom;
				width = pReaderPage->GetPageHeight() * m_fZoom;				
			}
			RenderPageByGE(&dcMem, (int)i->index, (int)width, (int)height, i->cx, (int)(i->cy * m_fZoom));

			//draw psi
			if (m_bPSI && m_hPSI)
			{
				FS_INT32 width, height;
				FS_LPVOID pBuf = NULL;
				FS_RESULT ret = FSCRT_Bitmap_GetSize(m_hBitmap, &width, &height);
				ASSERT(FSCRT_ERRCODE_SUCCESS == ret);
				ret = FSCRT_Bitmap_GetLineBuffer(m_hBitmap, 0, &pBuf);
				ASSERT(FSCRT_ERRCODE_SUCCESS == ret);
				StretchDIBits(dcMem, (int)m_rPSIViewRect.left, (int)m_rPSIViewRect.top, width, height, 
								(int)m_rPSIViewRect.left, (int)m_rPSIViewRect.top, width, height, 
								pBuf, (const BITMAPINFO*)m_pBitmapInfo, DIB_RGB_COLORS, SRCAND);
			}

			//Shadows on the surface of the page
			int cyTemp = (int)(i->cy * m_fZoom);
			int nShadow = m_pReaderDocument->GetPageShadowWidth();
			CRect rcShadowRight((int)(i->cx+width), cyTemp+nShadow, (int)(i->cx+width+nShadow), (int)(cyTemp+height+nShadow));
			dcMem.FillSolidRect(&rcShadowRight, RGB(0, 0, 0));
			CRect rcShadowBottom(i->cx+nShadow, (int)(cyTemp+height), (int)(i->cx+width), (int)(cyTemp+height+nShadow));
			dcMem.FillSolidRect(&rcShadowBottom, RGB(0, 0, 0));
		}

		IReader_ToolHandler* pCurToolHandler = theApp.m_pReaderApp->GetCurrentToolHandler();
		if(pCurToolHandler)
		{
			pCurToolHandler->Tool_OnDraw(m_pReaderDocument, dcMem.GetSafeHdc());
		}
		if (m_bDrawBox && m_nBoxPage == m_nCurPageIndex)
		{
			DrawBox(&dcMem, m_rPageArea, m_BoxColor);
		}
		pDC->BitBlt(0, 0, rc.Width(), rc.Height(), &dcMem, 0, 0, SRCCOPY);

		bm.DeleteObject();
		DeleteObject(hOldbmp);
		dcMem.DeleteDC(); 
	}
	

	if (m_bDrawAnnot)
	{
		//dc.Rectangle(1, 1, 100, 100);
		CRect rect(m_StartPoint, m_EndPonit);

		rect.NormalizeRect();
		if (!rect.IsRectEmpty() && IsInPageRect(m_StartPoint) && IsInPageRect(m_EndPonit))
		{
			CBrush br;
			CPen pen;
			pen.CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
			br.CreateStockObject(NULL_BRUSH);

			HGDIOBJ OldBr = pDC->SelectObject(&pen);
			HGDIOBJ OldPen = pDC->SelectObject(&br);

			rect.NormalizeRect();
			pDC->Rectangle(rect);

			pDC->SelectObject(&OldBr);
			pDC->SelectObject(&OldPen);
		}
		
	}

	if (!m_bSelecting && m_rSearchPage.GetSize()!= 0)
	{
		for (int j = 0;j<m_rSearchPage.GetSize();j++)
		{
			DrawSearchText(pDC, m_rSearchPage.GetAt(j));;
		}
	}

	if (m_bSelecting && m_bSelect && m_rSelectPage.GetSize()!= 0)
	{
		for (int j = 0;j<m_rSelectPage.GetSize();j++)
		{
			DrawSelectText(pDC, m_rSelectPage.GetAt(j));
		}
	}

	if (m_bSelect && m_nTextSelectPage == m_nCurPageIndex)
	{
		DrawCaret(m_rCaret);
	}
}

BOOL CPDFDisplayView::PreCreateWindow(CREATESTRUCT& cs)
{
	if(cs.lpszClass == NULL)
		cs.lpszClass = AfxRegisterWndClass(CS_DBLCLKS);


	if( !CView::PreCreateWindow(cs) )
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	// the CREATESTRUCT cs

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CPdfView diagnostics

#ifdef _DEBUG
void CPDFDisplayView::AssertValid() const
{
	CView::AssertValid();
}

void CPDFDisplayView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CPDFDisplayView functions

void CPDFDisplayView::SetPDFReaderView(CPDFReaderView* pPDFReaderView)
{
	m_pPDFReaderView = pPDFReaderView;
	if (m_pPDFReaderView != NULL)
	{
		CPDFReaderDoc* pPDFReaderDoc = (CPDFReaderDoc*)m_pPDFReaderView->GetDocument();
		if (pPDFReaderDoc)
		{
			m_pReaderDocument = pPDFReaderDoc->GetReaderDocument();
		}
	}
}

//set size of scroll bar
void CPDFDisplayView::InitScrollSize(int nLogPixelX, int nLogPixelY)
{
	if (!m_pReaderDocument) return;

	if(m_bRatateChange)
	{	
		m_pReaderDocument->InitPDFPages(nLogPixelX, nLogPixelY, m_nRotate);
		m_bRatateChange = FALSE;
	}

	CRect rc;
	GetClientRect(&rc);
	GetScrollInfo (SB_VERT, &m_Vertsi, SIF_ALL);
	GetScrollInfo (SB_HORZ, &m_Horzsi, SIF_ALL);

	m_Vertsi.fMask = SIF_ALL;
	m_Vertsi.nMax = (int)((m_pReaderDocument->GetRealHeight() * m_fZoom +1)/*+ (m_pReaderDocument->GetRealHeight()*m_fZoom - rc.Height()) * ((1 -m_fZoom)/m_fZoom)*/); //has a problem with multiply zoom
	m_Vertsi.nMin = 0;
	m_Vertsi.nPage = rc.Height();
	m_Vertsi.nPos = (int)m_Vertsi.nPos*(m_fZoom/m_fLastZoom); //m_Vertsi.nPos = 0;
	SetScrollInfo(SB_VERT, &m_Vertsi);

	
	m_Horzsi.fMask = SIF_ALL;
	m_Horzsi.nMax = (int)(m_pReaderDocument->GetRealWidth() * m_fZoom);//+ (m_pReaderDocument->GetRealWidth() * m_fZoom -rc.Width())* ((1 -m_fZoom)/m_fZoom) ;
	m_Horzsi.nMin = 0;
	m_Horzsi.nPage = rc.Width();
	m_Horzsi.nPos = -(int)(GetCenterPos(m_nCurPageIndex) / 2);//(int)(m_Horzsi.nPos*(m_fZoom/m_fLastZoom));//;
	SetScrollInfo(SB_HORZ, &m_Horzsi);
}

//get the position's middle of the page
float CPDFDisplayView::GetCenterPos(int nPage)
{
	if (!m_pReaderDocument) return 0;

	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(nPage);
	CRect rc;
	float width;
	GetClientRect(&rc);
	if (m_nRotate == 1 || m_nRotate == 3)
		width = pReaderPage->GetPageHeight() * m_fZoom;
	else width = pReaderPage->GetPageRealWidth() * m_fZoom;
	float pos = rc.Width()/2 - width/2;
	return pos;
}

//according to scroll bar's position , set the visible page area
void CPDFDisplayView::SetVisiblePageByScrollPos(int pose)
{
	if (!m_pReaderDocument) return;

	SCROLLINFO si ;
	float StartPos = 0, EndPos = 0, PrintPos = 0;
	m_nStartPageIndex = 0;
	m_nEndPageIndex = 0;
	float cy = 0 , cx = 0;
	CRect rc(0, 0, 0, 0);
	GetClientRect(&rc); 	
	if (!pose)
	{
		GetScrollInfo (SB_VERT, &si, SIF_ALL); 
		StartPos = (float)si.nPos;		
	}
	else StartPos = pose;

	EndPos = rc.Height() /*/ m_fZoom*/ + StartPos;
	PrintPos = StartPos;

	float fPageStartPos = StartPos/m_fZoom;
	float fPageEndPos = EndPos/m_fZoom;

	m_pReaderDocument->FindVisiblePage(fPageStartPos, fPageEndPos, m_nStartPageIndex, m_nEndPageIndex);	//get ranges and length of displayed page

	StartPos = (fPageStartPos/*+1*/)*m_fZoom;
	EndPos   = fPageEndPos*m_fZoom;
	//m_nCurPageIndex = m_StartPageIndex;

	CReader_Page* pReaderStartPage = (CReader_Page*)m_pReaderDocument->GetPage(m_nStartPageIndex);
	GetScrollInfo(SB_HORZ, &si, SIF_ALL);

	GetScrollInfo(SB_HORZ, &m_Horzsi);

	if (pReaderStartPage->GetPageRealWidth() < rc.Width())
	{
		if (GetCenterPos(m_nStartPageIndex) < 0)
		{
			cx = -(float)m_Horzsi.nPos;
		}
		else
		{
			cx = GetCenterPos(m_nStartPageIndex);
		}
	}
	else
		cx = -(float)si.nPos;


	ReleasePageTemData(m_nStartPageIndex, m_nEndPageIndex);
	m_vecPageToDisplay.clear();	

	PAGETODISPLAY pageToDisplay;
	pageToDisplay.cx = (int)cx ;
	pageToDisplay.cy = (int)(StartPos - PrintPos)/m_fZoom;
	pageToDisplay.index = m_nStartPageIndex;
	m_vecPageToDisplay.push_back(pageToDisplay);
	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(m_nStartPageIndex);
	
	if (pReaderPage)
	{
		cy = pReaderPage->GetPageTotalHeight() - PrintPos/m_fZoom;
	}

	for(int j = m_nStartPageIndex+1; j <= m_nEndPageIndex; j++)
	{
		CReader_Page* pReaderPageGet = (CReader_Page*)m_pReaderDocument->GetPage(j);

		if (pReaderPageGet->GetPageRealWidth() < rc.Width())
			cx = GetCenterPos(j);
		else
			cx = -(float)si.nPos;

		pageToDisplay.cx = (int)cx;
		pageToDisplay.cy = (int)cy;
		pageToDisplay.index = j;
		m_vecPageToDisplay.push_back(pageToDisplay);
		cy = pReaderPageGet->GetPageTotalHeight() - PrintPos/m_fZoom;
	}
}

//according to page number , set visible page area
void CPDFDisplayView::SetVisiblePageByIndex(int nIdex)
{
	if (!m_pReaderDocument) return;

	float fPageTotalHeight = 0;
	int nNextPos = 0;
	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(nIdex - 1);
	if (pReaderPage)
	{
		fPageTotalHeight = pReaderPage->GetPageTotalHeight();
	}

	nNextPos = (int)(fPageTotalHeight*m_fZoom+1);

	SCROLLINFO si ;
 	GetScrollInfo(SB_VERT, &si, SIF_ALL);

	if (nNextPos >= si.nMax-si.nPage+1)
		nNextPos = si.nMax-si.nPage+1;	

	si.nPos = (int)nNextPos;
	SetScrollInfo(SB_VERT, &si);

	SetVisiblePageByScrollPos(nNextPos);
}

BOOL CPDFDisplayView::MovePage(CPoint curPoint, CPoint toPoint)
{
	GetScrollInfo(SB_VERT, &m_Vertsi);
	GetScrollInfo(SB_HORZ, &m_Horzsi);
	CPoint pt;
	pt = toPoint - curPoint;
	m_Vertsi.nPos = m_Vertsi.nPos - pt.y;
	m_Horzsi.nPos = m_Horzsi.nPos - pt.x;
	
	SetScrollInfo(SB_VERT, &m_Vertsi);
	SetScrollInfo(SB_HORZ, &m_Horzsi);
	SetVisiblePageByScrollPos();

	return TRUE;
}

BOOL CPDFDisplayView::MovePage(short zDelta)
{
	GetScrollInfo(SB_VERT, &m_Vertsi);
	m_Vertsi.nPos = m_Vertsi.nPos - zDelta;
	SetScrollInfo(SB_VERT, &m_Vertsi);
	SetVisiblePageByScrollPos();
	
	return TRUE;
}

//input page
void CPDFDisplayView::RenderPage(CDC* pDC, int Index, int Width, int Height, int cx, int cy)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	int nRotate = 0;
	FSCRT_RENDERER render;
	
	ret = FSCRT_Renderer_CreateOnWindowsDC(pDC->GetSafeHdc(), &render);

	if (FSCRT_ERRCODE_INVALIDLICENSE == ret && m_pReaderDocument->m_bRender)
	{
		AfxMessageBox(_T("You don't have the right of \"FSCRT_Renderer_CreateOnWindowsDC\" Functions, please purchase \"Standard\" Modules!"));
		m_pReaderDocument->m_bRender = FALSE;
	}
	
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		return;
	}
	//renderOption Set Gamma
	if (((CPDFReaderApp*)AfxGetApp())->m_bRenderOption && m_bRenderOption)
	{
		if (((CPDFReaderApp*)AfxGetApp())->m_bGamma)
		{
			//ret = FSCRT_Renderer_SetGamma(((CPDFReaderApp*)AfxGetApp())->m_nGamma);
			if (IDYES == MessageBox(_T("Document need reopen, Save the document and reopen?"), _T("Save"), MB_YESNO))
			{
				m_pReaderDocument->ReLoadPDFDocument();
				m_bRatateChange = 1;
				m_pReaderDocument->InitPDFPages(m_nLogPixelX, m_nLogPixelY, nRotate);
			}
		}
		m_bRenderOption = FALSE;
	}

	if (((CPDFReaderApp*)AfxGetApp())->m_bBits)
	{
		ret = FSCRT_Renderer_SetDitherBits(render, ((CPDFReaderApp*)AfxGetApp())->m_nBits);
	}
	
	if (((CPDFReaderApp*)AfxGetApp())->m_bClipRect )
	{
		float cx = 	GetCenterPos(Index);
		FSCRT_RECT temp;
		temp.left = (int)(((CPDFReaderApp*)AfxGetApp())->m_rClip.left + cx);
		temp.right = (int)(((CPDFReaderApp*)AfxGetApp())->m_rClip.right + cx);
		temp.top = (int)(((CPDFReaderApp*)AfxGetApp())->m_rClip.top);
		temp.bottom = (int)(((CPDFReaderApp*)AfxGetApp())->m_rClip.bottom);
		if (temp.left < temp.right && temp.top < temp.bottom)
		{
			ret = FSCRT_Renderer_SetClipRect(render, &temp);
		}
		
	}
	if (((CPDFReaderApp*)AfxGetApp())->m_bFlags)
	{
		ret = FSCRT_Renderer_SetFlags(render, ((CPDFReaderApp*)AfxGetApp())->m_nFlags);
	}

	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(Index);
	
	if (pReaderPage)
	{
		pReaderPage->ParsePage();
		pReaderPage->LoadAnnots();
	}

	FSCRT_PAGE pdfPage = pReaderPage->GetPage();

	//Clear the canvas with the white color.
	RECT rcPage = {cx, cy, cx+Width, cy+Height};
	HBRUSH hBrush = 	CreateSolidBrush(RGB(0xff, 0xff, 0xff));
	FillRect(pDC->GetSafeHdc(), &rcPage, hBrush);
	DeleteObject(hBrush);

	int nSaveDC = pDC->SaveDC();
	pDC->IntersectClipRect(&rcPage);
	
	//Get page transformation matrix.
	FSCRT_MATRIX matrix;
	FSPDF_Page_GetMatrix(pdfPage, cx, cy, Width, Height, nRotate, &matrix);
	//Create a renderer on the printer device context.

	//Create a PDF rendering context. The PDF rendering context is designed specially to ensure rendering PDF contents on rendering engine.
	FSPDF_RENDERCONTEXT context;
	ret = FSPDF_RenderContext_Create(&context);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_Renderer_Release(render);
		return;
	}
	//Set a transformation matrix to a rendering context.
	FSPDF_RenderContext_SetMatrix(context, &matrix);

	//Set flags to a rendering context.
	FSPDF_RenderContext_SetFlags(context, FSPDF_RENDERCONTEXTFLAG_ANNOT|FSPDF_RENDERCONTEXTFLAG_NOWIDGET);
	
	//Set Layer Context
	if (m_pReaderDocument->IsChangeLayerContext())
	{
		FSPDF_LAYERCONTEXT layerContext =	m_pReaderDocument->GetLayerContext();
		FSPDF_RenderContext_SetLayerContext(context,layerContext);
	}
	
	//Start rendering a PDF page in a renderer with a PDF rendering context.
	FSCRT_PROGRESS progress;
	ret = FSPDF_RenderContext_StartPage(context, render, pdfPage, FSPDF_PAGERENDERFLAG_NORMAL, &progress);
	if (ret != FSCRT_ERRCODE_SUCCESS) 
	{
		FSPDF_RenderContext_Release(context);
		FSCRT_Renderer_Release(render);
		return;
	}

	//Continue a progressive process.
	ret = FSCRT_ERRCODE_TOBECONTINUED;
	while (ret == FSCRT_ERRCODE_TOBECONTINUED)
	{
		ret = FSCRT_Progress_Continue(progress, NULL);
	}
	if (ret != FSCRT_ERRCODE_FINISHED)
	{
		//Unknown error occurs, release resource and return.
	}

	//Release the current progressive process.
	FSCRT_Progress_Release(progress);

	//Render Form controls.
	progress = NULL;
	ret = FSPDF_RenderContext_StartPageFormControls(context, render, pdfPage, &progress);
	ret = FSCRT_ERRCODE_TOBECONTINUED;
	while (ret == FSCRT_ERRCODE_TOBECONTINUED)
	{
		ret = FSCRT_Progress_Continue(progress, NULL);
	}
	FSCRT_Progress_Release(progress);

	//Release rendering context.
	FSPDF_RenderContext_Release(context);
	//Release renderer.
	FSCRT_Renderer_Release(render);

	//draw annots
	CReader_AnnotHandlerMgr* pAnnotHandlerMgr = (CReader_AnnotHandlerMgr*)theApp.m_pReaderApp->GetAnnotHandlerMgr();
	int nAnnotCount = pReaderPage->CountAnnots();
	for (int i = 0; i<nAnnotCount; i++)
	{
		IReader_Annot* pReaderAnnot = pReaderPage->GetAnnot(i);
		pAnnotHandlerMgr->Annot_OnDraw(pReaderPage, pDC->GetSafeHdc(), pReaderAnnot, matrix, rcPage);
	}	
	pDC->RestoreDC(nSaveDC);
}

void CPDFDisplayView::RenderPageByGE(CDC* pDC, int Index, int Width, int Height, int cx, int cy)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;

	//Clear the canvas with the white color.
	RECT rcPage = {cx, cy, cx+Width, cy+Height};
	RECT rc;
	this->GetClientRect(&rc);
	rcPage.right = rcPage.right > rc.right ? rc.right : rcPage.right;
	rcPage.bottom = rcPage.bottom > rc.bottom ? rc.bottom : rcPage.bottom;

	HBRUSH hBrush = CreateSolidBrush(RGB(255, 255, 255));
	FillRect(pDC->GetSafeHdc(), &rcPage, hBrush);
	DeleteObject(hBrush);

	

	FSCRT_RENDERER render;



	FSCRT_RECT rectBitmap;
	rectBitmap.left = 0;
	rectBitmap.right = rcPage.right - rcPage.left;
	rectBitmap.top = 0;
	rectBitmap.bottom = rcPage.bottom - rcPage.top;
	FSCRT_BITMAP bitmap;
	FSCRT_Bitmap_Create((int)rectBitmap.right, (int)rectBitmap.bottom, FSCRT_BITMAPFORMAT_32BPP_BGRA, NULL, 0, &bitmap);

	ret = FSCRT_Renderer_CreateOnBitmap(bitmap, &render);
	if (FSCRT_ERRCODE_INVALIDLICENSE == ret && m_pReaderDocument->m_bRender)
	{
		AfxMessageBox(_T("You don't have the right of \"FSCRT_Renderer_CreateOnBitmap\" Functions, please purchase \"Standard\" Modules!"));
		m_pReaderDocument->m_bRender = FALSE;
	}

	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		return;
	}
	//renderOption Set Gamma
	if (((CPDFReaderApp*)AfxGetApp())->m_bRenderOption && m_bRenderOption)
	{
		if (((CPDFReaderApp*)AfxGetApp())->m_bGamma)
		{
			//ret = FSCRT_Renderer_SetGamma(((CPDFReaderApp*)AfxGetApp())->m_nGamma);
			if (IDYES == MessageBox(_T("Document need reopen, Save the document and reopen?"), _T("Save"), MB_YESNO))
			{
				m_pReaderDocument->ReLoadPDFDocument();
				m_bRatateChange = 1;
				m_pReaderDocument->InitPDFPages(m_nLogPixelX, m_nLogPixelY, m_nRotate);
			}
		}
		m_bRenderOption = FALSE;
	}

	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(Index);

	if (pReaderPage)
	{
		pReaderPage->ParsePage();
		pReaderPage->LoadAnnots();
	}
	FSCRT_PAGE pdfPage = pReaderPage->GetPage();

	if (pReaderPage->IsHasTransparency())
		FSCRT_Bitmap_FillRect(bitmap, FSCRT_ARGB_Encode(0, 0, 0, 0), &rectBitmap);
	else
		FSCRT_Bitmap_FillRect(bitmap, FSCRT_ARGB_Encode(0xff, 0xff, 0xff, 0xff), &rectBitmap);

	if (((CPDFReaderApp*)AfxGetApp())->m_bBits)
	{
		ret = FSCRT_Renderer_SetDitherBits(render, ((CPDFReaderApp*)AfxGetApp())->m_nBits);
	}

	if (((CPDFReaderApp*)AfxGetApp())->m_bClipRect)
	{
		float cx = 	GetCenterPos(Index);
		FSCRT_RECT temp;
		temp.left = (int)(((CPDFReaderApp*)AfxGetApp())->m_rClip.left + cx);
		temp.right = (int)(((CPDFReaderApp*)AfxGetApp())->m_rClip.right + cx);
		temp.top = (int)(((CPDFReaderApp*)AfxGetApp())->m_rClip.top);
		temp.bottom = (int)(((CPDFReaderApp*)AfxGetApp())->m_rClip.bottom);
		if (temp.left < temp.right && temp.top < temp.bottom)
		{
			ret = FSCRT_Renderer_SetClipRect(render, &temp);
		}
	}

	if (((CPDFReaderApp*)AfxGetApp())->m_bFlags)
	{
		ret = FSCRT_Renderer_SetFlags(render, ((CPDFReaderApp*)AfxGetApp())->m_nFlags);
	}
	
	int nSaveDC = pDC->SaveDC();
	pDC->IntersectClipRect(&rcPage);
	CPoint pos = pDC->GetCurrentPosition();

	//Get page transformation matrix.
	FSCRT_MATRIX matrix;
	FSPDF_Page_GetMatrix(pdfPage, -pos.x, -pos.y, Width, Height, m_nRotate, &matrix);
	//Create a renderer on the printer device context.

	//Create a PDF rendering context. The PDF rendering context is designed specially to ensure rendering PDF contents on rendering engine.
	FSPDF_RENDERCONTEXT context;
	ret = FSPDF_RenderContext_Create(&context);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_Renderer_Release(render);
		return;
	}
	//Set a transformation matrix to a rendering context.
	FSPDF_RenderContext_SetMatrix(context, &matrix);

	//Set flags to a rendering context.
	FSPDF_RenderContext_SetFlags(context, FSPDF_RENDERCONTEXTFLAG_ANNOT|FSPDF_RENDERCONTEXTFLAG_NOWIDGET);

	//Set Layer Context
	if (m_pReaderDocument->IsChangeLayerContext())
	{
		FSPDF_LAYERCONTEXT layerContext =	m_pReaderDocument->GetLayerContext();
		FSPDF_RenderContext_SetLayerContext(context,layerContext);
	}

	//Start rendering a PDF page in a renderer with a PDF rendering context.
	FSCRT_PROGRESS progress;
	ret = FSPDF_RenderContext_StartPage(context, render, pdfPage, FSPDF_PAGERENDERFLAG_NORMAL, &progress);
	if (ret != FSCRT_ERRCODE_SUCCESS) 
	{
		FSPDF_RenderContext_Release(context);
		FSCRT_Renderer_Release(render);
		return;
	}

	//Continue a progressive process.
	ret = FSCRT_ERRCODE_TOBECONTINUED;
	while (ret == FSCRT_ERRCODE_TOBECONTINUED)
	{
		ret = FSCRT_Progress_Continue(progress, NULL);
	}
	if (ret != FSCRT_ERRCODE_FINISHED)
	{
		//Unknown error occurs, release resource and return.
	}

	//Release the current progressive process.
	FSCRT_Progress_Release(progress);

	//Render Form controls.
	progress = NULL;
	ret = FSPDF_RenderContext_StartPageFormControls(context, render, pdfPage, &progress);
	ret = FSCRT_ERRCODE_TOBECONTINUED;
	while (ret == FSCRT_ERRCODE_TOBECONTINUED)
	{
		ret = FSCRT_Progress_Continue(progress, NULL);
	}
	FSCRT_Progress_Release(progress);

	//Release rendering context.
	FSPDF_RenderContext_Release(context);
	//Release renderer.
	FSCRT_Renderer_Release(render);
	FSCRT_Bitmap_ConvertFormat(bitmap, FSCRT_BITMAPFORMAT_32BPP_BGRx);

	FS_LPVOID pBuf;
	FSCRT_Bitmap_GetLineBuffer(bitmap, 0, &pBuf);

	FSCRT_BITMAPINFO* bitmapInfo = NULL;
	FS_DWORD dwSize = 0;
	FSCRT_Bitmap_GetBitmapInfo(bitmap, bitmapInfo, &dwSize);

	FSCRT_Memory_Alloc(dwSize, (FS_LPVOID*)&bitmapInfo);
	
	FSCRT_Bitmap_GetBitmapInfo(bitmap, bitmapInfo, &dwSize);

	StretchDIBits(pDC->GetSafeHdc(), (int)cx, (int)cy, rectBitmap.right, rectBitmap.bottom, 
		0, 0, rectBitmap.right, rectBitmap.bottom, pBuf, (const BITMAPINFO*)bitmapInfo, DIB_RGB_COLORS, SRCCOPY);

	FSCRT_Memory_Free(bitmapInfo);
	FSCRT_Bitmap_Release(bitmap);

	//draw annots
	CReader_AnnotHandlerMgr* pAnnotHandlerMgr = (CReader_AnnotHandlerMgr*)theApp.m_pReaderApp->GetAnnotHandlerMgr();
	int nAnnotCount = pReaderPage->CountAnnots();
	for (int i = 0; i<nAnnotCount; i++)
	{
		IReader_Annot* pReaderAnnot = pReaderPage->GetAnnot(i);
		pAnnotHandlerMgr->Annot_OnDraw(pReaderPage, pDC->GetSafeHdc(), pReaderAnnot, matrix, rcPage);
	}
	pDC->RestoreDC(nSaveDC);
}

CReader_Page* CPDFDisplayView::GetPage(CPoint point)
{
	if(!m_vecPageToDisplay.empty())
	{
		vector<PAGETODISPLAY>::iterator i;
		for(i = m_vecPageToDisplay.begin(); i!= m_vecPageToDisplay.end(); i++)
		{
			CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(i->index);
			float width = 0, height = 0;
			if(m_nRotate == 0 || m_nRotate == 2)
			{
				height = pReaderPage->GetPageHeight() * m_fZoom;
				width = pReaderPage->GetPageWidth() * m_fZoom;
			}
			else
			{
				height = pReaderPage->GetPageWidth() * m_fZoom;
				width = pReaderPage->GetPageHeight() * m_fZoom;
			}

			RECT rcTemp = {i->cx, (int)(i->cy * m_fZoom), (int)(i->cx+width), (int)(i->cy * m_fZoom+height)};
			if(point.x>= rcTemp.left && point.x<= rcTemp.right && point.y>= rcTemp.top && point.y<= rcTemp.bottom)
			{
				return pReaderPage;
			}
		}
	}
	return NULL;
}

BOOL CPDFDisplayView::GetPageMatrix(int nPageIndex, FSCRT_MATRIX* matrix)
{
	if(!m_vecPageToDisplay.empty())
	{
		vector<PAGETODISPLAY>::iterator i;
		for(i = m_vecPageToDisplay.begin(); i!= m_vecPageToDisplay.end(); i++)
		{
			CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(i->index);
			if (pReaderPage&&pReaderPage->GetPageIndex() == nPageIndex)
			{
				float width = 0, height = 0;
				if(m_nRotate == 0 || m_nRotate == 2)
				{
					height = pReaderPage->GetPageHeight() * m_fZoom;
					width = pReaderPage->GetPageWidth() * m_fZoom;
				}
				else
				{
					height = pReaderPage->GetPageWidth() * m_fZoom;
					width = pReaderPage->GetPageHeight() * m_fZoom;
				}
				FSPDF_Page_GetMatrix(pReaderPage->GetPage(), i->cx, (FS_INT32)(i->cy * m_fZoom), (FS_INT32)width, (FS_INT32)height, m_nRotate, matrix);
				return TRUE;
			}
		}
	}
	return FALSE;
}

BOOL CPDFDisplayView::GetPageMatrixForPrint(int nPageIndex, FSCRT_MATRIX* matrix)
{
	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(nPageIndex);
	if (pReaderPage)
	{
		float width = 0, height = 0;
		FS_INT32 nRotate = 0;
		FSPDF_Page_GetRotation(pReaderPage->GetPage(), &nRotate);
		if(nRotate == 0 || nRotate == 2)
		{
			height = pReaderPage->GetPageHeight() * m_fZoom;
			width = pReaderPage->GetPageWidth() * m_fZoom;
		}
		else
		{
			height = pReaderPage->GetPageWidth() * m_fZoom;
			width = pReaderPage->GetPageHeight() * m_fZoom;
		}

		FSCRT_PAGE pdfPage = NULL;
		FS_RESULT ret = FSPDF_Doc_GetPage(m_pReaderDocument->GetDocument(), nPageIndex, &pdfPage);
		FSPDF_Page_GetMatrix(pdfPage, 0, 0 ,(FS_INT32)width, (FS_INT32)height, nRotate, matrix);

		return TRUE;
	}

	return FALSE;
}

BOOL CPDFDisplayView::DocToWindow(int nPageIndex, FSCRT_RECTF rcDoc, CRect& rcWindow)
{
	if(!m_vecPageToDisplay.empty())
	{
		vector<PAGETODISPLAY>::iterator i;
		for(i = m_vecPageToDisplay.begin(); i!= m_vecPageToDisplay.end(); i++)
		{
			CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(i->index);
			if (pReaderPage->GetPageIndex() == nPageIndex)
			{
				float width = 0, height = 0;
				if(m_nRotate == 0 || m_nRotate == 2)
				{
					height = pReaderPage->GetPageHeight() * m_fZoom;
					width = pReaderPage->GetPageWidth() * m_fZoom;
				}
				else
				{
					height = pReaderPage->GetPageWidth() * m_fZoom;
					width = pReaderPage->GetPageHeight() * m_fZoom;
				}

				FSCRT_MATRIX matrix;
				FSPDF_Page_GetMatrix(pReaderPage->GetPage(), i->cx, (FS_INT32)(i->cy * m_fZoom), (FS_INT32)width, (FS_INT32)height, m_nRotate, &matrix);
				FSCRT_Matrix_TransformRectF(&matrix, &rcDoc);
				rcWindow.SetRect((int)(rcDoc.left+0.5), (int)(rcDoc.bottom+0.5), (int)(rcDoc.right+0.5), (int)(rcDoc.top+0.5));
				return TRUE;
			}
		}
	}
	return FALSE;
}

BOOL CPDFDisplayView::WindowToDoc(int nPageIndex, int window_x, int window_y, FS_FLOAT& doc_x, FS_FLOAT& doc_y)
{
	if(!m_vecPageToDisplay.empty())
	{		
		vector<PAGETODISPLAY>::iterator i;
		for(i = m_vecPageToDisplay.begin(); i!= m_vecPageToDisplay.end(); i++)
		{
			CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(i->index);
			if (pReaderPage->GetPageIndex() == nPageIndex)
			{
				float width = 0, height = 0;
				if(m_nRotate == 0 || m_nRotate == 2)
				{
					height = pReaderPage->GetPageHeight() * m_fZoom;
					width = pReaderPage->GetPageWidth() * m_fZoom;
				}
				else
				{
					height = pReaderPage->GetPageWidth() * m_fZoom;
					width = pReaderPage->GetPageHeight() * m_fZoom;
				}

				FSCRT_MATRIX matrix;
				FSPDF_Page_GetMatrix(pReaderPage->GetPage(), i->cx, (FS_INT32)(i->cy * m_fZoom), (FS_INT32)width, (FS_INT32)height, m_nRotate, &matrix);
				
				FSCRT_MATRIX dstMatrix;
				FSCRT_Matrix_GetReverse(&matrix, &dstMatrix);
				doc_x = (FS_FLOAT)window_x;
				doc_y = (FS_FLOAT)window_y;
				FSCRT_Matrix_TransformPointF(&dstMatrix, &doc_x, &doc_y);

				return TRUE;
			}
		}
	}
	return FALSE;
}

BOOL CPDFDisplayView::WindowToDoc(int nPageIndex, CRect rcWindow, FSCRT_RECTF& rcDoc)
{
	if(!m_vecPageToDisplay.empty())
	{
		vector<PAGETODISPLAY>::iterator i;
		for(i = m_vecPageToDisplay.begin(); i!= m_vecPageToDisplay.end(); i++)
		{
			CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(i->index);
			if (pReaderPage->GetPageIndex() == nPageIndex)
			{
				float width = 0, height = 0;
				if(m_nRotate == 0 || m_nRotate == 2)
				{
					height = pReaderPage->GetPageHeight() * m_fZoom;
					width = pReaderPage->GetPageWidth() * m_fZoom;
				}
				else
				{
					height = pReaderPage->GetPageWidth() * m_fZoom;
					width = pReaderPage->GetPageHeight() * m_fZoom;
				}

				FSCRT_MATRIX matrix;
				FSPDF_Page_GetMatrix(pReaderPage->GetPage(), i->cx, (FS_INT32)(i->cy * m_fZoom), (FS_INT32)width, (FS_INT32)height, m_nRotate, &matrix);

				FSCRT_MATRIX dstMatrix;
				FSCRT_Matrix_GetReverse(&matrix, &dstMatrix);
				rcDoc.left = (FS_FLOAT)rcWindow.left;
				rcDoc.bottom = (FS_FLOAT)rcWindow.top;
				rcDoc.right = (FS_FLOAT)rcWindow.right;
				rcDoc.top = (FS_FLOAT)rcWindow.bottom;
				FSCRT_Matrix_TransformRectF(&dstMatrix, &rcDoc);
				return TRUE;
			}
		}
	}
	return FALSE;
}

BOOL CPDFDisplayView::WindowToDocForPrint(int nPageIndex, CRect rcWindow, FSCRT_RECTF& rcDoc)
{
	FSCRT_MATRIX matrix;
	GetPageMatrixForPrint(nPageIndex, &matrix);

	FSCRT_MATRIX dstMatrix;
	FSCRT_Matrix_GetReverse(&matrix, &dstMatrix);
	rcDoc.left = (FS_FLOAT)rcWindow.left;
	rcDoc.bottom = (FS_FLOAT)rcWindow.top;
	rcDoc.right = (FS_FLOAT)rcWindow.right;
	rcDoc.top = (FS_FLOAT)rcWindow.bottom;
	FSCRT_Matrix_TransformRectF(&dstMatrix, &rcDoc);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CPdfView message handlers

void CPDFDisplayView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
	m_pFreeTextAnnotEditFont = new CFont();
	//horizontal scroll bar
	m_Horzsi.nMin = 0; 
	m_Horzsi.nMax = 0; 
	m_Horzsi.nPage = 0; 
	m_Horzsi.cbSize = sizeof(SCROLLINFO); 
	m_Horzsi.fMask = SIF_ALL; 
	m_Horzsi.nPos = 0; 
	m_Horzsi.nTrackPos = 0; 
	SetScrollInfo(SB_HORZ, &m_Horzsi); 
	//vertical scroll bar
	m_Vertsi.nMin = 0; 
	m_Vertsi.nMax = 0; 
	m_Vertsi.nPage = 0; 
	m_Vertsi.cbSize = sizeof(SCROLLINFO); 
	m_Vertsi.fMask = SIF_ALL; 
	m_Vertsi.nPos = 0; 
	m_Vertsi.nTrackPos = 0; 
	SetScrollInfo(SB_VERT, &m_Vertsi);

}

void CPDFDisplayView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	GetScrollInfo(SB_HORZ, &m_Horzsi);
	switch(nSBCode)
	{
	case SB_LINERIGHT:
		{
			m_Horzsi.nPos = m_Horzsi.nPos+m_Horzsi.nPage;
			SetScrollInfo(SB_HORZ, &m_Horzsi);
			SetVisiblePageByScrollPos();
			ResetCurrentPageIndex();
			Invalidate();
			break;
		}
	case SB_LINELEFT:
		{
			m_Horzsi.nPos = m_Horzsi.nPos-m_Horzsi.nPage;
			SetScrollInfo(SB_HORZ, &m_Horzsi);
			SetVisiblePageByScrollPos();
			ResetCurrentPageIndex();
			Invalidate();
			break;
		}
	case SB_THUMBTRACK: //position of drag scroll box
		{
			break;
		}
	case SB_THUMBPOSITION: //position of release mouse
		{
			SCROLLINFO si; 
			GetScrollInfo(SB_HORZ, &si, SIF_TRACKPOS);//nPos only has 16bit , nTrackPos is 32bit
			nPos = si.nTrackPos;
			m_Horzsi.nPos = nPos;
			SetScrollInfo(SB_HORZ, &m_Horzsi);
			SetVisiblePageByScrollPos();
			ResetCurrentPageIndex();
			Invalidate();
			break;
		}
	case SB_PAGERIGHT:
		{
			m_Horzsi.nPos = (int)(m_Horzsi.nPos + m_Horzsi.nMax * 0.2);
			SetScrollInfo(SB_HORZ, &m_Horzsi);
			SetVisiblePageByScrollPos();
			ResetCurrentPageIndex();
			Invalidate();
			break;
		}
	case SB_PAGELEFT:
		{	
			m_Horzsi.nPos = (int)(m_Horzsi.nPos - m_Horzsi.nMax * 0.2);
			SetScrollInfo(SB_HORZ, &m_Horzsi);
			SetVisiblePageByScrollPos();
			ResetCurrentPageIndex();
			Invalidate();
			break;
		}
	}
}

void CPDFDisplayView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	GetScrollInfo(SB_VERT, &m_Vertsi);
	switch(nSBCode)
	{
	case SB_LINEDOWN:
		{
			m_Vertsi.nPos = (int)(m_Vertsi.nPos+m_Vertsi.nPage*0.2);
			if (m_Vertsi.nPos + (int)m_Vertsi.nPage > m_Vertsi.nMax )
			{
				m_Vertsi.nPos = m_Vertsi.nMax - m_Vertsi.nPage;
			}
			SetScrollInfo(SB_VERT, &m_Vertsi);
			SetVisiblePageByScrollPos();
			ResetCurrentPageIndex();
			Invalidate();
			break;
		}
	case SB_LINEUP:
		{
			m_Vertsi.nPos = (int)(m_Vertsi.nPos-m_Vertsi.nPage*0.2);
			SetScrollInfo(SB_VERT, &m_Vertsi);
			SetVisiblePageByScrollPos();
			ResetCurrentPageIndex();
			Invalidate();
			break;
		}
	case SB_THUMBTRACK: // position of drag scroll box
		{				
			break;
		}
	case SB_THUMBPOSITION: // position of release mouse
		{	
			SCROLLINFO si ; 
			GetScrollInfo(SB_VERT, &si, SIF_TRACKPOS);//nPos only has 16bit , nTrackPos is 32bit
			nPos = si.nTrackPos;
			m_Vertsi.nPos = nPos;
			SetScrollInfo(SB_VERT, &m_Vertsi);
			SetVisiblePageByScrollPos();
			ResetCurrentPageIndex();
			Invalidate();
			break;
		}
	case SB_PAGEDOWN:
		{
			m_Vertsi.nPos = (int)(m_Vertsi.nPos+m_Vertsi.nMax*0.002);
			SetScrollInfo(SB_VERT, &m_Vertsi);
			SetVisiblePageByScrollPos();
			ResetCurrentPageIndex();
			Invalidate();
			break;
		}
	case SB_PAGEUP:
		{
			m_Vertsi.nPos = (int)(m_Vertsi.nPos-m_Vertsi.nMax*0.002);
			SetScrollInfo(SB_VERT, &m_Vertsi);
			SetVisiblePageByScrollPos();
			ResetCurrentPageIndex();
			Invalidate();
			break;
		}
	}
}

BOOL CPDFDisplayView::OnMouseWheel(UINT nFlags, short zDelta, CPoint point) 
{
	IReader_ToolHandler* pCurToolHandler = theApp.m_pReaderApp->GetCurrentToolHandler();
	if(pCurToolHandler == NULL)
		return FALSE;

	if (MK_CONTROL == nFlags)
	{
		if (zDelta < 0)
		{
			OnZoomOut();
		}
		else
		{
			OnZoomIn();	
		}
		return TRUE;
	}

	BOOL bRet = FALSE;
	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(point);
	
	/*if (m_bAnnotIsEdit)
	{
	OnAnnotAnnotTextEditKillFocus();
	m_bAnnotIsEdit = FALSE;
	}*/
	
	if(pReaderPage)
	{
		bRet = pCurToolHandler->Tool_OnMouseWheel((IReader_Page*)pReaderPage, nFlags, zDelta , point);
	}
	return TRUE;
}

BOOL CPDFDisplayView::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}

void CPDFDisplayView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	if(m_bScrollFlag == FALSE)
	{
		if(m_bFitPage == TRUE)
		{
			m_bScrollFlag = TRUE;
			Invalidate();
			return;
		}
		if(m_bFitWidth == TRUE)
		{
			if (m_pReaderDocument)
			{
				m_fLastZoom = m_fZoom;
				m_fZoom = cx / m_pReaderDocument->GetRealWidth();
			}
			m_bScrollFlag = TRUE;
			Invalidate();
			return;
		}
		if(m_bFitHeight == TRUE)
		{
			CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(m_nCurPageIndex);
			m_fLastZoom = m_fZoom;
			if(m_nRotate == 0 || m_nRotate == 2)
				m_fZoom = cy / pReaderPage->GetPageHeight();
			else
				m_fZoom = cy / pReaderPage->GetPageWidth();
			m_bScrollFlag = TRUE;
			Invalidate();
			return;
		}
	}
}

void CPDFDisplayView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	m_bLButtonDown = TRUE;
	if(m_bAnnotIsEdit)
	{
		CRect rc;
		m_pEdit->GetWindowRect(&rc);
		ScreenToClient(&rc);
		if (!rc.PtInRect(point))
		{
			OnAnnotAnnotTextEditKillFocus();
			m_pEdit->ShowWindow(SW_HIDE);
			m_bAnnotIsEdit = FALSE;
		}
	}

	if (m_bDrawAnnot)
	{
		if (IsInPageRect(point))
		{
			m_StartPoint = point;
			return;
		}
	}

	m_bClick = FALSE;
	m_pLbdown = point;
	m_bStarSelect = TRUE;

	IReader_ToolHandler* pCurToolHandler = theApp.m_pReaderApp->GetCurrentToolHandler();
	if(pCurToolHandler == NULL)
		return;

	BOOL bRet = FALSE;
	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(point);
	if(!pReaderPage) return;
	
	IReader_Annot* pReaderAnnot = pReaderPage->GetAnnotAtPoint(point);
	CReader_AnnotHandlerMgr* pAnnotHandlerMgr = (CReader_AnnotHandlerMgr*)theApp.m_pReaderApp->GetAnnotHandlerMgr();
	if (m_bAnnotTool && pAnnotHandlerMgr && !m_bSelect && !m_bPSI ) 
	{
		//FSPDF_Annot_ResetAppearance(pReaderAnnot->GetPDFAnnot());
		bRet = pAnnotHandlerMgr->Annot_OnLButtonDown((IReader_Page*)pReaderPage, pReaderAnnot, nFlags, point);
		pReaderPage->SetFocusAnnotFlag(TRUE);
		m_nAnnotCur = GetPage(point)->GetPageIndex();
	}

	if (!pReaderAnnot)
	{
		pReaderPage->SetFocusAnnotFlag(FALSE);
		m_pReaderDocument->SetSelectedAnnot(NULL);
	}

	if(!bRet && pReaderPage)
		bRet = pCurToolHandler->Tool_OnLButtonDown((IReader_Page*)pReaderPage, nFlags , point);

	CReader_Page* pPage = GetPage(point);
	if (pPage)
	{
		FSCRT_MATRIX matrix;
		GetPageMatrix(pPage->GetPageIndex(), &matrix);

		FSCRT_NOTEDATA_MOUSE mouseData;
		mouseData.deltaX = 0;
		mouseData.deltaY = 0;
		mouseData.flags = nFlags;
		mouseData.x = (FS_FLOAT)point.x;
		mouseData.y = (FS_FLOAT)point.y;
		FS_RESULT ret = FSPDF_FormFiller_TriggerWindowlessEvent(m_pReaderDocument->GetFormFiller(), pPage->GetPage(), &matrix, FSCRT_EVENT_LBUTTONDOWN, &mouseData);
		if (FSCRT_ERRCODE_SUCCESS == ret)
		{
			if (CReader_Document::GetMouseFlag())
			{
				LPARAM lParam = point.y * 65536 + point.x;
				::PostMessage(m_hWnd, WM_LBUTTONUP, 1, lParam);
				CReader_Document::SetMouseFlag(FALSE);
			}
			m_pReaderDocument->SetChangeMark();
			pReaderPage->SetFocusAnnot(NULL);
		}
	}
	if (m_bSelect && !m_bClick)
	{
		m_bSelecting = TRUE; 
		m_rSelectPage.RemoveAll();
		m_nTextSelectPage = pReaderPage->GetPageIndex();
		m_nCurPageIndex = pReaderPage->GetPageIndex();
		Invalidate();
		CReader_Page* page = GetPage(point);
		if (page)
		{
			FSCRT_PAGE pdfPage = page->GetPage();
			FS_RESULT ret = -1;
			m_pdfPage = (CReader_Page*)m_pReaderDocument->GetPage(m_nCurPageIndex);
			if (!m_pdfPage->GetPDFTextPage())
			{
				m_pdfPage->LoadPDFTextPage();
			}

			m_nTextSelectPage = page->GetPageIndex();
			m_bHasChar = GetStartCharIndexByPoint(point, m_nStartCharIndex);
			CreateCaret(point);
			m_ptSelectStart = point;
		}
	}
	if (m_bPSI)
	{
		FSCRT_Library_SetAppHandler(&m_hAppHandler);
		SetViewRect(point);
		InitPSI();
		if(NULL != m_hPSI)
		{
			FS_FLOAT x = point.x - m_rPSIViewRect.left;
			FS_FLOAT y = point.y - m_rPSIViewRect.top;
			if (x >= 0 && y >= 0)
			{
				FS_FLOAT fPressure = (FS_FLOAT)1024 / 1024;
				FS_RESULT ret = FSCRT_PSI_AddPoint(m_hPSI, x, y, fPressure, FSCRT_PSI_PT_MOVETO);
				m_bDrawing = TRUE;
				m_curPsiIndex = pReaderPage->GetPageIndex();
			}
		}
	}
	CView::OnLButtonDown(nFlags, point);
}

void CPDFDisplayView::OnLButtonUp(UINT nFlags, CPoint point) 
{	
	m_bLButtonDown = FALSE;
	if (m_bDrawAnnot)
	{
		if (m_StartPoint == point)
		{
			return;
		}
		CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(m_StartPoint);
		if (!pReaderPage)
		{
			return;
		}
		CRect rect(m_StartPoint, point);

		rect.NormalizeRect();
		FSCRT_RECTF AnnotRect;
		pReaderPage->WindowToDoc(rect, AnnotRect);
		if (AnnotRect.left > AnnotRect.right)
		{
			FS_FLOAT tem = AnnotRect.left;
			AnnotRect.left = AnnotRect.right;
			AnnotRect.right = tem;
		}

		if (AnnotRect.bottom > AnnotRect.top)
		{
			FS_FLOAT tem = 	AnnotRect.bottom ;
			AnnotRect.bottom = AnnotRect.top;
			AnnotRect.top = tem;
		}
		pReaderPage->AddAnnot(m_pAnnotType, AnnotRect);
		m_StartPoint.SetPoint(0, 0);
		m_EndPonit.SetPoint(0, 0);
		Invalidate();
		return;
	}

	m_bStarSelect = FALSE;
	if (!m_bAnnotIsEdit)
	{
		ReleaseCapture();
	}
	
	IReader_ToolHandler* pCurToolHandler = theApp.m_pReaderApp->GetCurrentToolHandler();

	BOOL bRet = FALSE;
	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(point);
	if (pReaderPage)
	{
		IReader_Annot* pReaderAnnot = pReaderPage->GetAnnotAtPoint(point);
		CReader_AnnotHandlerMgr* pAnnotHandlerMgr = (CReader_AnnotHandlerMgr*)theApp.m_pReaderApp->GetAnnotHandlerMgr();
		if (m_bAnnotTool && pAnnotHandlerMgr && !m_bPSI && !m_bSelect)
		{
			if ( m_nAnnotCur == pReaderPage->GetPageIndex())
			{
				bRet = pAnnotHandlerMgr->Annot_OnLButtonUp((IReader_Page*)pReaderPage, pReaderAnnot, nFlags, point);
			}
			else
			{
				if (GetPage(m_AnnotLastPoint))
				{
					bRet = pAnnotHandlerMgr->Annot_OnLButtonUp((IReader_Page*)GetPage(m_AnnotLastPoint), pReaderAnnot, nFlags, m_AnnotLastPoint);
				}
			}
			Invalidate();
		}

		if(!bRet && pReaderPage)
			bRet = pCurToolHandler->Tool_OnLButtonUp((IReader_Page*)pReaderPage, nFlags , point);
	}
	else
	{
		if (GetPage(m_AnnotLastPoint))
		{
			CReader_AnnotHandlerMgr* pAnnotHandlerMgr = (CReader_AnnotHandlerMgr*)theApp.m_pReaderApp->GetAnnotHandlerMgr();
			bRet = pAnnotHandlerMgr->Annot_OnLButtonUp((IReader_Page*)GetPage(m_AnnotLastPoint), NULL, nFlags, m_AnnotLastPoint);
			Invalidate();
		}
	}

	CReader_Page* pPage = GetPage(point);
	if (pPage)
	{
		FSCRT_MATRIX matrix;
		GetPageMatrix(pPage->GetPageIndex(), &matrix);

		FSCRT_NOTEDATA_MOUSE mouseData;
		mouseData.deltaX = 0;
		mouseData.deltaY = 0;
		mouseData.flags = nFlags;
		mouseData.x = (FS_FLOAT)point.x;
		mouseData.y = (FS_FLOAT)point.y;
		FS_RESULT ret = FSPDF_FormFiller_TriggerWindowlessEvent(m_pReaderDocument->GetFormFiller(), pPage->GetPage(), &matrix, FSCRT_EVENT_LBUTTONUP, &mouseData);
		if (FSCRT_ERRCODE_SUCCESS == ret)
		{
			m_pReaderDocument->SetChangeMark();
		}
	}
	if (!m_bClick)
	{
		m_rSearchPage.RemoveAll();
		m_rSelectPage.RemoveAll();
		Invalidate();
	}
	if (m_bPSI && m_bDrawing)
	{
		FS_FLOAT x; 
		FS_FLOAT y;
		CReader_Page* pPage = GetPage(point);
		if (pPage)
		{
			if (m_curPsiIndex == pPage->GetPageIndex())
			{
				x = point.x - m_rPSIViewRect.left;
				y = point.y - m_rPSIViewRect.top;
			}
			else
			{
				x = m_LastPoint.x - m_rPSIViewRect.left;
				y = m_LastPoint.y - m_rPSIViewRect.top;
			}
		}
		else
		{
			x = m_LastPoint.x - m_rPSIViewRect.left;
			y = m_LastPoint.y - m_rPSIViewRect.top;
		}
		
		if (x >= 0 && y >= 0)
		{
			FS_FLOAT fPressure = (FS_FLOAT)1024 / 1024;
			FS_RESULT ret = FSCRT_PSI_AddPoint(m_hPSI, x, y, fPressure, FSCRT_PSI_PT_LINETO | FSCRT_PSI_PT_ENDPATH);
			m_bDrawing = FALSE;
			if (pPage)
			{
				if (m_curPsiIndex == pPage->GetPageIndex())
				{
					PsiToAnnotation(point);
				}
				else
				{
					PsiToAnnotation(m_LastPoint);
				}
			}
			else
			{
				PsiToAnnotation(m_LastPoint);
			}
			
			ReleasePSI();
			m_bDrawing = FALSE;
			Invalidate();
		}

	}
	CView::OnLButtonUp(nFlags, point);
}

void CPDFDisplayView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	if (m_bDrawAnnot)
	{
		CWnd* pDocView = CWnd::FromHandle(m_pReaderDocument->GetDocViewHwnd());

		CRect rsClient;
		pDocView->GetClientRect(&rsClient);
		//GetClientRect(&rect);
		if (rsClient.PtInRect(point))
			::SetCursor(AfxGetApp()->LoadCursor(IDC_CUR_CROSS));
		else
			::SetCursor(AfxGetApp()->LoadStandardCursor(IDC_CROSS));

		if (m_bLButtonDown)
		{
			if (IsInPageRect(point))
			{
				CReader_Page *pPageStart = 	GetPage(m_StartPoint);
				CReader_Page *pPageEnd = GetPage(point);
				if (!pPageStart || !pPageEnd)
 				{
					return;
				}
				if (pPageStart->GetPageIndex() == pPageEnd->GetPageIndex())
				{
					m_EndPonit = point;
				//CRect rect(m_StartPoint, m_EndPonit);
					Invalidate();					
				}
			}	
		}
		return;
	}

	if (nFlags != MK_LBUTTON && !m_bPSI)
	{
		m_pReaderDocument->SetCapturedAnnot(NULL);
		/*if (m_rSelectPage.GetSize() == 0 && m_rSearchPage.GetSize() == 0)
			Invalidate();*/
	}

	IReader_ToolHandler* pCurToolHandler = theApp.m_pReaderApp->GetCurrentToolHandler();
	if(pCurToolHandler == NULL)
		return;

	BOOL bRet = FALSE;
	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(point);
	if (pReaderPage)
	{
		CReader_AnnotHandlerMgr* pAnnotHandlerMgr = (CReader_AnnotHandlerMgr*)theApp.m_pReaderApp->GetAnnotHandlerMgr();
		IReader_Annot* pCaptureAnnot = m_pReaderDocument->GetCapturedAnnot();
		
		if (pCaptureAnnot && pAnnotHandlerMgr)
		{
			if (m_nAnnotCur == GetPage(point)->GetPageIndex())
			{
				bRet = pAnnotHandlerMgr->Annot_OnMouseMove((IReader_Page*)pReaderPage, pCaptureAnnot, nFlags, point);
				m_AnnotLastPoint = point;
			}
			Invalidate();
		}

		IReader_Annot* pReaderAnnot = pReaderPage->GetAnnotAtPoint(point);
		if (m_bAnnotTool && !bRet && pReaderAnnot && pAnnotHandlerMgr && !pCurToolHandler->Tool_IsProcessing()& !m_bSelect && !m_bPSI)
		{
			bRet = pAnnotHandlerMgr->Annot_OnMouseMove((IReader_Page*)pReaderPage, pReaderAnnot, nFlags, point);
		}
	}
	
	CRect clientRc;
	GetClientRect(&clientRc);
	if(!bRet && pReaderPage)
	{
		if (m_bSelect)
		{
			if (clientRc.PtInRect(point) == 0)
			{
				::SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));
			}
			else
			{
				::SetCursor(AfxGetApp()->LoadStandardCursor(IDC_IBEAM));
			}
		}
		else if(m_bPSI)
		{

		}
		else
		{
			bRet = pCurToolHandler->Tool_OnMouseMove((IReader_Page*)pReaderPage, nFlags, point);			
		}
	}
	
	CReader_Page* pPage = GetPage(point);
	if (pPage)
	{
		FSCRT_MATRIX matrix;
		GetPageMatrix(pPage->GetPageIndex(), &matrix);

		FSCRT_NOTEDATA_MOUSE mouseData;
		mouseData.deltaX = 0;
		mouseData.deltaY = 0;
		mouseData.flags = nFlags;
		mouseData.x = (FS_FLOAT)point.x;
		mouseData.y = (FS_FLOAT)point.y;
		FSPDF_FormFiller_TriggerWindowlessEvent(m_pReaderDocument->GetFormFiller(), pPage->GetPage(), &matrix, FSCRT_EVENT_MOUSEMOVE, &mouseData);
	}
	if (abs(m_pLbdown.x - point.x)>= 1 && abs(m_pLbdown.y - point.y)>= 1 && nFlags == MK_LBUTTON)
	{
		m_bClick = TRUE;
	}
	
	if (m_bSelect && nFlags == MK_LBUTTON)
	{
		if (m_bHasChar && m_bStarSelect)
		{
			int index = 0;
			if(!GetCharIndexByPoint(point, index))
				return;
			if (index == m_nEndCharIndex) return;
			m_nEndCharIndex = index;
			CreateCaret(m_ptSelectStart);
			GetStartCharIndexByPoint(m_ptSelectStart, m_nStartCharIndex);
			GetSelectRect(m_nStartCharIndex, m_nEndCharIndex);
			Invalidate();
		}
	}
	if(m_bPSI && m_bDrawing && nFlags == MK_LBUTTON)
	{		
		FS_FLOAT x = point.x - m_rPSIViewRect.left;
		FS_FLOAT y = point.y - m_rPSIViewRect.top;
		if (x >= 0 && y >= 0)
		{
			CReader_Page* pPage = GetPage(point);
			if (pPage)
			{
				FS_FLOAT fPressure = (FS_FLOAT)1024 / 1024;
				FS_RESULT ret = FSCRT_PSI_AddPoint(m_hPSI, x, y, fPressure, FSCRT_PSI_PT_LINETO);
				if (m_curPsiIndex == pPage->GetPageIndex())
				{
					m_LastPoint = point;
				}
			}
		}
	}
	else if (m_bPSI && nFlags != MK_LBUTTON && m_bDrawing)
	{
		if (m_hPSI)
		{
			FS_FLOAT x = m_LastPoint.x - m_rPSIViewRect.left;
			FS_FLOAT y = m_LastPoint.y - m_rPSIViewRect.top;
			FS_FLOAT fPressure = (FS_FLOAT)1024 / 1024;
			FS_RESULT ret = FSCRT_PSI_AddPoint(m_hPSI, x, y, fPressure, FSCRT_PSI_PT_LINETO | FSCRT_PSI_PT_ENDPATH);
			PsiToAnnotation(m_LastPoint);
			ReleasePSI();
			m_bDrawing = FALSE;
		}
		
	}

	CView::OnMouseMove(nFlags, point);
}
void char2wchar_t(char *c, wchar_t **wc)
{
	size_t len = strlen(c) + 1;
	size_t converted = 0;
	*wc=(wchar_t*)malloc(len*sizeof(wchar_t));
	memset(*wc, 0, len*sizeof(wchar_t));

	size_t wclen = MultiByteToWideChar(CP_ACP, 0, c, len, NULL, 0);
	if (wclen > len * sizeof(wchar_t))
	{
		wclen = len* sizeof(wchar_t);
	}
	MultiByteToWideChar(CP_ACP, 0, c, len, *wc, wclen);
}

void CPDFDisplayView::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	IReader_ToolHandler* pCurToolHandler = theApp.m_pReaderApp->GetCurrentToolHandler();
	if(pCurToolHandler == NULL)
		return;

	BOOL bRet = FALSE;
	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(point);

	CReader_AnnotHandlerMgr* pAnnotHandlerMgr = (CReader_AnnotHandlerMgr*)theApp.m_pReaderApp->GetAnnotHandlerMgr();
	IReader_Annot* pCaptureAnnot = m_pReaderDocument->GetCapturedAnnot();
	if (!pReaderPage)
		return;
	IReader_Annot* pReaderAnnot = pReaderPage->GetAnnotAtPoint(point);
	if (m_bAnnotTool && pReaderAnnot && pAnnotHandlerMgr && !pCurToolHandler->Tool_IsProcessing())
	{
		FSCRT_BSTR TypeBstr;
		FSCRT_BStr_Init(&TypeBstr);
		FSPDF_Annot_GetType(pReaderAnnot->GetPDFAnnot(), &TypeBstr);
		if (0 == strcmp(FSPDF_ANNOTTYPE_FREETEXT, TypeBstr.str))
		{
			if (!m_pEdit)
			{
				CWnd* pDocView = CWnd::FromHandle(m_pReaderDocument->GetDocViewHwnd());
				if (pDocView)
				{
					m_pEdit = new CEdit;
					m_pEdit->Create(ES_AUTOVSCROLL|ES_WANTRETURN | ES_MULTILINE | WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_BORDER, 
						CRect(0, 0, 0, 0), pDocView, IDC_ANNOT_TEXT_EDIT);
				}
			}

			m_pFreeTextAnnotEditFont->Detach();

			FSPDF_DEFAULTAPPEARANCE da;
			FSPDF_Annot_GetDefaultAppearance(pReaderAnnot->GetPDFAnnot(), &da);
			FSCRT_MATRIX matrix = da.textMatrix;
			FSCRT_FONT font = da.font;
			FSCRT_BSTR strFontName;
			FSCRT_BStr_Init(&strFontName);
			FSCRT_Font_GetName(font, &strFontName);
		
			if (strFontName.len > 0) {
			wchar_t* tempChar;
			char2wchar_t(strFontName.str, &tempChar);

			m_pFreeTextAnnotEditFont->CreatePointFont(da.fontSize*10/2, tempChar);
			
			m_pEdit->SetFont(m_pFreeTextAnnotEditFont,TRUE);
			free(tempChar);
			}
			CRect winRect = pReaderAnnot->GetInnerRectMargin();
			winRect.bottom = winRect.bottom;
			m_pEdit->MoveWindow(winRect);
			m_pEdit->ShowWindow(SW_SHOW);

			FSCRT_BSTR ContentStr;
			FSCRT_BStr_Init(&ContentStr);
			FSPDF_Annot_GetContents(pReaderAnnot->GetPDFAnnot(), &ContentStr);

			wchar_t* buf = NULL;
			FS_DWORD length = 0;
			FS_RESULT hResult = FSCRT_UTF8_ToUTF16LE(&ContentStr, NULL, &length);
			if(hResult == FSCRT_ERRCODE_SUCCESS)
			{
				buf = new wchar_t[length+1];
				memset(buf, 0, (length+1)*sizeof(wchar_t));
				FSCRT_UTF8_ToUTF16LE(&ContentStr, (FS_WORD*)buf, &length);
			}
			CString tem = buf;
			delete []buf;
			buf = NULL; 
			FSCRT_BStr_Clear(&ContentStr);
			m_pEdit->SetWindowTextW(tem);
			m_pEdit->SetSel(0, -1);
			m_bAnnotIsEdit = TRUE;
			m_csFreeText = tem;
		}
		else
		{
			bRet = pAnnotHandlerMgr->Annot_OnLButtonDblClk((IReader_Page*)pReaderPage, pReaderAnnot, nFlags, point);
		}
		m_pReaderDocument->SetSelectedAnnot(pReaderAnnot);
		FSCRT_BStr_Clear(&TypeBstr);
	}

	if(!bRet && pReaderPage)
		bRet = pCurToolHandler->Tool_OnLButtonDblClk((IReader_Page*)pReaderPage, nFlags , point);

	CReader_Page* pPage = GetPage(point);
	if (pPage)
	{
		FSCRT_MATRIX matrix;
		GetPageMatrix(pPage->GetPageIndex(), &matrix);

		FSCRT_NOTEDATA_MOUSE mouseData;
		mouseData.deltaX = 0;
		mouseData.deltaY = 0;
		mouseData.flags = nFlags;
		mouseData.x = (FS_FLOAT)point.x;
		mouseData.y = (FS_FLOAT)point.y;
		FS_RESULT ret = FSPDF_FormFiller_TriggerWindowlessEvent(m_pReaderDocument->GetFormFiller(), pPage->GetPage(), &matrix, FSCRT_EVENT_LBUTTONDBLCLK, &mouseData);
		if (FSCRT_ERRCODE_SUCCESS == ret)
		{
			m_pReaderDocument->SetChangeMark();
		}
	}
	CView::OnLButtonDblClk(nFlags, point);
}

void CPDFDisplayView::OnRButtonDown(UINT nFlags, CPoint point)
{
	IReader_ToolHandler* pCurToolHandler = theApp.m_pReaderApp->GetCurrentToolHandler();
	if(pCurToolHandler == NULL)
		return;

	BOOL bRet = FALSE;
	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(point);
	if (!pReaderPage)
	{
		return;
	}
	IReader_Annot* pReaderAnnot = pReaderPage->GetAnnotAtPoint(point);
	if (pReaderAnnot && m_bAnnotTool)
	{
		m_pReaderDocument->SetSelectedAnnot(pReaderAnnot);
		Invalidate();
	}

	if(pReaderPage)
		bRet = pCurToolHandler->Tool_OnRButtonDown((IReader_Page*)pReaderPage, nFlags , point);

	//right menu
	if (m_bSelect)
	{
		ShowTextSelectMenu(point);
	}

	CView::OnRButtonDown(nFlags, point);
}

void CPDFDisplayView::OnRButtonUp(UINT nFlags, CPoint point)
{
	IReader_ToolHandler* pCurToolHandler = theApp.m_pReaderApp->GetCurrentToolHandler();
	if(pCurToolHandler == NULL)
		return;

	BOOL bRet = FALSE;
	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(point);
	if(!pReaderPage) return;

	IReader_Annot* pReaderAnnot = pReaderPage->GetAnnotAtPoint(point);
	CReader_AnnotHandlerMgr* pAnnotHandlerMgr = (CReader_AnnotHandlerMgr*)theApp.m_pReaderApp->GetAnnotHandlerMgr();
	if (pReaderAnnot && pAnnotHandlerMgr && m_bAnnotTool)
	{
		bRet = pAnnotHandlerMgr->Annot_OnRButtonUp((IReader_Page*)pReaderPage, pReaderAnnot, nFlags, point);
	}

	if(!bRet && pReaderPage)
		bRet = pCurToolHandler->Tool_OnRButtonUp((IReader_Page*)pReaderPage, nFlags , point);

	CView::OnRButtonUp(nFlags, point);
}

void CPDFDisplayView::OnRButtonDblClk(UINT nFlags, CPoint point)
{
	IReader_ToolHandler* pCurToolHandler = theApp.m_pReaderApp->GetCurrentToolHandler();
	if(pCurToolHandler == NULL)
		return;

	BOOL bRet = FALSE;
	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(point);
	if(pReaderPage)
		bRet = pCurToolHandler->Tool_OnRButtonDblClk((IReader_Page*)pReaderPage, nFlags , point);

	CView::OnRButtonDblClk(nFlags, point);
}

void CPDFDisplayView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	FSCRT_NOTEDATA_CHAR charData;
	charData.code = nChar;
	charData.flags = nFlags;
	FS_RESULT ret = FSPDF_FormFiller_TriggerWindowlessEvent(m_pReaderDocument->GetFormFiller(), NULL, NULL, FSCRT_EVENT_CHAR, &charData);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		m_pReaderDocument->SetChangeMark();
	}
	CView::OnChar(nChar, nRepCnt, nFlags);
}

void CPDFDisplayView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	FSCRT_NOTEDATA_KEY keyData;
	keyData.code = nChar;
	keyData.flags = nFlags;
	FS_RESULT ret = FSPDF_FormFiller_TriggerWindowlessEvent(m_pReaderDocument->GetFormFiller(), NULL, NULL, FSCRT_EVENT_KEYUP, &keyData);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		m_pReaderDocument->SetChangeMark();
	}

	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CPDFDisplayView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	FSCRT_NOTEDATA_KEY keyData;
	keyData.code = nChar;
	keyData.flags = nFlags;	FS_RESULT ret = FSPDF_FormFiller_TriggerWindowlessEvent(m_pReaderDocument->GetFormFiller(), NULL, NULL, FSCRT_EVENT_KEYDOWN, &keyData);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		m_pReaderDocument->SetChangeMark();
	}

	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CPDFDisplayView::OnSearch()
{
	 m_nSearchPageIndex = m_nCurPageIndex;
	 ChangeTool(HAND_TOOL);
	 if (!m_pSearchDlg)
	 {
		m_pSearchDlg = new FindDlg();
		m_pSearchDlg->Create(IDD_FINDDLG, this);
	 }
	
	 m_pSearchDlg->ShowWindow(TRUE);
	 m_pSearchDlg->GetDlgItem(IDC_EDIT_FINDTEXT)->SetFocus();
}

void CPDFDisplayView::SearchText(CString strFind, BOOL bCase, FS_BOOL bWholeword, FS_INT32 Direction)
{
	m_bSelecting = FALSE;
	if (NULL == m_pReaderDocument)
		return;

	m_pdfPage = (CReader_Page *)m_pReaderDocument->GetPage(m_nCurPageIndex);
	if (NULL == m_pdfPage)
		return;
	
	if (!m_pdfPage->GetPDFTextPage())
	{
		BOOL bRet = m_pdfPage->LoadPDFTextPage();
		if (!bRet)
			return;
	}

	FS_INT32 nFlag = 0;
	if(bCase) {nFlag |= FSCRT_SEARCHFLAG_MATCHCASE; }
	if(bWholeword) {nFlag |= FSCRT_SEARCHFLAG_MATCHWHOLEWORD; }

	if( Direction > 0)
	{
		Direction = -1;
	}

	FSCRT_BSTR bstr;
	FSCRT_BStr_Init(&bstr);
	FS_RESULT ret = FSCRT_UTF8_FromUTF16LE(&bstr, (FS_WORD*)(LPCWSTR)(strFind.GetBuffer()), (FS_DWORD)strFind.GetLength());
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSCRT_BStr_Clear(&bstr);
		return;
	}
	
	if (0 != strFind.Compare(m_strSearch) || nFlag != m_nSearchFlag || m_nCurPageIndex != m_nSearchPageIndex) //search text or search flags changed, for new search 
	{		
		BOOL bRet = 	m_pdfPage->StartSearch(&bstr, nFlag, Direction);
		if(!bRet)
		{
			FSCRT_BStr_Clear(&bstr);
			return;
		}
		//save the find info
		m_nSearchFlag = nFlag;
		m_strSearch = strFind;
	}

	FS_BOOL bIsmatch;
	if(Direction == 0) //down
		bIsmatch = m_pdfPage->FindNext();
	else
		bIsmatch = m_pdfPage->FindPrev();

	FS_INT32 nNextPageIndex = 0;
	if (!bIsmatch)
	{
		bIsmatch = m_pdfPage->ContinueSearch(Direction, &bstr, nFlag, Direction, &nNextPageIndex);
		if (!bIsmatch)
		{
			MessageBox(L"Find complete!", L"Find Information", MB_OK | MB_ICONINFORMATION);
		}
		else
		{
			m_pdfPage = (CReader_Page *)m_pReaderDocument->GetPage(nNextPageIndex);
		}
	}

	if (bIsmatch)
	{
		GetSearchTextRect();
	}

	FSCRT_BStr_Clear(&bstr);
}

void CPDFDisplayView::GetSearchTextRect()
 {
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	FS_BOOL isMatch = FALSE;

	if (!m_pdfPage)
		return;

	if (m_pReaderDocument && m_nCurPageIndex != m_pdfPage->GetPageIndex())
	{
		m_pReaderDocument->GotoPage(m_pdfPage->GetPageIndex());
		m_nCurPageIndex = m_pdfPage->GetPageIndex();
	}
	m_nSearchPageIndex = m_nCurPageIndex;

	FSPDF_TEXTSEARCH TextSearch = m_pdfPage->GetPDFTextSearch();

	if (!TextSearch)
		return;
	
	FSPDF_TEXTSELECTION TextSelection = NULL;
	FS_INT32 piecesCount = 0;
	ret = FSPDF_TextSearch_GetSelection(TextSearch, &TextSelection);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		return;
	
	ret = FSPDF_TextSelection_CountPieces(TextSelection, &piecesCount);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		return;

	m_rSearchPage.RemoveAll();
	
	CRect rc;
	FSCRT_RECTF rectf;
	FSCRT_MATRIX matrix;
	GetClientRect(&rc);
	GetPageMatrix(m_pdfPage->GetPageIndex(), &matrix);
	for (int i = 0; i<piecesCount; i++)
	{
		ret = FSPDF_TextSelection_GetPieceRect(TextSelection, i, &rectf);
		m_rSearchPage.Add(rectf);
		FSCRT_RECTF rect = rectf;
		FSCRT_Matrix_TransformRectF(&matrix, &rect);
		CPoint pt((int)rect.right, (int)rect.bottom);

		if (pt.y>rc.bottom)
		{
			int offset = abs(pt.y - rc.bottom/2);
			MovePage(-offset);
		}
		if (pt.y<rc.top)
		{
			int offset = abs(pt.y - rc.bottom/2);
			MovePage(offset);
		}
	}
	FSPDF_TextSelection_Release(TextSelection);
	Invalidate();
}

void CPDFDisplayView::DrawSearchText(CDC* pDc, FSCRT_RECTF rc)
{
	FSCRT_MATRIX matrix ;

	if(m_nSearchPageIndex > m_nEndPageIndex || m_nSearchPageIndex < m_nStartPageIndex)
		return;

	if (GetPageMatrix(m_nSearchPageIndex/*m_pdfPage->GetPageIndex()*/, &matrix))
	{
		FSCRT_Matrix_TransformRectF(&matrix, &rc);
	}
	
	FSCRT_RECTF rt = rc;
	rc.left = __min(rt.left, rt.right);
	rc.right = __max(rt.left, rt.right);
	rc.top = __min(rt.top, rt.bottom);
	rc.bottom = __max(rt.top, rt.bottom);

	CRect rcClient;
	GetClientRect(&rcClient);
	if(rcClient.bottom <= rc.top || rcClient.top >= rc.bottom)
		return;

	COLORREF clrSrc = RGB(255, 255, 0);

	int nSavedDC = pDc->SaveDC();//Save the original DC.

	int nWidth = (int)abs(rc.right-rc.left+1);//The width of the semitransparent zone.
	int nHeight = (int)abs(rc.bottom-rc.top+3);//The height of the semitransparent zone.

	//Copy the screen: create a new DC to copy the target area. 
	CDC bmpDC; 
	bmpDC.CreateCompatibleDC(pDc); 

	//Create a base bitmap
	CBitmap bmp ; 
	bmp.CreateCompatibleBitmap(pDc, nWidth, nHeight); 
	CBitmap* pOldBmp = (CBitmap*)bmpDC.SelectObject(&bmp); 
	bmpDC.BitBlt(0, 0, nWidth, nHeight, pDc, (int)rc.left, (int)rc.top, SRCCOPY); 

	//Retrieve the bitmap data from bmpDC and save the data to a BYTE array(pbtPixels).
	HDC hDIBDC = CreateCompatibleDC(NULL); 
	BITMAPINFO hdr; 
	ZeroMemory(&hdr , sizeof(BITMAPINFO)); 
	hdr.bmiHeader.biSize = sizeof(BITMAPINFOHEADER); 
	hdr.bmiHeader.biWidth = nWidth; 
	hdr.bmiHeader.biHeight = nHeight; 
	hdr.bmiHeader.biPlanes = 1; 
	hdr.bmiHeader.biBitCount = 32; 
	//Here we use a 32-bit bitmap. Because the structure of it is relatively fixed, and each pixel is presented by 4 bytes and no extra bytes need to be added to the trail of each line.
	//However, to a 24-bit bitmap, extra bytes need to be added to the trail of each line in order to make sure that the count of bytes of each line is an integer multiple of 4.
	//If you want to reduce the overhead and the count of bytes, you can choose to use a 24-bit bitmap. But you have to add some more codes for the extra judgment.

	BYTE * pbtPixels = NULL ; 
	HBITMAP hDIBitmap = CreateDIBSection(hDIBDC, (BITMAPINFO*)&hdr, DIB_RGB_COLORS, (void **)&pbtPixels, NULL, 0); 
	HBITMAP hOldBmp = (HBITMAP)SelectObject(hDIBDC, hDIBitmap); 
	BitBlt(hDIBDC, 0, 0, nWidth, nHeight, bmpDC.m_hDC, 0, 0, SRCCOPY); 
	SelectObject(hDIBDC, hOldBmp); 


	//Do the alpha mixing to the BYTE array(pbtPixels).
	int nPixelSize = 4 ;//Each pixel needs 4 bytes to store. 

	BYTE btSR = GetRValue(clrSrc); 
	BYTE btSG = GetGValue(clrSrc); 
	BYTE btSB = GetBValue(clrSrc); 

	for(int i = 0; i < nHeight; i ++) 
	{
		for(int j = 0; j < nWidth; j ++) 
		{
			BYTE btB = pbtPixels[i * nWidth * nPixelSize + j * nPixelSize ];
			BYTE btG = pbtPixels[i * nWidth * nPixelSize + j * nPixelSize + 1 ];
			BYTE btR = pbtPixels[i * nWidth * nPixelSize + j * nPixelSize + 2 ];

			btB = (btSB + btB ) >> 1 ;
			btG = (btSG + btG ) >> 1 ;
			btR = (btSR + btR ) >> 1 ; 


			pbtPixels[i * nWidth * nPixelSize + j * nPixelSize ]  = btB ;
			pbtPixels[i * nWidth * nPixelSize + j * nPixelSize + 1 ] = btG ; 
			pbtPixels[i * nWidth * nPixelSize + j * nPixelSize + 2 ] = btR ; 
		}
	}

	//Draw the final semitransparent graph to the target area.
	SetDIBitsToDevice(pDc->GetSafeHdc(), (int)rc.left, (int)rc.top, nWidth, nHeight, 
		0, 0, 0, nHeight, (void*)pbtPixels, (BITMAPINFO*)&hdr, DIB_RGB_COLORS); 

	// Release the resource
	bmpDC.SelectObject(pOldBmp); 
	bmp.DeleteObject(); 
	bmpDC.DeleteDC(); 
	DeleteObject(hDIBDC); 
	DeleteObject(hDIBitmap); 
	pDc->RestoreDC(nSavedDC);//Restore the original DC. 
	pbtPixels = NULL; 
}

void CPDFDisplayView::DrawSelectText(CDC* pDc, FSCRT_RECTF rc)
{
	FSCRT_MATRIX matrix ;
	IReader_Page* page = m_pReaderDocument->GetPage(m_nTextSelectPage);
	if (GetPageMatrix(page->GetPageIndex(), &matrix))
	{
		FSCRT_Matrix_TransformRectF(&matrix, &rc);
	}
	else
		return;
	COLORREF clrSrc = RGB(200, 500, 106);

	rc.top -= 3 *m_fZoom;
	rc.left -= 2 *m_fZoom;
	rc.bottom += 2*m_fZoom;
	
	int nSavedDC = pDc->SaveDC();//Save the original DC.

	int nWidth = (int)abs(rc.right-rc.left+2);//The width of the semitransparent zone.
	int nHeight = (int)abs(rc.top-rc.bottom);//The height of the semitransparent zone.

	//Copy the screen: create a new DC to copy the target area. 
	CDC bmpDC; 
	bmpDC.CreateCompatibleDC(pDc); 

	//Create a base bitmap
	CBitmap bmp ; 
	bmp.CreateCompatibleBitmap(pDc , nWidth , nHeight); 
	CBitmap* pOldBmp = (CBitmap*)bmpDC.SelectObject(&bmp); 
	bmpDC.BitBlt(0 , 0 , nWidth , nHeight , pDc , (int)rc.left , (int)rc.top , SRCCOPY); 

	//Retrieve the bitmap data from bmpDC and save the data to a BYTE array(pbtPixels).
	HDC hDIBDC = CreateCompatibleDC(NULL); 
	BITMAPINFO hdr; 
	ZeroMemory(&hdr , sizeof(BITMAPINFO)); 
	hdr.bmiHeader.biSize   = sizeof(BITMAPINFOHEADER); 
	hdr.bmiHeader.biWidth  = nWidth; 
	hdr.bmiHeader.biHeight  = nHeight; 
	hdr.bmiHeader.biPlanes  = 1; 
	hdr.bmiHeader.biBitCount = 32; 
	//Here we use a 32-bit bitmap. Because the structure of it is relatively fixed, and each pixel is presented by 4 bytes and no extra bytes need to be added to the trail of each line.
	//However, to a 24-bit bitmap, extra bytes need to be added to the trail of each line in order to make sure that the count of bytes of each line is an integer multiple of 4.
	//If you want to reduce the overhead and the count of bytes, you can choose to use a 24-bit bitmap. But you have to add some more codes for the extra judgment.

	BYTE * pbtPixels = NULL ; 
	HBITMAP hDIBitmap = CreateDIBSection(hDIBDC, (BITMAPINFO  *)&hdr, DIB_RGB_COLORS, (void **)&pbtPixels, NULL, 0); 
	HBITMAP hOldBmp = (HBITMAP)SelectObject(hDIBDC, hDIBitmap); 
	BitBlt(hDIBDC, 0, 0, nWidth, nHeight, bmpDC.m_hDC, 0, 0, SRCCOPY); 
	SelectObject(hDIBDC, hOldBmp); 


	//Do the alpha mixing to the BYTE array(pbtPixels).
	int nPixelSize = 4 ;//Each pixel needs 4 bytes to store. 

	BYTE btSR = GetRValue(clrSrc); 
	BYTE btSG = GetGValue(clrSrc); 
	BYTE btSB = GetBValue(clrSrc); 

	for(int i = 0 ; i < nHeight ; i ++) 
	{
		for(int j = 0 ;j < nWidth ; j ++) 
		{

			BYTE btB = pbtPixels[i * nWidth * nPixelSize + j * nPixelSize ] ; 
			BYTE btG = pbtPixels[i * nWidth * nPixelSize + j * nPixelSize + 1 ] ;
			BYTE btR = pbtPixels[i * nWidth * nPixelSize + j * nPixelSize + 2 ];

			btB = (btSB + btB ) >> 1 ;
			btG = (btSG + btG ) >> 1 ;
			btR = (btSR + btR ) >> 1 ; 


			pbtPixels[i * nWidth * nPixelSize + j * nPixelSize ]  = btB ;
			pbtPixels[i * nWidth * nPixelSize + j * nPixelSize + 1 ] = btG ; 
			pbtPixels[i * nWidth * nPixelSize + j * nPixelSize + 2 ] = btR ; 
		}
	}

	//Draw the final semitransparent graph to the target area.
	SetDIBitsToDevice(pDc->GetSafeHdc(), (int)rc.left, (int)rc.top, nWidth, nHeight, 
		0, 0, 0, nHeight, (void*)pbtPixels, (BITMAPINFO*)&hdr, DIB_RGB_COLORS); 

	// Release the resource
	bmpDC.SelectObject(pOldBmp); 
	bmp.DeleteObject(); 
	bmpDC.DeleteDC(); 
	DeleteObject(hDIBDC); 
	DeleteObject(hDIBitmap); 
	pDc->RestoreDC(nSavedDC);//Restore the original DC. 
	pbtPixels = NULL; 
}

void CPDFDisplayView::OnSelect()
{
	ChangeTool(TEXTSELECT_TOOL);
}

BOOL CPDFDisplayView::GetStartCharIndexByPoint(CPoint pt, int& index)
{
	int indexTmp = 0;
	if(!GetCharIndexByPoint(pt, indexTmp))
		return FALSE;

	FSPDF_TEXTPAGE_CHARINFO info;
	FS_RESULT ret = FSPDF_TextPage_GetCharInfo(m_pdfPage->GetPDFTextPage(), indexTmp, &info);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		// need to add error handling for ret value without FSCRT_ERRCODE_SUCCESS
		return FALSE;
	}

	FSCRT_MATRIX pageMatrix,deviceMatrix;
	float page_x = pt.x , page_y = pt.y;
	GetPageMatrix(m_nTextSelectPage, &pageMatrix);
	FSCRT_Matrix_GetReverse(&pageMatrix, &deviceMatrix);
	FSCRT_Matrix_TransformPointF(&deviceMatrix, &page_x, &page_y);

	float positionTmp = 0;

	if (m_nEndCharIndex - m_nStartCharIndex >= 0)
	{
		if(page_x > (info.bbox.left + info.bbox.right)/2)
			index = indexTmp + 1;
		else
			index = indexTmp;
	}
	else
	{
		if(page_x > (info.bbox.left + info.bbox.right)/2)
			index = indexTmp;
		else
			index = indexTmp - 1;
	}


	return TRUE;
}

BOOL CPDFDisplayView::GetCharIndexByPoint(CPoint pt, int& index)
{
	FS_FLOAT page_x = (FS_FLOAT)pt.x;
	FS_FLOAT page_y = (FS_FLOAT)pt.y;
	FSCRT_MATRIX pageMatrix;
	FSCRT_MATRIX deviceMatrix;
	BOOL bret = GetPageMatrix(m_nTextSelectPage, &pageMatrix);
	FS_RESULT ret = -1;
	FS_INT32 charIndex = -1;
	FSCRT_Matrix_GetReverse(&pageMatrix, &deviceMatrix);
	FSCRT_Matrix_TransformPointF(&deviceMatrix, &page_x, &page_y);
	if (m_pdfPage->GetPDFTextPage())
	{
		ret = FSPDF_TextPage_GetCharIndexAtPos(m_pdfPage->GetPDFTextPage(), page_x, page_y, 10.0f, &charIndex);
	}
	
	if (-1 == charIndex)
	{
		return FALSE;
	}
	index = charIndex;
	return TRUE;
}

void CPDFDisplayView::GetSelectRect(int starIndex, int endIndex)
{
	int temp, nCount;
	FS_RESULT ret = -1;
	nCount = endIndex - starIndex;
	if (starIndex == endIndex )
	{
		m_rSelectPage.RemoveAll();
		return;
	}
	
	if (nCount < 0)
	{
		temp = endIndex;
		endIndex = starIndex;
		starIndex = temp;

		nCount = abs(nCount);
	}
	
	nCount++;

	if(!m_pdfPage->GetPDFTextPage())
	{
		m_pdfPage->LoadPDFTextPage();
	}
	ret = FSPDF_TextPage_SelectByRange(m_pdfPage->GetPDFTextPage(), starIndex, nCount, &m_pTextSelection);

	m_rSelectPage.RemoveAll();
	int num = -1;
	ret = FSPDF_TextSelection_CountPieces(m_pTextSelection, &num);
	FSCRT_RECTF rect;
	FSCRT_RECTF lastRect;
	for (int i = 0; i<num; i++)
	{
		ret = FSPDF_TextSelection_GetPieceRect(m_pTextSelection, i, &rect);

/*		if (0 != i)
		{
			FS_FLOAT nHeight = 0; 
			if (lastRect.top - lastRect.bottom > rect.top - rect.bottom)
			{
				nHeight = lastRect.top - lastRect.bottom;
			}
			else
			{
				nHeight = rect.top - rect.bottom;
			}

	
			if ( lastRect.top - rect.top < nHeight)
			{
				rect.left = lastRect.left;
				if (rect.bottom > lastRect.bottom )
				{
					rect.bottom = lastRect.bottom;
				}

				if (rect.top < lastRect.top )
				{
					rect.top = lastRect.top;
				}
				if (!m_rSelectPage.IsEmpty())
				{
					INT_PTR index = m_rSelectPage.GetCount();
					m_rSelectPage.RemoveAt(index-1);
				}
 			}
 
 		}
 		lastRect = rect;*/
 		m_rSelectPage.Add(rect);
 	}
}

void CPDFDisplayView::CreateCaret(CPoint pt)
{
	int nIndex = 0;
	int nAcent = 0, nDecent = 0;
	FS_RESULT ret = -1;
	FSPDF_TEXTPAGE_CHARINFO info;

	// info need to be initialized
	memset(&info,0,sizeof(FSPDF_TEXTPAGE_CHARINFO));

	if(!GetCharIndexByPoint(pt, nIndex))
		return;
	
	ret = FSPDF_TextPage_GetCharInfo(m_pdfPage->GetPDFTextPage(), nIndex, &info);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		// need to add error handling for ret value without FSCRT_ERRCODE_SUCCESS
		return;
	}
	ret = FSCRT_Font_GetAscent(info.font, &nAcent);
	ret = FSCRT_Font_GetDescent(info.font, &nDecent);
	nAcent = (FS_INT32)((nAcent * info.fontSize / 1000.0f) * info.matrix.d + info.originY + 0.5f);
	nDecent = (FS_INT32)((nDecent * info.fontSize / 1000.0f) * info.matrix.d + info.originY + 0.5f);

	FSCRT_MATRIX pageMatrix,deviceMatrix;
	float page_x = pt.x , page_y = pt.y;
	GetPageMatrix(m_nTextSelectPage, &pageMatrix);
	FSCRT_Matrix_GetReverse(&pageMatrix, &deviceMatrix);
	FSCRT_Matrix_TransformPointF(&deviceMatrix, &page_x, &page_y);

	float positionTmp = 0;

	if(page_x > (info.bbox.left + info.bbox.right)/2)
		positionTmp = info.bbox.right;
	else
		positionTmp = info.originX;
		
	FSCRT_RECTF rect;
	rect.left = positionTmp;
	rect.top = (FS_FLOAT)nAcent;
	rect.right = positionTmp;
	rect.bottom = (FS_FLOAT)nDecent;
	m_rCaret = rect;

	FSCRT_MATRIX matrix;
	GetPageMatrix(m_nTextSelectPage, &matrix);
	FSCRT_Matrix_TransformRectF(&matrix, &rect);
	if(m_nRotate == 1 || m_nRotate == 3)
		this->CreateSolidCaret((int)abs(rect.right - rect.left), 2);
	else
		this->CreateSolidCaret(2, (int)abs(rect.bottom - rect.top));
	
	this->SetCaretPos(CPoint((int)rect.left, (int)rect.top));
	this->ShowCaret();
}

void CPDFDisplayView::OnPsi()
{
	if (m_bPSI)
	{
		m_bPSI = FALSE;
		ChangeTool(HAND_TOOL);
	}
	else
	{
		m_bPSI = TRUE;
		ChangeTool(PSI_TOOL);
	}	
}

void CPDFDisplayView::OnAnnotSelect()
{
	if (m_bAnnotTool)
	{
		m_bAnnotTool = FALSE;
		ChangeTool(HAND_TOOL);
	}
	else
	{
		m_bAnnotTool = TRUE;
		ChangeTool(ANNOT_TOOL);
	}	
}

void CPDFDisplayView::InitPSI()
{
	if (m_hRenderer != NULL)
	{
		FSCRT_Renderer_Release(m_hRenderer);
		m_hRenderer = NULL;
	}
	if (m_pBitmapInfo != NULL)
	{
		FSCRT_Memory_Free(m_pBitmapInfo);
		m_pBitmapInfo = NULL;
	}
	if (m_hBitmap != NULL)
	{
		FSCRT_Bitmap_Release(m_hBitmap);
		m_hBitmap = NULL;
	}
	if(m_hPSI != NULL)
	{
		FSCRT_PSI_Release(m_hPSI);
		m_hPSI = NULL;
	}
	//init psi
	FS_INT32 nWidth = (FS_INT32)abs(m_rPSIViewRect.right);
	FS_INT32 nHeight = (FS_INT32)abs(m_rPSIViewRect.bottom - m_rPSIViewRect.top);
	if (m_rPSIViewRect.top > 0)
	{
		nHeight = (FS_INT32)abs(m_rPSIViewRect.bottom);
	}
	FS_RESULT ret = FSCRT_PSI_Create(0, &m_hPSI);
	if (FSCRT_ERRCODE_INVALIDLICENSE == ret)
	{
		AfxMessageBox(_T("You don't have the right of \"FSCRT_PSI_Create\" Functions, please purchase \"PSI Add-on\" Modules!"));
		m_hPSI = NULL;
		return;
	}
	ret = FSCRT_PSI_InitCanvas(m_hPSI, (FS_FLOAT)nWidth, (FS_FLOAT)nHeight);
	m_dInkColor = 0xff0000ff;
	ret = FSCRT_PSI_SetInkColor(m_hPSI, m_dInkColor);
	float temInkDia = 3.0f * m_fZoom;

	m_nInkDia = (temInkDia > 0.0) ? (FS_INT32)floor(temInkDia + 0.5) : (FS_INT32)ceil(temInkDia - 0.5);  
	if (m_nInkDia < 1)
	{
		m_nInkDia = 1;
	}
	ret = FSCRT_PSI_SetInkDiameter(m_hPSI, m_nInkDia);
	m_fOpacity = 1.0f;
	ret = FSCRT_PSI_SetOpacity(m_hPSI, m_fOpacity);
	ret = FSCRT_Bitmap_Create(nWidth, nHeight, FSCRT_BITMAPFORMAT_32BPP_BGRx, NULL, 0, &m_hBitmap);
	FSCRT_RECT rect = {0, 0, nWidth, nHeight};
	ret = FSCRT_Bitmap_FillRect(m_hBitmap, 0xFFFFFFFF, &rect);

	FS_DWORD infoSize = 0;
	ret = FSCRT_Bitmap_GetBitmapInfo(m_hBitmap, NULL, &infoSize);
	ASSERT(FSCRT_ERRCODE_SUCCESS == ret);
	ret = FSCRT_Memory_Alloc(infoSize, (FS_LPVOID*)&m_pBitmapInfo);
	ASSERT(FSCRT_ERRCODE_SUCCESS == ret);
	ret = FSCRT_Bitmap_GetBitmapInfo(m_hBitmap, m_pBitmapInfo, &infoSize);
	ASSERT(ret == FSCRT_ERRCODE_SUCCESS);
	ret = FSCRT_Renderer_CreateOnBitmap(m_hBitmap, &m_hRenderer);
}

void CPDFDisplayView::ReleasePSI()
{
	if (m_hRenderer != NULL)
	{
		FSCRT_Renderer_Release(m_hRenderer);
		m_hRenderer = NULL;
	}
	if (m_pBitmapInfo != NULL)
	{
		FSCRT_Memory_Free(m_pBitmapInfo);
		m_pBitmapInfo = NULL;
	}
	if (m_hBitmap != NULL)
	{
		FSCRT_Bitmap_Release(m_hBitmap);
		m_hBitmap = NULL;
	}
	if(m_hPSI != NULL)
	{
		FSCRT_PSI_Release(m_hPSI);
		m_hPSI = NULL;
	}
}

void CPDFDisplayView::SetViewRect(CPoint point)
{
	vector<PAGETODISPLAY>::iterator i;
	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(point);

	for(i = m_vecPageToDisplay.begin(); i != m_vecPageToDisplay.end(); i++)
	{
		CReader_Page* pTempReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(i->index);
		if (pReaderPage != pTempReaderPage)
			continue;

		float width = 0.0f, height = 0.0f;
		if(0 == m_nRotate || 2 == m_nRotate)
		{
			height = pReaderPage->GetPageHeight() * m_fZoom;
			width = pReaderPage->GetPageWidth() * m_fZoom;
		}
		else
		{
			height = pReaderPage->GetPageWidth() * m_fZoom;
			width = pReaderPage->GetPageHeight() * m_fZoom;
		}
		m_rPSIViewRect.left = (FS_FLOAT)i->cx;
		m_rPSIViewRect.top = i->cy * m_fZoom;
		m_rPSIViewRect.right = i->cx + width;
		m_rPSIViewRect.bottom = i->cy * m_fZoom + height;
	}
}

FS_RESULT CPDFDisplayView::PsiFlush(const FSCRT_RECTF* rect)
{
	if (rect->bottom <= rect->top || rect->left >= rect->right)
		return FSCRT_ERRCODE_SUCCESS;

	if (NULL == m_bPSI || NULL == m_hPSI)
		return FSCRT_ERRCODE_SUCCESS;

	FSCRT_RECT rt;
	rt.left = (FS_INT32)(rect->left + m_rPSIViewRect.left);
	rt.right = (FS_INT32)(rect->right + m_rPSIViewRect.left);
	rt.top = (FS_INT32)(rect->top + m_rPSIViewRect.top);
	rt.bottom = (FS_INT32)(rect->bottom + m_rPSIViewRect.top);

	// Flush the pdf page
	FS_RESULT ret = FSCRT_Renderer_DrawBitmap(m_hRenderer, rt.left, rt.top, m_hBitmap, (FSCRT_RECT*)rect);
	FS_INT32 nWidth = rt.right - rt.left;
	FS_INT32 nHeight = rt.bottom - rt.top;
	
	ret = FSCRT_PSI_Render(m_hPSI, m_hRenderer, rt.left, rt.top, nWidth, nHeight, rect->left, rect->top);

	InvalidateRect((LPRECT)&rt);
	return ret;
}

void CPDFDisplayView::PsiToAnnotation(CPoint& point)
{
	if (!m_pReaderDocument)
		return;

	CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(point);
	if (!pReaderPage)
	{
		return;
	}
	FSCRT_PAGE hPdfpage = pReaderPage->GetPage();
	FSCRT_MATRIX matrix, pagematrix;
	FSPDF_Page_GetMatrix(hPdfpage, 0, 0, (FS_INT32)(m_rPSIViewRect.right-m_rPSIViewRect.left), (FS_INT32)(m_rPSIViewRect.bottom-m_rPSIViewRect.top), m_nRotate, &matrix);
	FSCRT_Matrix_GetReverse(&matrix, &pagematrix);
	FSCRT_RECTF conrect;
	FSCRT_PSI_GetContentsRect(m_hPSI, &conrect);
	FSCRT_RECTF annotrect = conrect;
	FSCRT_Matrix_TransformRectF(&pagematrix, &annotrect);

	FSCRT_RECTF psirect = {0, 0, m_rPSIViewRect.right - m_rPSIViewRect.left, m_rPSIViewRect.bottom - m_rPSIViewRect.top};
	FSCRT_ANNOT annot = NULL;
	FSCRT_PSI_ConvertToPDFAnnot(m_hPSI, &psirect, hPdfpage, &annotrect, &annot);
	FS_RESULT ret = FSCRT_Bitmap_FillRect(m_hBitmap, 0xFFFFFFFF, NULL);
	m_pReaderDocument->SetChangeMark();
}

void CPDFDisplayView::GetBox(FS_INT32 areaType)
{
	if (NULL == m_pReaderDocument)
	{
		return;
	}
	m_nBoxPage = m_nCurPageIndex;
	CReader_Page* page = (CReader_Page*)m_pReaderDocument->GetPage(m_nBoxPage);
	FSCRT_PAGE pdfpage = page->GetPage();
	FS_RESULT ret = -1;
	ret = FSPDF_Page_GetBox(pdfpage, areaType, &m_rPageArea);
	m_bDrawBox = TRUE;
	Invalidate();
}

void CPDFDisplayView::DrawBox(CDC* pDc, FSCRT_RECTF rect, COLORREF color)
{

	FSCRT_MATRIX matrix ;
	IReader_Page* page = m_pReaderDocument->GetPage(m_nBoxPage);
	if (GetPageMatrix(page->GetPageIndex(), &matrix))
	{
		FSCRT_Matrix_TransformRectF(&matrix, &rect);
	}
	CPen newPen(PS_SOLID, 2, color);
	CPen* oldPen = pDc->SelectObject(&newPen);
	pDc->MoveTo((int)rect.left, (int)rect.top);
	pDc->LineTo((int)rect.right, (int)rect.top);
	pDc->MoveTo((int)rect.right, (int)rect.top);
	pDc->LineTo((int)rect.right, (int)rect.bottom);
	pDc->MoveTo((int)rect.right, (int)rect.bottom);
	pDc->LineTo((int)rect.left, (int)rect.bottom);
	pDc->MoveTo((int)rect.left, (int)rect.bottom);
	pDc->LineTo((int)rect.left, (int)rect.top);
	pDc->SelectObject(oldPen);
	newPen.DeleteObject();
}

void CPDFDisplayView::ClearBox()
{
	m_bDrawBox = FALSE;
	Invalidate();
}

void CPDFDisplayView::GetContentBBox(FS_INT32 mode)
{
	if (NULL == m_pReaderDocument)
	{
		return;
	}
	m_nBoxPage = m_nCurPageIndex;
	CReader_Page* page = (CReader_Page*)m_pReaderDocument->GetPage(m_nBoxPage);
	FSCRT_PAGE pdfpage = page->GetPage();
	FS_RESULT ret = -1;
	ret = FSPDF_Page_CalcContentBBox(pdfpage, mode, &m_rPageArea);
	m_bDrawBox = TRUE;
	Invalidate();
}

void CPDFDisplayView::OnZoomIn()
{
	m_fLastZoom = m_fZoom;
	m_fZoom += 0.1f;
	if (m_fZoom >= 64.0f)
	{
		m_fZoom = 64.0f;
	}
	m_bScrollFlag = TRUE;
	m_bRatateChange = TRUE;
	Invalidate();
}

void CPDFDisplayView::OnZoomOut()
{
	m_fLastZoom = m_fZoom;
	m_fZoom -= 0.1f;
	if (m_fZoom<0.3f)
	{
		m_fZoom = 0.3f;
	}
	m_bScrollFlag = TRUE;
	m_bRatateChange = TRUE;
	Invalidate();
}

void CPDFDisplayView::OnFilePrint()
{
	m_pPDFReaderView->OnFilePrint();
}

void CPDFDisplayView::ChangeTool(Tools nIndex)
{
	if (m_bPSI)
	{
		if (PSI_TOOL != nIndex )
		{
			if (m_hPSI)
			{
				FS_FLOAT x = m_LastPoint.x - m_rPSIViewRect.left;
				FS_FLOAT y = m_LastPoint.y - m_rPSIViewRect.top;
				FS_FLOAT fPressure = (FS_FLOAT)1024 / 1024;
				FS_RESULT ret = FSCRT_PSI_AddPoint(m_hPSI, x, y, fPressure, FSCRT_PSI_PT_LINETO | FSCRT_PSI_PT_ENDPATH);
				PsiToAnnotation(m_LastPoint);
				ReleasePSI();
				m_bDrawing = FALSE;
			}
		}
	}

	m_bSelect = FALSE;
	m_bPSI = FALSE;
	m_bDrawAnnot = FALSE;
	m_bHand = FALSE;
	m_bAnnotTool = FALSE;
	m_rSelectPage.RemoveAll();
	CReader_AnnotHandlerMgr* pAnnotHandlerMgr = (CReader_AnnotHandlerMgr*)theApp.m_pReaderApp->GetAnnotHandlerMgr();	
	switch(nIndex)
	{
	case DRAW_ANNOT:	
		m_bDrawAnnot = TRUE;
		break;
	case HAND_TOOL:
		m_bHand = TRUE;
		pAnnotHandlerMgr->ValidAnnotPropertyDlg();
		break;
	case TEXTSELECT_TOOL:
		m_bSelect = TRUE;
		pAnnotHandlerMgr->ValidAnnotPropertyDlg();
		break;
	case PSI_TOOL:
		{
			HideCaret();
			m_bPSI = TRUE;
			pAnnotHandlerMgr->ValidAnnotPropertyDlg();
		}
		break;
	case ANNOT_TOOL:
		m_bAnnotTool = TRUE;
		pAnnotHandlerMgr->ValidAnnotPropertyDlg();
		break;
	default:
		HideCaret();
		pAnnotHandlerMgr->ValidAnnotPropertyDlg();
		break;
	}

	if (ANNOT_TOOL != nIndex)
	{
		m_pReaderDocument->SetCapturedAnnot(NULL);
		m_pReaderDocument->SetSelectedAnnot(NULL);
	}

	Invalidate();
}

float CPDFDisplayView::GetScale()
{
	return m_fZoom;
}

FS_INT32 CPDFDisplayView::GetRotate()
{
	return m_nRotate;
}

CPDFReaderView* CPDFDisplayView::GetPDFReaderView()
{
	return m_pPDFReaderView;
}

void CPDFDisplayView::DrawCaret(FSCRT_RECTF rc)
{
	FSCRT_MATRIX matrix;
	GetPageMatrix(m_nTextSelectPage, &matrix);
	FSCRT_Matrix_TransformRectF(&matrix, &rc);
	if(m_nRotate == 1 || m_nRotate == 3)
		this->CreateSolidCaret((int)abs(rc.right - rc.left), 2);
	else
		this->CreateSolidCaret(2, (int)abs(rc.bottom - rc.top));
	this->SetCaretPos(CPoint((int)rc.left, (int)rc.top));
	this->ShowCaret();
}

void CPDFDisplayView::OnAnnotUnderLine()
{
	ChangeTool(DRAW_ANNOT);
	m_bDrawAnnot = TRUE;
	m_pAnnotType = FSPDF_ANNOTTYPE_UNDERLINE;
}

void CPDFDisplayView::OnAnnotFreeText()
{
	ChangeTool(DRAW_ANNOT);
	m_pAnnotType = FSPDF_ANNOTTYPE_FREETEXT;
}

void CPDFDisplayView::OnAnnotLink()
{
	ChangeTool(DRAW_ANNOT);
	m_pAnnotType = FSPDF_ANNOTTYPE_LINK;
}

void CPDFDisplayView::OnAnnotNotes()
{
	ChangeTool(DRAW_ANNOT);
	//::SetCursor(AfxGetApp()->LoadStandardCursor(IDC_CROSS));
	m_pAnnotType = FSPDF_ANNOTTYPE_TEXT;
	
}

void CPDFDisplayView::OnAnnotHightLight()
{
	ChangeTool(DRAW_ANNOT);
	m_pAnnotType = FSPDF_ANNOTTYPE_HIGHLIGHT;
}

HBRUSH CPDFDisplayView::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
	// TODO: Change any attributes of the DC here
	if (pWnd-> GetDlgCtrlID() == IDC_ANNOT_TEXT_EDIT)
	{
		IReader_Annot* pAnnot = m_pReaderDocument->GetSelectedAnnot();
		if (pAnnot)
		{
			FS_ARGB BKArgb;
			FS_ARGB TextArgb;
			COLORREF Textclr = 0;
			COLORREF Bkclr = 0;
			HBRUSH hBrush = NULL;
			FSPDF_DEFAULTAPPEARANCE AnnotDefAp;

			FS_RESULT ret = FSPDF_Annot_GetColor(pAnnot->GetPDFAnnot(), TRUE, &BKArgb);
			if(FSCRT_ERRCODE_SUCCESS == ret)
			{
				COLORREF Bkclr = RGB((BKArgb >>16) &0xff, (BKArgb >>8) &0xff, (BKArgb) &0xff);
				pDC-> SetBkColor(Bkclr);
				if (NULL != m_brMine)
				{
					DeleteObject(m_brMine);
				}
				m_brMine = ::CreateSolidBrush(Bkclr); 
				//ΪԼرɫӦˢӾ 
			}
			else
			{
				hBrush = CView::OnCtlColor(pDC, pWnd, nCtlColor);
			}

			ret	 = FSPDF_Annot_GetDefaultAppearance(pAnnot->GetPDFAnnot(), &AnnotDefAp);
			if (FSCRT_ERRCODE_SUCCESS == ret)
			{
				TextArgb = AnnotDefAp.textColor;
				Textclr = RGB((TextArgb >>16) &0xff, (TextArgb >>8) &0xff, (TextArgb) &0xff);
				pDC-> SetTextColor(Textclr);
			}

			if (NULL != hBrush)
				return hBrush;
			return m_brMine; 
		}
		else
		{
			return NULL;
		}
	}
	else
	{
		HBRUSH hbr = CView::OnCtlColor(pDC, pWnd, nCtlColor);
		return hbr;
	}

	// TODO: Return a different brush if the default is not desired

}

void CPDFDisplayView::OnAnnotAnnotTextEditKillFocus()
{
	IReader_Annot* pAnnot = m_pReaderDocument->GetSelectedAnnot();

	CString str;
	m_pEdit->GetWindowTextW(str);
	if (pAnnot && str != m_csFreeText)
	{
		FSCRT_BSTR Contexts;
		FSCRT_BStr_Init(&Contexts);

		FS_RESULT ret = FSCRT_UTF8_FromUTF16LE(&Contexts, (FS_WORD*)(LPCWSTR)(str.GetBuffer()), (FS_DWORD)str.GetLength());
		ret = FSPDF_Annot_SetContents(pAnnot->GetPDFAnnot(), &Contexts);
		
		if (FSCRT_ERRCODE_SUCCESS == ret)
		{
			ret = FSPDF_Annot_ResetAppearance(pAnnot->GetPDFAnnot());
			CWnd* pWnd = CWnd::FromHandle(m_pReaderDocument->GetDocViewHwnd());
			CRect winRect = pAnnot->GetViewBBox(); //pAnnot->GetViewBBox();pAnnot->GetInnerRectMargin();
			pWnd->InvalidateRect(winRect);
			m_pReaderDocument->SetChangeMark();
		}
		FSCRT_BStr_Clear(&Contexts);
	}
	else if (pAnnot)
	{
		CWnd* pWnd = CWnd::FromHandle(m_pReaderDocument->GetDocViewHwnd());
		CRect winRect = pAnnot->GetViewBBox();
		pWnd->InvalidateRect(winRect);
	}
	m_pEdit->ShowWindow(SW_HIDE);
	m_bAnnotIsEdit = FALSE;
}

void CPDFDisplayView::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	IReader_Document* pReaderDocument = theApp.m_pReaderApp->GetCurrentDocument();
	if(!pReaderDocument)
	{
		pCmdUI->Enable(FALSE);
	}
}

FS_BOOL CPDFDisplayView::IsInPageRect(CPoint point)
{
	CReader_Page* pPage = GetPage(point);
	if (!pPage)
	{
		return FALSE;
	}
	FSCRT_RECTF PageBox ;
	FSPDF_Page_GetBox(pPage->GetPage(), FSPDF_PAGEBOX_MEDIABOX, &PageBox);

	FSCRT_MATRIX matrix ;
	if (GetPageMatrix(pPage->GetPageIndex(), &matrix))
	{
		FSCRT_Matrix_TransformRectF(&matrix, &PageBox);
	}
	
	if (point.x > PageBox.left && point.x < PageBox.right && point.y > PageBox.top && point.y < PageBox.bottom)
	{
		return TRUE;
	}
	return FALSE;
}

VOID CPDFDisplayView::RenderRefresh()
{
	CWinApp *pApp = AfxGetApp(); 
	POSITION p = pApp->GetFirstDocTemplatePosition();
	while (p) 
	{
		CDocTemplate *pDocTem = pApp->GetNextDocTemplate(p);
		POSITION p1 = pDocTem->GetFirstDocPosition();
		while (p1)
		{
			CDocument *pDoc = pDocTem->GetNextDoc(p1);
			POSITION p2 = pDoc->GetFirstViewPosition(); 
			while (p2) 
			{
				CPDFReaderView* pView = (CPDFReaderView*)pDoc->GetNextView(p2);
				CPDFDisplayView* pDisplayView = pView->GetPDFDisplayView();
				if (pDisplayView)
				{
					pDisplayView->m_bRenderOption = TRUE; 
				}
			}
		}
	}
	Invalidate();
}

VOID CPDFDisplayView::ReleasePageTemData(FS_INT32 iPageStart, FS_INT32 iPgaeEnd)
{
	if(!m_vecPageToDisplay.empty())
	{
		vector<PAGETODISPLAY>::iterator i;
		for(i = m_vecPageToDisplay.begin(); i!= m_vecPageToDisplay.end(); i++)
		{
			if (i->index < iPageStart || i->index > iPgaeEnd)
			{
				CReader_Page* pReaderPage = (CReader_Page*)m_pReaderDocument->GetPage(i->index);
				pReaderPage->ReleaseTempData();
			}	
		}
	}
}

BOOL CPDFDisplayView::IsVisiblePage(FS_INT32 index)
{
	if(!m_vecPageToDisplay.empty())
	{		
		vector<PAGETODISPLAY>::iterator i;
		for(i = m_vecPageToDisplay.begin(); i!= m_vecPageToDisplay.end(); i++)
		{
			if (i->index == index)
			{
				return TRUE;
			}
		}
	}
	return FALSE;
}

FS_INT32 CPDFDisplayView::GetCurVisiblePageIndex()
{
	return m_nCurPageIndex;
}

void CPDFDisplayView::ShowTextSelectMenu(CPoint point)
{
	FSCRT_RECTF bbox;
	CRect rcWindow;
	if(NULL == m_pTextSelection)
		return;
	FSPDF_TextSelection_GetBBox(m_pTextSelection,&bbox);
	m_pdfPage->DocToWindow(bbox,rcWindow);
	rcWindow.NormalizeRect();
	if (rcWindow.PtInRect(point) )
	{
		if (m_rSelectPage.IsEmpty())
		{
			return;
		}

		CMenu menu;
		menu.CreatePopupMenu();
		menu.AppendMenu(MF_STRING,1,_T("Copy To Clipbord"));
		CPoint pt= point;

		CWnd* pwnd = CWnd::FromHandle(this->GetSafeHwnd()) ;
		pwnd->ClientToScreen(&pt);
		int ret=menu.TrackPopupMenu(TPM_LEFTALIGN|TPM_RETURNCMD,pt.x,pt.y,pwnd);
		if (ret)
		{
			FSCRT_BSTR SelectText;
			FSCRT_BStr_Init(&SelectText);
			FSPDF_TextSelection_GetChars(m_pTextSelection,&SelectText);
		
			SetClipboardData(SelectText);
			FSCRT_BStr_Clear(&SelectText);
		}
		menu.DestroyMenu();
	}
	else
	{
		m_rSelectPage.RemoveAll();
		Invalidate();
	}

}

void	CPDFDisplayView::SetClipboardData(FSCRT_BSTR csText)
{
	//m_pReaderDocument->GetApp()->OnDidCopy(m_pReaderDocument);
	::OpenClipboard(NULL);
	::EmptyClipboard();

	FS_WORD* buffer = NULL;
	FS_DWORD bufLen = 0;
	FS_RESULT ret = FSCRT_UTF8_ToUTF16LE(&csText, buffer, &bufLen);
	if (FSCRT_ERRCODE_SUCCESS != ret || bufLen == 0) return;
	buffer = new FS_WORD[bufLen+1];
	memset(buffer, 0, bufLen+1);
	ret = FSCRT_UTF8_ToUTF16LE(&csText, buffer, &bufLen);
	buffer[bufLen] = 0;
	if ( FSCRT_ERRCODE_SUCCESS != ret)
	{
		delete []buffer;
		buffer = NULL;
		return;
	}

	if (IsSupportUnicodeSel())
	{
	
		HGLOBAL hMem;
		hMem = ::GlobalAlloc(0, bufLen*2+32);    

		wchar_t* pMem = (wchar_t*)::GlobalLock(hMem);
		memset(pMem,0,bufLen*2+32);

		if(pMem==NULL) {
			AfxMessageBox(_T("Can not open Clipboard")); 
			delete[] buffer;
			buffer = NULL;
			return ;
		}	

		wcscpy(pMem,(LPCWSTR)buffer);    

		::GlobalUnlock(hMem);
		DWORD LastError = GetLastError();

		HANDLE hh = ::SetClipboardData(CF_UNICODETEXT, hMem);
	}
	else
	{

		HGLOBAL hMem;
		hMem = ::GlobalAlloc(0, csText.len+32);    

		char* pMem = (char*)::GlobalLock(hMem);
		memset(pMem,0,csText.len+32);

		if(pMem==NULL) {
			AfxMessageBox(_T("Can not open Clipboard")); 
			delete[] buffer;
			buffer = NULL;
			return ;
		}	

		strcpy(pMem,csText.str);    

		::GlobalUnlock(hMem);
		DWORD ret = GetLastError();
		HANDLE hh = ::SetClipboardData(CF_TEXT, hMem);
	}

	CloseClipboard();
	delete[] buffer;
	buffer = NULL;
}

BOOL CPDFDisplayView::IsSupportUnicodeSel()
{
	if (m_bSupportUnicode == NOT_CHK_SUPPORT)
	{
		DWORD dwVersion=::GetVersion();
		DWORD dwWindowsMajorVersion =  (DWORD)(LOBYTE(LOWORD(dwVersion)));
		DWORD dwWindowsMinorVersion =  (DWORD)(HIBYTE(LOWORD(dwVersion)));

		if (dwVersion >= 0x80000000)
			m_bSupportUnicode = NOT_SUPPORT_UNICODE;
		else 
			m_bSupportUnicode = SUPPORT_UNICODE;
	}

	return m_bSupportUnicode;
}

void CPDFDisplayView::OnRotationLeft()
{
	// TODO: Add your command handler code here
	if (NULL == m_pReaderDocument)
		return;
	m_nRotate = m_nRotate - 1 < 0 ? 3 : m_nRotate - 1;
	m_bRatateChange = TRUE;
	m_bScrollFlag = TRUE;
	Invalidate();
}


void CPDFDisplayView::OnRotationRight()
{
	// TODO: Add your command handler code here
	if (NULL == m_pReaderDocument)
		return;
	m_nRotate = m_nRotate + 1 > 3 ? 0 : m_nRotate + 1;
	m_bRatateChange = TRUE;
	m_bScrollFlag = TRUE;
	Invalidate();
}

void CPDFDisplayView::KillFormfillerFocus()
{
	if (NULL == m_pReaderDocument)
		return;
	FSCRT_MATRIX pageMatrix;
	GetPageMatrix(m_nCurPageIndex, &pageMatrix);
	IReader_Page* readPage = m_pReaderDocument->GetPage(m_nCurPageIndex);
	FSCRT_PAGE page = NULL;
	if(readPage)
		page = readPage->GetPage();
	FSPDF_FormFiller_TriggerWindowlessEvent(m_pReaderDocument->GetFormFiller(), page, &pageMatrix, FSCRT_EVENT_KILLFOCUS, NULL);
}

void CPDFDisplayView::OnTimer(UINT_PTR nIDEvent)
{
	// TODO: Add your message handler code here and/or call default
	if(this->m_pReaderDocument == (CReader_Document*)nIDEvent && m_lpTimerFunc)
		m_lpTimerFunc(nIDEvent);

	CView::OnTimer(nIDEvent);
}

FS_INT32 CPDFDisplayView::CreateTimer(FS_LPVOID clientData,  FS_INT32 uElapse, FSCRT_CALLBACK_TIMER lpTimerFunc)
{
	m_lpTimerFunc = lpTimerFunc;
	SetTimer((UINT_PTR)clientData, uElapse, NULL);
	return 0;
}

void CPDFDisplayView::DestroyTime(FS_INT32 nTimerID)
{
	KillTimer((UINT_PTR)nTimerID);
}
