
// PDFReaderView.cpp : implementation of the CPDFReaderView class
//

#include "stdafx.h"
// SHARED_HANDLERS can be defined in an ATL project implementing preview, thumbnail
// and search filter handlers and allows sharing of document code with that project.
#ifndef SHARED_HANDLERS
#include "PDFReader.h"
#endif

#include "PDFReaderDoc.h"
#include "PDFReaderView.h"
#include "PDFDisplayView.h"
#include "ChildFrm.h"
#include "./FrameworkImpl/PDFReaderApp.h"
#include "./FrameworkImpl/PDFReaderDocument.h"
#include "./FrameworkImpl/PDFReaderPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CPDFReaderView

IMPLEMENT_DYNCREATE(CPDFReaderView, CView)

BEGIN_MESSAGE_MAP(CPDFReaderView, CView)
	ON_WM_SIZE()
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, &OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT, &CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, &CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, &CView::OnFilePrintPreview)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_CREATE()
END_MESSAGE_MAP()

// CPDFReaderView construction/destruction

CPDFReaderView::CPDFReaderView()
{
	// TODO: add construction code here
	m_pPDFDisplayView = NULL;
}

CPDFReaderView::~CPDFReaderView()
{
	
}

BOOL CPDFReaderView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	// the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

// CPDFReaderView drawing

void CPDFReaderView::OnDraw(CDC* pDC)
{
	CPDFReaderDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if (!pDoc)
		return;

	// TODO: add draw code for native data here
	CRect rt;
	this->GetClientRect(&rt);
	pDC->FillSolidRect(&rt, RGB(171, 171, 171));
}


// CPDFReaderView printing

BOOL CPDFReaderView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CPDFReaderView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CPDFReaderView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


// CPDFReaderView diagnostics

#ifdef _DEBUG
void CPDFReaderView::AssertValid() const
{
	CView::AssertValid();
}

void CPDFReaderView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CPDFReaderDoc* CPDFReaderView::GetDocument() const // non-debug version is inline
{
	if (!m_pDocument)
	{
		return NULL;
	}
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CPDFReaderDoc)));
	return (CPDFReaderDoc*)m_pDocument;
}
#endif //_DEBUG

void CPDFReaderView::OnInitialUpdate() 
{	
	CView::OnInitialUpdate();
	CReader_App* pReaderApp = theApp.m_pReaderApp;
	CPDFReaderDoc* pPDFReaderDoc = this->GetDocument();
	//pReaderApp->SetCurrentDocument((IReader_Document*)pPDFReaderDoc->GetReaderDocument());
	m_pDocument = pPDFReaderDoc;
	if (m_pPDFDisplayView)
	{
		delete m_pPDFDisplayView;
		m_pPDFDisplayView = NULL;
	}
	m_pPDFDisplayView = new CPDFDisplayView; 	
	
	CRect rc;
	this->GetClientRect(&rc);
	
	m_pPDFDisplayView->Create(NULL, L"PDFDisplayView", WS_CHILD|WS_VISIBLE|
		WS_TABSTOP, rc, this, ID_VIEW_PREVIEW);
	m_pPDFDisplayView->SetPDFReaderView(this);

	CWnd* pFrame = this;
	while (pFrame) 
	{		
		if( pFrame->IsKindOf(RUNTIME_CLASS(CChildFrame)))
		{
			((CChildFrame*)pFrame)->AddPagesToDockWindow(this->GetDocument());
			break; 
		}
		pFrame = pFrame->GetParent();
	}

	CReader_Document* pReaderDocument = pPDFReaderDoc->GetReaderDocument();
	pReaderDocument->SetPDFDisplayView(m_pPDFDisplayView);
	
	((CFrameWnd*)pFrame)->SetActiveView(m_pPDFDisplayView);
}

// CPDFReaderView message handlers
void CPDFReaderView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	if(m_pPDFDisplayView )
		m_pPDFDisplayView->MoveWindow(0, 0, cx, cy, TRUE);
}

void CPDFReaderView::OnDestroy() 
{
	if(m_pPDFDisplayView)
	{
		delete m_pPDFDisplayView;
		m_pPDFDisplayView = NULL;
	}

	CView::OnDestroy();
}

CPDFDisplayView* CPDFReaderView::GetPDFDisplayView()
{
	return m_pPDFDisplayView;
}

void CPDFReaderView::OnFilePrint() 
{
	CPDFReaderDoc* pPDFReaderDoc = (CPDFReaderDoc*)GetDocument();
	if (!pPDFReaderDoc)
	{
		return;
	}
	CReader_Document *pReaderDoc = pPDFReaderDoc->GetReaderDocument();
	if (!pReaderDoc)
	{
		return;
	}
	CString strDoc = m_pDocument->GetTitle(); 
	CPrintDialog dlg(FALSE, PD_PAGENUMS | PD_USEDEVMODECOPIES);
	dlg.m_pd.nMinPage = dlg.m_pd.nFromPage = 1;
	dlg.m_pd.nMaxPage = dlg.m_pd.nToPage = pReaderDoc->GetPageCount();
	if (dlg.DoModal() == IDOK)
	{
		int from_page, to_page;
		if (dlg.PrintAll())
		{
			from_page = dlg.m_pd.nMinPage;
			to_page = dlg.m_pd.nMaxPage;
		}
		else if (dlg.PrintRange())
		{
			from_page = dlg.GetFromPage();
			to_page = dlg.GetToPage();
		}
		else if (dlg.PrintSelection()) 
		{
			from_page = to_page = m_pPDFDisplayView->GetCurVisiblePageIndex() + 1;
		}

		HDC printDC;
		DOCINFO docInfo;

		printDC = dlg.CreatePrinterDC();
		RECT printRect;
		GetClipBox(printDC, &printRect);
		
		if(NULL == printDC) return;
		docInfo.cbSize = sizeof(DOCINFO);
		docInfo.fwType = 0;
		docInfo.lpszDatatype = NULL;
		docInfo.lpszOutput = NULL;
		docInfo.lpszDocName = strDoc;

		if(StartDoc(printDC, &docInfo) <= 0) return;

		CString str;
		for (int i = from_page-1; i < to_page; i++)
		{

			CReader_Page *pPage = (CReader_Page *)(pReaderDoc->GetPage(i));
		
			float width = 0, height = 0;
			FS_INT32 nRotate = 0;
			FSPDF_Page_GetRotation(pPage->GetPage(), &nRotate);
			float fScale = m_pPDFDisplayView->GetScale();
	
			FSCRT_MATRIX matrix;
			m_pPDFDisplayView->GetPageMatrixForPrint(i, &matrix);

			CRect WinRect(0, 0, (int)pPage->GetPageWidth(), (int)pPage->GetPageHeight());

			FSCRT_RECTF PageRect;
			PageRect.bottom = 0;
			PageRect.left = 0;
			PageRect.right = 0;
			PageRect.top = 0;

			pPage->WindowToDocForPrint(i, WinRect, PageRect);
			if(nRotate == 0 || nRotate == 2)
			{
				height = (PageRect.bottom - PageRect.top) * fScale;
				width = (PageRect.right- PageRect.left) * fScale;
			}
			else
			{
				height = (PageRect.bottom - PageRect.top) * fScale;
				width = (PageRect.right - PageRect.left) * fScale;
			}
		
			//calculate the page size
			int logpixelsx = GetDeviceCaps(printDC, LOGPIXELSX);
			int logpixelsy = GetDeviceCaps(printDC, LOGPIXELSY);
			int nsizeX = (int)(width / 72 *logpixelsx + 0.5f);
			int nsizeY = (int)(height / 72 *logpixelsy + 0.5f);

			float cx = m_pPDFDisplayView->GetCenterPos(i);

			if(StartPage(printDC) <= 0)
			{	
				str.Format(_T("one error occured when start the page %d"), i);
				MessageBox(str);
				return;
			}
			float x = (float)(printRect.right - printRect.left - nsizeX)/2;
			float y = (float)(printRect.bottom - printRect.top - nsizeY)/2;

			//render to print device
			BOOL bPrint = ((CPDFReaderApp*)AfxGetApp())->m_bClipRect;
			if (bPrint)
			{
				((CPDFReaderApp*)AfxGetApp())->m_bClipRect = FALSE;
			}
			m_pPDFDisplayView->RenderPage(CDC::FromHandle(printDC), i, nsizeX, nsizeY, (int)x , (int)y);
			if (bPrint)
			{
				((CPDFReaderApp*)AfxGetApp())->m_bClipRect = TRUE;
			}

			//	FPDF_RenderPage(printDC, pPage, 0, nPhOffsetY, nPageWidth, nPageHeight, m_nRotateFlag, 0);
			if(EndPage(printDC) <= 0)
			{
				str.Format(_T("one error occured when close the page %d"), i);
				MessageBox(str);
				return;
			}

		}//end for
		EndDoc(printDC);
		if(!DeleteDC(printDC))// delete printDC
			MessageBox(_T("can not delete the printer"));
	}	
}


int CPDFReaderView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	// TODO:  Add your specialized creation code here

	return 0;
}


BOOL CPDFReaderView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext)
{
	// TODO: Add your specialized code here and/or call the base class

	return CView::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}
