﻿/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a common source to implement some general functions which are used by examples.
 */

//Include common header file.
#include "fgsdk_common.h"

GSDKExport FS_RESULT  FSDK_SetAnnotProperties(FSCRT_ANNOT annot)
{
	//Prepare the string object of the annotation name.

	FSCRT_BSTR strAnnotType;
	FSCRT_BStr_Init(&strAnnotType);
	FS_RESULT ret = FSPDF_Annot_GetType(annot, &strAnnotType);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	//Set the flags, name of annotation.
	ret = FSPDF_Annot_SetFlags(annot, FSPDF_ANNOTFLAG_NOROTATE);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	ret = FSPDF_Annot_SetName(annot, &strAnnotType);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	ret = FSDK_SetAnnotBorder(annot);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;
	//Set the subject, title, contents, icon name of annotation.
	ret = FSPDF_Annot_SetSubject(annot, &strAnnotType);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	ret = FSPDF_Annot_SetTitle(annot, &strAnnotType);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	unsigned short* utf16le = (unsigned short*)L"C Sharp";
	FSCRT_BSTR content;
	FSCRT_BStr_Init(&content);

	FSCRT_UTF8_FromUTF16LE(&content, (const FS_WORD*)utf16le, wcslen((wchar_t*)utf16le));

	ret = FSPDF_Annot_SetContents(annot, &content);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	FSCRT_BStr_Clear(&content);
	//Set the stroke color and the opacity of annotation.
	ret = FSPDF_Annot_SetColor(annot, FALSE, 0x00FF0000);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	ret = FSPDF_Annot_SetOpacity(annot, 1.0);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	//Set the creation time and the modification time of annotation.
	FSCRT_DATETIMEZONE dt;
	dt.day = 14;
	dt.month = 1;
	dt.year = 2014;
	ret = FSPDF_Annot_SetCreationDateTime(annot, &dt);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	ret = FSPDF_Annot_SetModifiedDateTime(annot, &dt);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	//Clear the string object.
	FSCRT_BStr_Clear(&strAnnotType);

	return ret;
}

//Set Highlight annotation properties.
GSDKExport FS_RESULT  FSDK_SetHighlightAnnotProperties(FSCRT_ANNOT annot, float left, float top
	,float right, float bottom)
{
	FSCRT_BSTR strAnnotType;
	FSCRT_BStr_Init(&strAnnotType);
	FS_RESULT ret = FSPDF_Annot_GetType(annot, &strAnnotType);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	//Deal with the Highlight annotation only.
	if(strcmp(strAnnotType.str, FSPDF_ANNOTTYPE_HIGHLIGHT)) return FSCRT_ERRCODE_ERROR;

	FSCRT_RECTF rect;
	rect.left = left;
	rect.top = top;
	rect.right = right;
	rect.bottom = bottom;

	//Set the quadrilaterals points of annotation.
	FSCRT_QUADPOINTSF quadPoints = {rect.left, rect.top, rect.right, rect.top, rect.left, rect.bottom, rect.right, rect.bottom};//Reference:{100, 110, 300, 110, 300, 300, 100, 300};
	FSPDF_Annot_SetQuadPoints(annot, &quadPoints, 1);
	//Set the stroke color and opacity of annotation.
	FSPDF_Annot_SetColor(annot, FALSE, 0x0000FFf0);
	FSPDF_Annot_SetOpacity(annot, (FS_FLOAT)0.55);

	//Add the specified state annotation to an annotation.
	FSCRT_ANNOT stateAnnot = NULL;
	ret = FSPDF_Annot_AddState(annot, FSPDF_ANNOTSTATEMODEL_REVIEW, FSPDF_ANNOTSTATE_REVIEW_ACCEPTED, &stateAnnot);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		//Remove the state annotation.
		FSPDF_Annot_Remove(stateAnnot);

	return ret;
}

GSDKExport FS_RESULT FSDK_SetAnnotBorder(FSCRT_ANNOT annot)
{
	//Set the border of annotation.
	//Prepare FSPDF_ANNOTBORDER data.
	FSPDF_ANNOTBORDER border;
	border.borderWidth = 3.0;
	border.borderStyle = FSPDF_ANNOT_BORDERSTYLE_DASHED;
	border.cloudIntensity = -1;
	border.dashPhase = 5;
	border.dashCount = 2;
	border.dashPattern[0] = border.dashPattern[1] = 1.0;
	//Set the border.
	return FSPDF_Annot_SetBorder(annot, &border);
}

GSDKExport FS_RESULT FSDK_AnnotResetAP(FSCRT_ANNOT annot)
{
	return FSPDF_Annot_ResetAppearance(annot);
}