/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a common header to provide some general functions which are used by examples.
 */

#ifndef _FSDK_COMMON_H_
#define _FSDK_COMMON_H_

#define GSDKExport extern "C" _declspec(dllexport)

/* Common header files. */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <string>
#include <iostream>
#include <vector>
#include <locale.h>
#include <cstdio>

using namespace std;

#if defined(_MSC_VER)
	#if _MSC_VER >= 1200
	#endif
	#include <windows.h>
	#include <direct.h>
#endif

//#define INTERNAL
#ifdef INTERNAL

	#if defined(_WIN64)  //windows 64bit platforms.
	#if defined(_DEBUG)
	#pragma comment(lib, "../../../../../lib/fsdk_win64.lib")
	#else
	#pragma comment(lib, "../../../../../lib/fsdk_win64.lib")
	#endif

	#elif defined(_WIN32) //windows 32bit platforms.
	#if defined(_DEBUG)
	#pragma comment(lib, "../../../../../lib/fsdk_win32.lib")
	#else
	#pragma comment(lib, "../../../../../lib/fsdk_win32.lib")
	#endif
	#endif
	
#else
	#if defined(_WIN64)  //windows 64bit platforms.
	#if defined(_DEBUG)
	#pragma comment(lib, "../../../../lib/fsdk_win64.lib")
	#else
	#pragma comment(lib, "../../../../lib/fsdk_win64.lib")
	#endif

	#elif defined(_WIN32) //windows 32bit platforms.
	#if defined(_DEBUG)
	#pragma comment(lib, "../../../../lib/fsdk_win32.lib")
	#else
	#pragma comment(lib, "../../../../lib/fsdk_win32.lib")
	#endif
#endif

#endif

/* Include Foxit GSDK header files. */
#ifdef INTERNAL
	#include "../../../../../../include/fsdk.h"
#else
	#include "../../../../include/fsdk.h"
#endif

/*******************************************************************************/
/* CRT                                                         */
/*******************************************************************************/

GSDKExport FS_RESULT	FSDK_CreateMgr(bool useDefMgr);

GSDKExport void	FSDK_FinalizeLibrary();

GSDKExport FS_RESULT	FSDK_UnlockLibrary(const char* strLicenseID, const char* strUnlockCode);

GSDKExport FS_RESULT FSDK_LicenseGetType(FS_INT32* type);

GSDKExport FS_RESULT FSDK_InitPDFModule();

GSDKExport FS_RESULT FSDK_FinalizePDFModule();

GSDKExport FSCRT_FILE FSDK_OpenFileW(const wchar_t* fileName, const wchar_t* fileMode, bool useDef);

GSDKExport FSCRT_FILE FSDK_OpenFileW_Win(const TCHAR* fileName);

GSDKExport FS_RESULT FSDK_CloseFile(FSCRT_FILE file);

GSDKExport FS_RESULT FSDK_SetAppHander();

GSDKExport FS_RESULT FSDK_ProgressContinue(FSCRT_PROGRESS progress);

GSDKExport FS_RESULT FSDK_CreateBitmap(FS_INT32 width, FS_INT32 height, FS_INT32 format, FS_LPVOID buffer, FS_INT32 stride, FSCRT_BITMAP* bitmap);

GSDKExport FS_RESULT FSDK_ReleaseBitmap(FSCRT_BITMAP bitmap,  FS_LPVOID* buffer);

GSDKExport FS_RESULT FSDK_GetBitmapBuffer(FSCRT_BITMAP bitmap, FS_LPVOID* buffer);

GSDKExport FS_RESULT FSDK_FillBitmapRect(FSCRT_BITMAP bitmap, FS_ARGB color, int left, int top, int right, int bottom);

GSDKExport FS_RESULT FSDK_RenderPageByGE(FSCRT_PAGE page, int width, int height, int rotate, FSCRT_BITMAP bitmap);

GSDKExport int FSDK_GetBitmapSize(FSCRT_BITMAP bitmap);
/************************************************************************/
/* Document                                                                     */
/************************************************************************/

GSDKExport FS_RESULT FSDK_LoadPDFDocument(FSCRT_FILE file, const char* password, FS_DWORD flag, FSCRT_DOCUMENT* document);
GSDKExport FS_RESULT FSDK_ClosePDFDocument(FSCRT_DOCUMENT document);
GSDKExport FS_RESULT FSDK_CountPages(FSCRT_DOCUMENT document, int* count);
GSDKExport FS_RESULT FSDK_GetPage(FSCRT_DOCUMENT document, int index, FSCRT_PAGE* page);
GSDKExport FS_RESULT FSDK_LoadForm(FSCRT_DOCUMENT document, FSPDF_FORM* form);
GSDKExport FS_RESULT FSDK_SetActionHandler(FSCRT_DOCUMENT document, FS_LPVOID clientData);
/************************************************************************/
/* Page                                                                    */
/************************************************************************/
GSDKExport FS_RESULT FSDK_ParsePage(FSCRT_PAGE page, FS_INT32 flag);
GSDKExport FS_RESULT FSDK_GetPageSize(FSCRT_PAGE page, FS_FLOAT* width, FS_FLOAT* height);
GSDKExport FS_RESULT FSDK_GetPageMatrix(FSCRT_PAGE page, FS_INT32 x, FS_INT32 y, FS_INT32 width, FS_INT32 height, FS_INT32 rotation
	,float* a, float* b, float* c, float* d, float* e, float* f);
GSDKExport FS_RESULT FSDK_TransformPageMatrixToRectF(FSCRT_PAGE page, FS_INT32 pageX, FS_INT32 pageY, FS_INT32 pageWidth, FS_INT32 pageHeight, FS_INT32 rotation, float* left, float* top, float* right, float* bottom);
GSDKExport FS_RESULT FSDK_GetPageRotation(FSCRT_PAGE page, FS_INT32* rotation);
GSDKExport FS_RESULT FSDK_PageIsHasTransparency(FSCRT_PAGE page, FS_BOOL* bHasTransparency);
GSDKExport FS_RESULT FSDK_LoadTextPage(FSCRT_PAGE page, FSPDF_TEXTPAGE* textPage);
GSDKExport FS_RESULT FSDK_StartSearch(FSPDF_TEXTPAGE textPage, const wchar_t* searchPattern, FS_DWORD flags, FS_INT32 startIndex, FSPDF_TEXTSEARCH* textSearch);
GSDKExport FS_RESULT FSDK_SearchFindNext(FSPDF_TEXTSEARCH textSearch, FS_BOOL* isMatch);
GSDKExport FS_RESULT FSDK_SearchFindPrev(FSPDF_TEXTSEARCH textSearch, FS_BOOL* isMatch);
GSDKExport FS_RESULT FSDK_CountTextSelectionPieces(FSPDF_TEXTSELECTION textSelection, FS_INT32* piecesCount);
GSDKExport FS_RESULT FSDK_GetTextSelectionPiectRectF(FSPDF_TEXTSELECTION textSelection, FS_INT32 pieceIndex, FS_FLOAT* left, FS_FLOAT* top, FS_FLOAT* right, FS_FLOAT* bottom);
GSDKExport FS_RESULT FSDK_SelectTextCaretRect(FSPDF_TEXTPAGE textPage, float pageX, int index, float* left, float* top, float* right, float* bottom);
GSDKExport FS_RESULT FSDK_GetStartCharIndexByPoint(FSPDF_TEXTPAGE textPage, float pageX, int charIndex, int startCharIndex, int endCharIndex, int* index);
GSDKExport FS_RESULT FSDK_GetCharIndexAtPos(FSPDF_TEXTPAGE textPage, FS_FLOAT x, FS_FLOAT y, FS_FLOAT tolerance, FS_INT32* charIndex);
GSDKExport FS_RESULT FSDK_ReleaseTextPage(FSPDF_TEXTPAGE textPage);
GSDKExport FS_RESULT FSDK_ReleaseTextSearch(FSPDF_TEXTSEARCH textSearch);
GSDKExport FS_RESULT FSDK_ReleaseTextSelection(FSPDF_TEXTSELECTION textSelection);
GSDKExport FS_RESULT FSDK_ClosePage(FSCRT_PAGE page);
GSDKExport FS_RESULT FSDK_PageLoadAnnots(FSCRT_PAGE page);
GSDKExport FS_RESULT FSDK_PageUnLoadAnnots(FSCRT_PAGE page);
GSDKExport FS_RESULT FSDK_PageAddAnnot(FSCRT_PAGE page, float left, float top, float right, float bottom
	,const char* annotType, const char* filter, int index, FSCRT_ANNOT* annot);
GSDKExport FS_RESULT FSDK_GetPageBox(FSCRT_PAGE page, FS_INT32 pageBoxType, FS_FLOAT* left, FS_FLOAT* top, FS_FLOAT* right, FS_FLOAT* bottom);

/************************************************************************/
/* FormFiller                                                                     */
/************************************************************************/

GSDKExport FS_RESULT FSDK_TriggerFormFillerWindowlessCharEvent(FSPDF_FORMFILLER formFiller, int eventType, int code, int flag);
GSDKExport FS_RESULT FSDK_TriggerFormFillerWindowlessMouseEvent(FSPDF_FORMFILLER formFiller, FSCRT_PAGE page, float a, float b, float c,
	float d, float e,float f, int eventType, int flag, float pointX, float pointY);
GSDKExport FS_RESULT FSDK_ShowFormFillerHL(FSPDF_FORMFILLER formFiller, FS_BOOL show);
GSDKExport FS_RESULT FSDK_SetFormFillerHighlightColor(FSPDF_FORMFILLER formFiller, FS_INT32 fieldType, FS_ARGB color);
GSDKExport FS_RESULT FSDK_EndFormFiller(FSPDF_FORMFILLER formFiller);
GSDKExport FS_RESULT FSDK_BeginFormFiller(FSPDF_FORM form, FSPDF_FORMFILLER* formFiller,FS_LPVOID clientData);

/************************************************************************/
/* Annotations                                                                     */
/************************************************************************/

GSDKExport FS_RESULT  FSDK_SetAnnotProperties(FSCRT_ANNOT annot);
GSDKExport FS_RESULT  FSDK_SetAnnotBorder(FSCRT_ANNOT annot);
GSDKExport FS_RESULT  FSDK_SetHighlightAnnotProperties(FSCRT_ANNOT annot, float left, float top
	,float right, float bottom);
GSDKExport FS_RESULT FSDK_AnnotResetAP(FSCRT_ANNOT annot);
#endif /* _FSDK_COMMON_H_ */

