/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a common source to implement some general functions which are used by examples.
 */

//Include common header file.
#include "fgsdk_common.h"

FS_RESULT	OutputSelectedRect(FS_LPVOID clientData, FS_INT32 pageIndex, FSCRT_RECTF* pdfRect)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	SetCursor(FS_LPVOID clientData, FS_INT32 nCursorType)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	SetTimer(FS_LPVOID clientData, FS_INT32 uElapse, FSCRT_CALLBACK_TIMER lpTimerFunc, FS_INT32* timeID)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	KillTimer(FS_LPVOID clientData, FS_INT32 timerID)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	GotFocusOnControl(FS_LPVOID clientData, FSPDF_FORMCONTROL control, const FSCRT_BSTR* filedValue)
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;

	FSCRT_ANNOT widgetAnnot = NULL;
	FSPDF_FormControl_GetWidgetAnnot( control, &widgetAnnot);
	if (!widgetAnnot) return ret;

	FSCRT_PAGE page = NULL;
	FSCRT_Annot_GetPage(widgetAnnot, &page);
	if (!page) return ret;

	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	LostFocusOnControl(FS_LPVOID clientData, FSPDF_FORMCONTROL control, const FSCRT_BSTR* filedValue)
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;

	FSCRT_ANNOT widgetAnnot = NULL;
	FSPDF_FormControl_GetWidgetAnnot( control, &widgetAnnot);

	if (!widgetAnnot) return ret;

	FSCRT_PAGE page = NULL;
	FSCRT_Annot_GetPage(widgetAnnot, &page);

	if (!page) return ret;

	return FSCRT_ERRCODE_SUCCESS;
}

GSDKExport FS_RESULT FSDK_BeginFormFiller(FSPDF_FORM form, FSPDF_FORMFILLER* formFiller,FS_LPVOID clientData)
{
	FSPDF_FORMINTERACTION_WINDOWLESS forminterAction;
	forminterAction.clientData = clientData;
	forminterAction.lStructSize = sizeof(FSPDF_FORMINTERACTION_WINDOWLESS);
	forminterAction.GotFocusOnControl = GotFocusOnControl;
	forminterAction.LostFocusOnControl = LostFocusOnControl;
	forminterAction.KillTimer = KillTimer;
	forminterAction.SetCursor = SetCursor;
	forminterAction.OutputSelectedRect = OutputSelectedRect;
	forminterAction.SetTimer = SetTimer;

	return FSPDF_FormFiller_Begin(form, &forminterAction, formFiller);
}

GSDKExport FS_RESULT FSDK_EndFormFiller(FSPDF_FORMFILLER formFiller)
{
    return FSPDF_FormFiller_End(formFiller);
}

GSDKExport FS_RESULT FSDK_SetFormFillerHighlightColor(FSPDF_FORMFILLER formFiller, FS_INT32 fieldType, FS_ARGB color)
{
	return FSPDF_FormFiller_SetHighlightColor(formFiller, fieldType, color);
}

GSDKExport FS_RESULT FSDK_ShowFormFillerHL(FSPDF_FORMFILLER formFiller, FS_BOOL show)
{
	return FSPDF_FormFiller_ShowHighlight(formFiller, TRUE);
}

GSDKExport FS_RESULT FSDK_TriggerFormFillerWindowlessMouseEvent(FSPDF_FORMFILLER formFiller, FSCRT_PAGE page, float a, float b, float c,
	float d, float e,float f, int eventType, int flag, float pointX, float pointY)
{
	FSCRT_NOTEDATA_MOUSE mouseData;
	mouseData.deltaX = 0;
	mouseData.deltaY = 0;
	mouseData.flags = flag;
	mouseData.x = pointX;
	mouseData.y = pointY;

	FSCRT_MATRIX  matrix;
	matrix.a = a;
	matrix.b = b;
	matrix.c = c;
	matrix.d = d;
	matrix.e = e;
	matrix.f = f;

	FS_RESULT ret = FSPDF_FormFiller_TriggerWindowlessEvent(formFiller, page, &matrix, eventType, &mouseData);
	return ret;
}

GSDKExport FS_RESULT FSDK_TriggerFormFillerWindowlessCharEvent(FSPDF_FORMFILLER formFiller, int eventType, int code, int flag)
{
	FSCRT_NOTEDATA_CHAR charData;
	charData.code = code;
	charData.flags = flag;

	FS_RESULT ret = FSPDF_FormFiller_TriggerWindowlessEvent(formFiller, NULL, NULL, eventType, &charData);
	return ret;
}
