/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a common source to implement some general functions which are used by examples.
 */

//Include common header file.
#include "fgsdk_common.h"

FSCRT_DOCUMENT g_document = NULL;

GSDKExport FS_RESULT FSDK_LoadPDFDocument(FSCRT_FILE file, const char* password, FS_DWORD flag, FSCRT_DOCUMENT* document)
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	FSCRT_BSTR bstrPW;
	FSCRT_BStr_Init(&bstrPW);
	bstrPW.str = (FS_LPSTR)password;
	bstrPW.len = strlen(password);
	ret = FSPDF_Doc_StartLoadWithFlag(file, &bstrPW, flag, document, NULL);
	g_document = *document;
	return ret;
}

GSDKExport FS_RESULT FSDK_ClosePDFDocument(FSCRT_DOCUMENT document)
{
	g_document = NULL;
	return FSPDF_Doc_Close(document);
}

GSDKExport FS_RESULT FSDK_CountPages(FSCRT_DOCUMENT document, int* count)
{
	return FSPDF_Doc_CountPages(document, count);
}

GSDKExport FS_RESULT FSDK_GetPage(FSCRT_DOCUMENT document, int index, FSCRT_PAGE* page)
{
	return FSPDF_Doc_GetPage(document, index, page);
}

GSDKExport FS_RESULT FSDK_LoadForm(FSCRT_DOCUMENT document, FSPDF_FORM* form)
{
	FS_RESULT ret = FSPDF_Form_Load(document, form);
	if (ret == FSCRT_ERRCODE_NOTFOUND)
		ret = FSPDF_Doc_CreateForm(document, form);

	return ret;
}

#define	INVAILDATERECT			0x01 + WM_USER
#define GETCURRENTPAGEINDEX		0x02 + WM_USER
#define SETCURRENTPAGEINDEX		0x03 + WM_USER
#define GETCHANGEMARK			0x04 + WM_USER
#define ALERT					0x05 + WM_USER
#define BEEP					0x06 + WM_USER
#define GETROTATION				0x07 + WM_USER
#define	GETOPENEDDOC			0x08 + WM_USER
#define EXECUTENAMEDACTION		0x09 + WM_USER
#define	GETFILEPATH				0x10 + WM_USER
#define	MAIL					0x11 + WM_USER
#define PRINT					0x12 + WM_USER
#define RELEASE					0x13 + WM_USER
#define RESPONSE				0x14 + WM_USER
#define SUBMITFORM				0x15 + WM_USER
#define LAUNCHURL				0x16 + WM_USER
#define BROWSEFILE				0x17 + WM_USER
#define SETCHANGEMARK			0x18 + WM_USER
#define GETIDENTITYPROPERTIES	0x19 + WM_USER
#define GETLANGUAGE				0x20 + WM_USER
#define POPUPMENU				0x21 + WM_USER

FS_RESULT InvalidateRect(FS_LPVOID clientData, FSCRT_DOCUMENT, FS_INT32, FSCRT_RECTF*)
{
	if(!clientData)
		return FSCRT_ERRCODE_ERROR;
	::SendMessage((HWND)clientData, INVAILDATERECT, NULL, NULL);
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	GetCurrentPageIndex(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_INT32* pageIndex)
{
	wchar_t str[MAX_PATH];
	swprintf(str, L"GetCurrentPageIndex : %d", pageIndex);
	if (IDOK == ::MessageBox((HWND)clientData, (LPCWSTR)str, L"C Sharp PDF Reader", 1))
	{
		::SendMessage((HWND)clientData, GETCURRENTPAGEINDEX, (WPARAM)str, NULL);
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	SetCurrentPageIndex(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_INT32 pageIndex)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	GetChangeMark(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_BOOL* changeMark)
{
	wchar_t str[MAX_PATH];
	swprintf(str, L"GetChangeMark : %d", changeMark);
	if (IDOK == ::MessageBox((HWND)clientData, (LPCWSTR)str, L"C Sharp PDF Reader", 1))
	{
		::SendMessage((HWND)clientData, GETCHANGEMARK, NULL, NULL);
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	Alert(FS_LPVOID clientData, const FSCRT_BSTR* Msg, const FSCRT_BSTR* Title, FS_INT32 Type, FS_INT32 Icon, FS_INT32* retCode)
{
	FS_DWORD len = 0;
	FSCRT_UTF8_ToUTF16LE(Msg, NULL, &len);
	FS_WORD* pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(Msg, pBuff, &len);
	if (2 == Type) Type = MB_YESNO;
	int ret = ::MessageBox((HWND)clientData, (LPCWSTR)pBuff, L"C Sharp PDF Reader", Type);
	if (IDOK == ret)
	{
		::SendMessage((HWND)clientData, ALERT, NULL, NULL);
	}
	else if (IDYES == ret)
		*retCode = 4;

	delete[] pBuff;
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	Beep(FS_LPVOID clientData, FS_INT32 nType)
{
	wchar_t str[MAX_PATH];

	switch (nType)
	{
	case 0: 
		swprintf(str, L"Beep : Error");
		break;
	case 1: 
		swprintf(str, L"Beep : Warning");
		break;
	case 2: 
		swprintf(str, L"Beep : Question");
		break;
	case 3:
		swprintf(str, L"Beep : Status");
		break;
	case 4:
		swprintf(str, L"Beep : Default");
		break;
	default:
		swprintf(str, L"Beep : Not Defined");
		break;
	}

	if (IDOK == ::MessageBox((HWND)clientData, (LPCWSTR)str, L"C Sharp PDF Reader", 1))
	{
		::SendMessage((HWND)clientData, BEEP, NULL, NULL);
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	GetRotation(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_INT32 pageIndex, FS_INT32* rotation)
{
	wchar_t temp[MAX_PATH];
	swprintf(temp, L"page index : %d\n\nrotation : %d\n\n", pageIndex, rotation);
	wstring strRotate(temp);
	wstring str = L"GetRotation\n\n" + strRotate;

	if (IDOK == ::MessageBox((HWND)clientData, (LPCWSTR)str.c_str(), L"C Sharp PDF Reader", 1))
	{
		::SendMessage((HWND)clientData, GETROTATION, NULL, NULL);
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT GetOpenedDocs(FS_LPVOID clientData, FSCRT_DOCUMENT* document, FS_INT32* count)
{
	if (document)
	{
		document[0] = g_document;
		*count = 1;
	} 
	else
	{
		*count = 1;
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	ExecuteNamedAction(FS_LPVOID clientData, FSCRT_DOCUMENT document, const FSCRT_BSTR* namedAction)
{
	FS_DWORD len = 0;
	FSCRT_UTF8_ToUTF16LE(namedAction, NULL, &len);
	FS_WORD* pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(namedAction, pBuff, &len);
	wstring strNamedAction((wchar_t*)pBuff);
	wstring str = L"ExecuteNamedAction\n\n" + strNamedAction;

	if (IDOK == ::MessageBox((HWND)clientData, (LPCWSTR)str.c_str(), L"C Sharp PDF Reader", 1))
	{
		::SendMessage((HWND)clientData, EXECUTENAMEDACTION, NULL, NULL);
	}

	delete[] pBuff;

	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	GetFilePath(FS_LPVOID clientData, FSCRT_DOCUMENT document, FSCRT_BSTR* filePath)
{
	FS_DWORD len = 0;
	FSCRT_UTF8_ToUTF16LE(filePath, NULL, &len);
	FS_WORD* pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(filePath, pBuff, &len);

	wstring strfilepath((wchar_t*)pBuff);
	wstring str = L"GetFilePath\n\n" + strfilepath;
	if (IDOK == ::MessageBox((HWND)clientData, (LPCWSTR)str.c_str(), L"C Sharp PDF Reader", 1))
	{
		::SendMessage((HWND)clientData, GETFILEPATH, NULL, NULL);
	}

	delete[] pBuff;

	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	Mail(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_BOOL bUI, const FSCRT_BSTR* To, 
	const FSCRT_BSTR* Subject, const FSCRT_BSTR* CC, const FSCRT_BSTR* BCC, const FSCRT_BSTR* Msg)
{
	FS_DWORD len = 0;
	FSCRT_UTF8_ToUTF16LE(To, NULL, &len);
	FS_WORD* pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(To, pBuff, &len);
	wstring wstrTo((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	FSCRT_UTF8_ToUTF16LE(Subject, NULL, &len);
	pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(Subject, pBuff, &len);
	wstring wstrSubject((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	FSCRT_UTF8_ToUTF16LE(CC, NULL, &len);
	pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(CC, pBuff, &len);
	wstring wstrCC((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	FSCRT_UTF8_ToUTF16LE(BCC, NULL, &len);
	pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(BCC, pBuff, &len);
	wstring wstrBCC((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	FSCRT_UTF8_ToUTF16LE(Msg, NULL, &len);
	pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(Msg, pBuff, &len);
	wstring wstrMsg((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	wstring funname(L"Mail\n\n");
	wstring n1(L"To:"); 
	wstring n2(L"Subject:"); 
	wstring n3(L"CC:"); 
	wstring n4(L"BCC:"); 
	wstring n5(L"Msg:"); 
	wstring CL(L"\n");
	wstring str = funname + n1 + wstrTo + CL + n2 + wstrSubject + CL + n3 + wstrCC + CL + 
		n4 + wstrBCC + CL + n5 + wstrMsg + CL;

	if (IDOK == ::MessageBox((HWND)clientData, (LPCWSTR)str.c_str(), L"C Sharp PDF Reader", 1))
	{
		::SendMessage((HWND)clientData, MAIL, NULL, NULL);
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	Print(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_BOOL bUI, FS_INT32 nStart, FS_INT32 nEnd, FS_BOOL bSilent , 
	FS_BOOL bShrinkToFit, FS_BOOL bPrintAsImage, FS_BOOL bReverse, FS_BOOL bAnnotations)
{
	wchar_t strPrint[1024];
	swprintf(strPrint, L"bUI: %d\nrange: from %d to %d\nbSilent: %d\nbShrinkToFit: %d\nbPrintAsImage: %d\nbReverse: %d\nbAnnotations: %d", 
		bUI, nStart, nEnd, bSilent, bShrinkToFit, bPrintAsImage, bReverse, bAnnotations);

	wstring funname(L"Print\n\n");
	wstring str = funname + strPrint;
	if (IDOK == ::MessageBox((HWND)clientData, (LPCWSTR)str.c_str(), L"C Sharp PDF Reader", 1))
	{
		::SendMessage((HWND)clientData, PRINT, NULL, NULL);
	}
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	Release(FS_LPVOID clientData)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	Response(FS_LPVOID clientData, const FSCRT_BSTR* Question, const FSCRT_BSTR* Title, const FSCRT_BSTR* Default, 
	const FSCRT_BSTR* cLabel, FS_BOOL bPassword, FSCRT_BSTR* response)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	SubmitForm(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_LPVOID formData, FS_DWORD length, const FSCRT_BSTR* URL)
{
	FS_DWORD len = 0;
	FSCRT_UTF8_ToUTF16LE(URL, NULL, &len);
	FS_WORD* pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(URL, pBuff, &len);

	wstring strURL((wchar_t*)pBuff);
	wstring funname(L"SubmitForm URL\n\n");
	wstring str = funname + strURL;
	if (IDOK == ::MessageBox((HWND)clientData, (LPCWSTR)str.c_str(), L"C Sharp PDF Reader", 1))
	{
		::SendMessage((HWND)clientData, SUBMITFORM, NULL, NULL);
	}
	delete[] pBuff;
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	LaunchURL(FS_LPVOID clientData, const FSCRT_BSTR* URL)
{
	FS_DWORD len = 0;
	FSCRT_UTF8_ToUTF16LE(URL, NULL, &len);
	FS_WORD* pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(URL, pBuff, &len);

	wstring strURL((wchar_t*)pBuff);
	wstring str = L"LaunchURL\n\n" + strURL;
	if (IDOK == ::MessageBox((HWND)clientData, (LPCWSTR)str.c_str(), L"C Sharp PDF Reader", 1))
	{
		::SendMessage((HWND)clientData, LAUNCHURL, NULL, NULL);
	}

	delete[] pBuff;

	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	BrowseFile(FS_LPVOID clientData, FSCRT_BSTR* filePath)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	SetChangeMark(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_BOOL changeMark)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	GetIdentityProperties(FS_LPVOID clientData, FSPDF_IDENTITYPROPERTIES* identityProperties)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	GetLanguage(FS_LPVOID clientData, FS_INT32* language)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	PopupMenu(FS_LPVOID clientData, const FSPDF_MENULIST* menus, FS_INT32 count, FSCRT_BSTR* seletedMenu)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	MailData(FS_LPVOID clientData, FS_LPVOID data, FS_DWORD dataType, FS_BOOL bUI, const FSCRT_BSTR* To, 
	const FSCRT_BSTR* Subject, const FSCRT_BSTR* CC, const FSCRT_BSTR* BCC, const FSCRT_BSTR* Msg)
{
	wchar_t* mailType = NULL;
	switch(dataType)
	{
	case FSPDF_MAILTYPE_DOC:
		mailType = L"doc";
		break;
	case FSPDF_MAILTYPE_MSG:
		mailType = L"message";
		break;
	case FSPDF_MAILTYPE_FORM:
		mailType = L"form";
		break;
	}
	wstring wsMailType(mailType);

	FS_DWORD len = 0;
	FSCRT_UTF8_ToUTF16LE(To, NULL, &len);
	FS_WORD* pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(To, pBuff, &len);
	wstring wstrTo((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	FSCRT_UTF8_ToUTF16LE(Subject, NULL, &len);
	pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(Subject, pBuff, &len);
	wstring wstrSubject((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	FSCRT_UTF8_ToUTF16LE(CC, NULL, &len);
	pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(CC, pBuff, &len);
	wstring wstrCC((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	FSCRT_UTF8_ToUTF16LE(BCC, NULL, &len);
	pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(BCC, pBuff, &len);
	wstring wstrBCC((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	FSCRT_UTF8_ToUTF16LE(Msg, NULL, &len);
	pBuff = new FS_WORD[len+1];
	pBuff[len] = 0;
	FSCRT_UTF8_ToUTF16LE(Msg, pBuff, &len);
	wstring wstrMsg((wchar_t*)pBuff);
	delete[] pBuff;pBuff = NULL;

	wstring funname(L"Mail\n\n");
	wstring n0(L"Mail type:");
	wstring n1(L"To:"); 
	wstring n2(L"Subject:"); 
	wstring n3(L"CC:"); 
	wstring n4(L"BCC:"); 
	wstring n5(L"Msg:"); 
	wstring CL(L"\n");
	wstring str = funname + n0 + wsMailType + CL + n1 + wstrTo + CL + n2 + wstrSubject + CL + n3 + wstrCC + CL + 
		n4 + wstrBCC + CL + n5 + wstrMsg + CL;

	if (IDOK == ::MessageBox((HWND)clientData, (LPCWSTR)str.c_str(), L"C Sharp PDF Reader", 1))
	{
		::SendMessage((HWND)clientData, MAIL, NULL, NULL);
	}
	return FSCRT_ERRCODE_SUCCESS;
}

#define FORM_VERSION	"5.0"
#define VIEWERTYPET		"C Sharp PDF Reader"
#define VIEWERVARIATION	"full authorized"
#define APP_VERSION		"5.0"
#define VIEWERVERSION	"5.0"

FS_RESULT	GetAppInfo(FS_LPVOID clientData, FS_INT32 type, FSCRT_BSTR* info)
{
	switch(type)
	{
	case FSPDF_APPINFOTYPE_FORMSVERSION:
		FSCRT_BStr_Set(info, FORM_VERSION, strlen(FORM_VERSION));
		break;
	case FSPDF_APPINFOTYPE_VIEWERTYPE:
		FSCRT_BStr_Set(info, VIEWERTYPET, strlen(VIEWERTYPET));
		break;
	case FSPDF_APPINFOTYPE_VIEWERVARIATION:
		FSCRT_BStr_Set(info, VIEWERVARIATION, strlen(VIEWERVARIATION));
		break;
	case FSPDF_APPINFOTYPE_VIEWERVERSION:
		FSCRT_BStr_Set(info, VIEWERVERSION, strlen(VIEWERVERSION));
		break;
	case FSPDF_APPINFOTYPE_APPVERSION:
		FSCRT_BStr_Set(info, APP_VERSION, strlen(APP_VERSION));
		break;

	}
	return FSCRT_ERRCODE_SUCCESS;
}


GSDKExport FS_RESULT FSDK_SetActionHandler(FSCRT_DOCUMENT document, FS_LPVOID clientData)
{
	FSPDF_JAVASCRIPTACTION_HANDLER actionHandler;
	memset(&actionHandler, 0, sizeof(FSPDF_JAVASCRIPTACTION_HANDLER));
	actionHandler.actionHandleSize = sizeof(FSPDF_JAVASCRIPTACTION_HANDLER);

	actionHandler.clientData = clientData;
	actionHandler.InvalidateRect = InvalidateRect;
	actionHandler.Alert = Alert;
	actionHandler.Beep = Beep;
	actionHandler.BrowseFile = BrowseFile;
	actionHandler.ExecuteNamedAction = ExecuteNamedAction;
	actionHandler.GetChangeMark = GetChangeMark;
	actionHandler.GetCurrentPageIndex = GetCurrentPageIndex;
	actionHandler.GetFilePath = GetFilePath;
	actionHandler.GetIdentityProperties = GetIdentityProperties;
	actionHandler.GetLanguage = GetLanguage;
	actionHandler.GetOpenedDocs = GetOpenedDocs;
	actionHandler.GetRotation = GetRotation;
	actionHandler.LaunchURL = LaunchURL;
	actionHandler.Mail = Mail;
	actionHandler.PopupMenu = PopupMenu;
	actionHandler.Print = Print;
	actionHandler.Release = Release;
	actionHandler.Response = Response;
	actionHandler.SetChangeMark = SetChangeMark;
	actionHandler.SetCurrentPageIndex = SetCurrentPageIndex;
	actionHandler.SubmitForm = SubmitForm;
	actionHandler.MailData = MailData;
	actionHandler.GetAppInfo = GetAppInfo;

	return FSPDF_Doc_SetJavaScriptActionHandler(document, &actionHandler);
}

GSDKExport FS_RESULT FSDK_InitiateJavaScript(FSCRT_DOCUMENT document)
{
	return FSPDF_Doc_InitiateJavaScript(document);
}

GSDKExport FS_RESULT FSDK_DoJavaScriptAction(FSCRT_DOCUMENT document,  FS_INT32 trigger)
{
	return FSPDF_Doc_DoJavaScriptAction(document, trigger);
}