/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK nto any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a common source to implement some general functions which are used by examples.
 */

//Include common header file.
#include "fgsdk_common.h"

GSDKExport FS_RESULT FSDK_ParsePage(FSCRT_PAGE page, FS_INT32 flag)
{
	FSCRT_PROGRESS progress = NULL;
	FS_RESULT ret = FSPDF_Page_StartParse(page, flag, &progress);
	if (ret == FSCRT_ERRCODE_SUCCESS)
	{
		ret = FSCRT_ERRCODE_TOBECONTINUED;
		while(FSCRT_ERRCODE_TOBECONTINUED == ret)
		{
			ret = FSCRT_Progress_Continue(progress, NULL);
		}
		FSCRT_Progress_Release(progress);
		if (ret == FSCRT_ERRCODE_SUCCESS || ret == FSCRT_ERRCODE_FINISHED)
			return FSCRT_ERRCODE_SUCCESS;
		else
			return ret;
	}
	return ret;
}

GSDKExport FS_RESULT FSDK_PageLoadAnnots(FSCRT_PAGE page)
{
	return FSPDF_Page_LoadAnnots(page);
}

GSDKExport FS_RESULT FSDK_PageUnLoadAnnots(FSCRT_PAGE page)
{
	return FSPDF_Page_UnloadAnnots(page);
}

GSDKExport FS_RESULT FSDK_PageAddAnnot(FSCRT_PAGE page, float left, float top, float right, float bottom
	,const char* annotType, const char* filter, int index, FSCRT_ANNOT* annot)
{
	FSCRT_RECTF rect;
	rect.left = left;
	rect.top = top;
	rect.right = right;
	rect.bottom = bottom;

	FSCRT_BSTRC(strAnnotType, (FS_LPSTR)annotType);
	strAnnotType.len = strlen(annotType);

	FSCRT_BSTRC(strFilter, (FS_LPSTR)filter);
	strFilter.len = strlen(filter);

	return FSPDF_Annot_Add(page, &rect, &strAnnotType, &strFilter, index, annot);
}

GSDKExport FS_RESULT FSDK_GetPageSize(FSCRT_PAGE page, FS_FLOAT* width, FS_FLOAT* height)
{
	return FSPDF_Page_GetSize(page, width, height);
}
 
GSDKExport FS_RESULT FSDK_GetPageBox(FSCRT_PAGE page, FS_INT32 pageBoxType, FS_FLOAT* left, FS_FLOAT* top, FS_FLOAT* right, FS_FLOAT* bottom)
{
	FSCRT_RECTF rectF; 
	int ret = FSPDF_Page_GetBox(page, pageBoxType, &rectF); 
	*left = rectF.left;
	*top = rectF.top;
	*right = rectF.right;
	*bottom = rectF.bottom;

	return ret;
}

GSDKExport FS_RESULT FSDK_GetPageRotation(FSCRT_PAGE page, FS_INT32* rotation)
{
	return FSPDF_Page_GetRotation(page, rotation);
}

GSDKExport FS_RESULT FSDK_SetPageRotation(FSCRT_PAGE page, FS_INT32 rotation)
{
	return FSPDF_Page_SetRotation(page, rotation);
}

GSDKExport FS_RESULT FSDK_GetPageMatrix(FSCRT_PAGE page, FS_INT32 x, FS_INT32 y, FS_INT32 width, FS_INT32 height, FS_INT32 rotation
	,float* a, float* b, float* c, float* d, float* e, float* f)
{
	FSCRT_MATRIX matrix;
	FS_RESULT  ret = FSPDF_Page_GetMatrix(page, x, y, width, height, rotation, &matrix);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		return  ret;
	}

	*a = matrix.a;
	*b = matrix.b;
	*c = matrix.c;
	*d = matrix.d;
	*e = matrix.e;
	*f = matrix.f;
	return ret;
}

GSDKExport FS_RESULT FSDK_TransformWindowsToDocRectF(FSCRT_PAGE page, FS_INT32 pageX, FS_INT32 pageY, FS_INT32 pageWidth, FS_INT32 pageHeight, FS_INT32 rotation, float* left, float* top, float* right, float* bottom)
{
	FSCRT_MATRIX pageMatrix, deviceMatrix;
	FS_RESULT ret = FSPDF_Page_GetMatrix(page, pageX, pageY, pageWidth, pageHeight, rotation, &pageMatrix);
	if (ret != FSCRT_ERRCODE_SUCCESS)
		return ret;

	FSCRT_RECTF rectF;
	rectF.left = *left;
	rectF.right = *right;
	rectF.top = *top;
	rectF.bottom = *bottom;

	ret = FSCRT_Matrix_GetReverse(&pageMatrix, &deviceMatrix);
	if (ret != FSCRT_ERRCODE_SUCCESS)
		return ret;

	ret = FSCRT_Matrix_TransformRectF(&deviceMatrix, &rectF);

	*left = rectF.left;
	*top = rectF.top;
	*right = rectF.right;
	*bottom = rectF.bottom;
	return ret;
}

GSDKExport FS_RESULT FSDK_TransformPageMatrixToRectF(FSCRT_PAGE page, FS_INT32 pageX, FS_INT32 pageY, FS_INT32 pageWidth, FS_INT32 pageHeight, FS_INT32 rotation, float* left, float* top, float* right, float* bottom)
{
	FSCRT_MATRIX matrix;
	FS_RESULT ret = FSPDF_Page_GetMatrix(page, pageX, pageY, pageWidth, pageHeight, rotation, &matrix);
	if (ret != FSCRT_ERRCODE_SUCCESS)
		return ret;

	FSCRT_RECTF rectF;
	rectF.left = *left;
	rectF.right = *right;
	rectF.top = *top;
	rectF.bottom = *bottom;

	ret = FSCRT_Matrix_TransformRectF(&matrix, &rectF);

	*left = rectF.left;
	*top = rectF.top;
	*right = rectF.right;
	*bottom = rectF.bottom;
	return ret;
}
 
GSDKExport FS_RESULT FSDK_CopyTextSelecionCharsToCilpBoard(FSPDF_TEXTSELECTION textSelection)
{
	FS_RESULT ret = -1;
	FSCRT_BSTR strTextChar;
	ret = FSCRT_BStr_Init(&strTextChar);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;
	
	ret = FSPDF_TextSelection_GetChars(textSelection, &strTextChar);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	::OpenClipboard(NULL);
	::EmptyClipboard();

	FS_WORD* buffer = NULL;
	FS_DWORD bufLen = 0;
	ret = FSCRT_UTF8_ToUTF16LE(&strTextChar, buffer, &bufLen);
	if (FSCRT_ERRCODE_SUCCESS != ret || bufLen == 0) return ret;
	buffer = new FS_WORD[bufLen+1];
	memset(buffer, 0, bufLen+1);
	ret = FSCRT_UTF8_ToUTF16LE(&strTextChar, buffer, &bufLen);
	buffer[bufLen] = 0;
	if ( FSCRT_ERRCODE_SUCCESS != ret)
	{
		delete []buffer;
		buffer = NULL;
		return ret;
	}

	HGLOBAL hMem;
	hMem = ::GlobalAlloc(0, strTextChar.len*2 + sizeof(wchar_t));    

	wchar_t* pMem = (wchar_t*)::GlobalLock(hMem);
	memset(pMem,0,strTextChar.len*2+sizeof(wchar_t));

	wcscpy(pMem,(const wchar_t*)buffer);    

	::GlobalUnlock(hMem);

	HANDLE hh = ::SetClipboardData(CF_UNICODETEXT, hMem);

	CloseClipboard();
	delete[] buffer;
	buffer = NULL;

	FSCRT_BStr_Clear(&strTextChar);
	return ret;
}

GSDKExport FS_RESULT FSDK_TransformPageMatrixToPointF(FSCRT_PAGE page, FS_INT32 pageX, FS_INT32 pageY, FS_INT32 pageWidth, FS_INT32 pageHeight, FS_INT32 rotation, FS_FLOAT* pointX, FS_FLOAT* pointY)
{
	FSCRT_MATRIX pageMatrix, deviceMatrix;
	FS_RESULT ret = FSPDF_Page_GetMatrix(page, pageX, pageY, pageWidth, pageHeight, rotation, &pageMatrix);
	if (ret != FSCRT_ERRCODE_SUCCESS)
		return ret;

	ret = FSCRT_Matrix_GetReverse(&pageMatrix, &deviceMatrix);
	if (ret != FSCRT_ERRCODE_SUCCESS)
		return ret;

	ret = FSCRT_Matrix_TransformPointF(&deviceMatrix, pointX, pointY);

	return ret;
}

GSDKExport FS_RESULT FSDK_GetStartCharIndexByPoint(FSPDF_TEXTPAGE textPage, float pageX, int charIndex, int startCharIndex, int endCharIndex, int* index)
{
	FSPDF_TEXTPAGE_CHARINFO info;
	FS_RESULT ret = FSPDF_TextPage_GetCharInfo(textPage, charIndex, &info);
	if (FSCRT_ERRCODE_SUCCESS != ret) return ret;

	float positionTmp = 0;

	if (endCharIndex - startCharIndex >= 0)
	{
		if(pageX > (info.bbox.left + info.bbox.right)/2)
			*index = charIndex + 1;
		else
			*index = charIndex;
	}
	else
	{
		if(pageX > (info.bbox.left + info.bbox.right)/2)
			*index = charIndex;
		else
			*index = charIndex - 1;
	}
	return ret;
}

GSDKExport FS_RESULT FSDK_SelectTextCaretRect(FSPDF_TEXTPAGE textPage, float pageX, int index, float* left, float* top, float* right, float* bottom)
{
	int nAcent = 0, nDecent = 0;
	FSPDF_TEXTPAGE_CHARINFO info;
	memset(&info, 0, sizeof(FSPDF_TEXTPAGE_CHARINFO));

	FS_RESULT ret = FSPDF_TextPage_GetCharInfo(textPage, index, &info);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		return ret;

	ret = FSCRT_Font_GetAscent(info.font, &nAcent);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		return ret;

	ret = FSCRT_Font_GetDescent(info.font, &nDecent);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		return ret;

	nAcent = (FS_INT32)((nAcent * info.fontSize / 1000.0f) * info.matrix.d + info.originY + 0.5f);
	nDecent = (FS_INT32)((nDecent * info.fontSize / 1000.0f) * info.matrix.d + info.originY + 0.5f);

	float positionTmp = 0;

	if(pageX > (info.bbox.left + info.bbox.right)/2)
		positionTmp = info.bbox.right;
	else
		positionTmp = info.originX;

	*left = positionTmp;
	*top = (FS_FLOAT)nAcent;
	*right = positionTmp;
	*bottom = (FS_FLOAT)nDecent;

	return ret;
}


GSDKExport FS_RESULT FSDK_TextPageSelectByRange(FSPDF_TEXTPAGE textPage, FS_INT32 start, FS_INT32 count, FSPDF_TEXTSELECTION* textSelection)
{
	return 	FSPDF_TextPage_SelectByRange(textPage, start, count, textSelection);
}

GSDKExport FS_RESULT FSDK_PageIsHasTransparency(FSCRT_PAGE page, FS_BOOL* bHasTransparency)
{
	return FSPDF_Page_HasTransparency(page, bHasTransparency);
}

GSDKExport FS_RESULT FSDK_LoadTextPage(FSCRT_PAGE page, FSPDF_TEXTPAGE* textPage)
{
	return FSPDF_TextPage_Load(page, textPage);
}

GSDKExport FS_RESULT FSDK_StartSearch(FSPDF_TEXTPAGE textPage, const wchar_t* searchPattern, FS_DWORD flags, FS_INT32 startIndex, FSPDF_TEXTSEARCH* textSearch)
{
	FSCRT_BSTR strSearchPattern;
	FSCRT_BStr_Init(&strSearchPattern);

	FS_RESULT ret = FSCRT_UTF8_FromUTF16LE(&strSearchPattern, (FS_WORD*)(LPCWSTR)(searchPattern), (FS_DWORD)wcslen(searchPattern));
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_BStr_Clear(&strSearchPattern);
		return ret;
	}

	ret = FSPDF_TextPage_StartSearch(textPage, &strSearchPattern, flags, startIndex, textSearch);
	FSCRT_BStr_Clear(&strSearchPattern);
	return ret;
}

GSDKExport FS_RESULT FSDK_SearchFindNext(FSPDF_TEXTSEARCH textSearch, FS_BOOL* isMatch)
{
	return FSPDF_TextSearch_FindNext(textSearch, isMatch);
}

GSDKExport FS_RESULT FSDK_SearchFindPrev(FSPDF_TEXTSEARCH textSearch, FS_BOOL* isMatch)
{
	return FSPDF_TextSearch_FindPrev(textSearch, isMatch);
}

GSDKExport FS_RESULT FSDK_ReleaseTextPage(FSPDF_TEXTPAGE textPage)
{
	return FSPDF_TextPage_Release(textPage);
}

GSDKExport FS_RESULT FSDK_ReleaseTextSearch(FSPDF_TEXTSEARCH textSearch)
{
	return FSPDF_TextSearch_Release(textSearch);
}

GSDKExport FS_RESULT FSDK_GetTextSelection(FSPDF_TEXTSEARCH textSearch, FSPDF_TEXTSELECTION* textSelection)
{
	return FSPDF_TextSearch_GetSelection(textSearch, textSelection);
}

GSDKExport FS_RESULT FSDK_CountTextSelectionPieces(FSPDF_TEXTSELECTION textSelection, FS_INT32* piecesCount)
{
	return FSPDF_TextSelection_CountPieces(textSelection, piecesCount);
}

GSDKExport FS_RESULT FSDK_GetTextSelectionPiectRectF(FSPDF_TEXTSELECTION textSelection, FS_INT32 pieceIndex, FS_FLOAT* left, FS_FLOAT* top, FS_FLOAT* right, FS_FLOAT* bottom)
{
	FSCRT_RECTF rectF;
	FS_RESULT ret = FSPDF_TextSelection_GetPieceRect(textSelection, pieceIndex, &rectF);
	*left = rectF.left;
	*right = rectF.right;
	*top = rectF.top;
	*bottom = rectF.bottom;

	return ret;
}

GSDKExport FS_RESULT FSDK_GetCharIndexAtPos(FSPDF_TEXTPAGE textPage, FS_FLOAT x, FS_FLOAT y, FS_FLOAT tolerance, FS_INT32* charIndex)
{
	return FSPDF_TextPage_GetCharIndexAtPos(textPage, x, y, tolerance, charIndex);  
}

GSDKExport FS_RESULT FSDK_ReleaseTextSelection(FSPDF_TEXTSELECTION textSelection)
{
	return FSPDF_TextSelection_Release(textSelection);
}

GSDKExport FS_RESULT FSDK_ClosePage(FSCRT_PAGE page)
{
	return FSPDF_Page_Clear(page);
}