﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Runtime.InteropServices;
using PDFReader_Demo_CSharp.Interface.GSDK;

namespace PDFReader_Demo_CSharp.BLLayer
{
    class FSDKBitmap
    {
        public IntPtr m_bitmap = IntPtr.Zero;
        public int m_size = 0;

        public enum bitmapFormat
        {
            /** @brief	24bpp format, bits order: Blue, Green, Red. Blue is the lowest order. */
            FSCRT_BITMAPFORMAT_24BPP_BGR = 1,
            /** @brief	32bpp format, bits order: Blue, Green, Red, not used. Blue is the lowest order. */
            FSCRT_BITMAPFORMAT_32BPP_BGRx = 2,
            /** @brief	32bpp format, bits order: Blue, Green, Red, Alpha. Blue is the lowest order. */
            FSCRT_BITMAPFORMAT_32BPP_BGRA = 3,
            /** @brief	8bpp format, gray scale. */
            FSCRT_BITMAPFORMAT_8BPP_GRAY = 4,
            /** @brief	24bpp format, bits order: Red, Green, Blue. Red is the lowest order. */
            FSCRT_BITMAPFORMAT_24BPP_RGB = 5,
            /** @brief	32bpp format, bits order: Red, Green, Blue, not used. Red is the lowest order. */
            FSCRT_BITMAPFORMAT_32BPP_RGBx = 6,
            /** @brief	32bpp format, bits order: Red, Green, Blue, Alpha. Red is the lowest order. */
            FSCRT_BITMAPFORMAT_32BPP_RGBA = 7,
            /** @brief	16bpp format, bits order: Red 5 bits, Green 6 bits, Blue 5 bits. Red is the lowest order. */
            FSCRT_BITMAPFORMAT_16BPP_RGB565 = 8,
            /** @brief	8bpp alpha mask. */
            FSCRT_BITMAPFORMAT_8BPP_MASK = 9,
        }

        public FSDKBitmap(int width, int height, int format, IntPtr buffer, int stride)
        {
            createBitmap(width, height, format, buffer, stride);
        }

        ~FSDKBitmap()
        {
        }

        private uint encodeARGB(Color color)
        {
            return ((((uint)((color.B | ((uint)(color.G) << 8)) | (((uint)(color.R)) << 16)))) | (((uint)(color.A)) << 24));
        }

        private int createBitmap(int width, int heigth, int format, IntPtr buffer, int stride)
        {
            return CRTInterface.FSDK_CreateBitmap(width, heigth, format, buffer, stride, ref m_bitmap);
        }

        public int fillBitmapRect(Color color, Rectangle rect)
        {
            uint encodeColor = encodeARGB(color);
            return CRTInterface.FSDK_FillBitmapRect(m_bitmap, encodeColor, rect.Left, rect.Top, rect.Right, rect.Bottom);
        }

        public int getSize()
        {
            m_size = CRTInterface.FSDK_GetBitmapSize(m_bitmap);
            return m_size;
        }

        public Byte[] getBuffer()
        {
            Byte[] buffer = null; 
            if (m_bitmap != IntPtr.Zero)
            {
                IntPtr bitmapData = new IntPtr();
                buffer = new Byte[getSize()];
               
                CRTInterface.FSDK_GetBitmapBuffer(m_bitmap, ref bitmapData);
                Marshal.Copy(bitmapData, buffer, 0, m_size);
                CRTInterface.FSDK_ReleaseBitmap(m_bitmap, bitmapData);
            }

            return buffer;
        }
     
    }
}
