﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PDFReader_Demo_CSharp.Interface.GSDK;
using PDFReader_Demo_CSharp.Interface.GSDK.PDF;
using PDFReader_Demo_CSharp.UI;

namespace PDFReader_Demo_CSharp.BLLayer.PDF
{
    class Document
    {
        public IntPtr m_document = IntPtr.Zero;
        public IntPtr m_form = IntPtr.Zero;
        public IntPtr m_formFiller = IntPtr.Zero;
        public float m_fZoom = 1.0f;
        private List<Page> m_pageList = null;
        private int m_nPageCount = 0;

        public Document()
        {
         
        }

        ~Document()
        {
            if (m_document != IntPtr.Zero)
            {
                closeDocument();
                m_document = IntPtr.Zero;
            }
        }

        public Page getPage(int index)
        {
            if (index >= GetPageCount())
                return null;

            return m_pageList[index];
        }

        public int GetPageCount()
        {
            return m_nPageCount;
        }

        public int loadDocument(IntPtr file, string password, int flag)
        {
            int ret = DocInterface.FSDK_LoadPDFDocument(file, password, flag, ref m_document);

            if (m_document != IntPtr.Zero && ret == (int)CRTInterface.retCode.FSCRT_ERRCODE_SUCCESS)
            {
                return (int)CRTInterface.retCode.FSCRT_ERRCODE_SUCCESS;
            }

            return ret;
        }

        public int closeDocument()
        {
            for (int i = m_nPageCount - 1; i >= 0; i--)
            {
                if (m_pageList[i] != null)
                {
                    m_pageList[i].closePage();
                    m_pageList[i] = null;
                    m_pageList.RemoveAt(i);
                }
            }
            if (m_pageList != null)
                m_nPageCount = m_pageList.Count;
            
            if (m_formFiller != IntPtr.Zero)
            {
                endFormFiller();
                m_formFiller = IntPtr.Zero;
            }
            int ret = DocInterface.FSDK_ClosePDFDocument(m_document);
            m_document = IntPtr.Zero;
            return ret;
        }

        private int countPages()
        {
            int ret = DocInterface.FSDK_CountPages(m_document, ref m_nPageCount);
            if (ret != (int)CRTInterface.retCode.FSCRT_ERRCODE_SUCCESS)
                return ret;

            return ret;
        }

        public void loadAllPages()
        {
            if(null == m_pageList)
                m_pageList = new List<Page>();
            countPages();

            for (int i = 0; i < m_nPageCount; i++)
            {
                IntPtr pdfPage = IntPtr.Zero;
                int ret = DocInterface.FSDK_GetPage(m_document, i, ref pdfPage);
                Page page = new Page(this, pdfPage, i);
                page.SetPropertyOfPage();

                m_pageList.Add(page);
            }
        }

        public void loadForm()
        {
            if (m_document != IntPtr.Zero)
            {
                DocInterface.FSDK_LoadForm(m_document, ref m_form);

                DocInterface.FSDK_InitiateJavaScript(m_document);

                DocInterface.FSDK_DoJavaScriptAction(m_document, (int)DocInterface.trigger.FSPDF_TRIGGER_DOC_DO);
                DocInterface.FSDK_DoJavaScriptAction(m_document, (int)DocInterface.trigger.FSPDF_TRIGGER_DOC_WC);
                DocInterface.FSDK_DoJavaScriptAction(m_document, (int)DocInterface.trigger.FSPDF_TRIGGER_DOC_WS);
                DocInterface.FSDK_DoJavaScriptAction(m_document, (int)DocInterface.trigger.FSPDF_TRIGGER_DOC_WP);
                DocInterface.FSDK_DoJavaScriptAction(m_document, (int)DocInterface.trigger.FSPDF_TRIGGER_DOC_DP);
                DocInterface.FSDK_DoJavaScriptAction(m_document, (int)DocInterface.trigger.FSPDF_TRIGGER_DOC_DS);
            }
        }

        public void beginFormFiller(IntPtr cliendData)
        {
            FormFillerInterface.FSDK_BeginFormFiller(m_form, ref m_formFiller, cliendData);

            uint color = 0x8000ff00;
            FormFillerInterface.FSDK_SetFormFillerHighlightColor(m_formFiller, (int)FormFillerInterface.formFieldType.FSPDF_FORMFIELDTYPE_PUSHBUTTON, (int)color);
            FormFillerInterface.FSDK_SetFormFillerHighlightColor(m_formFiller, (int)FormFillerInterface.formFieldType.FSPDF_FORMFIELDTYPE_CHECKBOX, (int)color);
            FormFillerInterface.FSDK_SetFormFillerHighlightColor(m_formFiller, (int)FormFillerInterface.formFieldType.FSPDF_FORMFIELDTYPE_COMBOBOX, (int)color);
            FormFillerInterface.FSDK_SetFormFillerHighlightColor(m_formFiller, (int)FormFillerInterface.formFieldType.FSPDF_FORMFIELDTYPE_LISTBOX, (int)color);
            FormFillerInterface.FSDK_SetFormFillerHighlightColor(m_formFiller, (int)FormFillerInterface.formFieldType.FSPDF_FORMFIELDTYPE_RADIOBUTTON, (int)color);
            FormFillerInterface.FSDK_SetFormFillerHighlightColor(m_formFiller, (int)FormFillerInterface.formFieldType.FSPDF_FORMFIELDTYPE_TEXTFIELD, (int)color);

            FormFillerInterface.FSDK_ShowFormFillerHL(m_formFiller, true);
        }

        public void endFormFiller()
        {
            FormFillerInterface.FSDK_EndFormFiller(m_formFiller);
        }
    }
}
