﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using PDFReader_Demo_CSharp.Interface.GSDK.PDF;
using PDFReader_Demo_CSharp.Interface.GSDK;
using PDFReader_Demo_CSharp.Interface.Windows;
using System.Drawing.Drawing2D;

namespace PDFReader_Demo_CSharp.BLLayer.PDF
{
    class Page
    {
        public enum parsePageFlag
        {
            /** @brief	PDF page parsing flag for normal mode. */
            FSPDF_PAGEPARSEFLAG_NORMAL = 0,
            /** @brief	PDF page parsing flag for text-only mode. */
            FSPDF_PAGEPARSEFLAG_TEXTONLY = 1,
        }

        public enum searchFlag
        {
            /** @brief	Whether is it a matching case. */
            FSCRT_SEARCHFLAG_MATCHCASE = 1,
            /** @brief	Whether is it a matching whole word. */
            FSCRT_SEARCHFLAG_MATCHWHOLEWORD = 2,
            /** @brief	Whether matching consecutively (for example, "CC" will match twice in "CCC").*/
            FSCRT_SEARCHFLAG_CONSECUTIVE = 4,
        }

        public enum pageRotation
        {
            FSCRT_PAGEROTATION_0 = 0,
            FSCRT_PAGEROTATION_90 = 1,
            FSCRT_PAGEROTATION_180 = 2,
            FSCRT_PAGEROTATION_270 = 3,
        }

        public enum pageBox
        {
            /** 
             * @brief	Media Box for page boundary.
             * 
             * @details	The boundary of the physical medium on which page is to be displayed or printed. 
             */
            FSPDF_PAGEBOX_MEDIABOX = 0,
            /** 
             * @brief	Crop Box for page boundary.
             *
             * @details	The region to which the contents of page are to be clipped (cropped) while displaying or printing. 
             */
            FSPDF_PAGEBOX_CROPBOX = 1,
            /** 
             * @brief	Trim Box for page boundary.
             * 
             * @details	The region to which the contents of page should be clipped while outputting in a production environment. 
             */
            FSPDF_PAGEBOX_TRIMBOX = 2,
            /** 
             * @brief	Art Box for page boundary.
             *
             * @details	The intended dimensions of a finished page after trimming. 
             */
            FSPDF_PAGEBOX_ARTBOX = 3,
            /** 
             * @brief	Bleed Box for page boundary.
             *
             * @details	The extent of page's meaningful content (including potential white space) as intended by page's creator. 
             */
            FSPDF_PAGEBOX_BLEEDBOX = 4,
        }

        public Document m_document = null;
        public IntPtr m_page = IntPtr.Zero;
        public IntPtr m_textPage = IntPtr.Zero;
        public IntPtr m_textSearch = IntPtr.Zero;
        public int m_nPageIndex = -1;
        public int m_pageRotation = (int)pageRotation.FSCRT_PAGEROTATION_0;
        public float m_pageWidth;
        public float m_pageHeight;
        public float m_fTotalHeight = 0.0f;
        public float m_fRealWidth = 0.0f;
        public int m_nLogPixelX;
        public int m_nLogPixelY;
        public bool m_hasTranspancy;
        public bool m_bParse;
        public bool m_bLoadAnnots = true;
        public bool m_bReLoadAnnots = false;

        public List<Annot> m_annotList = null;

        public Page(Document document, IntPtr page, int pageIndex)
        {
            m_page = page;
            m_pageRotation = 0;
            m_pageWidth = 0.0f;
            m_pageHeight = 0.0f;
            m_nLogPixelX = 0;
            m_nLogPixelY = 0;
            m_hasTranspancy = false;
            m_bParse = false;
            m_bLoadAnnots = true;
            m_nPageIndex = pageIndex;
            m_document = document;
        }

        public void SetPropertyOfPage()
        {
            float pdfPageWidth = 0.0f;
            float pdfPageHeight = 0.0f;

            PageInterface.FSDK_GetPageSize(m_page, ref pdfPageWidth, ref pdfPageHeight);

            Panel p = new Panel();
            Graphics g = Graphics.FromHwnd(p.Handle);
            IntPtr hdc = g.GetHdc();
            m_nLogPixelX = WindowsAPI.GetDeviceCaps(hdc, 88);     // HORZRES 
            m_nLogPixelY = WindowsAPI.GetDeviceCaps(hdc, 90);     // BITSPIXEL
            g.ReleaseHdc(hdc);
            
            m_pageWidth = pdfPageWidth;// / 72.0f * m_nLogPixelX + 0.5f;
            m_pageHeight = pdfPageHeight; //0/ 72.0f * m_nLogPixelY + 0.5f;

            PageInterface.FSDK_PageIsHasTransparency(m_page, ref m_hasTranspancy);
        }

        public void parsePage(int flag)
        {
            if (m_bParse == false)
            {
                if ((int)CRTInterface.retCode.FSCRT_ERRCODE_SUCCESS == PageInterface.FSDK_ParsePage(m_page, flag))
                {
                    m_bParse = true;
                }
            }
        }

        public void SetRotation(int pageRotation)
        {
            m_pageRotation = pageRotation;
        }

        public void loadAnnots()
        {
            if (m_bLoadAnnots || m_bReLoadAnnots)
            {
                int ret = PageInterface.FSDK_PageLoadAnnots(m_page);
                if ((int)CRTInterface.retCode.FSCRT_ERRCODE_SUCCESS == ret)
                {
                    m_bLoadAnnots = true;
                    m_bReLoadAnnots = false;
                }
                if ((int)CRTInterface.retCode.FSCRT_ERRCODE_INVALIDLICENSE == ret)
                {
                    m_bLoadAnnots = false;
                    m_bReLoadAnnots = false;
                    MessageBox.Show("You don't have the right of \"FSDK_PageLoadAnnots\" Functions, please purchase \"Annot Add-on\" Modules!");
                }
            }
        }
         
        public int transformWindowsToDocRectf(int x, int y, float zoom, ref float left, ref float top, ref float right, ref float bottom)
        {
            float width = 0, height = 0;
            if (m_pageRotation == 0 || m_pageRotation == 2)
            {
                height = m_pageHeight * zoom;
                width = m_pageWidth * zoom;
            }
            else
            {
                height = m_pageWidth * zoom;
                width = m_pageHeight * zoom;
            }
            return PageInterface.FSDK_TransformWindowsToDocRectF(m_page, x, y, (int)(width + 0.5), (int)(height + 0.5), m_pageRotation, ref left, ref top, ref right, ref bottom);
        }

        public int transformPageMatrixToRectf(int x, int y, float zoom, ref float left, ref float top, ref float right, ref float bottom)
        {
            float width = 0, height = 0;
            if (m_pageRotation == 0 || m_pageRotation == 2)
            {
                height = m_pageHeight * zoom;
                width = m_pageWidth * zoom;
            }
            else
            {
                height = m_pageWidth * zoom;
                width = m_pageHeight * zoom;
            }
            return PageInterface.FSDK_TransformPageMatrixToRectF(m_page, x, y, (int)(width + 0.5), (int)(height + 0.5), m_pageRotation, ref left, ref top, ref right, ref bottom);
        }

        public int transformPageMatrixToPointF(int x, int y, float zoom, ref float pointX, ref float pointY)
        {
            float width = 0, height = 0;
            if (m_pageRotation == 0 || m_pageRotation == 2)
            {
                height = m_pageHeight * zoom;
                width = m_pageWidth * zoom;
            }
            else
            {
                height = m_pageWidth * zoom;
                width = m_pageHeight * zoom;
            }
            return PageInterface.FSDK_TransformPageMatrixToPointF(m_page, x, y, (int)(width + 0.5), (int)(height + 0.5), m_pageRotation, ref pointX, ref pointY);
        }

        public int getPageMatrix(int x, int y, float zoom, ref float a, ref float b,ref float c,ref float d,ref float e,ref float f)
        {
            float width = 0, height = 0;
            if (m_pageRotation == 0 || m_pageRotation == 2)
            {
                height = m_pageHeight * zoom;
                width = m_pageWidth * zoom;
            }
            else
            {
                height = m_pageWidth * zoom;
                width = m_pageHeight * zoom;
            }
            return PageInterface.FSDK_GetPageMatrix(m_page, x, y, (int)width, (int)height, m_pageRotation, ref a, ref b, ref c, ref d, ref e, ref f);
        }
        public IntPtr getTextPage()
        {
            return m_textPage;
        }

        public IntPtr getTextSearch()
        {
            return m_textSearch;
        }

        public bool loadTextPage()
        {           
            if (m_page == IntPtr.Zero) return false;
	
	        int ret = PageInterface.FSDK_LoadTextPage(m_page, ref m_textPage);
	        if ((int)CRTInterface.retCode.FSCRT_ERRCODE_INVALIDLICENSE == ret)
	        {
		       MessageBox.Show("You don't have the right of \"FSPDF_TextPage_Load\" Functions, please purchase \"Edit Add-on\" Modules!");
	        }
            if ((int)CRTInterface.retCode.FSCRT_ERRCODE_SUCCESS == ret)
	        {
		        return true;
	        }
	        return false;
        }

        public int startSearch(string strFind, int flags, int startIndex)
        {
            if (m_textSearch != IntPtr.Zero)
            {
                PageInterface.FSDK_ReleaseTextSearch(m_textSearch);
            }
            return PageInterface.FSDK_StartSearch(m_textPage, strFind, flags, startIndex, ref m_textSearch);
        }

        public bool continueSearch(int nDirection, string strFind, int nFlags, int startIndex, ref int nextPageIndex)
        {
            bool isMatch = false;
            int nPageIndex = m_nPageIndex;
            while (!isMatch)
            {
                if (0 == nDirection)
                {
                    nPageIndex++;
                }
                else
                {
                    nPageIndex--;
                }

                if (nPageIndex >= m_document.GetPageCount() || nPageIndex < 0)
                {
                    break;
                }

                if (null == m_document)
                {
                    break;
                }

                Page page = m_document.getPage(nPageIndex);
                if (null == page)
                {
                    break;
                }

                if (!page.m_bParse)
                    page.parsePage(0);

                if (page.getTextPage() == IntPtr.Zero)
                {
                    page.loadTextPage();
                }

                page.startSearch(strFind, nFlags, startIndex);
                if (0 == nDirection)// find down
                {
                    isMatch = page.findNext();
                }
                else
                {
                    isMatch = page.findPrev();
                }

                if (!isMatch)
                {
                    page.releaseTextPage();
                }
                else
                {
                    nextPageIndex = page.m_nPageIndex;
                }
            }
            return isMatch;
        }

        public bool findNext()
        {
            bool isMatch = false;
            PageInterface.FSDK_SearchFindNext(m_textSearch, ref isMatch);
            return isMatch;
        }

        public bool isInPageRect(int pageBoxType, float x, float y)
        {
            bool bInPageRect = false;
            float left = 0.0f, top = 0.0f, right = 0.0f, bottom = 0.0f;
            PageInterface.FSDK_GetPageBox(m_page, pageBoxType, ref left, ref top, ref right, ref bottom);
            transformPageMatrixToRectf(0, 0, m_document.m_fZoom, ref left, ref top, ref right, ref bottom);
            
            if (x > left && x < right && y > top && y < bottom)
            {
                bInPageRect = true;
            }

            return bInPageRect;
        }

        public bool findPrev()
        {
            bool isMatch = false;
            PageInterface.FSDK_SearchFindPrev(m_textSearch, ref isMatch);
            return isMatch;
        }

        public void releaseTextPage()
        {
            if (IntPtr.Zero == m_textPage)
            {
                PageInterface.FSDK_ReleaseTextPage(m_textPage);
                m_textPage = IntPtr.Zero;
            }

            if (IntPtr.Zero == m_textSearch)
            {
                PageInterface.FSDK_ReleaseTextSearch(m_textSearch);
                m_textSearch = IntPtr.Zero;
            }
        }

        public void closePage()
        {
            if (m_page != IntPtr.Zero)
            {
                PageInterface.FSDK_ClosePage(m_page);
                m_page = IntPtr.Zero; 
            }
        }

        public int addAnnot(float left, float top, float right, float bottom, string annotType, string annotFilter, int index)
        {
            if (m_annotList == null)
            {
                m_annotList = new List<Annot>();
            }
            int ret = -1;
            IntPtr annot = IntPtr.Zero;
            ret = PageInterface.FSDK_PageAddAnnot(m_page, left, top, right, bottom, annotType, annotFilter, index, ref annot);
            if ((int)CRTInterface.retCode.FSCRT_ERRCODE_SUCCESS == ret)
            {
                Annot pAnnot = new Annot(annot);
                pAnnot.SetAnnotProperties();
                pAnnot.SetAnnotBorder();
                pAnnot.SetHighLightAnnotProperties(left, top, right, bottom);
                pAnnot.resetAp();
                m_annotList.Add(new Annot(annot));
            }
            return ret;
        }
    }
}
