﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using PDFReader_Demo_CSharp.BLLayer;
using PDFReader_Demo_CSharp.BLLayer.PDF;
using PDFReader_Demo_CSharp.Interface.GSDK;
using PDFReader_Demo_CSharp.Interface.GSDK.PDF;
using System.Drawing.Drawing2D;
using PDFReader_Demo_CSharp.Interface.Windows;


namespace PDFReader_Demo_CSharp.UI
{
    public partial class MainForm : Form
    {       
        private const int recentFileCountInMenu = 4;
        private List<string> m_sRecentFilePathList = null;

        private string m_fileName = String.Empty;
        private IntPtr m_file = IntPtr.Zero;
        private Document m_document = null;
        private int m_vScrollValue = 0;
        public PictureBox pb = null;
        public List<PictureBox> m_pbSearchList = null;
        public List<PictureBox> m_pbSelectList = null;
        private Page m_curPage = null;
        private int m_nCurPageIndex = 0;
        private int m_nSearchPageIndex = 0;
        private bool bFirstLoad = true;
        private float m_fZoom = 1.0f;
        private bool m_bCtrlKeyDown = false;
        public string m_strSearchText = "";
        private int m_nSearchFlag = 0;
        private bool m_bSelect = false;
        private bool m_bSelecting = false;
        private bool m_bHasChar = false;
        public List<RectangleF> m_rectSearchPageList = null;
        public List<RectangleF> m_rectSelectPageList = null;
        private int m_nStartCharIndex = -1;
        private int m_nEndCharIndex = -1;
        private IntPtr m_TextSelection = IntPtr.Zero;
        private RectangleF m_rCaret;
        private Point m_ptSelectStart;
        bool m_bClick = false;
        bool m_bStartSelect = false;
        Point m_pLbdown;
        bool m_bLButtonDown = false;
        SearchDlg searchDlg = null;
        public bool m_bSearching = false;
        private bool m_bResize = false;
        private static bool m_bMouse = false;
        private bool m_bDrawAnnot = false;
        Point m_startPoint;
        Point m_endPoint;
        private int m_iVViewPosition = 0;
        private int m_iHViewPosition = 0;
        private Panel m_annotPanel = null;
        private bool m_bHandTool = true;
#region Msg
        private const int invalidateRectMsg = 0x01 + 0x0400;
        private const int getCurrentPageIndexMsg = 0x02 + 0x0400;
        private const int setCurrentPageIndexMsg = 0x03 + 0x0400;
        private const int getChangeMarkMsg = 0x04 + 0x0400;
        private const int alertMsg = 0x05 + 0x0400;
        private const int beepMsg = 0x06 + 0x0400;
        private const int getRotationMsg = 0x07 + 0x0400;
        private const int getOpenedDocMsg = 0x08 + 0x0400;
        private const int executeNamedActionMsg = 0x09 + 0x0400;
        private const int getFilePathMsg = 0x10 + 0x0400;
        private const int mailMsg =	0x11 + 0x0400;
        private const int printMsg = 0x12 + 0x0400;
        private const int releaseMsg = 0x13 + 0x0400;
        private const int responseMsg = 0x14 + 0x0400;
        private const int subMitFormMsg = 0x15 + 0x0400;
        private const int launchURLMsg = 0x16 + 0x0400;
        private const int BrowseFileMsg = 0x17 + 0x0400;
        private const int SetChangeMarkMsg = 0x18 + 0x0400;
        private const int GetIdentityPropertiesMsg = 0x19 + 0x0400;
        private const int GetLanguageMsg = 0x20 + 0x0400;
        private const int PopupMenuMsg = 0x21 + 0x0400;
#endregion

#region UI
        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            m_sRecentFilePathList = new List<string>();
            m_rectSearchPageList = new List<RectangleF>();
            m_rectSelectPageList = new List<RectangleF>();
            m_rCaret = new RectangleF();

            ShowRecentFileInMenu();

            m_vScrollValue = panel.VerticalScroll.Value;

            pb = new PictureBox();
            this.panel.Controls.Add((Control)pb);
            pb.Cursor = Cursors.Hand;
            pb.BorderStyle = BorderStyle.None;

            this.toolToolStripMenuItem.Click += new EventHandler(MainForm_Resize);
            this.statusBarToolStripMenuItem.Click += new EventHandler(MainForm_Resize);

            this.MouseWheel += new MouseEventHandler(Panel_MouseWheel);

            GetFocus(Controls);

            this.pb.MouseMove += new MouseEventHandler(pb_MouseMove);
            this.pb.MouseDown += new MouseEventHandler(pb_MouseDown);
            this.pb.MouseUp += new MouseEventHandler(pb_MouseUp);
            this.pb.MouseDoubleClick += new MouseEventHandler(pb_MouseDoubleClick);
            this.pb.KeyPress += new KeyPressEventHandler(pb_KeyPress);

            bFirstLoad = false;
        }

        void pb_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (m_document != null)
            {
                int charCode = Convert.ToInt32(e.KeyChar);
                float a = 0.0f, b = 0.0f, c = 0.0f, d = 0.0f, eM = 0.0f, f = 0.0f;
                m_curPage.getPageMatrix(0, m_iVViewPosition, m_fZoom, ref a, ref b, ref c, ref d, ref eM, ref  f);
                FormFillerInterface.FSDK_TriggerFormFillerWindowlessCharEvent(m_document.m_formFiller, (int)FormFillerInterface.eventType.FSCRT_EVENT_CHAR, charCode, 0);
            }
        }

        void GetFocus(Control.ControlCollection cs)
        {
            foreach (Control c in cs)
            {
                c.MouseHover +=new EventHandler(c_MouseHover); 
                GetFocus(c.Controls);
            }
        }

        void  c_MouseHover(object sender, EventArgs e)
        {
            this.Activate();	        
        }

        private void EnableButton(bool bEnable)
        {
            this.closeToolStripMenuItem.Enabled = bEnable;
            this.handToolStripButton.Enabled = bEnable;
            this.zoomInToolStripButton.Enabled = bEnable;
            this.zoomOutToolStripButton.Enabled = bEnable;
            this.rotateLeftToolStripButton.Enabled = bEnable;
            this.rotateRightToolStripButton.Enabled = bEnable;
            this.searchToolStripButton.Enabled = bEnable;
            this.selectToolStripButton.Enabled = bEnable;
            this.annotToolStripMenuItem.Visible = bEnable;
            if (bEnable == false)
            {
                m_bDrawAnnot = false;
            }
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.openPDFFileDialog.FileName = "";
            this.openPDFFileDialog.Filter = "*.pdf|*.pdf";
            this.openPDFFileDialog.ShowDialog(this);
            openPDFFile(this.openPDFFileDialog.FileName);
        }

        private void openToolStripButton_Click(object sender, EventArgs e)
        {
            this.openPDFFileDialog.FileName = "";
            this.openPDFFileDialog.Filter = "*.pdf|*.pdf";
            this.openPDFFileDialog.ShowDialog(this);
            openPDFFile(this.openPDFFileDialog.FileName);
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            pb.Image = null;
            if(m_document.m_formFiller != IntPtr.Zero)
                  killFormFillerFocus();
            CloseDocumentAndFile();
            this.closeToolStripMenuItem.Enabled = false;
        }

        private void killFormFillerFocus()
        {
            float a = 0.0f, b = 0.0f, c = 0.0f, d = 0.0f, eM = 0.0f, f = 0.0f;
            if (m_curPage != null)
                m_curPage.getPageMatrix(0, 0, m_fZoom, ref a, ref b, ref c, ref d, ref eM, ref  f);
            FormFillerInterface.FSDK_TriggerFormFillerWindowlessMouseEvent(m_document.m_formFiller
                , m_curPage.m_page, a, b, c, d, eM, f, (int)FormFillerInterface.eventType.FSCRT_EVENT_KILLFOCUS, 0, 0, 0);
        }

        private void prePageToolStripButton_Click(object sender, EventArgs e)
        {
            m_nCurPageIndex--;
            goToPage(m_nCurPageIndex);
            m_curPage = m_document.getPage(m_nCurPageIndex);
        }

        private void nextPageToolStripButton_Click(object sender, EventArgs e)
        {
            m_nCurPageIndex++;
            goToPage(m_nCurPageIndex);
            m_curPage = m_document.getPage(m_nCurPageIndex);
        }

        private void zoomInToolStripButton_Click(object sender, EventArgs e)
        {
            zoomIn();
        }

        private void zoomOutToolStripButton_Click(object sender, EventArgs e)
        {
            zoomOut();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (DialogResult.OK == MessageBox.Show("The application will be closed, whether to continue?", "", MessageBoxButtons.OKCancel))
            {
                CloseDocumentAndFile();
                CloseProgram();
            }
            else
                e.Cancel = true;
        }

        private void toolToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem toolCheckMenuItem = sender as ToolStripMenuItem;
            if (toolCheckMenuItem.Checked == false)
            {
                this.toolStripMain.Visible = false;
            }
            else
                this.toolStripMain.Visible = true;
        }

        private void statusBarToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem statusBarCheckMenuItem = sender as ToolStripMenuItem;
            if (statusBarCheckMenuItem.Checked == false)
            {
                this.statusStripMain.Visible = false;
            }
            else
                this.statusStripMain.Visible = true;
        }

        private void aboutPDFReaderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutCSharpPDFReader aboutCSharpPDFReader = new AboutCSharpPDFReader();
            aboutCSharpPDFReader.ShowDialog();
        }

        private delegate Bitmap MethodCaller();

        private Bitmap GetRenderPageBuffer()
        {
            Page page = m_document.getPage(m_nCurPageIndex);
            IntPtr pdfPage = page.m_page;

            page.parsePage(0);
            page.loadAnnots();

            int pageRotation = page.m_pageRotation;
            int nWidth = 0, nHeight = 0;

            if (pageRotation == 0 || pageRotation == 2)
            {
                nWidth = (int)(page.m_pageWidth * m_fZoom + 0.5f);
                nHeight = (int)(page.m_pageHeight * m_fZoom + 0.5f);
            }
            else if (pageRotation == 1 || pageRotation == 3)
            {
                nHeight = (int)(page.m_pageWidth * m_fZoom + 0.5f);
                nWidth = (int)(page.m_pageHeight * m_fZoom + 0.5f);
            }

            try
            {
                FSDKBitmap sdkBitmap = new FSDKBitmap(nWidth, nHeight, (int)FSDKBitmap.bitmapFormat.FSCRT_BITMAPFORMAT_24BPP_BGR, IntPtr.Zero, 0);

                Color c = Color.White;
                Rectangle rect = new Rectangle(0, 0, (int)nWidth, (int)nHeight);
                sdkBitmap.fillBitmapRect(c, rect);

                CRTInterface.FSDK_RenderPageByGE(pdfPage, nWidth, nHeight, pageRotation, sdkBitmap.m_bitmap);
                Byte[] bitmapBuffer = sdkBitmap.getBuffer();
                MemoryStream bufferStream;
                Bitmap bitmap;

                bufferStream = new MemoryStream(bitmapBuffer);
                bitmap = (Bitmap)Bitmap.FromStream(bufferStream); 

                sdkBitmap = null ;
                bitmapBuffer = null;
                GC.Collect();
                return bitmap;
            }
            catch (System.Exception e)
            {
            	
            }
            return null;
        }

        private void RenderPageByGE(bool bFormFillerRender = false)
        {
            if (m_document == null) return;

            m_iVViewPosition = panel.AutoScrollPosition.Y;
            m_iHViewPosition = panel.AutoScrollPosition.X;

            int nWidth = 0, nHeight = 0;

            Page page = m_document.getPage(m_nCurPageIndex);
            int pageRotation = page.m_pageRotation;
            if (pageRotation == 0 || pageRotation == 2)
            {
                nWidth = (int)(page.m_pageWidth * m_fZoom + 0.5f);
                nHeight = (int)(page.m_pageHeight * m_fZoom + 0.5f);
            }
            else if (pageRotation == 1 || pageRotation == 3)
            {
                nHeight = (int)(page.m_pageWidth * m_fZoom + 0.5f);
                nWidth = (int)(page.m_pageHeight * m_fZoom + 0.5f);
            }
            Bitmap bitmap = null;
            if (bFormFillerRender == true)
            {
                bitmap = GetRenderPageBuffer();
            }
            else
            {
                MethodCaller mc = new MethodCaller(GetRenderPageBuffer);
                IAsyncResult result = mc.BeginInvoke(null, null);
                bitmap = mc.EndInvoke(result);
            }
             
            if (pb.Image != null)
            {
                pb.Image.Dispose();
            }

            pb.Image = null;
            GC.Collect();

            pb.Image = bitmap;

            pb.Size = new Size(nWidth+5, nHeight+5);
            pb.Top = m_iVViewPosition;
            pb.Left = (panel.Width - nWidth - 17) / 2;
            
            if (pb.Left < 0)
            {
                pb.Left = m_iHViewPosition;
            }

            if (m_pbSearchList != null)
            {
                foreach (PictureBox searchPb in m_pbSearchList)
                {
                    this.pb.Controls.Remove(searchPb);
                    searchPb.Dispose();
                }
                m_pbSearchList.RemoveRange(0, m_pbSearchList.Count);
                m_pbSearchList = null;
            }


            if (m_pbSelectList != null)
            {
                foreach (PictureBox selectPb in m_pbSelectList)
                {
                    this.pb.Controls.Remove(selectPb);
                    selectPb.Dispose();
                }
                m_pbSelectList.RemoveRange(0, m_pbSelectList.Count);
                m_pbSelectList = null;
            }

            if (!m_bSearching && m_bSelecting && m_bSelect && m_rectSelectPageList.Count != 0)
            {
                m_pbSelectList = new List<PictureBox>();
                for (int j = 0; j < m_rectSelectPageList.Count; j++)
                {
                    DrawSelectText(m_rectSelectPageList[j]);
                }
                pb.ContextMenuStrip = contextMenuStrip1;
                WindowsAPI.HideCaret(pb.Handle);
            }  

            if (m_rectSearchPageList != null && m_rectSearchPageList.Count != 0)
            {
                m_pbSearchList = new List<PictureBox>();
                for (int i = 0; i < m_rectSearchPageList.Count; i++)
                {
                    DrawSearchText(m_rectSearchPageList[i]);
                }
                m_bSearching = false;
            }

            bitmap = null;
            GC.Collect();
        }
        
        void DrawCaret(RectangleF rc)
        {
            float left = rc.Left, top = rc.Top, right = rc.Right, bottom = rc.Bottom;
            m_curPage.transformPageMatrixToRectf(0, 0, m_fZoom, ref left, ref top, ref right, ref bottom);

            if ((int)Page.pageRotation.FSCRT_PAGEROTATION_90 == m_curPage.m_pageRotation
                || (int)Page.pageRotation.FSCRT_PAGEROTATION_270 == m_curPage.m_pageRotation)
                WindowsAPI.CreateCaret(pb.Handle, IntPtr.Zero, (int)Math.Abs(right - left), 2);
            else
                WindowsAPI.CreateCaret(pb.Handle, IntPtr.Zero, 2, (int)Math.Abs(bottom - top));
            WindowsAPI.SetCaretPos((int)left, (int)top);
	        WindowsAPI.ShowCaret(pb.Handle);
        }

        private void CloseProgram()
        {
            WriteFilePathToIni();
            Base.finalizePDFModule();
            Base.finalizeLibrary();
        }

        private void MainForm_Resize(object sender, EventArgs e)
        {
            m_bResize = true;
            int menuHeight = menuStripMain.Height;
            if (menuStripMain.Visible == false && bFirstLoad == false)
                menuHeight = 0;

            int toolHeight = toolStripMain.Height;
            if (toolStripMain.Visible == false && bFirstLoad == false)
                toolHeight = 0;

            int statusBarHeight = statusStripMain.Height;
            if (statusStripMain.Visible == false && bFirstLoad == false)
                statusBarHeight = 0;

            this.panel.Left = 0;
            this.panel.Top = menuHeight + toolHeight;
            this.panel.Width = this.ClientSize.Width;
            this.panel.Height = this.ClientSize.Height - (toolHeight + menuHeight + statusBarHeight);
        }

        private void panel_Paint(object sender, PaintEventArgs e)
        {
            int menuHeight = menuStripMain.Height;
            if (menuStripMain.Visible == false && bFirstLoad == false)
                menuHeight = 0;

            int toolHeight = toolStripMain.Height;
            if (toolStripMain.Visible == false && bFirstLoad == false)
                toolHeight = 0;

            int statusBarHeight = statusStripMain.Height;
            if (statusStripMain.Visible == false && bFirstLoad == false)
                statusBarHeight = 0;

            this.panel.Left = 0;
            this.panel.Top = menuHeight + toolHeight;
            this.panel.Width = this.ClientSize.Width;
            this.panel.Height = this.ClientSize.Height - (toolHeight + menuHeight + statusBarHeight);

            if (m_document != null && m_bResize == true && panel.VerticalScroll.Value == m_vScrollValue)
            {
                m_bResize = false;
                RenderPageByGE();
            }
            else
                m_vScrollValue = panel.VerticalScroll.Value;
        }

        private void zoomOut()
        {
            m_fZoom -= 0.1f;
            if (m_fZoom < 0.3f)
            {
                m_fZoom = 0.3f;
            }
            if (m_fZoom != m_document.m_fZoom)
            {
                m_document.m_fZoom = m_fZoom;

                RenderPageByGE();
                if (m_bSelect)
                {
                    DrawCaret(m_rCaret);
                }
            }
        }

        private void zoomIn()
        {
            m_fZoom += 0.1f;
            if (m_fZoom >= 7.0f)
            {
                m_fZoom = 7.0f;
            }
            if (m_fZoom != m_document.m_fZoom)
            {
                m_document.m_fZoom = m_fZoom;

                RenderPageByGE();

                if (m_bSelect)
                {
                    DrawCaret(m_rCaret);
                }
            }
        }

        private void rotateRightToolStripButton_Click(object sender, EventArgs e)
        {
            this.panel.AutoScrollPosition = new Point(0, 0);
            for (int i = 0; i < m_document.GetPageCount(); i++)
            {
                Page page = m_document.getPage(i);
                int rotation = page.m_pageRotation;
                rotation = (rotation == 3 ? 0 : (rotation + 1));
                page.SetRotation(rotation);
            }
            if (m_bSelect)
            {
                DrawCaret(m_rCaret);
            }
            RenderPageByGE();
        }

        private void rotateLeftToolStripButton_Click(object sender, EventArgs e)
        {
            this.panel.AutoScrollPosition = new Point(0, 0);
            for (int i = 0; i < m_document.GetPageCount(); i++)
            {
                Page page = m_document.getPage(i);
                int rotation = page.m_pageRotation;
                rotation = (rotation == 0 ? 3 : (rotation - 1));
                page.SetRotation(rotation);
            }
            if (m_bSelect)
            {
                DrawCaret(m_rCaret);
            }
            RenderPageByGE();
        }

        private void Panel_MouseWheel(object sender, MouseEventArgs e)
        {
            if (m_bCtrlKeyDown)
            {
                if (e.Delta < 0)
                    zoomOut();
                else
                    zoomIn();
            }
            else
            {
                Point mousePoint = new Point(e.X, e.Y);
                mousePoint.Offset(this.Location.X, this.Location.Y);
                if (panel.RectangleToScreen(panel.DisplayRectangle).Contains(mousePoint))
                {
                    panel.AutoScrollPosition = new Point(0, panel.VerticalScroll.Value - e.Delta);
                    m_iVViewPosition = panel.AutoScrollPosition.Y;
                }
            }
        }

        private void MainForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control)
            {
                m_bCtrlKeyDown = true;
            }
        }

        private void MainForm_KeyUp(object sender, KeyEventArgs e)
        {
            m_bCtrlKeyDown = false;
        }

        void pb_MouseUp(object sender, MouseEventArgs e)
        {
           if (e.Button == MouseButtons.Left)
            {
                if (m_bClick == false)
                {
                    //if (m_rectSearchPageList != null)
                    //    m_rectSearchPageList.RemoveRange(0, m_rectSearchPageList.Count);
                    if (m_rectSelectPageList != null)
                        m_rectSelectPageList.RemoveRange(0, m_rectSelectPageList.Count);
                }

                m_bStartSelect = false;
                if (m_document != null && m_document.m_formFiller != IntPtr.Zero && m_bDrawAnnot == false)
                {
                    float a = 0.0f, b = 0.0f, c = 0.0f, d = 0.0f, eM = 0.0f, f = 0.0f;
                    if (m_curPage != null)
                        m_curPage.getPageMatrix(0, 0, m_fZoom, ref a, ref b, ref c, ref d, ref eM, ref  f);
                    FormFillerInterface.FSDK_TriggerFormFillerWindowlessMouseEvent(m_document.m_formFiller, m_curPage.m_page,
                        a, b, c, d, eM, f, (int)FormFillerInterface.eventType.FSCRT_EVENT_LBUTTONUP, 0, e.Location.X, e.Location.Y);
                }

                if (m_bDrawAnnot)
                {
                    if (m_annotPanel != null)
                    {
                        pb.Controls.Remove(m_annotPanel);
                        m_annotPanel.Dispose();
                        m_annotPanel = null;
                    }

                    if (m_startPoint.X == e.Location.X && m_startPoint.Y == e.Location.Y)
                    {
                        m_bLButtonDown = false;
                        return;
                    }
                    float left = (float)m_startPoint.X , right = (float)e.Location.X, top = (float)m_startPoint.Y, bottom = (float)e.Location.Y;
                    
                    float nTemp = 0.0f;
                    if (left > right)
                    {
                        nTemp = left;
                        left = right;
                        right = nTemp;
                    }
                    if (top > bottom)
                    {
                        nTemp = top;
                        top = bottom;
                        bottom = nTemp;
                    }
                    m_curPage.transformWindowsToDocRectf(0, 0, m_fZoom, ref left, ref top, ref right, ref bottom);
        
                    if (left > right)
                    {
                        float temp = left;
                        left = right;
                        right = temp;
                     }

                    if (bottom > top)
                    {
                        float tem = bottom;
                        bottom = top;
                        top = tem;
                    }
                    m_curPage.addAnnot(left, top, right,bottom, "Highlight", "", 0);
                    m_startPoint.X = 0;
                    m_startPoint.Y = 0;
                    m_curPage.m_bReLoadAnnots = true;
                    RenderPageByGE();
                }
            }

            m_bLButtonDown = false;
        }
        
        void pb_MouseDown(object sender, MouseEventArgs e)
        {
            m_iVViewPosition = panel.AutoScrollPosition.Y;
            if (e.Button == MouseButtons.Left)
            {
                m_bLButtonDown = true;
                if (m_pbSelectList != null)
                {
                    foreach (PictureBox pbSelect in m_pbSelectList)
                    {
                        pb.Controls.Remove(pbSelect);
                        pbSelect.Dispose();
                    }

                    if (m_rectSelectPageList != null)
                    {
                        m_rectSelectPageList.RemoveRange(0, m_rectSelectPageList.Count);
                    }
                    RenderPageByGE();
                    pb.ContextMenuStrip = null;
                }

                m_bClick = false;
                m_pLbdown = new Point(e.Location.X, e.Location.Y);
                if (m_bSelect && !m_bClick)
                {
                    PointF pointF = new PointF((float)e.Location.X, (float)e.Location.Y);
                    m_bStartSelect = true;
                    m_bSelecting = true;
                    if (m_rectSelectPageList != null)
                    {
                        m_rectSelectPageList.RemoveRange(0, m_rectSelectPageList.Count);
                    }

                    if (m_curPage != null)
                    {
                        if (m_curPage.getTextPage() == IntPtr.Zero)
                        {
                            m_curPage.loadTextPage();
                        }

                        int index = -1;
                        index = GetStartCharIndexByPoint(pointF);
                        if (index > -1)
                        {
                            m_bHasChar = true;
                        }

                        Point pt = new Point((int)pointF.X, (int)pointF.Y);
                        CreateCaret(pt);
                        m_ptSelectStart = pt;
                    }
                }

                if (m_document != null && m_document.m_formFiller != IntPtr.Zero && m_bDrawAnnot == false)
                {
                    float a = 0.0f, b = 0.0f, c = 0.0f, d = 0.0f, eM = 0.0f, f = 0.0f;
                    if (m_curPage != null)
                        m_curPage.getPageMatrix(0, 0, m_fZoom, ref a, ref b, ref c, ref d, ref eM, ref  f);
                    FormFillerInterface.FSDK_TriggerFormFillerWindowlessMouseEvent(m_document.m_formFiller
                        , m_curPage.m_page, a, b, c, d, eM, f, (int)FormFillerInterface.eventType.FSCRT_EVENT_LBUTTONDOWN, 0, e.Location.X, e.Location.Y);
                }

                if (m_bDrawAnnot)
                {
                    if (m_curPage.isInPageRect((int)Page.pageBox.FSPDF_PAGEBOX_MEDIABOX, (float)e.Location.X, (float)e.Location.Y))
                    {
                        m_startPoint.X = e.Location.X;
                        m_startPoint.Y = e.Location.Y;
                    }
                }
                if(m_bDrawAnnot == false)
                    this.pb.Focus();
            }
        }

        void pb_MouseMove(object sender, MouseEventArgs e)
        {
            m_iVViewPosition = panel.AutoScrollPosition.Y;
            m_iHViewPosition = panel.AutoScrollPosition.X;

            if (m_bLButtonDown && m_bHandTool)
            {
                int x = panel.HorizontalScroll.Value + e.Location.X - m_pLbdown.X;
                int y = panel.VerticalScroll.Value - e.Location.Y + m_pLbdown.Y;
                if (x < 0) x = 0;
                if (x > panel.HorizontalScroll.Maximum) x = panel.VerticalScroll.Maximum;
                if (y < 0) y = 0;
                if (y > panel.VerticalScroll.Maximum) y = panel.VerticalScroll.Maximum;

                panel.AutoScrollPosition = new Point(x, y);
                m_iVViewPosition = panel.AutoScrollPosition.Y;
                m_iHViewPosition = panel.AutoScrollPosition.X;
            }

            if (m_bSelecting && e.Button == MouseButtons.Left)
            {
                if (Math.Abs(m_pLbdown.X - e.Location.X) >= 1 && Math.Abs(m_pLbdown.Y - e.Location.Y) >= 1)
                {
                    m_bClick = true;
                }

                if (m_bHasChar && m_bStartSelect)
                {
                    PointF pointF = new PointF((float)e.Location.X, (float)e.Location.Y);
                    int index = 0;
                    if (!GetCharIndexByPoint(pointF, ref index))
                        return;
                    if (index == m_nEndCharIndex) return;
                    m_nEndCharIndex = index;
                    CreateCaret(m_ptSelectStart);
                    m_nStartCharIndex = GetStartCharIndexByPoint(m_ptSelectStart);
                    GetSelectRect(m_nStartCharIndex, m_nEndCharIndex);
                    RenderPageByGE();
                }
            }

            if (m_document != null && m_bDrawAnnot == false)
            {
                float a = 0.0f, b = 0.0f, c = 0.0f, d = 0.0f, eM = 0.0f, f = 0.0f;
                if (m_curPage != null)
                    m_curPage.getPageMatrix(0, 0, m_fZoom, ref a, ref b, ref c, ref d, ref eM, ref  f);
                FormFillerInterface.FSDK_TriggerFormFillerWindowlessMouseEvent(m_document.m_formFiller,
                    m_curPage.m_page, a, b, c, d, eM, f, (int)FormFillerInterface.eventType.FSCRT_EVENT_MOUSEMOVE, 0, e.Location.X, e.Location.Y);
            }

            if (m_bLButtonDown && m_bDrawAnnot)
            {
                if (m_curPage.isInPageRect((int)Page.pageBox.FSPDF_PAGEBOX_MEDIABOX, (float)e.Location.X, (float)e.Location.Y))
                {
                    m_endPoint.X = e.Location.X;
                    m_endPoint.Y = e.Location.Y;

                    if (m_bDrawAnnot)
                    {
                        this.Focus();
                        bool bReverse = false;
                        float left = (float)m_startPoint.X, right = (float)m_endPoint.X, top = (float)m_startPoint.Y, bottom = (float)m_endPoint.Y;

                        float nTemp = 0.0f;
                        if (left > right)
                        {
                            nTemp = left;
                            left = right;
                            right = nTemp;
                            bReverse = true;
                        }
                        else
                            bReverse = false;

                        if (top > bottom)
                        {
                            nTemp = top;
                            top = bottom;
                            bottom = nTemp;
                            bReverse = true;
                        }


                        if (m_curPage.isInPageRect((int)Page.pageBox.FSPDF_PAGEBOX_MEDIABOX, m_startPoint.X, m_startPoint.Y) &&
                            m_curPage.isInPageRect((int)Page.pageBox.FSPDF_PAGEBOX_MEDIABOX, m_endPoint.X, m_endPoint.Y))
                        {
                            if (m_annotPanel == null)
                            {
                                m_annotPanel = new Panel();
                                m_annotPanel.Location = new Point((int)left, (int)top);
                                m_annotPanel.BorderStyle = BorderStyle.FixedSingle;
                                m_annotPanel.BackColor = Color.Transparent;
                                m_annotPanel.Size = new Size(0, 0);
                                pb.Controls.Add(m_annotPanel);
                            }
                            if (bReverse)
                            {
                                m_annotPanel.Location = new Point((int)left, (int)top);
                            }

                            m_annotPanel.Size = new Size((int)(right - left), (int)(bottom - top));
                            pb.Refresh();
                        }
                    }
                }
            }
        }

        void pb_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (m_document != null)
            {
                float a = 0.0f, b = 0.0f, c = 0.0f, d = 0.0f, eM = 0.0f, f = 0.0f;
                if (m_curPage != null)
                    m_curPage.getPageMatrix(0, 0, m_fZoom, ref a, ref b, ref c, ref d, ref eM, ref  f);
                FormFillerInterface.FSDK_TriggerFormFillerWindowlessMouseEvent(m_document.m_formFiller,
                    m_curPage.m_page, a, b, c, d, eM, f, (int)FormFillerInterface.eventType.FSCRT_EVENT_LBUTTONDBLCLK, 0, e.Location.X, e.Location.Y);
            }
        }

        private void searchToolStripButton_Click(object sender, EventArgs e)
        {
            m_nSearchPageIndex = m_nCurPageIndex;
            EnableSearchToolStripButon(false);
            searchDlg = new SearchDlg(this);
            searchDlg.Show();
            m_bSearching = true;
            m_bHandTool = false;
        }

        public void EnableSearchToolStripButon(bool enable)
        {
            this.searchToolStripButton.Enabled = enable;
        }

        private void selectToolStripButton_Click(object sender, EventArgs e)
        {
            if (pb == null)
                return;
            m_bDrawAnnot = false;
            pb.Cursor = Cursors.IBeam;
            m_bSelect = true;
            m_bHandTool = false;
        }

        private void handToolStripButton_Click(object sender, EventArgs e)
        {
            pb.Cursor = Cursors.Hand;
            m_bSelect = false;
            EnableSearchToolStripButon(true);
            m_bDrawAnnot = false;
            RenderPageByGE();
            WindowsAPI.HideCaret(pb.Handle);
        }

        private void goToPage(int pageIndex)
        {
            this.panel.AutoScrollPosition = new Point(0, 0);

            if (m_pbSelectList != null)
            {
                foreach (PictureBox selectPb in m_pbSelectList)
                {
                    this.pb.Controls.Remove(selectPb);
                    selectPb.Dispose();
                }
                m_pbSelectList.RemoveRange(0, m_pbSelectList.Count);
                m_pbSelectList = null;
            }
            if (m_rectSelectPageList != null)
            {
                m_rectSelectPageList.RemoveRange(0, m_rectSelectPageList.Count);
            }

            m_nCurPageIndex = pageIndex;
            WindowsAPI.HideCaret(pb.Handle);

            RenderPageByGE();
            
            if (m_nCurPageIndex > 0)
            {
                nextPageToolStripButton.Enabled = true;
                prePageToolStripButton.Enabled = true;
                if (m_nCurPageIndex == m_document.GetPageCount() - 1)
                {
                    nextPageToolStripButton.Enabled = false;
                }
            }
            else if (m_nCurPageIndex == 0)
            {
                prePageToolStripButton.Enabled = false;
                nextPageToolStripButton.Enabled = true;
                if (m_nCurPageIndex == m_document.GetPageCount() - 1)
                {
                    nextPageToolStripButton.Enabled = false;
                }
            }
        }

        private void highLightToolStripMenuItem_Click(object sender, EventArgs e)
        {
            m_bDrawAnnot = true;
            pb.Cursor = Cursors.Cross;
            m_bSelect = false;
            m_bSelecting = false;
            WindowsAPI.HideCaret(this.pb.Handle);
            m_bHandTool = false;
        }

        protected override void DefWndProc(ref System.Windows.Forms.Message m)
        {
            switch (m.Msg)
            {
                case invalidateRectMsg:
                    RenderPageByGE(true);
                    break;
                case getCurrentPageIndexMsg:
                case setCurrentPageIndexMsg:
                case getChangeMarkMsg:
                case alertMsg:
                case beepMsg:
                case getRotationMsg:
                case launchURLMsg:
                case subMitFormMsg:
                case printMsg:
                case mailMsg:
                case getFilePathMsg:
                case executeNamedActionMsg:
                    m_bMouse = true;
                    break;
                default:
                    base.DefWndProc(ref m);
                    break;
            }
        }
#endregion

#region File Opeartion
        //for first load
        private void ShowRecentFileInMenu()
        {
            int index = this.fileToolStripMenuItem.DropDownItems.Count - 1;

            FileStream fs = new FileStream("../../../PDFFileName.ini", FileMode.OpenOrCreate);
            File.SetAttributes("../../../PDFFileName.ini", FileAttributes.Hidden);
            StreamReader sr = new StreamReader(fs);

            if (sr.Peek() >= 0)
            {
                for (int i = 0; i < 2; i++)
                {
                    ToolStripSeparator separatorMenuItem = new ToolStripSeparator();
                    this.fileToolStripMenuItem.DropDownItems.Insert(index, separatorMenuItem);
                }
            }

            while (sr.Peek() >= 0)
            {
                m_sRecentFilePathList.Add(sr.ReadLine());
            }
            sr.Close();

            index = this.fileToolStripMenuItem.DropDownItems.Count - 2;
            int count = m_sRecentFilePathList.Count;
            if (count > recentFileCountInMenu)
                count = recentFileCountInMenu;

            for (int i = 0; i < count; i++)
            {
                ToolStripMenuItem recentFileMenuItem = new ToolStripMenuItem(m_sRecentFilePathList[i]);
                this.fileToolStripMenuItem.DropDownItems.Insert(index, recentFileMenuItem);
                index++;
                recentFileMenuItem.Click += new EventHandler(recentFileMenuItem_Click);
            }
        }

        private void SetFilePathToRecentMenu(string filePath)
        {
            if (filePath == String.Empty) return;

            int index = 0;
            if (3 == this.fileToolStripMenuItem.DropDownItems.Count)
            {
                index = this.fileToolStripMenuItem.DropDownItems.Count - 1;
                for (int i = 0; i < 2; i++)
                {
                    ToolStripSeparator separatorMenuItem = new ToolStripSeparator();
                    this.fileToolStripMenuItem.DropDownItems.Insert(index, separatorMenuItem);
                }
            }

            index = this.fileToolStripMenuItem.DropDownItems.Count - 3;

            for (int i = index; i > 0; i--)
            {
                if (this.fileToolStripMenuItem.DropDownItems[i].GetType().Name.Equals("ToolStripSeparator"))
                    break;
                this.fileToolStripMenuItem.DropDownItems.RemoveAt(i);
            }

            for (int i = m_sRecentFilePathList.Count - 1; i >= 0; i--)
            {
                if (filePath.Equals(m_sRecentFilePathList[i]))
                    m_sRecentFilePathList.RemoveAt(i);
            }

            if (File.Exists(filePath))
            {
                if (m_sRecentFilePathList.Count == recentFileCountInMenu)
                {
                    m_sRecentFilePathList.RemoveAt(m_sRecentFilePathList.Count - 1);
                }
                m_sRecentFilePathList.Insert(0, filePath);
            }
            else
                MessageBox.Show("File is not exist!");

            int count = m_sRecentFilePathList.Count;
            if (m_sRecentFilePathList.Count >= recentFileCountInMenu)
                count = recentFileCountInMenu;

            index = this.fileToolStripMenuItem.DropDownItems.Count - 2;
            for (int i = 0; i < count; i++)
            {
                ToolStripMenuItem recentFileMenuItem = new ToolStripMenuItem(m_sRecentFilePathList[i]);
                this.fileToolStripMenuItem.DropDownItems.Insert(index + i, recentFileMenuItem);
                recentFileMenuItem.Click += new EventHandler(recentFileMenuItem_Click);
            }
        }

        private void RemoveFilePathToRecentMenu(string filePath)
        {
            if (filePath == String.Empty) return;

            int index = this.fileToolStripMenuItem.DropDownItems.Count - 3;

            for (int i = index; i > 0; i--)
            {
                if (this.fileToolStripMenuItem.DropDownItems[i].GetType().Name.Equals("ToolStripSeparator"))
                    break;
                this.fileToolStripMenuItem.DropDownItems.RemoveAt(i);
            }

            for (int i = m_sRecentFilePathList.Count - 1; i >= 0; i--)
            {
                if (filePath.Equals(m_sRecentFilePathList[i]))
                {
                    m_sRecentFilePathList.RemoveAt(i);
                    break;
                }
            }

            int count = m_sRecentFilePathList.Count;
            if (m_sRecentFilePathList.Count >= recentFileCountInMenu)
                count = recentFileCountInMenu;

            index = this.fileToolStripMenuItem.DropDownItems.Count - 2;
            for (int i = 0; i < count; i++)
            {
                ToolStripMenuItem recentFileMenuItem = new ToolStripMenuItem(m_sRecentFilePathList[i]);
                this.fileToolStripMenuItem.DropDownItems.Insert(index + i, recentFileMenuItem);
                recentFileMenuItem.Click += new EventHandler(recentFileMenuItem_Click);
            }
        }

        private bool openPDFFile(string pdfFileName)
        {
            if (pdfFileName == String.Empty || (pdfFileName.Equals(m_fileName) && m_document != null))
                return true;

            panel.AutoScrollPosition = new Point(0, 0);

            if (m_document != null)
            {
                CloseDocumentAndFile();
            }

            //2.Open PDF File
            IntPtr file = Base.openFileW_win(pdfFileName);
            if (file != IntPtr.Zero)
                m_file = file;
            else
            {
                MessageBox.Show("Open PDF file error!");
                return false;
            }
            //3.Open PDF document
            m_document = new Document();
            int ret = m_document.loadDocument(m_file, "", 0);

            while (ret == (int)CRTInterface.retCode.FSCRT_ERRCODE_PASSWORD)
            {
                PasswordDlg pwDlg = new PasswordDlg();
                DialogResult diaResult = pwDlg.ShowDialog();

                if (pwDlg.m_password != String.Empty)
                {
                    ret = m_document.loadDocument(m_file, pwDlg.m_password, 0);
                    if (ret != (int)CRTInterface.retCode.FSCRT_ERRCODE_PASSWORD)
                        break;
                    else
                        m_fileName = pdfFileName;
                }

                if (DialogResult.Cancel == diaResult && pwDlg.m_pressKey != 13)
                {
                    m_document = null;
                    return false;
                }
                MessageBox.Show("Invalid password, please enter password again!");
            }

            if (m_document.m_document == IntPtr.Zero || ret != (int)CRTInterface.retCode.FSCRT_ERRCODE_SUCCESS)
            {
                closePDFFile();
                MessageBox.Show("Load PDF document error!");
            }
            else
            {
                DocInterface.FSDK_SetActionHandler(m_document.m_document, this.Handle);
                m_document.loadForm();
                m_document.loadAllPages();
                m_document.beginFormFiller(this.Handle);

                m_fileName = pdfFileName;
                SetFilePathToRecentMenu(pdfFileName);
                RenderPageByGE();
                m_curPage = m_document.getPage(m_nCurPageIndex);
                if (m_nCurPageIndex < m_document.GetPageCount() - 1)
                {
                    this.nextPageToolStripButton.Enabled = true;
                }
                EnableButton(true);

                return true;
            }
            return false;
        }

        private void recentFileMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem recentFileMenuItem = sender as ToolStripMenuItem;
            if (!File.Exists(recentFileMenuItem.Text))
            {
                MessageBox.Show("File " + recentFileMenuItem.Text +  " is not exist!");
                RemoveFilePathToRecentMenu(recentFileMenuItem.Text);
                return;
            }
            if (!openPDFFile(recentFileMenuItem.Text))
            {
                RemoveFilePathToRecentMenu(recentFileMenuItem.Text);
            }
        }

        private bool CloseDocumentAndFile()
        {
            WindowsAPI.HideCaret(pb.Handle);

            m_bSelect = false;
            m_bStartSelect = false;
            m_bSelecting = false;
            m_fZoom = 1.0f;
            this.pb.Cursor = Cursors.Hand;

            if (searchDlg != null)
            {
                searchDlg.Close();
                searchDlg = null;
            }

            EnableButton(false);
            this.closeToolStripMenuItem.Enabled = false;
            this.prePageToolStripButton.Enabled = false;
            this.nextPageToolStripButton.Enabled = false;

            pb.Image = null;

            if (m_pbSearchList != null)
            {
                foreach (PictureBox searchPb in m_pbSearchList)
                {
                    this.pb.Controls.Remove(searchPb);
                    searchPb.Dispose();
                }
                m_pbSearchList.RemoveRange(0, m_pbSearchList.Count);
                m_pbSearchList = null;
            }

            if (m_pbSelectList != null)
            {
                foreach (PictureBox selectPb in m_pbSelectList)
                {
                    this.pb.Controls.Remove(selectPb);
                    selectPb.Dispose();
                }
                m_pbSelectList.RemoveRange(0, m_pbSelectList.Count);
                m_pbSelectList = null;
            }

            if (m_document != null)
            {
                m_document.closeDocument();
                m_document = null;
            }
            if (m_file != IntPtr.Zero)
            {
                closePDFFile();
                m_file = IntPtr.Zero;
            }
            m_nCurPageIndex = 0;
            return true;
        }

        private bool closePDFFile()
        {
            if (m_file == IntPtr.Zero)
                return true;
            else
            {
                if ((int)CRTInterface.retCode.FSCRT_ERRCODE_SUCCESS == Base.closeFile(m_file))
                {
                    m_file = IntPtr.Zero;
                    return true;
                }
                else
                    return false;
            }
        }

        private void WriteFilePathToIni()
        {
            int count = m_sRecentFilePathList.Count;
            if (m_sRecentFilePathList.Count > recentFileCountInMenu)
                count = recentFileCountInMenu;

            string filePath = "";
            for (int j = 0; j < count; j++)
                filePath += m_sRecentFilePathList[j] + "\n";

            FileStream fs = new FileStream("../../../PDFFileName.ini", FileMode.OpenOrCreate);
            File.SetAttributes("../../../PDFFileName.ini", FileAttributes.Hidden);

            StreamWriter sw = new StreamWriter(fs);

            sw.Write(filePath);
            sw.Flush();
            sw.Close();
        }
#endregion

        public void SearchText(string strFind, bool bCase, bool bWholeword, int Direction)
        {
	        if (null == m_document)
		        return;

            m_bSearching = true;
            if (m_pbSearchList != null)
            {
                foreach (PictureBox searchPb in m_pbSearchList)
                {
                    this.pb.Controls.Remove(searchPb);
                    searchPb.Dispose();
                }
                m_pbSearchList.RemoveRange(0, m_pbSearchList.Count);
                m_pbSearchList = null;
            }

            m_curPage = m_document.getPage(m_nCurPageIndex);
            if (null == m_curPage)
		        return;

            if (IntPtr.Zero == m_curPage.getTextPage())
	        {
                bool bRet = m_curPage.loadTextPage();
		        if (!bRet)
			        return;
	        }

	        int nFlag = 0;
	        if(bCase) {nFlag |= (int)Page.searchFlag.FSCRT_SEARCHFLAG_MATCHCASE; }
	        if(bWholeword) {nFlag |= (int)Page.searchFlag.FSCRT_SEARCHFLAG_MATCHWHOLEWORD; }

	        if(Direction > 0)
	        {
		        Direction = -1;
	        }

            if (!strFind.Equals(m_strSearchText) || nFlag != m_nSearchFlag || m_nCurPageIndex != m_nSearchPageIndex)
            {
                int ret = m_curPage.startSearch(strFind, nFlag, Direction);
                if (ret != (int)CRTInterface.retCode.FSCRT_ERRCODE_SUCCESS)
                {
                    return;
                }
                m_nSearchFlag = nFlag;
                m_strSearchText = strFind;
            }

	        bool bIsmatch;
	        if(Direction == 0) //down
                bIsmatch = m_document.getPage(m_nCurPageIndex).findNext();
	        else
                bIsmatch = m_document.getPage(m_nCurPageIndex).findPrev();

	        int nNextPageIndex = 0;
	        if (!bIsmatch)
	        {
                bIsmatch = m_curPage.continueSearch(Direction, strFind, nFlag, Direction, ref nNextPageIndex);
		        if (!bIsmatch)
		        {
                    RenderPageByGE();
                    MessageBox.Show("Find complete!", "Information", MessageBoxButtons.OK);
		        }
		        else
		        {
                    m_curPage = m_document.getPage(nNextPageIndex);
		        }
	        }

	        if (bIsmatch)
	        {
		        GetSearchTextRect();
	        }
        }

        public void releaseSearchText()
        {
            if (m_document == null) return;

            for (int i = 0; i < m_document.GetPageCount(); i++)
            {
                Page pageTmp = m_document.getPage(i);
                if (pageTmp != null)
                    pageTmp.releaseTextPage();
            }
        }

        private void GetSearchTextRect()
        {
	        int ret = (int)CRTInterface.retCode.FSCRT_ERRCODE_ERROR;
	
	        if (m_document.getPage(m_nCurPageIndex) == null)
		        return;

            if (m_document != null && m_nCurPageIndex != m_curPage.m_nPageIndex)
	        {
                goToPage(m_curPage.m_nPageIndex);
	        }
	        m_nSearchPageIndex = m_nCurPageIndex;

            IntPtr textSearch = m_curPage.m_textSearch;

	        if (textSearch == IntPtr.Zero)
		        return;
	
	        IntPtr textSelection = IntPtr.Zero;
	        int piecesCount = 0;

	        ret = PageInterface.FSDK_GetTextSelection(textSearch, ref textSelection);
	        if ((int)CRTInterface.retCode.FSCRT_ERRCODE_SUCCESS != ret)
		        return;

            ret = PageInterface.FSDK_CountTextSelectionPieces(textSelection, ref piecesCount);
	        if ((int)CRTInterface.retCode.FSCRT_ERRCODE_SUCCESS != ret)
		        return;

            if (m_rectSearchPageList != null)
                m_rectSearchPageList.RemoveRange(0, m_rectSearchPageList.Count);
            
	   
            for (int i = 0; i<piecesCount; i++)
            {
                if (i == 0 && m_rectSearchPageList == null)
                {
                    m_rectSearchPageList = new List<RectangleF>();
                }
                float left = 0.0f, right = 0.0f, top = 0.0f, bottom = 0.0f;
                ret = PageInterface.FSDK_GetTextSelectionPiectRectF(textSelection, i, ref left, ref top, ref right, ref bottom);
                RectangleF rectf = new RectangleF(left, top, right - left, bottom - top);
                m_rectSearchPageList.Add(rectf);
            }
	        PageInterface.FSDK_ReleaseTextSelection(textSelection);
            RenderPageByGE();
        }
        
        private void DrawSearchText(RectangleF rc)
        {
	        if(m_nSearchPageIndex != m_nCurPageIndex)
		        return;

            float left = rc.Left, right = rc.Right, top = rc.Top, bottom = rc.Bottom;
            m_document.getPage(m_nSearchPageIndex).transformPageMatrixToRectf(0, 0, m_fZoom, ref left, ref top, ref right, ref bottom);

            RectangleF rect = new RectangleF(left, top, right - left, bottom - top);

            Point pt = new Point((int)rect.Right, (int)rect.Bottom);

            panel.AutoScrollPosition = new Point(0, 0);
            if (pt.Y > panel.ClientRectangle.Bottom)
            {
                int offset = Math.Abs(pt.Y - panel.ClientRectangle.Bottom);
                panel.AutoScrollPosition = new Point(0, offset);
            }

            float leftTmp = Math.Min(left, right); 
            float rightTmp = Math.Max(left, right);
	        float topTmp = Math.Min(top, bottom);
	        float bottomTmp = Math.Max(top, bottom);

            PictureBox searchPb = new PictureBox();
            searchPb.Left = (int)(leftTmp + 0.5);
            searchPb.Top = (int)(topTmp + 0.5);
            searchPb.Size = new Size((int)(rightTmp - leftTmp), (int)(bottomTmp - topTmp));
            Color c = Color.FromArgb(0x95, 0xff, 0xff, 0x00);
            searchPb.BackColor = c;

            this.pb.Controls.Add(searchPb);
            if (m_pbSearchList != null)
                m_pbSearchList.Add(searchPb);
        }

        private void DrawSelectText(RectangleF rc)
        {
            float left = rc.Left, right = rc.Right, top = rc.Top, bottom = rc.Bottom;
            m_curPage.transformPageMatrixToRectf(0, m_iVViewPosition, m_fZoom, ref left, ref top, ref right, ref bottom);

            RectangleF rect = new RectangleF(left, top, right - left, bottom - top);

            float leftTmp = Math.Min(left, right);
            float rightTmp = Math.Max(left, right);
            float topTmp = Math.Min(top, bottom);
            float bottomTmp = Math.Max(top, bottom);

            PictureBox selectPb = new PictureBox();

            selectPb.Left = (int)(leftTmp + 0.5);
            selectPb.Top = (int)(topTmp + 0.5 - m_iVViewPosition);
            selectPb.Size = new Size((int)(rightTmp - leftTmp), (int)(bottomTmp - topTmp));
            Color c = Color.FromArgb(0x20, 0x00, 0x00, 0xff);
            selectPb.BackColor = c;

            this.pb.Controls.Add(selectPb);
        
            if (m_pbSelectList != null)
                m_pbSelectList.Add(selectPb);
        }

        bool GetCharIndexByPoint(PointF pt, ref int index)
        {
	        float x = pt.X;
            float y = pt.Y;
	        if (m_curPage != null && m_curPage.m_textPage != IntPtr.Zero)
	        {
                m_curPage.transformPageMatrixToPointF(0, 0, m_fZoom, ref x, ref y);
                PageInterface.FSDK_GetCharIndexAtPos(m_curPage.m_textPage, x, y, 10.0f, ref index); 
	        }
	
	        if (-1 == index)
	        {
		        return false;
	        }
	        return true;
        }

        int GetStartCharIndexByPoint(PointF pt)
        {
	        int indexTmp = 0, index = -1;
	        if(!GetCharIndexByPoint(pt, ref indexTmp))
		        return -1;

            float page_x = pt.X, page_y = pt.Y;
            m_curPage.transformPageMatrixToPointF(0, 0, m_fZoom, ref page_x, ref page_y);

            PageInterface.FSDK_GetStartCharIndexByPoint(m_curPage.m_textPage, page_x, indexTmp, m_nStartCharIndex, m_nEndCharIndex, ref index);
            return index;
        }

        void GetSelectRect(int starIndex, int endIndex)
        {
	        int temp, nCount;
	        int ret = -1;
	        nCount = endIndex - starIndex;
	        if (starIndex == endIndex )
	        {
                if (m_rectSelectPageList != null)
		            m_rectSelectPageList.RemoveRange(0, m_rectSelectPageList.Count);
		        return;
	        }
	
	        if (nCount < 0)
	        {
		        temp = endIndex;
		        endIndex = starIndex;
		        starIndex = temp;

		        nCount = Math.Abs(nCount);
	        }
	
	        nCount++;

	        if(m_curPage.m_textPage == IntPtr.Zero)
	        {
		        m_curPage.loadTextPage();
	        }

	        ret = PageInterface.FSDK_TextPageSelectByRange(m_curPage.m_textPage, starIndex, nCount, ref m_TextSelection);

            if (m_rectSelectPageList != null)
                m_rectSelectPageList.RemoveRange(0, m_rectSelectPageList.Count);

	        int num = -1;
	        ret = PageInterface.FSDK_CountTextSelectionPieces(m_TextSelection, ref num);

            //float lastLeft = 0.0f, lastRight = 0.0f, lastTop = 0.0f, lastBottom = 0.0f;

	        for (int i = 0; i<num; i++)
	        {
                float left = 0.0f, right = 0.0f, top = 0.0f, bottom = 0.0f;
		        ret = PageInterface.FSDK_GetTextSelectionPiectRectF(m_TextSelection, i, ref left, ref top, ref right, ref bottom);

		      /*  if (0 != i)
		        {
			        float nHeight = 0; 
			        if (lastTop - lastBottom > top - bottom)
			        {
				        nHeight = lastTop - lastBottom;
			        }
			        else
			        {
				        nHeight = top - bottom;
			        }

	
			        if ( lastTop - top < nHeight)
			        {
				        left = lastLeft;
				        if (bottom > lastBottom )
				        {
					        bottom = lastBottom;
				        }

				        if (top < lastTop )
				        {
					        top = lastTop;
				        }
				        if (m_rectSelectPageList.Count != 0)
				        {
					        int index = m_rectSelectPageList.Count;
					        m_rectSelectPageList.RemoveAt(index-1);
				        }
 			        }
 		        }
 		        lastTop = top; lastLeft = left; lastRight = right; lastBottom = bottom;*/

                RectangleF rectF = new RectangleF(left, top, right - left, bottom - top);
 		        m_rectSelectPageList.Add(rectF);
 	        }
        }

        void CreateCaret(Point pt)
        {
	        int nIndex = 0;

	        if(!GetCharIndexByPoint(pt, ref nIndex))
		        return;

            float page_x = pt.X , page_y = pt.Y;
            m_curPage.transformPageMatrixToPointF(0, m_iVViewPosition, m_fZoom, ref page_x, ref page_y);

            float left = 0.0f, top = 0.0f, right = 0.0f, bottom = 0.0f;
            PageInterface.FSDK_SelectTextCaretRect(m_curPage.m_textPage, page_x, nIndex, ref left, ref top, ref right, ref bottom);
		
	        RectangleF rect = new RectangleF(left, top, right - left, bottom - top);

	        m_rCaret = rect;

            m_curPage.transformPageMatrixToRectf(0, 0, m_fZoom, ref left, ref top, ref right, ref bottom);

            if ((int)Page.pageRotation.FSCRT_PAGEROTATION_90 == m_curPage.m_pageRotation
                || (int)Page.pageRotation.FSCRT_PAGEROTATION_270 == m_curPage.m_pageRotation)
                WindowsAPI.CreateCaret(pb.Handle, IntPtr.Zero, (int)Math.Abs(right - left), 2);
            else
                WindowsAPI.CreateCaret(pb.Handle, IntPtr.Zero, 2, (int)Math.Abs(bottom - top));

            WindowsAPI.SetCaretPos((int)left, (int)top);
            WindowsAPI.ShowCaret(pb.Handle);

           
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PageInterface.FSDK_CopyTextSelecionCharsToCilpBoard(m_TextSelection);
        }
    }
}
